#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/aliases/cookbook_aliases.rb'
# =========================================================================== #
require 'rbt/constants/file_and_directory_constants.rb'
require 'rbt/requires/require_yaml.rb'

module RBT

  # ========================================================================= #
  # === RBT::COOKBOOK_ALIASES
  #
  # All aliases for the cookbook files are stored here.
  #
  # For the individual yaml files, the constant
  # INDIVIDUAL_COOKBOOKS_DIRECTORY should be
  # used instead.
  # ========================================================================= #
  _ = RBT.file_cookbook_aliases
  if File.exist? _
    COOKBOOK_ALIASES = YAML.load_file(_) # load up the file cookbook_aliases.yml
  else
    COOKBOOK_ALIASES = {} # This will be for most everyone else.
  end

  # ========================================================================= #
  # === RBT.cookbook_aliases?
  # ========================================================================= #
  def self.cookbook_aliases?
    COOKBOOK_ALIASES
  end

  # ========================================================================= #
  # === RBT.cookbook_aliases
  #
  # This will return the registered/available keys, in a sorted manner.
  #
  # This will have about 331 entries (as of August 2018).
  # ========================================================================= #
  def self.cookbook_aliases
    COOKBOOK_ALIASES.keys.sort
  end; self.instance_eval { alias aliases? cookbook_aliases } # === Cookbook.aliases?

end

if __FILE__ == $PROGRAM_NAME
  puts 'How many aliases are registered? '+
        RBT.cookbook_aliases.size.to_s 
end