#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/base/change_directory.rb'
# =========================================================================== #
module RBT

class Base # === RBT::Base

  # ========================================================================= #
  # === change_directory                                             (cd tag)
  #
  # Change into the given directory.
  #
  # The second argument to this method can be used to do additional tasks,
  # such as creating the directory if it does not yet exist.
  # ========================================================================= #
  def change_directory(
      i, optional_arguments = nil
    )
    be_verbose = false
    # ======================================================================= #
    # First, handle Symbol-keywords given as first argument.
    # ======================================================================= #
    case i
    when :log_directory
      i = log_directory?
    end
    if block_given?
      yielded = yield
      case yielded
      when :be_verbose
        be_verbose = true
      end
    end 
    # ======================================================================= #
    # Past this point, the variable `i` ought to be a String.
    # ======================================================================= #
    i = i.to_s unless i.is_a? String
    case optional_arguments
    when :be_quiet,
         :quiet,
         :silent,
         :be_silent,
         :do_not_report_anything
        be_verbose = false
    when true,
         :be_verbose
      be_verbose = true
    when :create_the_directory_if_it_does_not_exist
      FileUtils.mkdir_p(i) unless File.directory? i
    when :verbose_create_the_directory_if_it_does_not_exist,
         :ensure_that_the_directory_exists
      unless File.directory? i
        if be_verbose
          opnn; e "The directory at `#{sdir(i)}` does not exist."
          opnn; e 'We will create it now.'
        end
        FileUtils.mkdir_p(i)
      end
    end
    i = i.dup if i.frozen?
    i << '/' unless i.end_with? '/'
    i = rds(i) # Added this here as of May 2014.
    # ======================================================================= #
    # Perform the cd-action next.
    # ======================================================================= #
    if File.directory? i
      if be_verbose
        e "Changing into the directory `#{sdir(i.to_s)}` now."
      end
      Dir.chdir(i) if File.directory? i # This is the actual cd-action.
    end
  end; alias chdir      change_directory # === chdir
       alias change_dir change_directory # === change_dir
       alias cd         change_directory # === cd

end; end