#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/base/colours.rb'
# =========================================================================== #
require 'rbt/colours/colours.rb'

module RBT

class Base # === RBT::Base

  # ========================================================================= #
  # === KONSOLE
  # ========================================================================= #
  KONSOLE = Colours

  # ========================================================================= #
  # === ALL_COLOUR_METHODS
  # ========================================================================= #
  ALL_COLOUR_METHODS = ::Colours::AllColourMethods

  # ========================================================================= #
  # === ARRAY_KDE_KONSOLE_COLOURS_IN_USE
  #
  # Next, we will define the KONSOLE Colours that can be used.
  # ========================================================================= #
  ARRAY_KDE_KONSOLE_COLOURS_IN_USE = %w(
    aliceblue
    aquamarine
    blueviolet
    cadetblue
    cornflowerblue
    crimson
    cyan
    darkcyan
    darkgoldenrod
    darkorchid
    darkseagreen
    darkslateblue
    darkturquoise
    deepskyblue
    dodgerblue
    firebrick
    gold
    gray
    grey
    green
    hotpink
    khaki
    lightblue
    lightseagreen
    lightgoldenrodyellow
    lightgreen
    lightsalmon
    lightskyblue
    lightslategray
    lightsteelblue
    limegreen
    mediumaquamarine
    mediumorchid
    mediumpurple
    mediumseagreen
    mediumslateblue
    mediumspringgreen
    mediumturquoise
    mediumvioletred
    navajowhite
    olivedrab
    orange
    orangebrown
    orangered
    orchid
    palevioletred
    paleturquoise
    peru
    plum
    powderblue
    royalblue
    salmon
    saddlebrown
    sandybrown
    seagreen
    silver
    skyblue
    slateblue
    slategray
    springgreen
    steelblue
    rosybrown
    teal
    turquoise
    violet
    yellow
    yellowgreen
  )

  # ========================================================================= #
  # === use_colours?
  #
  # Query whether we may use colours or not.
  # ========================================================================= #
  def use_colours?
    @use_colours
  end; alias we_may_use_colours? use_colours? # === we_may_use_colours?

  # ========================================================================= #
  # === enable_colours
  # ========================================================================= #
  def enable_colours
    @use_colours = true
  end

  # ========================================================================= #
  # === use_colours=
  # ========================================================================= #
  def use_colours=(i = true)
    @use_colours = i
  end

  # ========================================================================= #
  # === edir
  # ========================================================================= #
  def edir(i = '')
    e sdir(i)
  end

  # ========================================================================= #
  # === ecomment
  # ========================================================================= #
  def ecomment(i = '')
    ::Colours.ecomment(i)
  end

  # ========================================================================= #
  # === simp
  # ========================================================================= #
  def simp(i = '')
    return ::Colours.simp(i) if use_colours?
    i
  end; alias simportant simp # === simportant

  # ========================================================================= #
  # === sdir
  # ========================================================================= #
  def sdir(i = '')
    return ::Colours.sdir(i) if use_colours?
    i
  end

  # ========================================================================= #
  # === sfile
  # ========================================================================= #
  def sfile(i = '')
    return ::Colours.sfile(i) if use_colours?
    i
  end

  # ========================================================================= #
  # === efancy
  # ========================================================================= #
  def efancy(i = '')
    e sfancy(i)
  end

  # ========================================================================= #
  # === sfancy
  # ========================================================================= #
  def sfancy(i = '')
    return ::Colours.sfancy(i) if use_colours?
    i
  end
 
  # ========================================================================= #
  # === ssym
  # ========================================================================= #
  def ssym(i)
    return Colours.ssym(i) if use_colours?
    i
  end

  # ========================================================================= #
  # === simp
  # ========================================================================= #
  def eimp(i)
    e simp(i)
  end

  # ========================================================================= #
  # === rev
  # ========================================================================= #
  def rev
    ::Colours.rev
  end

  # ========================================================================= #
  # === colourize_directory_for_system_results
  #
  # This method is to colourize directories that are given via system()
  # or similar calls. In theory, we could use sdir(), but I wanted to
  # have another colour.
  # ========================================================================= #
  def colourize_directory_for_system_results(i)
    lightslategray(i)
  end

  # ========================================================================= #
  # === KDE colours
  #
  # Define the KDE colours in use next.
  # ========================================================================= #
  ARRAY_KDE_KONSOLE_COLOURS_IN_USE.each {|this_colour|
    define_method(this_colour.to_sym) {|i = ''|
      return KONSOLE.send(this_colour.to_sym, i) if use_colours?
      i
    }
  }

  # ========================================================================= #
  # === eparse
  # ========================================================================= #
  def eparse(i)
    Colours.eparse(i)
  end

  # ========================================================================= #
  # === disable_colours
  #
  # Use this method if you wish to disable colours. Invoke it only when
  # you really do wish to disable the colours.
  # ========================================================================= #
  def disable_colours
    @use_colours = false
  end; alias do_not_use_colours disable_colours # === do_not_use_colours
       alias disable_colors     disable_colours # === disable_colors

  # ========================================================================= #
  # === colourize_this_file_path
  #
  # This method will colourize a "file path". So for example, if you have
  # a file path such as "/opt/foo/bar.rb", then this method will colourize
  # the directory in one colour, and the file itself (bar.rb) in another
  # colour.
  #
  # The whole idea behind this is to be easily able to separate which
  # part is the directory and which part is the file.
  #
  # If no '/' has been given then the input will be returned unmodified.
  #
  # The colour for this can be controlled by the input-arguments.
  # ========================================================================= #
  def colourize_this_file_path(
      i,
      colour1 = :slateblue, # <- This is the colour for the directory.
      colour2 = :royalblue  # <- This is the colour for the file.
    )
    if i.include? '/'
      if use_colours?
        dirname  = File.dirname(i)
        basename = File.basename(i)
        dirname  = send(colour1, dirname+'/')
        basename = send(colour2, basename)
        i = "#{dirname}#{basename}"
      end
    end
    i
  end

  # ========================================================================= #
  # === ecrimson
  #
  # This is mostly for debug-related output, e. g. colourize something
  # via the colour red (or rather, crimson) quickly.
  # ========================================================================= #
  def ecrimson(i = '')
    e crimson(i)
  end

  # ========================================================================= #
  # === colourize_this_error
  #
  # This method in particular attempts to colourize some errors.
  # ========================================================================= #
  def colourize_this_error(i)
    return darkgoldenrod(i) if use_colours?
    i
  end

  # ========================================================================= #
  # === colourize_this_warning
  # ========================================================================= #
  def colourize_this_warning(i)
    return firebrick(i) if use_colours?
    i
  end; alias colourize_for_warnings colourize_this_warning # === colourize_for_warnings

  # ========================================================================= #
  # === swarn
  # ========================================================================= #
  def swarn(i = '')
    return Colours.swarn(i) if use_colours?
    i
  end

  # ========================================================================= #
  # === ewarn
  # ========================================================================= #
  def ewarn(i = '')
    e swarn(i) # <- This method will already check for use of colours.
  end

end; end