#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/base/copy_files.rb'
# =========================================================================== #
require 'fileutils'

module RBT

class Base # === RBT::Base

  # ========================================================================= #
  # === copy_files                          (copy tag, cp tag, copy file tag)
  #
  # Use this if you need to copy a file.
  # ========================================================================= #
  def copy_files(
      from,
      to         = Dir.pwd,
      be_verbose = false
    )
    case be_verbose # case tag
    when :be_verbose
      be_verbose = true
    end
    case to
    when '.'
      to = Dir.pwd
    end
    # ======================================================================= #
    # The next method is defined in the file 'prototype.rb'.
    # ======================================================================= #
    from = sanitize_for_environment_variable(from) if from.include? '$'
    to   = sanitize_for_environment_variable(to)   if to.include? '$'
    # ======================================================================= #
    # Since as of June 2011, we will also automatically create
    # the base directory should it not yet exist.
    # ======================================================================= #
    unless File.exist? File.dirname(to)
      create_directory(File.dirname(to))
    end
    if File.exist? from
      if be_verbose
        e "Now copying the file `#{sfile(from)}` to `#{sfile(to)}`."
      end
      FileUtils.cp(from, to)
    else
      e "Can not copy the file `#{sfile(from)}` as it "\
        "does not appear to exist."
    end
  end; alias copy_file copy_files # === copy_file
       alias copy copy_files      # === copy (copy tag)
       alias cp   copy_files      # === cp

end; end