#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/base/create_directory.rb'
# =========================================================================== #
require 'rbt/colours/sdir.rb'

module RBT

class Base # === RBT::Base

  # ========================================================================= #
  # === create_directory                                          (mkdir tag)
  #
  # Consistently use this when you want to create a directory
  # for the RBT-Project.
  # ========================================================================= #
  def create_directory(
      i           = '/Users/Packages/',
      be_verbose  = true,
      permissions = :default
    )
    unless i.end_with? '/'
      i = i.dup if i.frozen?
      i << '/'
    end
    if block_given?
      yielded = yield
      case yielded
      when :be_quiet
        be_verbose = false
      when :be_verbose
        be_verbose = true
      end
    end
    unless File.directory? i
      if be_verbose.is_a? Hash
        # =================================================================== #
        # We have to handle other keys first, in particular the permissions
        # key:
        # =================================================================== #
        if be_verbose.has_key? :permissions
          permissions = be_verbose.delete(:permissions)
        end
        if be_verbose.has_key? :be_quiet
          be_verbose = be_verbose.delete(:be_quiet)
        elsif be_verbose.has_key? :verbosity
          be_verbose = be_verbose.delete(:verbosity)
        end
      end
      case be_verbose
      when :be_quiet,
           :be_silent
        be_verbose = false
      end
      unless File.directory? i
        if be_verbose
          opnn; e "Creating the directory `#{sdir(i)}` now."
        end
        RBT.create_directory(
          i, permissions, :be_quiet, :do_not_use_opn
        ) # ^^^ We are already verbose above, and handle opn on
          # our own.
      end
    end
  end; alias mkdir                                     create_directory # === mkdir (mkdir tag)
       alias ensure_that_this_directory_exists         create_directory # === ensure_that_this_directory_exists
       alias create_directory_if_it_does_not_yet_exist create_directory # === create_directory_if_it_does_not_yet_exist

end; end