#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file will store code that has something to do with expanded
# cookbooks - that is, cookbook entries that reside in a special
# directory, and that are "complete" and correct.
# =========================================================================== #
# require 'rbt/base/expanded_cookbooks.rb'
# =========================================================================== #
require 'rbt/constants/file_and_directory_constants.rb'

module RBT
             
class Base

  # ========================================================================= #
  # === does_this_expanded_cookbook_file_exist_for_this_program?
  # ========================================================================= #
  def does_this_expanded_cookbook_file_exist_for_this_program?(i)
    File.exist? path_to_this_expanded_cookbooks_dataset(i)
  end; alias does_the_expanded_cookbook_file_exist_for_this_program? does_this_expanded_cookbook_file_exist_for_this_program? # === does_the_expanded_cookbook_file_exist_for_this_program?

  # ========================================================================= #
  # === path_to_this_expanded_cookbooks_dataset
  #
  # This method shall attempt to return the full path to the expanded
  # cookbooks directory dataset of a given program.
  #
  # For example, if the input is 'htop' then this method should return
  # 'htop.yml'.
  # ========================================================================= #
  def path_to_this_expanded_cookbooks_dataset(i)
    i = i.dup
    i = remove_archive_at_the_end(
      File.basename(i)
    )
    if i =~ /\d+/
      i = ProgramInformation.return_program_name(i)
    end
    i << '.yml' unless i.end_with? '.yml'
    "#{directory_expanded_cookbooks?}#{i}"
  end

  # ========================================================================= #
  # === directory_expanded_cookbooks?
  # ========================================================================= #
  def directory_expanded_cookbooks?
    RBT.directory_expanded_cookbooks?
  end; alias directory_expanded_cookbooks          directory_expanded_cookbooks? # === directory_expanded_cookbooks
       alias base_dir_to_store_expanded_cookbooks? directory_expanded_cookbooks? # === base_dir_to_store_expanded_cookbooks?

  # ========================================================================= #
  # === expanded_cookbooks_directory_exists?
  #
  # This method will try and check to see if the expanded-cookbooks
  # directory exists. If this directory exists then this method will
  # return true; otherwise this method will return false.
  # ========================================================================= #
  def expanded_cookbooks_directory_exists?
    target = directory_expanded_cookbooks?
    File.directory? target
  end; alias expanded_directory_exists?           expanded_cookbooks_directory_exists? # === expanded_directory_exists?
       alias directory_expanded_cookbooks_exists? expanded_cookbooks_directory_exists? # === directory_expanded_cookbooks_exists?

end; end