#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/base/remove_directory.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/rds.rb'

module RBT

class Base

  # ========================================================================= #
  # === remove_directory
  #
  # Consistently use this method in order to remove one or more
  # directories.
  # ========================================================================= #
  def remove_directory(
      this_directory,
      be_verbose = false
    )
    if this_directory.is_a? Array
      this_directory.each {|entry| remove_directory(entry, be_verbose) }
    elsif this_directory.is_a? String
      case be_verbose
      when :be_verbose
        be_verbose = true
      end
      this_directory = rds(this_directory).strip # Don't want double slashes in it.
      if File.directory? this_directory
        case this_directory # case tag
        # =================================================================== #
        # / can never be removed through this method here:
        # =================================================================== #
        when '/' # We will never remove '/', ever.
          e "The #{sdir('/')} directory can not be removed "\
            "via this method." # Tiny "safeguard".
        # =================================================================== #
        # RBT.temp_directory?
        # =================================================================== #
        when RBT.temp_directory?
          e "Will not remove directory `#{sdir(this_directory)}`." # Another tiny "safeguard".
        else
          if File.exist?(this_directory) and (this_directory.size > 1)
            if be_verbose
              e "Removing the directory `#{sdir(this_directory)}'` next."
            end
            FileUtils.rm_rf(this_directory) # Finally remove the directory.
          end
        end
      end
    else
      e "Unknown input: #{this_directory.class}"
    end
  end; alias remove_dir               remove_directory # === remove_dir (Remove directory)
       alias remove_these_directories remove_directory # === remove_these_directories
       alias remove_directories       remove_directory # === remove_directories
       alias remove_this_directory    remove_directory # === remove_this_directory

end; end