#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/base/symlink.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/symlink.rb'
require 'rbt/base/remove_file.rb'

module RBT

class Base

  # ========================================================================= #
  # === symlink                                                 (symlink tag)
  #
  # Wrapper to symlinking something.
  #
  # The first argument should be the existing target.
  #
  # The second argument shall be the name of the new location.
  # ========================================================================= #
  def symlink(
      existing,
      new_location = nil,
      be_verbose   = true,
      use_colours  = RBT.use_colours? # Here we need colours/colours.rb
    )
    if block_given?
      yielded = yield
      case yielded
      when :be_quiet
        be_verbose = false
      end
    end
    ::RBT.symlink(existing, new_location, be_verbose, use_colours)
  end; alias do_symlink   symlink # === do_symlink
       alias symlink_file symlink # === symlink_file

  # ========================================================================= #
  # === symlink_all_files_from_this_directory_to_that_directory
  # ========================================================================= #
  def symlink_all_files_from_this_directory_to_that_directory(
      this_dir,
      that_dir
    )
    delete_target_file = false
    if block_given?
      yielded = yield
      case yielded
      when :delete_target_file_if_it_exists
        delete_target_file = true
      end
    end
    all_files = Dir[this_dir+'*'].select {|entry| is_file?(entry) }
    all_files.each {|entry|
      if delete_target_file
        _ = that_dir+File.basename(entry)
        if File.exist? _
          remove_file(_)
        end
      end
      symlink(entry, that_dir)
    }
  end

  # ========================================================================= #
  # === remove_symlink
  #
  # Simply remove a symlink. Use this method consistently when you
  # wish to remove any symlink.
  # ========================================================================= #
  def remove_symlink(
      i,
      be_verbose = false
    )
    case be_verbose
    when :be_quiet
      be_verbose = false
    when :be_verbose
      be_verbose = true
    end if be_verbose.is_a? Symlink
    if i.is_a? Array
      i.each {|entry| remove_symlink(entry, be_verbose) }
    else
      if File.symlink?(i)
        if be_verbose
          e "Now deleting the symlink `#{sfancy(i)}`."
        end
        File.delete(i)
      end
    end
  end; alias delete_symlink      remove_symlink # === delete_symlink
       alias delete_this_symlink remove_symlink # === delete_this_symlink
       alias remove_this_symlink remove_symlink # === remove_this_symlink

end; end