#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Autoconf-related code will be gathered in this file.
# =========================================================================== #
# require 'rbt/compile/autoconf.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === decide_whether_to_invoke_the_autoconf_command
  #
  # Whether to run "autoconf" or whether we will not. Only very few
  # programs will need this.
  #
  # Note that the invocation of autoconf must happen AFTER the
  # pre-installation step, and after checking whether sed must be
  # used or not. Afterwards, we can then check whether we will use
  # a build directory, and whether we must use another prefix.
  #
  # A program that may require this is "bridgeutils".
  #
  # Invocation example:
  #
  #   ry bridgeutils
  #
  # ========================================================================= #
  def decide_whether_to_invoke_the_autoconf_command
    if use_autoconf?
      cmd = 'autoconf' # 2>&1'
      cliner { 
        opnn; e "Doing `#{teal(cmd)}` in directory #{sdir_return_pwd}"
        opnn; e 'using this command next:'
        opnn; e sfancy("  #{cmd}")
      }
      `#{cmd}`
    end
  end; alias decide_whether_to_invoke_autoconf_command decide_whether_to_invoke_the_autoconf_command # === decide_whether_to_invoke_autoconf_command
       alias consider_invoking_autoconf                decide_whether_to_invoke_the_autoconf_command # === consider_invoking_autoconf

  # ========================================================================= #
  # === use_autoconf?
  #
  # This method determines as to whether we will use autoconf or whether
  # we will not. By default the value of @internal_hash[:use_autoconf]
  # is nil. If a user modifies this value from the commandline, to
  # either true or false, then we will use that value; otherwise we
  # will use the value defined in the cookbook-dataset for this
  # particular program at hand.
  # ========================================================================= #
  def use_autoconf?
    _ = cookbook_dataset?.use_autoconf? # <- This is the default.
    unless @internal_hash[:use_autoconf].nil?
      _ = @internal_hash[:use_autoconf]
    end
    _
  end

  # ========================================================================= #
  # === do_use_autoconf
  # ========================================================================= #
  def do_use_autoconf
    @internal_hash[:use_autoconf] = true
    opnn; e 'We will invoke autoconf.'
  end

  # ========================================================================= #
  # === do_not_use_autoconf
  # ========================================================================= #
  def do_not_use_autoconf
    @internal_hash[:use_autoconf] = false
    opnn; e 'We will not invoke autoconf.'
  end

end; end