#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/autogen.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === FILE_AUTOGEN
  # ========================================================================= #
  FILE_AUTOGEN = 'autogen.sh'

  # ========================================================================= #
  # === decide_whether_to_invoke_autogen
  #
  # Invoke autogen.sh if specified so, and only if the file autogen.sh
  # actually exists.
  #
  # There may be some situations where it does not make sense to
  # invoke autogen, such as when the build type specifically tells
  # us to use meson.
  # ========================================================================= #
  def decide_whether_to_invoke_autogen
    _ = FILE_AUTOGEN
    if use_autogen?
      if uses_the_meson_build_system?
        opne crimson('The value for autogen is set to ')+
             royalblue('true')+crimson(', meaning "do use autogen".')
        opne crimson('The .yml file specifies to use the meson-build system, though.')
        opne crimson('Autogen is only useful for GNU configure based system, so')
        opne crimson('autogen is set to false for this run - thus, not making '\
                     'use of autogen.')
        e
        opne crimson('If you need autogen, consider changing the information in the')
        opne crimson('.yml file ')+sfile("#{compile_which_program?}.yml")
        e
        opne 'Do note that this behaviour may be changed at a later time.'
        disable_autogen
      end
      if use_autogen? and File.exist?(_)
        do_use_autogen # This is the do-action that invokes autogen.sh
      end
    end
  end

  # ========================================================================= #
  # === use_autogen?
  #
  # Query method over as to whether we will use autogen.
  # ========================================================================= #
  def use_autogen?
    _ = cookbook_dataset_use_autogen? 
    unless @internal_hash[:use_autogen].nil?
      _ = @internal_hash[:use_autogen]
    end
    _
  end

  # ========================================================================= #
  # === do_use_autogen
  #
  # This invokes the autogen script, by using 'sh'.
  #
  # In oldschool shell script code, this means that we will run the
  # following command:
  #
  #   sh autogen.sh
  #
  # ========================================================================= #
  def do_use_autogen(
      use_this_cmd = "sh #{FILE_AUTOGEN}" # 2>&1'
    )
    cliner {
      _ = return_pwd
      opne simp('Doing `')+sfancy(use_this_cmd)+simp('` from ')+sdir(_)
      opne simp('using the following command:')
      opn; efancy '  '+use_this_cmd
    }
    `#{use_this_cmd}`
  end

  # ========================================================================= #
  # === verbose_report_will_we_use_autogen
  #
  # This method will output, in a verbose manner, whether we will use
  # autogen or whether we will not.
  # ========================================================================= #
  def verbose_report_will_we_use_autogen(
      i = compile_which_program?
    )
    will_we_use_autogen = use_autogen?
    if will_we_use_autogen.nil? and cookbook_dataset_use_autogen?
      will_we_use_autogen = true
    end
    if will_we_use_autogen
      opne "Yes, autogen will be used for the "\
           "program `#{sfancy(i)}`."
    else
      opne "No, autogen will #{swarn('NOT')} be used for "\
           "the program #{sfancy(i)}."
    end
  end

  # ========================================================================= #
  # === disable_autogen
  # ========================================================================= #
  def disable_autogen(
      be_verbose = true
    )
    if be_verbose
      opne 'We will not make use of autogen.'
    end
    @internal_hash[:use_autogen] = false # Enable the use of autogen in our dataset here.
  end

  # ========================================================================= #
  # === enable_autogen
  #
  # This method will enable autogen.
  # ========================================================================= #
  def enable_autogen(
      be_verbose = true
    )
    if be_verbose
      opne 'We will make use of autogen.'
    end
    @internal_hash[:use_autogen] = true # Enable the use of autogen in our dataset here.
  end

end; end