#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/blfs.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === try_to_show_the_BLFS_content_of_this_program
  #
  # This method can be used to show the BLFS content of a program on the
  # commandline.
  #
  # Invocation example:
  #   ry gcc --paste-blfs-page
  #   ry libffi --paste-blfs-page
  # ========================================================================= #
  def try_to_show_the_BLFS_content_of_this_program(
      i = compile_which_program?
    )
    # ======================================================================= #
    # First, obtain the remove URL that points to the BLFS page.
    # ======================================================================= #
    remote_url = RBT.return_blfs_page_of_this_program(i)
    if remote_url
      opne "A remote URL has been found, at `#{sfancy(remote_url)}`."
      opne 'We will next display the content of this remote page '\
           'on the commandline.'
      remote_dataset = open(remote_url).read
      opne 'The data was obtained - we will next show the STRING on'
      opne 'the commandline, but we will remove all HTML tags.'
      sanitized_dataset = remote_dataset.gsub(/<\/?[^>]*>/, '')
      sanitized_dataset = sanitized_dataset.split(N).reject {|line|
        line.strip.empty?
      }
      # ===================================================================== #
      # Some HTML entities may have to be re-formatted, for display on
      # the commandline. For example, '&amp;' is actually Ampersand aka
      # '&' token.
      # ===================================================================== #
      if sanitized_dataset.map! {|entry|
        if entry.include? '&amp;'
          entry.gsub!(/&amp;/,'&')
        end
        entry
      }
      end
      cliner {
        sanitized_dataset.each {|line|
          e  sfancy('  '+line)
        }
      }
    else
      opne "No BLFS entry is registered for the program `#{sfancy(i)}`."
    end
  end

  # ========================================================================= #
  # === show_programs_with_a_blfs_entry
  #
  # Invocation example:
  #   ry --show-all-blfs-entries
  # ========================================================================= #
  def show_programs_with_a_blfs_entry
    _ = Dir[individual_cookbooks_dir?+'*.yml']
    matches = _.select {|file|
      dataset = File.read(file)
      dataset.include? ' blfs:'
    }.map {|entry|
      File.basename(entry).sub(/\.yml$/,'')
    }
    unless matches.empty?
      opne "The following #{sandybrown(matches.size.to_s)} programs "\
           "have a #{sfancy('blfs')} entry:"
      e
      matches.each_with_index {|name_of_the_program, index| index += 1
        index = (index.to_s+') ').rjust(5)
        remote_url = return_blfs_page_of_this_program(name_of_the_program)
        e royalblue(index)+slateblue(name_of_the_program.ljust(24))+
          lightblue(remote_url.to_s)
      }
      e
    end
  end

end; end