#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/ccache.rb'
# =========================================================================== #
require 'rbt/utility_scripts/url.rb'

module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === use_ccache?
  # ========================================================================= #
  def use_ccache?
    @internal_hash[:use_ccache]
  end

  # ========================================================================= #
  # === disable_ccache
  #
  # Use this method to temporarily disable ccache, that is - for the
  # current run. If you do not want to use ccache at all, simply set
  # the content of the corresponding .yml file to false.
  # ========================================================================= #
  def disable_ccache
    @internal_hash[:use_ccache] = false
  end

  # ========================================================================= #
  # === notify_the_user_that_ccache_is_not_available
  #
  # Provide some information to the user about the use of ccache.
  # ========================================================================= #
  def notify_the_user_that_ccache_is_not_available
    opnn; ewarn 'You are trying to use ccache, as specified by the '\
          'configuration'
    opnn; ewarn 'file, but ccache could not be found. Please install '\
          'ccache if'
    opnn; ewarn 'you wish to use it. The URL to the latest source would be:'
    e
    e sfancy("  #{::RBT.return_url1_of_this_program(:ccache)}")
    e
    opnn; ewarn 'If you do not want to use ccache, you can '\
                'disable ccache, via'
    opnn; ewarn 'either of the following two options:'
    e
    e sfancy('  rbt --permanently-disable-ccache')
    e
    e sfancy('  rbt --disable-ccache # this variant will avoid '\
             'ccache just for this run')
    e
    opnn; ewarn 'RBT will now continue without ccache.'
  end

  # ========================================================================= #
  # === verbose_disable_ccache
  # ========================================================================= #
  def verbose_disable_ccache
    opne 'We will not use ccache.'
    disable_ccache
  end

  # ========================================================================= #
  # === do_permanently_enable_ccache
  # ========================================================================= #
  def do_permanently_enable_ccache(
      into = FILE_CCACHE,
      what = "true\n"
    )
    write_what_into(what, into)
    opne "Permanently enabling the use of ccache "\
         "(Stored into this file: `#{sfile(into)}`)."
    if is_on_roebe?
      into = "#{RUBY_SRC_DIRECTORY}rbt/lib/rbt/yaml/configuration/"\
             "#{File.basename(into)}"
      write_what_into(what, into)
      opne "Permanently enabling the use of ccache "\
           "(Stored into this file: `#{sfile(into)}`)."
    end
  end

  # ========================================================================= #
  # === do_permanently_disable_ccache
  # ========================================================================= #
  def do_permanently_disable_ccache(
      into = FILE_CCACHE,
      what = "false\n"
    )
    write_what_into(what, into)
    opne "Permanently disabling the use of ccache "\
         "(Stored into this file: `#{sfile(into)}`)."
    if is_on_roebe?
      into = "#{RUBY_SRC_DIRECTORY}rbt/lib/rbt/yaml/configuration/"\
             "#{File.basename(into)}"
      write_what_into(what, into)
      opne "Permanently disabling the use of ccache "\
           "(Stored into this file: `#{sfile(into)}`)."
    end
  end

  # ========================================================================= #
  # === show_whether_we_are_using_ccache_or_whether_we_are_not
  #
  # To invoke this method, try:
  #   ry --use-ccache?
  # ========================================================================= #
  def show_whether_we_are_using_ccache_or_whether_we_are_not
    _ = use_ccache?
    opne "Will the RBT project use ccache: "\
         "#{sfancy(verbose_truth(_))}"
  end

end; end