#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/cflags.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === set_cflags                                               (cflags tag)
  #
  # This method will enable you to set i.e. new cflags. This then also
  # allows you to compile some programs in a different manner, such as
  # if you wish to compile that particular program statically.
  #
  # The default argument to this method will be a copy of ENV['CFLAGS'].
  #
  # You can overrule this via a shortcut, if you want to, by passing
  # true as argument, or even better, a Symol such as :static. This
  # will then allow you to perform the static build, if the program
  # at hand allows for this.
  #
  # -static-libgcc may also be an option here.
  #
  # The variable LDFLAGS may be important as well.
  #
  # For instance:
  #
  #   make LDFLAGS=-all-static
  #
  # ========================================================================= #
  def set_cflags(
      i = default_cflags?,
      &block
    )
    case i # case tag
    when :clear
      i = ''
    # ======================================================================= #
    # Enable static builds for the CFLAGS at hand
    # ======================================================================= #
    when :overrule_static_build,
         :static,
         :build_static,
         true
      i = '-static -O2 -g'
    end
    # ======================================================================= #
    # Do the actual assignment next.
    # ======================================================================= #
    set_this_env_variable(
      'CFLAGS: '+i.to_s,
      :do_set_these_variables,
      &block
    )
  end

  # ========================================================================= #
  # === cflags?
  #
  # Just a convenience wrapper over the CFLAGS environment variable.
  # A getter method, which will tell us the CFLAGS in use of the
  # environment.
  # ========================================================================= #
  def cflags?
    ENV['CFLAGS'].to_s.dup
  end; alias default_cflags? cflags? # === default_cflags?

  # ========================================================================= #
  # === consider_reporting_the_cflags_we_will_use
  #
  # Notify the user which CFLAGS he is using.
  # ========================================================================= #
  def consider_reporting_the_cflags_we_will_use
    _ = cflags?
    unless _.empty?
      opne "The CFLAGS in use are: #{simp(_)}"
    end
  end

end; end