#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/clang.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === do_use_the_clang_compiler
  #
  # Use this method if you wish to make use of the (LLVM) clang compiler
  # rather than gcc.
  #
  # Do note that you can also use special arguments or combine clang
  # with ccache, such as shown in the following examples:
  #
  #   export CC='ccache clang -fcolor-diagnostics -Qunused-arguments -fcatch-undefined-behavior -ftrapv'
  #   export CXX='ccache clang++ -fcolor-diagnostics -Qunused-arguments -fcatch-undefined-behavior -ftrapv'
  #
  # Invocation example:
  #
  #   ry htop --clang
  #
  # ========================================================================= #
  def do_use_the_clang_compiler(
      shall_we_notify_the_user = :notify_the_user
    )
    case shall_we_notify_the_user
    when :notify_the_user,
         :and_notify_the_user
      opnn; e "We will attempt to use (#{teal('LLVM')}) #{slateblue('clang')} "\
              "to compile the code at hand."
    end
    set_this_env_variable 'CC=clang',    :be_quiet
    set_this_env_variable 'CXX=clang++', :be_quiet
    @internal_hash[:compiler_to_use] = :clang # Keep track of the compiler here.
  end; alias do_use_clang_compiler do_use_the_clang_compiler # === do_use_clang_compiler

end; end