#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/class_methods.rb'
# =========================================================================== #
require 'rbt/compile/base/compile_base.rb'
require 'rbt/toplevel_methods/e.rb'
require 'rbt/colours/colours.rb'
require 'rbt/toplevel_methods/available_programs.rb'

module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === RBT::Compile.eimp
  # ========================================================================= #
  def self.eimp(i = '')
    e RBT.simp(i)
  end

  # ========================================================================= #
  # === RBT::Compile.efancy
  # ========================================================================= #
  def self.efancy(i = '')
    e RBT.sfancy(i)
  end

  # ========================================================================= #
  # === RBT::Compile.sfancy
  # ========================================================================= #
  def self.sfancy(i = '')
    RBT.sfancy(i)
  end

  # ========================================================================= #
  # === RBT::Compile.sdir
  # ========================================================================= #
  def self.sdir(i = '')
    RBT.sdir(i)
  end

  # ========================================================================= #
  # === RBT::Compile.simp
  # ========================================================================= #
  def self.simp(i = '')
    RBT.simp(i)
  end

  # ========================================================================= #
  # === RBT::Compile.e
  # ========================================================================= #
  def self.e(i = '')
    RBT.e(i)
  end

  # ========================================================================= #
  # === @is_included
  # ========================================================================= #
  @is_included = false

  # ========================================================================= #
  # === Compile.the_program_is_included
  # ========================================================================= #
  def self.the_program_is_included
    @is_included = true
  end

  # ========================================================================= #
  # === the_program_is_included
  # ========================================================================= #
  def the_program_is_included
    RBT::Compile.the_program_is_included
  end

  # ========================================================================= #
  # === RBT::Compile.is_this_program_included?
  #
  #   ry --included?
  #
  # ========================================================================= #
  def self.is_this_program_included?(
      i,
      be_verbose = :default
    )
    case be_verbose
    when :default
      be_verbose = true
    when :be_quiet
      be_verbose = false
    end
    if i.is_a? Array
      i.each {|entry| is_this_program_included?(entry, be_verbose) }
    else
      i = i.to_s.dup
      i.delete!('-')
      i.delete!('_')
      # ===================================================================== #
      # Note that RBT.does_include? tries harder to find a program
      # than does RBT.is_this_program_included?.
      # ===================================================================== #
      is_included = RBT.does_include?(i)
      if is_included
        if be_verbose
          RBT::Compile.opnn
          e "The program `#{Colours.simp(i)}` is included."
        end
        @is_included = true
      else
        if be_verbose
          RBT::Compile.opnn
          e "The program `#{Colours.simp(i)}` is #{swarn('not')} included."
        end
        @is_included = false
      end
    end
  end

  # ========================================================================= #
  # === Compile.program_is_included?
  # ========================================================================= #
  def self.program_is_included?
    @is_included
  end

  # ========================================================================= #
  # === RBT::Compile.compile
  #
  # The first argument should be the name of the program that you wish
  # to compile, such as :htop or 'htop'.
  # ========================================================================= #
  def self.compile(
      this_program       = nil,
      optional_arguments = nil
    )
    _ = RBT::Compile.new(this_program, :do_not_run_already)
    # ======================================================================= #
    # Pass any extra arguments into the menu() interface.
    # ======================================================================= #
    _.menu(optional_arguments) if optional_arguments
    _.run
  end; self.instance_eval { alias do_compile compile } # === RBT.do_compile 

  # ========================================================================= #
  # === RBT::Compile['php']
  # ========================================================================= #
  def self.[](i = 'php')
    self.new(i)
  end

end

# =========================================================================== #
# === RBT::do_compile
# =========================================================================== #
def self.do_compile(this_program)
  ::RBT::Compile.compile(this_program, nil)
end

end