#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/cmake.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === cmake_exists?
  # ========================================================================= #
  def cmake_exists?
    result = `cmake #{ERROR_LINE}`
    !result.include?('cmake: command not found')
  end; alias is_cmake_available? cmake_exists? # === is_cmake_available?

  # ========================================================================= #
  # === run_cmake_then_make_then_make_install
  #
  # Just a convenience method to run "cmake", then "make" and then
  # "make install".
  # ========================================================================= #
  def run_cmake_then_make_then_make_install
    run_cmake_command
    run_make
    run_make_install
  end

  # ========================================================================= #
  # === depends_on_cmake?
  # ========================================================================= #
  def depends_on_cmake?
    RBT.depends_on_cmake?(required_deps_on?)
  end; alias program_uses_cmake? depends_on_cmake? # === program_uses_cmake?

  # ========================================================================= #
  # === do_use_cmake
  #
  # If you wish to use cmake, you can do so via this method here.
  # ========================================================================= #
  def do_use_cmake
    opne 'We will use cmake rather than make.'
    set_use_this_make_command 'cmake'
  end; alias use_cmake do_use_cmake # === use_cmake

  # ========================================================================= #
  # === cmake_file?
  # ========================================================================= #
  def cmake_file_exists?
    File.exist? 'CMakeLists.txt'
  end; alias cmake_file? cmake_file_exists? # === cmake_file?

  # ========================================================================= #
  # === run_cmake_command
  #
  # Run the corresponding "cmake" command.
  # ========================================================================= #
  def run_cmake_command
    cmd = ''.dup
    cmd << "cmake -DCMAKE_INSTALL_PREFIX=#{prefix?}"
    if use_configure_options?
      configure_options = configure_options?.dup
      if configure_options and !configure_options.empty?
        # =================================================================== #
        # Append additional commands to the configure-string:
        # =================================================================== #
        cmd << " #{configure_options}"
      end
    end
    cmd << ' '
    # ======================================================================= #
    # Next we have to keep the relative path in mind, which is why we
    # will simply append the configure base directory next.
    # ======================================================================= #
    cmd << configure_base_dir?
    # ======================================================================= #
    # === Check for more than one -DCMAKE_INSTALL_PREFIX entry next
    # ======================================================================= #
    if cmd.scan(/-DCMAKE_INSTALL_PREFIX=\//).size > 1
      # ===================================================================== #
      # In this case, something must be wrong. Notify the user about it.
      # ===================================================================== #
      opnewarn "There are at the least two definitions of "\
               "#{royalblue('-DCMAKE_INSTALL_PREFIX=')}"
      opnewarn 'One of them must be incorrect and/or superfluous. Please correct'
      opnewarn 'this logic-problem because RBT can continue to work.'
      e
      e 'See:'
      e
      e "  #{sfancy(cmd)}"
      e
      exit_program
    end
    opnn; e 'Next running'
    e
    e "  #{sfancy(cmd)}"
    e
    cmd.strip!
    reset_error_map_line_to_a_specific_error
    set_full_configure_command(cmd)
    # ======================================================================= #
    # We should check whether cmake exists on the host system or whether
    # it does not.
    # ======================================================================= #
    if cmake_exists?
      run_this_sys_command cmd # <- Run the sys-command here.
      run_through_the_registered_problems_registered_errors_and_behaviour_changes
      reset_error_map_line_to_a_specific_error
    else
      opnn; e "The program #{sfancy('cmake')} could not "\
              "be found, thus we can not"
      opnn; e 'compile the program called `'+
               orange(compile_which_program?)+'`.'
      we_can_not_continue
    end
  end

end; end