#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Colour-related code for RBT::Compile is stored here. This also includes
# code that has something to do with colour-related changes.
# =========================================================================== #
# require 'rbt/compile/colours.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === do_permanently_enable_the_colours
  #
  # This method can be used to permanently enable the colours.
  #
  # Invoke this method by issuing:
  #
  #   ry --permanent-enable-colours
  #
  # ========================================================================= #
  def do_permanently_enable_the_colours
    opne 'Now permanently enable the colours.'
    configuration?.use_colours = true
    write_what_into(false, RBT.file_use_colours)
    if is_on_roebe?
      into = RUBY_SRC_DIR_RBT_YAML_AT_HOME+'configuration/'+
             File.basename(into)
      opne "Also storing in `#{sfile(into)}`."
      write_what_into(what, into)
    end
  end

  # ========================================================================= #
  # === do_permanently_disable_the_colours
  #
  # This method can be used to permanently disable the colours.
  #
  # Invoke this method by issuing:
  #
  #   ry --permanent-no-colours
  #
  # ========================================================================= #
  def do_permanently_disable_the_colours
    opne 'Now permanently disabling the colours.'
    configuration?.use_colours = false
    what = false
    into = RBT.file_use_colours
    write_what_into(what, into)
    if is_on_roebe?
      into = RUBY_SRC_DIR_RBT_YAML_AT_HOME+'configuration/'+
             File.basename(into)
      opne "Also storing in `#{sfile(into)}`."
      write_what_into(what, into)
    end
  end

  # ========================================================================= #
  # === do_use_a_white_background
  #
  # We will silently assume that the user has a white background,
  # and will re-assign to use a green font colour in this case, on
  # KDE Konsole.
  # ========================================================================= #
  def do_use_a_white_background
    ::Colours.table_colour_methods_to_specific_colour[:simp] = :seagreen
  end

  # ========================================================================= #
  # === report_whether_colours_will_be_used
  # ========================================================================= #
  def report_whether_colours_will_be_used
    e 'Will we use colours: '+
      royalblue(
        verbose_truth?(use_colours?)
      )
  end

  # ========================================================================= #
  # === show_which_colours_will_be_used
  #
  # To show which colours will be used for the RBT project.
  # ========================================================================= #
  def show_which_colours_will_be_used
    opne 'These colours will be used:'
    e
    YAML.load_file(RBT_COLOURS_FILE).each_pair {|key, value|
      e "  #{key.ljust(12)} -> #{value}"
    }; e
  end

  # ========================================================================= #
  # === RBT::Compile.use_colours?
  # ========================================================================= #
  def self.use_colours?
    config?.use_colours.to_s
  end

  # ========================================================================= #
  # === disable_colours                                         (colours tag)
  #
  # Disable colours with this method. This will not be permanent, it will
  # be valid only for the running instance (this object). If you want to
  # permanently disable colours, use:
  #
  #   rbt_config --disable-colours
  #
  # To use this specific method here, do something like this:
  #
  #   ry htop disablecolours
  #
  # Remember that you can chain together instructions, so:
  #
  #   ry htop disablecolours ntrad
  #
  # Would compile in an appdir-like fashion, without using any colours.
  # ========================================================================= #
  def disable_colours(
      be_verbose = true
    )
    RBT.disable_colours  # <- Defined in rbt/colours/colours.rb
    @use_colours = false # This one is defined in base.rb
    if be_verbose
      opne 'We will not use any colours.'
    end
  end; alias no_colours disable_colours # === no_colours

end; end