#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Commandline-related information is kept in this file here.
# =========================================================================== #
# require 'rbt/compile/commandline.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @internal_hash[:commandline_arguments]
  end; alias commandline?           commandline_arguments? # === commandline?
       alias commandline_options?   commandline_arguments? # === commandline_options?
       alias commandline            commandline_arguments? # === commandline
       alias commandline_args?      commandline_arguments? # === commandline_args?

  # ========================================================================= #
  # === check_for_early_commandline_arguments
  #
  # Check for compound-input here. But only for some input - for now we
  # only respond to "update-entry", "--remove-global-flag" and
  # "--add-global-flag".
  # ========================================================================= #
  def check_for_early_commandline_arguments(i)
    if i.is_a? Array
      i = i.join(' ').strip
    end
    case i # case tag
    # ======================================================================= #
    # === --add_global_flag
    # ======================================================================= #
    when /^-?-?add(-|_)?global(-|_)?flag (-?-?.+)$/
      add_this_global_flag($3)
    # ======================================================================= #
    # === --remove_global_flag
    # ======================================================================= #
    when /^-?-?remove(-|_)?global(-|_)?flag (-?-?.+)$/
      remove_this_global_flag($3)
    # ======================================================================= #
    # === --update-entry
    # ======================================================================= #
    when /^-?-?update-?entry (http.+)$/
      update_this_entry($1.to_s.dup)
      exit_program
    end
  end

  # ========================================================================= #
  # === obtain_and_set_commandline_arguments_from
  # ========================================================================= #
  def obtain_and_set_commandline_arguments_from(i)
    if i and i.respond_to?(:any?) and i.any? {|entry| entry.to_s.start_with?('--') }
      commandline_arguments? << i.select {|inner_entry|
        inner_entry.to_s.start_with?('--')
      }
      commandline_arguments?.flatten!
      # ===================================================================== #
      # === filter_away_commandline_options
      #
      # Filter away the commandline-arguments from the rest of the input:
      # ===================================================================== #
      i.reject! {|inner_entry| inner_entry.to_s.start_with?('--') }
    end
    i # And we must return the modified variant here.
  end

  # ========================================================================= #
  # === append_these_options_to_the_commandline
  #
  # Simply append to the commandline options through this method here.
  # ========================================================================= #
  def append_these_options_to_the_commandline(i)
    # ======================================================================= #
    # First check whether the input is a Hash:
    # ======================================================================= #
    if i.is_a? Hash
      if i.has_key? :commandline_arguments
        i = i.delete(:commandline_arguments)
      end
    end
    @internal_hash[:commandline_arguments] << i
    @internal_hash[:commandline_arguments].flatten!
  end; alias append_to_the_commandline append_these_options_to_the_commandline # === append_to_the_commandline

  # ========================================================================= #
  # === clear_commandline_options
  # ========================================================================= #
  def clear_commandline_options
    @internal_hash[:commandline_arguments] = []
  end; alias clear_commandline_arguments clear_commandline_options # === clear_commandline_arguments

  # ========================================================================= #
  # === set_commandline_options
  #
  # This will be guaranteed to be an Array.
  # ========================================================================= #
  def set_commandline_options(i = [])
    case i
    when nil
      i = []
    end
    i = [i].flatten.compact
    @internal_hash[:commandline_arguments] = i
  end; alias set_commandline           set_commandline_options # === set_commandline
       alias set_commandline_arguments set_commandline_options # === set_commandline_arguments

end; end