#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/configure.rb'
# =========================================================================== #
require 'rbt/compile/ccache.rb'

module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === consider_running_the_pre_configure_steps
  # ========================================================================= #
  def consider_running_the_pre_configure_steps
    _ = pre_configure_steps?
    unless _.empty?
      _.each {|entry|
        esystem entry
      }
    end
  end
    
  # ========================================================================= #
  # === configure_options?
  #
  # This method will retrieve the configure options. We default to using
  # the configure-options stored in the cookbook dataset, but we can
  # overrule this on the commandline (when we store it onto the
  # @internal_hash).
  # ========================================================================= #
  def configure_options?
    _ = cookbook_dataset_configure_options?
    if @internal_hash[:configure_options]
      _ = @internal_hash[:configure_options]
    end
    _
  end

  # ========================================================================= #
  # === return_default_configure_command
  # ========================================================================= #
  def return_default_configure_command
    "configure --prefix=#{use_this_prefix?.to_s}"
  end

  # ========================================================================= #
  # === all_configure_options?
  #
  # Use this method to show all configure options for a given program.
  #
  # This means all options available through "./configure --help".
  #
  # Invocation example:
  #
  #   ry htop allconfigureoptions?
  #
  # ========================================================================= #
  def all_configure_options?
    _ = extract_to? # TEMP_DIR+File.basename(compile_this_program?)
    # ======================================================================= #
    # The target may be empty though. Rescue this case.
    # ======================================================================= #
    _ = rbt_log_directory? if _.empty?
    opne 'Will extract to '+sdir(_)+'. In the event that you need '\
         'to know '
    opne 'where the extracted archive resides.'
    opne "All configure options for "\
         "#{simp(compile_this_program?)} will be shown next, "\
         "before exiting."
    set_extract_to(_)
    consider_extracting_this_program(
      program_path?
    )
    _ = rbt_log_directory?+program_name_and_version?
    opne "We will next cd (change directory) into `#{sdir(_)}`."
    cd _, :be_quiet # Now we should be in the proper directory.
    e
    RBT::BeautifyConfigureHelpOutput.new # Delegate to this class next.
    e
    exit_program
  end

  # ========================================================================= #
  # === run_the_configure_command
  #
  # Run the "./configure" command, with an appropriate prefix to be used.
  #
  # The optional argument to this method allows us to use a different
  # set of configure-options as-is.
  # ========================================================================= #
  def run_the_configure_command(
      use_these_configure_options = nil
    )
    if run_configure_script?
      use_this_prefix = prefix?.to_s
      RBT.set_last_prefix_used(use_this_prefix)
      _ = ''.dup # <- We will start with this here.
      _ << configure_base_directory?.dup
      # ===================================================================== #
      # Add the configure-command that is to be used next:
      # ===================================================================== #
      if cookbook_dataset_configure_command?
        _ << cookbook_dataset_configure_command?.dup
      else # This else-clause is the default.
        _ << 'configure'
      end
      # ===================================================================== #
      # The following _.include? check is valid for e. g. applications
      # such as "lsof".
      # ===================================================================== #
      _ << " --prefix=#{use_this_prefix.to_s}" unless _.include? './Configure -n linux'
      # ===================================================================== #
      # === Honour ccache
      #
      # Honour the CCACHE setting, if it has been provided.
      # ===================================================================== #
      if use_ccache?
        # =================================================================== #
        # Find out whether ccache is available or whether it is not. If
        # it is we will use it, while also specifying which compiler
        # to use.
        # =================================================================== #
        if ccache_is_available?
          if will_gcc_be_used_for_compilation?
            _.prepend 'CC="ccache gcc" ' # In this case prepend it.
          end
        else # It must be available.
          notify_the_user_that_ccache_is_not_available
        end
      end
      # ===================================================================== #
      # The following two checks are necessary, due to two reasons:
      #
      # - The user may not want to append any configure-options in
      #   the first place. Hence, we must check for this via
      #   use_configure_options?.
      #
      # - Second the user may also input a command such as "ry sed-4.5",
      #   and in this case we have to work on an existing file, rather
      #   than on the dataset in the cookbook-yaml file.
      # ===================================================================== #
      if use_configure_options? and cookbooks_dataset?
        if use_these_configure_options
          configure_options = use_these_configure_options
        else
          configure_options = configure_options?.dup
        end
        if configure_options and !configure_options.empty?
          # ================================================================= #
          # Finally, append these additional commands to the
          # configure-string:
          # ================================================================= #
          _ << ' '+configure_options
        end
      end
      _.strip! # Just ensure that there are no trailing ' ' characters.
      # ===================================================================== #
      # Next, we will append the ERROR_LINE.
      # ===================================================================== #
      _ << ERROR_LINE
      # ===================================================================== #
      # === Handle strange configuration options involving sysconfdir next
      # ===================================================================== #
      if use_appdir_prefix? and _.include?('--sysconfdir=/etc')
        regexp_quoted_old_sysconfdir_prefix = Regexp.quote('--sysconfdir=/etc')
        rewritten_sysconfdir = '--sysconfdir='+rds(use_which_prefix?.to_s+'/etc')
        opne 'An '+salmon('AppDir prefix')+' is used, but also "'+
             royalblue('--sysconfdir=/etc')+'". '\
             'This is not logical.'
        opne 'It is now assumed that the --sysconfdir=/etc is meant to target'
        opne "that #{salmon('AppDir prefix')}, so it will be rewritten"
        opne "to `#{royalblue(rewritten_sysconfdir)}`."
        _.sub!(/#{regexp_quoted_old_sysconfdir_prefix}/, rewritten_sysconfdir)
      end
      set_full_configure_command(_) # <- Keep a copy too.
      # ===================================================================== #
      # Next, output the configure command that we will use - in
      # teal, but also use dodgerblue for the actual prefix.
      # ===================================================================== #
      # ===================================================================== #
      # Colourize this in dodgerblue.
      # ===================================================================== #
      if use_colours? and RBT.konsole_available?
        use_this_colour = :konsole_dodgerblue # <- dodgerblue
        cliner(use_colour: use_this_colour)
        copy_of_configure = _.dup
        if copy_of_configure.include? '--prefix='
          # ================================================================= #
          # === Colourize the prefix, in cornflowerblue().
          # ================================================================= #
          copy_of_configure.sub!(/(--prefix=)(\/.+?)\s/, # See: http://rubular.com/r/Df8OzLqXGd
            '\1'+
            cornflowerblue('\2 ')+
            dodgerblue('')
          )
        end
        # =================================================================== #
        # Display the line we will use to the user.
        # =================================================================== #
        e dodgerblue(_)
        cliner(use_colour: use_this_colour)
      else
        cliner
        e _ # Else just display the command, without any colours.
        cliner
      end
      reset_error_map_line_to_a_specific_error
      run_this_sys_command _ # <- Run the sys-command here.
      run_through_the_registered_problems_registered_errors_and_behaviour_changes
      reset_error_map_line_to_a_specific_error
    end
  end; alias run_configure            run_the_configure_command # === run_configure
       alias run_configure_via_system run_the_configure_command # === run_configure_via_system

  # ========================================================================= #
  # === run_configure_script?
  #
  # Determine whether we will invoke "./configure" or whether we will not.
  # ========================================================================= #
  def run_configure_script?
    _ = @internal_hash[:run_configure_script]
    # ======================================================================= #
    # Next, fetch the entry that exists in the cookbook-dataset, and use
    # it in this case.
    # ======================================================================= #
    if _.nil? and !cookbook_dataset_run_configure_script?.nil?
      _ = cookbook_dataset_run_configure_script?
    end
    _
  end; alias run_configure? run_configure_script? # === run_configure?

  # ========================================================================= #
  # === configure_base_directory?
  #
  # This should always have a trailing '/' character IF it is defined.
  # class Cookbook will make sure of that.
  # ========================================================================= #
  def configure_base_directory?
    @internal_hash[:configure_base_directory]
  end; alias configure_base_dir?              configure_base_directory? # === configure_base_dir?
       alias configure_base_directory_to_use? configure_base_directory? # === configure_base_directory_to_use?

  # ========================================================================= #
  # === enable_shared_disable_static
  # ========================================================================= #
  def enable_shared_disable_static
    append_enable_shared_to_the_configure_options
    append_disable_static_to_the_configure_options
  end

  # ========================================================================= #
  # === append_this_to_the_configure_options
  # ========================================================================= #
  def append_this_to_the_configure_options(
      i = :enable_bootstrap
    )
    # ======================================================================= #
    # The following case-when menu could be replaced with generic code
    # that turns the Symbol-input into the equivalent String e. g.
    # --disable-something or --enable-something. However had, for the
    # time being (Dec 2018) I will leave it as it is.
    # ======================================================================= #
    case i
    when :disable_static
      i = ' --disable-static'
    when :enable_shared
      i = ' --enable-shared'
    when :enable_bootstrap
      i = ' --enable-bootstrap'
    end
    _ = @internal_hash[:configure_options]
    _ = ''.dup if _.nil?
    unless i.start_with? ' '
      i = i.dup if i.frozen?
      i.prepend(' ')
    end
    _ << i
    @internal_hash[:configure_options] = _
  end

  # ========================================================================= #
  # === internal_configure_options?
  # ========================================================================= #
  def internal_configure_options?
    @internal_hash[:configure_options]
  end

  # ========================================================================= #
  # === append_disable_static_to_the_configure_options
  #
  # This method is somewhat similar to the method called
  # append_enable_shared_to_the_configure_options in that
  # it will also modify the configure options; but it will
  # add "--disable-static" to the configure options.
  # ========================================================================= #
  def append_disable_static_to_the_configure_options
    append_this_to_the_configure_options(:disable_static)
  end

  # ========================================================================= #
  # === show_last_configure_options_then_exit
  #
  # This method will show the last configure option that was used,
  # then it will exit.
  #
  # Invoke this like so:
  #
  #   rbt lconfigure?
  #
  # ========================================================================= #
  def show_last_configure_options_then_exit
    show_last_options
    exit_program
  end

  # ========================================================================= #
  # === do_not_run_configure
  #
  # Do not run the configure script. Since Jan 2012 we also disable running
  # ldconfig in this method.
  # ========================================================================= #  
  def do_not_run_configure(
      be_verbose = true
    )
    if be_verbose
      opne 'We will not compile - or run any configure script.'
    end
    do_not_run_the_configure_script
    do_not_use_ldconfig
    no_postinstall_actions
  end; alias do_not_invoke_configure do_not_run_configure # === do_not_invoke_configure

  # ========================================================================= #
  # === do_not_run_configure_script
  # ========================================================================= #
  def do_not_run_configure_script
    @internal_hash[:run_configure_script] = false
  end; alias do_not_run_the_configure_script do_not_run_configure_script # === do_not_run_the_configure_script

  # ========================================================================= #
  # === append_enable_shared_to_the_configure_options
  #
  # This method will simply add --enable-shared onto the configure options.
  #
  # This can then be used by GNU configure sripts or meson builds or
  # even cmake builds (though the latter may use another format, so
  # keep this in mind for the time being; the primary use cases are
  # for GNU configure scripts and meson builds).
  # ========================================================================= #
  def append_enable_shared_to_the_configure_options
    append_this_to_the_configure_options(:enable_shared)
  end

  # ========================================================================= #
  # === show_last_options
  #
  # Use this method to show the last configure options that were used
  # in order to compile a program from source.
  #
  # To grab this, try:
  #   ry htop --last-options
  # ========================================================================= #
  def show_last_options(
      name_of_the_program = compile_which_program?
    )
    full_configure_command = return_last_configure_command
    if full_configure_command
      opne "The last configure command for `#{simp(name_of_the_program)}` was:"
      opne "  #{simp(full_configure_command)}"
    else
      opne "No configure command was used for `#{simp(name_of_the_program)}`."
    end
  end

  # ========================================================================= #
  # === return_last_configure_command
  # ========================================================================= #
  def return_last_configure_command
    hash = compiled_programs? # The full dataset is assigned here.
    last_key = hash.keys.last # Grab the last key.
    dataset = hash[last_key]  # Now we have obtained the dataset.
    full_configure_command = dataset['full_configure_command']
    return full_configure_command
  end

end; end