#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/cookbook_dataset.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === cookbook_dataset_run_make?
  # ========================================================================= #
  def cookbook_dataset_run_make?
    cookbook_dataset?.run_make?
  end

  # ========================================================================= #
  # === modify_the_makefile?
  #
  # If this is used then it should be used before invoking the "make" step.
  # ========================================================================= #
  def modify_the_makefile?
    cookbook_dataset?.modify_the_makefile?
  end

  # ========================================================================= #
  # === use_this_make_install_command?
  # ========================================================================= #
  def use_this_make_install_command?
    cookbook_dataset?.use_this_make_install_command?
  end

  # ========================================================================= #
  # === pre_configure_steps?
  # ========================================================================= #
  def pre_configure_steps?
    cookbook_dataset?.pre_configure_steps?
  end

  # ========================================================================= #
  # === cookbook_dataset?                                       (dataset tag)
  #
  # Keep this query method on top of this file here, ideally.
  #
  # This method can be used whenever we wish to query the main data
  # structure of the target program at hand.
  #
  # The dataset variable will typically be an instance of
  # class RBT::Cookbooks::Cookbook.
  # ========================================================================= #
  def cookbook_dataset?
    @internal_hash[:yaml_dataset]
  end; alias yaml_dataset?      cookbook_dataset? # === yaml_dataset?
       alias cookbooks_dataset? cookbook_dataset? # === cookbooks_dataset?
       alias dataset            cookbook_dataset? # === dataset
       alias dataset?           cookbook_dataset? # === cookbooks_dataset?
       alias dataset_object?    cookbook_dataset? # === dataset_object?
       alias data?              cookbook_dataset? # === data
       alias sload              cookbook_dataset? # === sload() (sload tag)
       alias sanitized_data     cookbook_dataset? # === sanitized_data()
       alias yaml_dataset?      cookbook_dataset? # === yaml_dataset?
       alias my_program         cookbook_dataset? # === my_program
       alias main_dataset?      cookbook_dataset? # === main_dataset?
       alias main_dataset       cookbook_dataset? # === main_dataset

  # ========================================================================= #
  # === initialize_cookbook_dataset_for
  #
  # This method is different to load_yaml_dataset_for_this_program()
  # in that we will also assign to the main internal Hash.
  # ========================================================================= #
  def initialize_cookbook_dataset_for(
      this_program       = @internal_hash[:compile_this_program],
      optional_arguments = nil
    )
    this_program = this_program.to_s.dup # We need a String.
    if this_program.include? '_'
      this_program.delete!('_') # <- Is already unfrozen by default.
    end
    yaml_dataset = load_yaml_dataset_for_this_program(
      this_program, optional_arguments
    )
    # ======================================================================= #
    # Next, assign to the internal Hash:
    # ======================================================================= #
    assign_to_the_main_dataset(
      yaml_dataset
    )
  end; alias initialize_dataset_for                      initialize_cookbook_dataset_for # === initialize_dataset_for
       alias initialize_dataset                          initialize_cookbook_dataset_for # === initialize_dataset
       alias initialize_yaml_dataset                     initialize_cookbook_dataset_for # === initialize_yaml_dataset
       alias initialize_dataset_for_this_program         initialize_cookbook_dataset_for # === initialize_dataset_for_this_program
       alias instantiate_a_new_cookbook_for              initialize_cookbook_dataset_for # === instantiate_a_new_cookbook_for
       alias determine_the_dataset_for_this_program      initialize_cookbook_dataset_for # === determine_the_dataset_for_this_program
       alias determine_cookbook_dataset_for_this_program initialize_cookbook_dataset_for # === determine_cookbook_dataset_for_this_program
       alias initialize_cookbook_dataset                 initialize_cookbook_dataset_for # === initialize_cookbook_dataset
       alias ensure_that_the_dataset_exists              initialize_cookbook_dataset_for # === ensure_that_the_dataset_exists
       alias ensure_that_the_main_dataset_exists         initialize_cookbook_dataset_for # === ensure_that_the_main_dataset_exists
       alias prepare_dataset_for                         initialize_cookbook_dataset_for # === prepare_dataset_for

  # ========================================================================= #
  # === no_cookbook_dataset?
  # ========================================================================= #
  def no_cookbook_dataset?
    !(
      @internal_hash and @internal_hash[:yaml_dataset] and !@internal_hash[:yaml_dataset].nil?
    )
  end

  # ========================================================================= #
  # === clear_old_configure_options
  # ========================================================================= #
  def clear_old_configure_options
    set_configure_options('')
  end

  # ========================================================================= #
  # === set_configure_options
  # ========================================================================= #
  def set_configure_options(i = '')
    cookbook_dataset?.configure_options = i
  end

  # ========================================================================= #
  # === cookbooks_dataset_configure_options?
  # ========================================================================= #
  def cookbooks_dataset_configure_options?
    cookbooks_dataset?.configure_options?
  end; alias cookbook_dataset_configure_options? cookbooks_dataset_configure_options? # === cookbook_dataset_configure_options?

  # ========================================================================= #
  # === cookbook_dataset_program_path?
  # ========================================================================= #
  def cookbook_dataset_program_path?
    if cookbook_dataset?
      cookbook_dataset?.program_path
    end
  end; alias cookbook_dataset_path?  cookbook_dataset_program_path? # === cookbook_dataset_path?
       alias full_path?              cookbook_dataset_program_path? # === full_path?
       alias cookbooks_program_path? cookbook_dataset_program_path? # === cookbooks_program_path?
       alias cookbook_program_path?  cookbook_dataset_program_path? # === cookbook_program_path?

  # ========================================================================= #
  # === cookbook_dataset_parameters_to_make?
  # ========================================================================= #
  def cookbook_dataset_parameters_to_make?
    cookbook_dataset?.parameters_to_make?
  end; alias cookbook_dataset_make_options? cookbook_dataset_parameters_to_make? # === cookbook_dataset_make_options? 

  # ========================================================================= #
  # === cookbook_installation_steps?
  #
  # The installation-steps contain the instructions in how to install
  # a given program at hand.
  # ========================================================================= #
  def cookbook_installation_steps?
    cookbook_dataset?.installation_steps?
  end; alias installation_steps? cookbook_installation_steps? # === cookbook_installation_steps?

  # ========================================================================= #
  # === cookbook_dataset_with_extended_configure_options?
  #
  # This method will have most of the cookbooks-dataset, but will
  # also honour the configure_options should they exist.
  # ========================================================================= #
  def cookbook_dataset_with_extended_configure_options?
    _ = cookbook_dataset?.dup
    if @internal_hash[:configure_options]
      _.append_this_to_the_configure_options(
        @internal_hash[:configure_options]
      )
    end
    if @internal_hash[:parameters_to_make]
      _.set_parameters_to_make(
        @internal_hash[:parameters_to_make]
      )
    end
    _
  end

  # ========================================================================= #
  # === cookbook_dataset_meson_configure_options?
  # ========================================================================= #
  def cookbook_dataset_meson_configure_options?
    cookbook_dataset?.meson_configure_options?
  end; alias meson_configure_options? cookbook_dataset_meson_configure_options? # === meson_configure_options?

  # ========================================================================= #
  # === disallow_changing_the_cookbook_dataset
  # ========================================================================= #
  def disallow_changing_the_cookbook_dataset
    cookbook_dataset?.freeze
  end

  # ========================================================================= #
  # === cookbook_dataset_use_this_build_system?
  # ========================================================================= #
  def cookbook_dataset_use_this_build_system?
    cookbook_dataset?.use_which_build_system?
  end; alias use_this_build_system?                      cookbook_dataset_use_this_build_system? # === use_this_build_system?
       alias cookbookset_dataset_use_which_build_system? cookbook_dataset_use_this_build_system? # === cookbookset_dataset_use_which_build_system?
       alias cookbook_build_system?                      cookbook_dataset_use_this_build_system? # === cookbook_build_system?

  # ========================================================================= #
  # === cookbook_dataset_program_has_to_be_compiled?
  # ========================================================================= #
  def cookbook_dataset_program_has_to_be_compiled?
    cookbooks_dataset? and cookbooks_dataset?.has_to_be_compiled?
  end; alias cookbook_dataset_has_to_be_compiled? cookbook_dataset_program_has_to_be_compiled? # === cookbook_dataset_has_to_be_compiled?

  # ========================================================================= #
  # === dataset_program_short_name?
  # ========================================================================= #
  def dataset_program_short_name?
    cookbook_dataset?.short_name?
  end; alias dataset_program_name?               dataset_program_short_name? # === dataset_program_name?
       alias cookbook_dataset_program_name?      dataset_program_short_name? # === cookbook_dataset_program_name?
       alias cookbook_dataset_real_program_name? dataset_program_short_name? # === cookbook_dataset_real_program_name?

  # ========================================================================= #
  # === real_short_name?
  #
  # The real_short_name is a String such as 'gnomemahjonng'. If you need
  # to obtain the String that includes the '-' token then you should
  # use short_name? instead.
  # ========================================================================= #
  def real_short_name?
    cookbook_dataset?.real_short_name?
  end

  # ========================================================================= #
  # === update_program_full_name
  # ========================================================================= #
  def update_program_full_name
    cookbook_dataset?.update_program_full_name
  end

  # ========================================================================= #
  # === set_archive_type
  # ========================================================================= #
  def set_archive_type(i)
    cookbook_dataset?.set_archive_type(i)
  end

  # ========================================================================= #
  # === cookbook_dataset_prefix?
  #
  # This method name may be a slight misnomer, since class Prefix may
  # handle the prefix here.
  # ========================================================================= #
  def cookbook_dataset_prefix?
    if cookbook_dataset?
      cookbook_dataset?.prefix?
    else
      nil
    end
  end; alias cookbook_prefix?                           cookbook_dataset_prefix? # === cookbook_prefix?
       alias dataset_prefix?                            cookbook_dataset_prefix? # === dataset_prefix?
       alias determine_prefix_from_the_cookbook_dataset cookbook_dataset_prefix? # === determine_prefix_from_the_cookbook_dataset
       alias cookbooks_prefix?                          cookbook_dataset_prefix? # === cookbooks_prefix?

  # ========================================================================= #
  # === set_program_full_name
  # ========================================================================= #
  def set_program_full_name(i = :default)
    cookbook_dataset?.set_program_full_name(i)
  end

  # ========================================================================= #
  # === dataset_program_name_and_program_version?
  #
  # This method will always return the program name and the
  # associated program version, such as:
  #
  #   "htop-2.0.2"
  #
  # ========================================================================= #
  def dataset_program_name_and_program_version?
    cookbook_dataset?.program_name?
  end; alias cookbooks_program_name? dataset_program_name_and_program_version? # === cookbooks_program_name?

  # ========================================================================= #
  # === archive_type?
  # ========================================================================= #
  def archive_type?
    if cookbook_dataset?
      cookbook_dataset?.archive_type?
    end
  end; alias archive?                archive_type? # === archive?
       alias cookbooks_archive_type? archive_type? # === cookbooks_archive_type?

  # ========================================================================= #
  # === program_full_name?
  #
  # This will always query the dataset object.
  # ========================================================================= #
  def program_full_name?
    cookbook_dataset?.program_full_name? # This returns e. g. 'gnome-mahjonng-5.08.tar.xz'
  end

  # ========================================================================= #
  # === update_program_name
  # ========================================================================= #
  def update_program_name
    cookbook_dataset?.update_program_name
  end

  # ========================================================================= #
  # === program_name_and_version?
  #
  # This method must return the program_name and the program version
  # of any given program.
  # ========================================================================= #
  def program_name_and_version?
    cookbook_dataset?.program_name? # Delegate to dataset? here.
  end

  # ========================================================================= #
  # === set_program_short_name
  # ========================================================================= #
  def set_program_short_name(i)
    cookbook_dataset?.set_program_short_name(i)
  end

  # ========================================================================= #
  # === apply_patch?
  # ========================================================================= #
  def apply_patch?
    cookbook_dataset?.apply_patch?
  end

  # ========================================================================= #
  # === cookbook_dataset_configure_base_directory?
  #
  # We ask the cookbook-dataset object for the configure base directory.
  #
  # Most programs will NOT have such a setting.
  # ========================================================================= #
  def cookbook_dataset_configure_base_directory?
    cookbook_dataset?.configure_base_directory?
  end; alias cookbook_dataset_configure_base_dir? cookbook_dataset_configure_base_directory? # === cookbook_dataset_configure_base_dir?

  # ========================================================================= #
  # === required_deps_on?
  # ========================================================================= #
  def required_deps_on?
    cookbook_dataset?.required_deps_on?
  end; alias required_deps?            required_deps_on? # === required_deps?
       alias required_deps             required_deps_on? # === required_deps
       alias required_dependencies?    required_deps_on? # === required_dependencies?
       alias required_dependencies_on? required_deps_on? # === required_dependencies_on?

  # ========================================================================= #
  # === cookbook_dataset_postinstall?
  #
  # Query from the cookbook-yaml file at hand whether we will run
  # a postinstall-step or not.
  # ========================================================================= #
  def cookbook_dataset_postinstall?
    cookbook_dataset?.postinstall?
  end; alias cookbook_dataset_postinstallation? cookbook_dataset_postinstall? # === cookbook_dataset_postinstallation?

  # ========================================================================= #
  # === cookbook_dataset_make_command?
  # ========================================================================= #
  def cookbook_dataset_make_command?
    cookbook_dataset?.use_which_make_command?
  end

  # ========================================================================= #
  # === cookbook_dataset_pre_make_commands?
  # ========================================================================= #
  def cookbook_dataset_pre_make_commands?
    cookbook_dataset?.pre_make_commands?
  end

  # ========================================================================= #
  # === cookbook_dataset_preinstall?
  # ========================================================================= #
  def cookbook_dataset_preinstall?
    cookbook_dataset?.preinstall?
  end; alias preinstall? cookbook_dataset_preinstall? # === preinstall?

  # ========================================================================= #
  # === set_preinstall_action
  #
  # This method can be used to set different pre-install actions.
  # ========================================================================= #
  def set_preinstall_action(i)
    if i.is_a? String
      if    i.include? ','
        i = i.split(',').map(&:strip)
      elsif i.include? ';'
        i = i.split(';').map(&:strip)
      end
    end
    cookbook_dataset?.preinstall = i
  end

  # ========================================================================= #
  # === cookbook_dataset_libraries?
  # ========================================================================= #
  def cookbook_dataset_libraries?
    cookbook_dataset?.libraries?
  end; alias libraries? cookbook_dataset_libraries? # === libraries?

  # ========================================================================= #
  # === cookbook_dataset_use_autogen?
  #
  # Query-method over as to the cookbook-dataset has an entry
  # called use_autogen set to true.
  # ========================================================================= #
  def cookbook_dataset_use_autogen?
    cookbook_dataset?.use_autogen?
  end

  # ========================================================================= #
  # === pre_make_install_sed
  # ========================================================================= #
  def pre_make_install_sed?
    cookbook_dataset?.pre_make_install_sed?
  end; alias pre_make_install_sed pre_make_install_sed? # === pre_make_install_sed

  # ========================================================================= #
  # === use_which_configure_command?
  # ========================================================================= #
  def use_which_configure_command?
    @internal_hash[:configure_command]
  end

  # ========================================================================= #
  # === set_use_this_configure_command
  # ========================================================================= #
  def set_use_this_configure_command(i)
    @internal_hash[:configure_command] = i.to_s
  end

  # ========================================================================= #
  # === cookbook_dataset_set_configure_command
  # ========================================================================= #
  def cookbook_dataset_set_configure_command(i)
    case i
    when :meson
      i = i.to_s
    end
    cookbook_dataset?.set_configure_command(i)
  end; alias set_configure_command cookbook_dataset_set_configure_command # === set_configure_command

  # ========================================================================= #
  # === cookbook_dataset_configure_command?
  #
  # Most programs may use GNU autoconfigure, and thus "configure", but a
  # few programs such as openssl, require another program name, such as
  # "/config". This method here will query whether the given program
  # needs such a name. If this is the case then this will be used as
  # name rather than the alternative name "configure".
  # ========================================================================= #
  def cookbook_dataset_configure_command?
    cookbook_dataset?.configure_command
  end; alias configure_command? cookbook_dataset_configure_command? # === configure_command?

  # ========================================================================= #
  # === set_cookbook_dataset_program_version
  #
  # For this method to work, cookbook_dataset? must have been instantiated
  # before.
  # ========================================================================= #
  def set_cookbook_dataset_program_version(i)
    cookbook_dataset?.set_program_version(i)
  end

  # ========================================================================= #
  # === cookbook_dataset_headers?
  # ========================================================================= #
  def cookbook_dataset_headers?
    cookbook_dataset?.headers?
  end; alias headers? cookbook_dataset_headers? # === headers?

  # ========================================================================= #
  # === tags?
  #
  # Note that this method will attempt to return an Array.
  # ========================================================================= #
  def tags?
    cookbook_dataset?.tags?
  end; alias tags tags? # === tags

  # ========================================================================= #
  # === set_cookbook_dataset_program
  # ========================================================================= #
  def set_cookbook_dataset_program(i)
    cookbooks_dataset?.set_program(i)
  end

  # ========================================================================= #
  # === set_cookbook_dataset_program_path
  # ========================================================================= #
  def set_cookbook_dataset_program_path(i)
    cookbooks_dataset?.set_program_path(i)
  end

  # ========================================================================= #
  # === cookbooks_dataset_extracted_towards?
  # ========================================================================= #
  def cookbooks_dataset_extracted_towards?
    cookbooks_dataset? and cookbooks_dataset?.extracted_towards?
  end

  # ========================================================================= #
  # === do_the_dependencies_include_this_program?
  #
  # This method can be used to check whether the dependencies include a
  # given program at hand (the input).
  #
  # Keep in mind that dependencies can be written in either "name" or
  # "name >= 1.2.3" style. So we also have to check for the second
  # possibility there.
  # ========================================================================= #
  def do_the_dependencies_include_this_program?(
      i = compile_which_program?
    )
    dependencies = required_dependencies_on?
    # ======================================================================= #
    # Simplify the dependencies next.
    # ======================================================================= #
    dependencies.map! {|entry|
      if entry.include?(' >') or entry.include?(' <')
        entry = entry.split(' ').first.strip
      end
      entry
    }
    dependencies.include? i
  end

  # ========================================================================= #
  # === cookbook_dataset_run_configure?
  # ========================================================================= #
  def cookbook_dataset_run_configure?
    cookbook_dataset?.run_configure?
  end; alias cookbook_dataset_run_configure_script? cookbook_dataset_run_configure? # === cookbook_dataset_run_configure_script?

  # ========================================================================= #
  # === set_base_dir
  # ========================================================================= #
  def set_base_dir(i)
    cookbook_dataset?.set_base_dir = i
  end

  # ========================================================================= #
  # === rebuild_path
  # ========================================================================= #
  def rebuild_path
    cookbook_dataset?.reassemble_program_path
  end

  # ========================================================================= #
  # === url1?
  # ========================================================================= #
  def url1?
    cookbook_dataset?.url1?
  end; alias url? url1? # === url?
       alias url1 url1? # === url1

  # ========================================================================= #
  # === set_url1
  # ========================================================================= #
  def set_url1(i)
    cookbook_dataset?.set_url1(i)
  end; alias set_url set_url1 # === set_url

  # ========================================================================= #
  # === set_url2
  # ========================================================================= #
  def set_url2(i)
    cookbook_dataset?.set_url2(i)
  end

  # ========================================================================= #
  # === extra_information?
  # ========================================================================= #
  def extra_information?
    cookbook_dataset?.extra_information?
  end

  # ========================================================================= #
  # === pkgconfig_files?
  #
  # Query method over the registered .pc files of a particular program.
  # ========================================================================= #
  def pkgconfig_files?
    cookbook_dataset?.pkgconfig_files?
  end; alias pc_files? pkgconfig_files? # === pc_files?

  # ========================================================================= #
  # === cookbooks_binaries?
  # ========================================================================= #
  def cookbooks_binaries?
    cookbooks_dataset?.binaries?
  end; alias binaries? cookbooks_binaries? # === binaries?

  # ========================================================================= #
  # === cookbooks_dataset_use_glib_schema?
  # ========================================================================= #
  def cookbooks_dataset_use_glib_schema?
    cookbook_dataset?.use_glib_schema?
  end

  # ========================================================================= #
  # === cookbook_dataset_use_glib_schema?
  #
  # Whether our dataset uses glib install schema for some .xml files.
  # ========================================================================= #
  def cookbook_dataset_use_glib_schema?
    cookbook_dataset?.use_glib_schema
  end; alias use_glib_schema? cookbook_dataset_use_glib_schema? # === use_glib_schema?

  # ========================================================================= #
  # === base_dir?
  # ========================================================================= #
  def base_dir?
    cookbook_dataset?.base_dir?.to_s
  end

  # ========================================================================= #
  # === set_license
  # ========================================================================= #
  def set_license(i)
    cookbook_dataset?.set_license(i)
  end

  # ========================================================================= #
  # === set_description
  # ========================================================================= #
  def set_description(i)
    cookbook_dataset?.set_description(i)
  end; alias set_desc set_description # === set_desc

  # ========================================================================= #
  # === set_short_desc
  # ========================================================================= #
  def set_short_desc(i)
    cookbook_dataset?.set_short_desc(i)
  end

  # ========================================================================= #
  # === attach_program_name_and_program_version
  # ========================================================================= #
  def attach_program_name_and_program_version
    cookbook_dataset?.attach_program_name_and_program_version
  end

  # ========================================================================= #
  # === assemble_program_path
  # ========================================================================= #
  def assemble_program_path
    cookbook_dataset?.assemble_program_path
  end

  # ========================================================================= #
  # === set_real_short_name
  # ========================================================================= #
  def set_real_short_name(i)
    cookbook_dataset?.set_real_short_name(i)
  end

  # ========================================================================= #
  # === set_short_name
  # ========================================================================= #
  def set_short_name(i)
    cookbook_dataset?.set_short_name(i)
  end

  # ========================================================================= #
  # === main_eight?
  # ========================================================================= #
  def main_eight?
    cookbook_dataset?.main_eight?
  end

  # ========================================================================= #
  # === cookbook_dataset_set_this_archive
  # ========================================================================= #
  def cookbook_dataset_set_this_archive(i)
    cookbook_dataset?.set_this_archive(i)
  end
  
  # ========================================================================= #
  # === chop_off_revision_number
  # ========================================================================= #
  def chop_off_revision_number
    cookbook_dataset?.chop_off_revision_number
  end

  # ========================================================================= #
  # === empty_archive_type
  #
  # This will clear the archive type.
  # ========================================================================= #
  def empty_archive_type
    cookbook_dataset?.archive_type = ''
  end

  # ========================================================================= #
  # === determine_program_full_name
  # ========================================================================= #
  def determine_program_full_name
    cookbook_dataset?.determine_program_full_name
  end

  # ========================================================================= #
  # === cookbook_dataset_use_build_directory?
  # ========================================================================= #
  def cookbook_dataset_use_build_directory?
    cookbook_dataset?.use_build_directory?
  end

  # ========================================================================= #
  # === program_compile_name?
  # ========================================================================= #
  def program_compile_name?
    cookbook_dataset?.program_compile_name?.to_s
  end

  # ========================================================================= #
  # === assume_default_archive_type
  # ========================================================================= #
  def assume_default_archive_type
    cookbook_dataset?.assume_default_archive_type
  end

  # ========================================================================= #
  # === use_default_configure_command
  #
  # This method will hard-set the configure command.
  # ========================================================================= #
  def use_default_configure_command(
      i = 'configure'
    )
    cookbook_dataset?.configure_command = i
  end

  # ========================================================================= #
  # === cookbook_dataset_keep_extracted?
  #
  # This method simply wraps over the cookbook_dataset keep_extracted value.
  # ========================================================================= #
  def cookbook_dataset_keep_extracted?
    cookbook_dataset?.keep_extracted?
  end; alias dataset_keep_extracted? cookbook_dataset_keep_extracted? # === dataset_keep_extracted?

  # ========================================================================= #
  # === update_program_path
  # ========================================================================= #
  def update_program_path
    cookbook_dataset?.update_program_path
  end; alias assemble_full_path update_program_path # === assemble_full_path

end; end