#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/create_program.rb'
# =========================================================================== #
require 'rbt/create_program/create_program.rb'

module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === consider_creating_appdir_skeleton
  #
  # This method will create an AppDir skeleton, by delegating towards
  # class RBT::CreateProgram. 
  #
  # We will only do so if the prefix includes the AppDir base directory
  # though. By default, that base directory would hold the value of
  # the $PROGRAMS environment variable - at the least on my system,
  # this defaults to /Programs/. You can change this value if you
  # modify a yaml file though. That yaml file is called
  # "programs_dir.yml" and resides in the yaml/configuration/
  # subdirectory of the RBT project.
  #
  # The method accepts one argument, which is either the program name,
  # or alternatively the full name to the program in question.
  #
  # Normally, the program name will be defined in the yaml data object, 
  # but since you can also overrule this setting in the yaml file,
  # we need an argument here to allow this too.
  # ========================================================================= #
  def consider_creating_appdir_skeleton(
      i = return_proper_prefix
    )
    # ======================================================================= #
    # Check for an AppDir prefix next:
    # ======================================================================= #
    i = i.to_s.dup
    if i.include? programs_dir?
      unless i =~ /\d+/
        i << program_version?
      end
      i << '/' unless i.end_with? '/'
      # ===================================================================== #
      # Next, delegate towards class RBT::CreateProgram.
      # ===================================================================== #
      _ = RBT::CreateProgram.new(i, :dont_run_yet) {{
        use_colours: use_colours?,
        use_opn:     use_opn?
      }}
      _.run_simulation = run_simulation?
      # _.set_base_directory = programs_directory? # <-- I think this is no longer needed.
      _.run
    end
  end; alias create_program_skeleton consider_creating_appdir_skeleton # === create_program_skeleton
       alias create_appdir_skeleton  consider_creating_appdir_skeleton # === create_appdir_skeleton
       alias create_app_skeleton     consider_creating_appdir_skeleton # === create_app_skeleton

end; end