#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/ensure.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === ensure_that_the_program_exists
  #
  # This method will control whether we can continue to run. We can
  # continue to run if the program exists as part of the cookbooks
  # project. One exception to this rule is when the last character
  # is a '*' token. In this case we assume that he user wants to
  # simply get a list of available programs instead, starting
  # with that specific name, e. g. "e*" should yield a list of
  # all programs starting with the character "e".
  #
  # You can try this out by doing:
  #   ry e*
  # ========================================================================= #
  def ensure_that_the_program_exists(
      i
    )
    i = i.to_s
    if honour_check_to_ensure_that_the_program_exists?
      unless cookbooks_include_this_program?(i)
        # =================================================================== #
        # Handle not-registered programs next:
        # =================================================================== #
        opne "#{crimson('The program `')}#{seagreen(i)}#{crimson('` could not be found. We thus ')}"\
             "#{crimson('can not compile')}"
        opne crimson('it and hence, can not continue. If this '\
             'file should exist, it')
        opne crimson('could be found at:')
        e
        e "  #{sfile(return_location_to_this_programs_yaml_file(i))}"
        e
        we_can_not_continue
      end unless i.end_with? '*'
    end
    return can_we_continue?
  end

  # ========================================================================= #
  # === ensure_that_we_can_extract_to_a_proper_directory
  # ========================================================================= #
  def ensure_that_we_can_extract_to_a_proper_directory
    if extract_to?.to_s.empty?
      set_extract_to(temp_directory?)
    end
  end

  # ========================================================================= #
  # === ensure_that_the_log_directory_exists
  #
  # This method will ensure that the log directory, usually the one
  # at "/Depot/Temp/rbt/", exists.
  # ========================================================================= #
  def ensure_that_the_log_directory_exists
    _ = log_directory?
    mkdir(_) unless File.directory? _
  end

end; end