#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file collects archive and extract-related code belonging to
# class RBT::Compile.
# =========================================================================== #
# require 'rbt/compile/extract.rb'
# =========================================================================== #
require 'rbt/compile/base/compile_base.rb'
require 'extracter'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === set_extract_to_this_directory_as_specified_by_the_user
  # ========================================================================= #
  def set_extract_to_this_directory_as_specified_by_the_user(i)
    unless i.end_with? '/'
      i = i.dup if i.frozen?
      i << '/'
    end
    @internal_hash[:extract_to_this_directory_as_specified_by_the_user] = i
  end

  # ========================================================================= #
  # === skip_extracting?
  #
  # Query-method over as to whether we will skip extracting the archive
  # at hand. This can usually only be done when the archive has already
  # been extracted somehow.
  # ========================================================================= #
  def skip_extracting?
    _ = @internal_hash[:skip_extracting]
    _ == true # This check prohibits nil and false.
  end; alias skip_extracting_the_archive? skip_extracting? # === skip_extracting_the_archive?

  # ========================================================================= #
  # === do_not_skip_extracting
  #
  # This method will set towards extracting the archive.
  # ========================================================================= #
  def do_not_skip_extracting
    @internal_hash[:skip_extracting] = false
  end

  # ========================================================================= #
  # === set_skip_extracting
  #
  # Use this method when we shall skip the extracting phase.
  # ========================================================================= #
  def set_skip_extracting(
      be_verbose = true
    )
    if be_verbose
      opnn; e 'We will not extract our source, thus we will skip '\
              'extracting the archive.'
    end
    @internal_hash[:skip_extracting] = true
  end; alias do_not_extract set_skip_extracting # === do_not_extract

  # ========================================================================= #
  # === extract_to_this_directory?
  #
  # This query-method tells us where we will extract (our archive) to.
  #
  # The temp-directory should usually be available through
  # RBT.rbt_log_directory?.
  # ========================================================================= #
  def extract_to_this_directory?
    _ = @internal_hash[:extract_to_this_directory]
    if @internal_hash[:extract_to_this_directory_as_specified_by_the_user]
      _ = @internal_hash[:extract_to_this_directory_as_specified_by_the_user]
    end
    _
  end; alias extract_to?                 extract_to_this_directory? # === extract_to?
       alias extract_where_to?           extract_to_this_directory? # === extract_where_to?
       alias extracted_directory?        extract_to_this_directory? # === extracted_directory?
       alias dir?                        extract_to_this_directory? # === dir?
       alias extract_to_this_dir?        extract_to_this_directory? # === extract_to_this_dir?
       alias extract_to_which_directory? extract_to_this_directory? # === extract_to_which_directory?

  # ========================================================================= #
  # === extract_to_default_directory
  #
  # By default, we will extract to this directory - but the value is
  # lateron refined to the full path instead; so the assignment done
  # here is just so that we have a starting value we can work with.
  #
  # This method will be called from within the method reset().
  # ========================================================================= #
  def extract_to_default_directory(
      i = RBT.log_directory?
    )
    @internal_hash[:extract_to_this_directory] = i
  end

  # ========================================================================= #
  # === set_extract_to_this_directory
  #
  # This method can be used to keep track onto which directory archives
  # will be extracted.
  #
  # Within RBT, the target is also known as the "homedir" variable.
  # ========================================================================= #
  def set_extract_to_this_directory(
      i          = RBT.log_directory?,
      be_verbose = false
    )
    case be_verbose
    when :be_verbose
      be_verbose = true
    end
    i = i.to_s
    case i
    # ======================================================================= #
    # === pwd
    # ======================================================================= #
    when 'pwd','homedir'
      i = rds(return_pwd+'/')
    end unless File.exist? i
    unless i.end_with? '/'
      i = i.dup if i.frozen?
      i << '/'
    end
    # ======================================================================= #
    # Display some information to the user, if we operate in a verbose manner.
    # ======================================================================= #
    if be_verbose
      opnn; e 'We will use '+sdir(i)+' as our directory into which '\
              'we will extract the source archive into.'
    end
    @internal_hash[:extract_to_this_directory] = i.dup
  end; alias set_extract_to set_extract_to_this_directory # === set_extract_to
       alias extract_to     set_extract_to_this_directory # === extract_to
       alias set_homedir    set_extract_to_this_directory # === set_homedir

  # ========================================================================= #
  # === extract_this_program_if_it_exists        (extract tag, extracter tag)
  #
  # This method will extract the given archive directory, by delegating
  # towards class Extracter.
  #
  # This method ought to be invoked quite early, logically; the method
  # "consider_extracting_this_program()" may be used to invoke this method.
  # ========================================================================= #
  def extract_this_program_if_it_exists(
      i                                                         = cookbook_dataset_program_path?,
      extract_to_this_location                                  = extract_to?,
      shall_we_report_to_the_user_if_the_archive_does_not_exist = true
    )
    if File.exist? i
      if compile_which_program?.nil?
        set_program_name(ProgramInformation.return_name(i))
      end
      if program_version?.nil?
        set_program_version(
          ProgramInformation.return_program_version(i)
        )
      end
      @program_name_and_program_version = compile_which_program?+'-'+program_version?.to_s
      if run_simulation?
        opnn; e "We would normally extract `#{sfile(i)}` "\
                "to #{sdir(extract_to_this_location)} next,"
        opnn; e 'but since we run in simulation-mode, we will not '\
                'extract anything.'
      else
        invoke_extracter(i) # <- Delegate to invoke_extracter() here.
      end
      # ===================================================================== #
      # Next we will exit if the user told us that he/she is only
      # interested in extracting the program.
      # ===================================================================== #
      exit_program if extract_only?
    else
      if shall_we_report_to_the_user_if_the_archive_does_not_exist
        opnn; e swarn('No file could be found at `')+
                sfile(i)+
                swarn('` - thus,')
        opnn; e swarn('we can not extract anything.')
        # =================================================================== #
        # Also set a specific error, so that we won't continue.
        # =================================================================== #
        error_is 'We can not continue. Has the archive '\
                 'been properly extracted?'
      end
    end
  end; alias extract_this_program             extract_this_program_if_it_exists # === extract_this_program
       alias extract_this_archive             extract_this_program_if_it_exists # === extract_this_archive
       alias extract_package                  extract_this_program_if_it_exists # === extract_package
       alias extract_program                  extract_this_program_if_it_exists # === extract_program
       alias extract                          extract_this_program_if_it_exists # === extract
       alias consider_extracting_this_program extract_this_program_if_it_exists # === consider_extracting_this_program
       alias default_extract                  extract_this_program_if_it_exists # === default_extract
       alias do_extract                       extract_this_program_if_it_exists # === do_extract
       alias try_to_extract_this_archive      extract_this_program_if_it_exists # === try_to_extract_this_archive
 
  # ========================================================================= #
  # === set_keep_the_extracted_archive
  #
  # This method will keep the source-archive extracted. Note that this
  # is not the default - by default we will remove the extracted
  # archive.
  # ========================================================================= #
  def set_keep_the_extracted_archive(i = true)
    @internal_hash[:keep_the_extracted_archive] = i
  end; alias set_keep_extracted_archive          set_keep_the_extracted_archive # === set_keep_extracted_archive
       alias set_keep_extracted                  set_keep_the_extracted_archive # === set_keep_extracted
       alias do_not_remove_the_extracted_archive set_keep_the_extracted_archive # === do_not_remove_the_extracted_archive
       alias do_not_remove_extracted_archive     set_keep_the_extracted_archive # === do_not_remove_extracted_archive

  # ========================================================================= #
  # === do_set_keep_extracted
  #
  # This delegates towards the method set_keep_the_extracted_archive
  # and may also report to the end-user as to what is being done. 
  # ========================================================================= #
  def do_set_keep_extracted(be_verbose = true)
    case be_verbose
    when :be_silent,
         :be_quiet
      be_verbose = false
    end
    if be_verbose
      opnn; e 'We will not remove the extracted directory '\
              'after we have finished.'
    end
    set_keep_the_extracted_archive
  end; alias do_set_keep_extracted_archive do_set_keep_extracted # === do_set_keep_extracted_archive

  # ========================================================================= #
  # === keep_the_extracted_archive?
  #
  # This is a query method towards a setting kept in the internal Hash
  # variable.
  #
  # A boolean value will be returned. This boolean value determines whether
  # the archive will be kept after it was extracted, or whether it will be
  # removed again after it was extracted. 
  # ========================================================================= #
  def keep_the_extracted_archive?
    _ = cookbook_dataset_keep_extracted? # <- The default. 
    unless @internal_hash[:keep_the_extracted_archive].nil?
      _ = @internal_hash[:keep_the_extracted_archive]
    end
    _
  end; alias keep_extracted_archive? keep_the_extracted_archive? # === keep_extracted_archive?
       alias keep_extracted?         keep_the_extracted_archive? # === keep_extracted?

  # ========================================================================= #
  # === set_dont_keep_archive
  #
  # Use this method when you don't want to keep the extracted archive.
  # ========================================================================= #
  def set_dont_keep_archive(
      be_verbose = true
    )
    if be_verbose
      opnn; e 'We will not keep our extracted archive.'
    end
    @internal_hash[:keep_the_extracted_archive] = false
  end

  # ========================================================================= #
  # === extract_only?
  # ========================================================================= #
  def extract_only?
    @internal_hash[:extract_only]
  end

  # ========================================================================= #
  # === verbose_do_extract_only
  # ========================================================================= #
  def verbose_do_extract_only
    opnn; e 'We will merely extract the archive, then exit.'
    @internal_hash[:extract_only] = true
  end

  # ========================================================================= #
  # === set_extracted_base_directory
  # ========================================================================= #
  def set_extracted_base_directory(i)
    @internal_hash[:extracted_base_directory] = i
  end

  # ========================================================================= #
  # === extracted_base_directory?
  # ========================================================================= #
  def extracted_base_directory?
    @internal_hash[:extracted_base_directory]
  end

  # ========================================================================= #
  # === do_skip_extracting
  # ========================================================================= #
  def do_skip_extracting
    @internal_hash[:skip_extracting] = true
  end

  # ========================================================================= #
  # === remove_all_temp_directories
  #
  # This method will remove all directories in the /Depot/Temp
  # hierarchy. It was added since it is sometimes easier to just
  # remove all extracted archives via batch-process.
  # ========================================================================= #
  def remove_all_temp_directories
    _ = rbt_log_dir?
    all_directories_there = Dir["#{_}*"].select {|entry| File.directory? entry }
    opnn; e "All #{sfancy(all_directories_there.size.to_s)}"\
            " directories in the #{sdir(_)} hierarchy will be removed."
    remove_these_directories(all_directories_there)
  end

  # ========================================================================= #
  # === extract_the_archive?
  #
  # Query method over as to whether RBT will extract the archive or not.
  # ========================================================================= #
  def extract_the_archive?
    @internal_hash[:extract_the_archive]
  end

  # ========================================================================= #
  # === skip_extracting
  #
  # Not every archive has to be extracted. For example, .gem files do not
  # have to be extracted.
  # ========================================================================= #
  def skip_extracting
    @internal_hash[:extract_the_archive] = false
  end

  # ========================================================================= #
  # === verbose_skip_extracting
  # ========================================================================= #
  def verbose_skip_extracting
    opnn; e 'The archive will not be extracted.'
    skip_extracting
  end

  # ========================================================================= #
  # === remove_extracted_archive
  #
  # This method simply removes the extracted archive. If @keep_extracted
  # was set to true we will not run it, but this is decided in another
  # method called consider_removing_extracted_archive().
  #
  # Note that the invocation of this method must happen last, as otherwise
  # we may run into problems such as:
  #
  #   in `pwd': No such file or directory - getcwd (Errno::ENOENT)
  #
  # We can not use the string returned by the method extract_to_this_directory?
  # because this will refer to the rbt/ base directory.
  # ========================================================================= #
  def remove_extracted_archive(
      where = :build_anew
    )
    case where
    when :build_anew, :default
      where = rds(
        extract_to_this_directory?+
        cookbook_dataset_real_program_name?.to_s+'-'+program_version?.to_s+
        '/'
      )
    end
    if run_simulation? # Run in simulation mode here.
      opnn; e sfancy('We run in simulation mode right now. Normally the directory ')
      opnn; e sdir(where)+sfancy('would next be removed.')
    else # ================================================================== #
         # We must honour the keep_extracted setting. We also will not
         # remove the log directory.
         # ================================================================== #
      if keep_the_extracted_archive? or (where == RBT.log_directory?)
        # =================================================================== #
        # In this case we do not have to do anything.
        # =================================================================== #
      else
        opnn; e "Removing the extracted archive at `#{sdir(where)}`."
        unless File.directory? where
          _ = 'However had, no such directory exists. Perhaps '\
              'it was already removed, or it has never existed.'
          opnn; e _
        end
        # =================================================================== #
        # We must check whether the directory exists, before trying to
        # remove it.
        # =================================================================== #
        if File.directory? where
          remove_directory(where)
        else
          # No directory exists there.
        end
      end
    end
  end; alias consider_removing_the_extracted_directory remove_extracted_archive # === consider_removing_the_extracted_directory
       alias consider_removing_the_extracted_archive   remove_extracted_archive # === consider_removing_the_extracted_archive
       alias consider_removing_extracted_archive       remove_extracted_archive # === consider_removing_extracted_archive
       alias remove_the_extracted_archive              remove_extracted_archive # === remove_the_extracted_archive

  # ========================================================================= #
  # === invoke_extracter
  #
  # Delegate towards class Extracter next.
  # ========================================================================= #
  def invoke_extracter(
      i                         = program_path?,
      extract_to_this_directory = extract_to_this_directory?
    )
    unless skip_extracting?
      # ===================================================================== #
      # Remove the extracted directory if it exists next. This will,
      # however had, NOT be done if we extract to another target
      # directory, such as when issuing a command like
      # "ry htop --extract-to=/opt/test".
      # ===================================================================== #
      # possibly_existing_old_extracted_archive = cookbooks_dataset_extracted_towards?
      # unless possibly_existing_old_extracted_archive.include? extract_to?
        # =================================================================== #
        # In this case we reasonably assume that the user has NOT given
        # a --extract-to value.
        # =================================================================== #
        if cookbooks_dataset?
          _ = cookbooks_dataset_extracted_towards?.dup
          if File.directory?(_)
            opnn; e 'Removing the old directory at `'+
                    sdir(_)+'`.'
            remove_directory(_)
          end
        end
      # end
      # ===================================================================== #
      # Delegate towards class Extracter next.
      # ===================================================================== #
      Extracter.extract_this(
        i,
        to:                      extract_to_this_directory,
        prepend_this_namespace:  'RBT->',
        use_colours:             use_colours?,
        verbosity:               :be_verbose,
        run_simulation:          run_simulation?,
        pad_opn_with_n_tokens:   RBT.n_padding_to_use_for_opn?,
        use_opn:                 show_the_classname?
      )
    end
  end; alias extract_what_to invoke_extracter # === extract_what_to

end; end