#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# All FTP-related code is to be stored here.
# =========================================================================== #
# require 'rbt/compile/ftp.rb'
# =========================================================================== #
require 'rbt/compile/base/compile_base.rb'
begin
  require 'ftp_paradise/without_gui_bindings'
rescue LoadError; end

module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === upload_to
  #
  # This can be used to upload a RBT file (or any other file) to a
  # remote host, via FTP.
  #
  # We will make use of the FTP-Paradise project for this.
  #
  # Usage examples:
  #   ry php upload /tmp/python-3.1.1.recipe
  #   ry php upload
  # ========================================================================= #
  def upload_to(
      upload_this_file = '/Depot/Temp/xchat.desktop',
      this_url         = 'shevegen.square7.ch',
      optional_cd_into_this_directory = nil
    )
    case this_url
    when :default
      this_url = 'shevegen.square7.ch'
    end
    this_recipe = '/Depot/Temp/xchat.desktop' if this_recipe.nil?
    ftp = FtpParadise::Connection.new
    ftp.login_name = 'shevegen'
    ftp.password   = ENV['MY_FTP_PASSWORD']
    ftp.url        = this_url
    if optional_cd_into_this_directory
      ftp.change_directory(optional_cd_into_this_directory)
    end
    ftp.upload(upload_this_file)
  end

  # ========================================================================= #
  # === do_binary_upload_of_this_file
  # ========================================================================= #
  def do_binary_upload_of_this_file(this_file)
    _ = this_file
    if File.exist? _
      FtpParadise.upload_this_binary_file(_, :be_verbose)
    else
      e "No file at `#{sfile(_)}` exists."
    end
  end

  # ========================================================================= #
  # === do_upload_ruby_binary
  #
  # To invoke this, try:
  #   rbt --upload-ruby-binary
  # ========================================================================= #
  def do_upload_ruby_binary
    ensure_that_the_log_directory_exists
    # ======================================================================= #
    # Obtain the path to the local ruby version in use:
    # ======================================================================= #
    path_to_ruby = "#{programs_dir?}Ruby/"
    base_directory_to_upload = "#{path_to_ruby}Current" # This is a symlink.
    ruby_version = File.readlink(base_directory_to_upload)
    cd log_dir?
    _ = path_to_ruby+ruby_version+'/' # This is the full, real path.
    # ======================================================================= #
    # Next, use tar to create an archive.
    # ======================================================================= #
    readelf_command_to_use  = "readelf -h #{_}bin/ruby"
    binary_32_bit_or_64_bit = `#{readelf_command_to_use}`
    binary_32_bit_or_64_bit = binary_32_bit_or_64_bit.split(N).select {|line|
      line.include? 'Class:'
    }
    binary_32_bit_or_64_bit = binary_32_bit_or_64_bit.to_s.dup
    binary_32_bit_or_64_bit =~ /(\d+)/
    binary_32_bit_or_64_bit = $1.to_s.dup+'-bit'
    e 'We will put this archive into `'+sdir((Dir.pwd+'/').squeeze('/'))+'`.'
    name_of_the_ruby_archive = 'ruby-'+ruby_version+'-'+binary_32_bit_or_64_bit+'.tar.xz'
    _ = 'tar cfJv '+name_of_the_ruby_archive+' '+_
    esystem _
    _ = (Dir.pwd+'/'+name_of_the_ruby_archive).squeeze '/'
    if File.exist? _
      e 'The file now resides at `'+sfile(_)+'`.'
      do_binary_upload_of_this_file(_) # Here we finally upload the archive.
    else
      e 'Some error appears to have happened - no archive at '+sfile(_)
      e ' could be found.'
    end
  end

end; end