#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Everything related to headers, that is, C-style header files .h,
# will be stored here.
# =========================================================================== #
# require 'rbt/compile/headers.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === install_only_headers?
  # ========================================================================= #
  def install_only_headers?
    @internal_hash[:install_only_the_headers]
  end; alias install_only_the_headers? install_only_headers? # === install_only_the_headers?

  # ========================================================================= #
  # === show_available_headers_of_this_program
  #
  # This method will show the available headers of the given program
  # at hand.
  #
  # Invocation example:
  #   ry glibc --headers?
  # ========================================================================= #
  def show_available_headers_of_this_program(
      i = compile_which_program?
    )
    array_headers = cookbook_dataset_headers?
    if array_headers.empty?
      opnn; e "The program `#{sfancy(i)}` will not install any .h files."
    else
      opnn; e "The program `#{sfancy(i)}` will install these .h files:"
      e
      array_headers.each {|this_header_file|
        e "  #{sfancy(this_header_file)}"
      }
      e
    end
  end

  # ========================================================================= #
  # === do_install_only_the_headers
  #
  # This method will only intall the headers of a program. You must
  # call the method action_install_only_headers() for real action.
  # ========================================================================= #
  def do_install_only_the_headers(
      of_this_program = compile_these_programs?.first
    )
    do_not_compile
    do_not_run_configure
    disable_logging # We dont want to log.
    opnn; e "Installing only headers for the program `"\
            "#{sfancy(of_this_program)}` now."
    @internal_hash[:install_only_the_headers] = true
  end

  # ========================================================================= #
  # === action_install_only_headers
  #
  # This method handles the actual copying of all headers. It
  # must be called at the proper time.
  # ========================================================================= #
  def action_install_only_headers(
      from_this_directory = extract_to?+program_name?+'/'
    )
    from_this_directory = from_this_directory.to_s.dup
    from_this_directory << '/' unless from_this_directory.end_with? '/'
    _ = ''.dup
    all_headers = Dir["#{from_this_directory}*.h"]
    _ = all_headers.join(',')[0..3] unless all_headers.empty?
    opnn; e "Now copying all found headers from '{sdir(from_this_directory)}"\
            " to #{sdir('/usr/include/')}:"
    copy_recursively(all_headers, '/usr/include/', true)
    do_not_continue_after_configure_step
    do_not_continue_after_make_step
    be_silent
  end

end; end