#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/help.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/e.rb'
require 'rbt/toplevel_methods/programs_directory.rb'
require 'rbt/toplevel_methods/cliner.rb'
require 'rbt/toplevel_methods/gobolinux.rb'
require 'rbt/compile/class_methods.rb'
require 'rbt/compile/opn.rb'
require 'rbt/colours/colours.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === show_help_for_this_program
  #
  # Use this method if you wish to feedback how to use this class.
  # ========================================================================= #
  def show_help_for_this_program(
      i         = '',
      be_silent = false
    )
    be_silent = true if be_silent == :be_silent
    e simp(i)+' has these options available:' unless be_silent
    display_standard_help_options # This is stored in help.rb
  end

  # ========================================================================= #
  # === RBT::Compile.display_standard_help_options
  #
  # Set the standard help-options only here.
  #
  # You can view them by issuing this command:
  #
  #   rbt --help
  #
  # ========================================================================= #
  def self.display_standard_help_options(
      exit_now = true
    )
    pad_to_most_right = ' ' * 40
    lpad = '  '
    RBT.cliner {
      middle_pad = 44
      e
      RBT::Compile.output_namespace?
      e
      e
      efancy 'These are the available help options for the Compile-Section '\
             'of the RBT scripts:'
      e
      efancy 'The basic command to compile a program is:'
      e
      e RBT.gold('    rbt [NAME_OF_THE_PROGRAM]')
      e RBT.gold('    rbt python')+RBT.orange(' # <- example input; here we '\
        'would try to compile python')
      e
      efancy lpad+'-b      --build-dir=[]                        Set a specific build directory'
      efancy lpad+'-h      --help                                Display this help subsection here'
      efancy '          --permanently_extract_to=[]           permanently extract the '\
             'archives to another directory'
      efancy lpad+'-e      --extract_to=[]                       extract the archives to '\
             'another directory'
      efancy '                                                (--homedir= is an alias to this setting)'
      efancy lpad+'-k      --keep_extracted                      Keep the sourceball extracted '\
             '(normally the extracted'
      efancy '                                                archive is '\
             'removed after compilation)'
      efancy '          --batch-compile=                      Batch-compile some '\
             'programs such as batch compiling'
      e      (' ' * middle_pad)+RBT.royalblue('      rbt --batch-compile=mate')
      efancy '          --dont-use-configure-options          Do not use any configure options '\
             'from the .yml file'
      efancy lpad+'-n      --no-harm                             Run in simulation mode'
      efancy lpad+'-nc     --nocolour / --nocolor                Disable colours'
      efancy '          --use-old-configure-options           Re-use old configure '\
             'options, e. g. from GCC'
      efancy lpad+'-sl     --sleep<number>                       Sleep for <number> seconds before continuing'
      efancy '          --permanently-disable-ccache          Permanently disable ccache, '\
             'via the commandline'
      efancy '          --disable-ccache                      Do not use ccache for the current run'
      efancy lpad+'-S      --no-symlink                          Do not symlink the AppDir that was compiled'
      efancy lpad+'-P      --no-postinstall                      Do not run the postinstall instructions'
      efancy '          --preinstall="sh bootstrap"           Run the specified pre-install action'
      efancy '          --use-gobolinux-symlinkprogram        Use the GoboLinux SymlinkProgram rather than '
      efancy '                                                class RBT::SymlinkProgram, for symlinking'
      efancy '          --use-meson                           Use the meson build system'
      efancy lpad+'-no     --no-extract                          Do not extract tarball again '\
             '(skip extracting the source archive)'
      efancy lpad+'-cmake  --cmake                               Use cmake rather than make'
      efancy lpad+'-clang  --clang                               Use clang rather than the default GCC '\
             'to compile C/C++ programs'
      efancy '          --create-yaml-file-for=subversion     Create a .yml file for this program, '\
             'holding the relevant information'
      efancy '                                                for compiling the program from source'
      efancy '                                                (Note that this requires the clang '\
                                                              'compiler of LLVM being'
      efancy '                                                available, of course.)'
      efancy '          --create-log-directory                Create the log directory.'
      efancy '          --paste-blfs-page                     Fetch and paste remote '\
             'data from the BLFS page of the given program'
      efancy lpad+'-slack  --slack-desc                          Create a Slackware slack-desc file'
      efancy '          --purge-traditional-binaries-of=gnupg Purge binaries of this program'
      efancy '          --valid-sys-tags?                     Show which '+sdir('/System/Tag')+
              sfancy(' entries are valid')
      efancy '          --symlink-headers                     Symlink all .h files into '\
             'the '+sdir('/usr/include/')+sfancy(' hierarchy')
      efancy '          --symlink-all-upcased-programs-to-downcased-variants Create '\
             'symlinks'
      efancy (' ' * (middle_pad-2))+'      for all upcased program entries under '+
             sdir(RBT.programs_dir?)+'.' 
      efancy '          --homedir                             Compile the program into your '\
             'HOME directory such as '+sdir('/home/name/')+sfancy('.')
      efancy lpad+'-ver    --verbose                             Be verbose in the output'
      efancy '          --do-not-use-special-flags            Do not use ENV (environmental '\
             'flags) at all.'
      efancy '          --update-entry                        Update a program, such '\
             'as via '
      e      RBT.royalblue((' ' * middle_pad)+
             '       rbt --update-entry http://ftp.gnome.org/pub/gnome/sources/glib/2.53/glib-2.53.4.tar.xz')
      efancy '          --edit-file=htop                      Open the htop.yml '\
             'cookbook-file, in order to inspect and/or edit it;'
      efancy '                                                the '+simp('$EDITOR')+
             sfancy(' variable can be used to specify which editor is to be used.')
      efancy "          --remove-la-files                     Remove "\
             ".la files of this program (requires the --prefix to include "\
             "#{sdir(::RBT.programs_dir?)})"
      efancy '          --suggest-cookbook-for=glib           Show a possible '\
             'content of glib.yml file'
      efancy lpad+'-v      --version                             Use a specific '\
             'version, such as, for instance: '
      efancy (' ' * middle_pad)+
             RBT.royalblue('       rbt bluefish --version=1.0.7')
      if RBT.is_on_gobolinux?
        # =================================================================== #
        # If we are on gobolinux then we also mention how to create
        # gobolinux-recipes.
        # =================================================================== #
        efancy '   --gobolinux-recipe                            Create a GoboLinux '\
               'recipe. (The "-" can be omitted.'
        efancy pad_to_most_right+'        A usage example follows next.)'
        efancy pad_to_most_right+'        Usage example: '+
               simp('rbt zenity --gobolinux-recipe')
      end
      e
      e 'Some shorter commandline options exist for toplevel methods:'
      e
      efancy '  rbt --download htop # This will download htop into '\
             'the local directory'
      efancy '  rbt --extract htop  # This will extract htop; or download '\
             'it if it does not exist, and then extract it'
      e # Need a spacer before we display the next part.
      e
      eimp 'Also note that these options can be chained together. Abbreviations '\
           'can be used as'
      eimp 'well. The options '+sfancy('noext nocol')+
           simp(' will all work fine, being equal to ')+
           sfancy('--no-extract --nocolor')+simp('.')
      eimp 'Abbreviations allow you to type less, but the longer variants '\
           'work just as well,'
      eimp 'in the event that you wish to be more explicit.'
      e
    }
    exit if exit_now
  end

  # ========================================================================= #
  # === display_standard_help_options                              (help tag)
  # ========================================================================= #
  def display_standard_help_options(
      exit_now = true
    )
    RBT::Compile.display_standard_help_options(exit_now)
  end

end; end