#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/infobox.rb'
# =========================================================================== #
# Require the infobox.
# =========================================================================== #
require 'rbt/infobox/infobox.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile.new

  # ========================================================================= #
  # === consider_showing_the_infobox
  # ========================================================================= #
  def consider_showing_the_infobox
    if show_infobox?
      show_infobox(
        cookbook_dataset_with_extended_configure_options?
      )
    end
  end

  # ========================================================================= #
  # === show_infobox
  #
  # The first argument should be the dataset for the program that we
  # are about to install/compile.
  # ========================================================================= #
  def show_infobox(
      i = compile_which_programs?
    )
    if i.is_a? Array
      i.each {|entry| show_infobox(entry) }
    else
      if show_the_infobox?
        pass_this_dataset_into_the_infobox = cookbook_dataset_with_extended_configure_options?.dup
        pass_this_dataset_into_the_infobox.set_extract_to(
          extract_to_this_directory?
        )
        # ===================================================================== #
        # We can modify the dataset, for example, by using another prefix.
        # ===================================================================== #
        if user_prefix? and !user_prefix?.empty?
          pass_this_dataset_into_the_infobox.internal_hash?[:yaml_dataset]['prefix'] = user_prefix?.dup
        end
        # ===================================================================== #
        # Check whether the user actually makes use of any configure options
        # or not. If not, the cookbook dataset passed into Infobox will be
        # modified.
        # Keep in mind that @internal_hash[:try_to_use_configure_options]
        # is set to nil within the method reset(). 
        # ===================================================================== #
        unless @internal_hash[:try_to_use_configure_options].nil?
          pass_this_dataset_into_the_infobox.configure_options = nil
        end
        unless @internal_hash[:postinstallation].nil?
          pass_this_dataset_into_the_infobox.postinstallation = @internal_hash[:postinstallation]
        end
        pass_this_dataset_into_the_infobox.compiler_to_use = compiler_to_use?
        use_this_build_system = use_which_build_system?
        case use_this_build_system
        when :infer_automatically
          use_this_build_system = RBT.infer_build_system
        end
        pass_this_dataset_into_the_infobox.set_use_this_build_system(
          use_this_build_system
        )
        pass_this_dataset_into_the_infobox.set_configure_options(
          configure_options?
        )
        do_this = if extract_only?
                    'Extracting'
                  elsif has_to_be_compiled?
                    'Compiling'
                  else
                    'Installing'
                  end
        RBT::Infobox.new(i) {{
          use_ccache:                    use_ccache?,
          show_required_dependencies_on: always_show_dependencies?,
          use_this_dataset:              pass_this_dataset_into_the_infobox,
          do_what:                       do_this
        }}
      end
    end
  end

  # ========================================================================= #
  # === show_infobox?
  # ========================================================================= #
  def show_infobox?
    @internal_hash[:show_the_infobox]
  end; alias show_the_infobox? show_infobox? # === show_the_infobox?

  # ========================================================================= #
  # === do_not_display_the_infobox
  # ========================================================================= #
  def do_not_display_the_infobox
    @internal_hash[:show_the_infobox] = false
  end; alias do_not_show_the_infobox do_not_display_the_infobox # === do_not_show_the_infobox

  # ========================================================================= #
  # === do_show_the_infobox
  # ========================================================================= #
  def do_show_the_infobox
    @internal_hash[:show_the_infobox] = true
  end

end; end