#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/initialize.rb'
# =========================================================================== #
require 'rbt/compile/base/compile_base.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === initialize
  #  
  # The first argument to this method, called `compile_these_programs`,
  # should be the name of the program (or programs) that you wish to
  # compile.
  #
  # So, for example, "htop" could be passed as the first argument to
  # the method. You can also pass in an Array here, and this class
  # will then attempt to batch-compile these programs.
  # ========================================================================= #
  def initialize(
      compile_these_programs = ARGV, # <- This should usually be an Array of programs that is to be compiled.
      run_already            = true
    )
    register_sigint
    reset
    time_snapshot # Store the first snapshot, which is the start-mark.
    set_original_input(compile_these_programs)
    # ======================================================================= #
    # Handle symbols given as first argument next.
    # ======================================================================= #
    case compile_these_programs
    when :do_not_run_yet,
         :do_not_yet_run,
         :dont_run_yet
      run_already = false
      compile_these_programs = []
    end
    compile_these_programs = [compile_these_programs].flatten.compact
    compile_these_programs = obtain_and_set_commandline_arguments_from(
      compile_these_programs
    )
    if compile_these_programs.is_a?(Array) and
      (compile_these_programs.size > 1) and # <- More than one entry.
      (!RBT.includes_this_program?(compile_these_programs[1]))
      # ===================================================================== #
      # Push entries into the commandline then, if it is not an
      # original program.
      # ===================================================================== #
      these_programs = compile_these_programs[1 .. -1]
      append_to_the_commandline(these_programs)
      compile_these_programs = compile_these_programs.first
    end
    # ======================================================================= #
    # Handle blocks passed to this method next:
    # ======================================================================= #
    if block_given?
      yielded = yield
      # ===================================================================== #
      # Handle Symbols first:
      # ===================================================================== #
      if yielded.is_a? Symbol
        case yielded
         # =================================================================== #
        # === :do_not_run_yet
        # =================================================================== #
        when :do_not_run_yet,
             :do_not_run
          run_already = false
        # =================================================================== #
        # === :do_not_check_whether_the_program_exists
        # =================================================================== #
        when :do_not_check_whether_the_program_exists
          do_not_check_whether_the_program_exists # <- This method exists, so use it too.
        end
      # ===================================================================== #
      # Next handle Hashes:
      # ===================================================================== #
      elsif yielded.is_a? Hash
        # =================================================================== #
        # === :prefix
        #
        # Handle prefix-related keys first.
        # =================================================================== #
        if yielded.has_key? :prefix
          set_user_prefix(yielded.delete(:prefix)) # <- Set the new prefix via a method call.
        # =================================================================== #
        # === :use_this_prefix
        # =================================================================== #
        elsif yielded.has_key? :use_this_prefix
          set_user_prefix(yielded.delete(:use_this_prefix))
        # =================================================================== #
        # === :use_this_as_prefix
        # =================================================================== #
        elsif yielded.has_key? :use_this_as_prefix
          set_user_prefix(yielded.delete(:use_this_as_prefix))
        end
        # =================================================================== #
        # === :symlink_after_compilation_has_finished
        # =================================================================== #
        if yielded.has_key? :symlink_after_compilation_has_finished
          value = yielded.delete(:symlink_after_compilation_has_finished)
          if value == false
            set_do_not_symlink(:be_verbose) # Disable symlinking after compilation.
          end
        end
        # =================================================================== #
        # === :run_already
        # =================================================================== #
        if yielded.has_key? :run_already
          case yielded[:run_already]
          when :do_not_run_yet
            run_already = false
          end
        end
        # =================================================================== #
        # Add commandline_arguments if the key is :commandline_arguments
        #
        # Such a Hash may have a key like this:
        #
        #   {:commandline_arguments=>["--dont-use-colours", "ntrad"]}
        
        # =================================================================== #
        if yielded.has_key? :commandline_arguments
          append_to_the_commandline(yielded.delete(:commandline_arguments))
        end
      else
        append_to_the_commandline(yielded)
      end
    end
    set_compile_these_programs(
      compile_these_programs
    )
    # ======================================================================= #
    # Handle special symbols passed to run_already next:
    # ======================================================================= #
    case run_already
    when :dont_run_yet,
         :do_not_run_yet,
         :do_not_run_already
      run_already = false
    end
    run if run_already
  end

end; end