#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/kde.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === try_to_compile_this_portingaids_component
  # ========================================================================= #
  def try_to_compile_this_portingaids_component(
      i = 1
    )
    result = nil
    big_array = return_porting_aids
    big_array.flatten!
    if i.include? '..'
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ===================================================================== #
      # Prevent too high numbers next:
      # ===================================================================== #
      i = i.to_i # <- We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result
    end
    set_compile_these_programs(result) # And assign it for compilation purposes.
  end

  # ========================================================================= #
  # === return_kde5_applications
  #
  # This method will conveniently return the Array holding all programs
  # that belong to the KDE5 Application pool.
  # ========================================================================= #
  def return_kde5_applications
    result = return_chained_programs['kde5_applications']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === compile_kde_apps
  # ========================================================================= #
  def compile_kde_apps
    set_compile_these_programs(
      return_kde5_applications
    )
  end

  # ========================================================================= #
  # === return_kde5_plasma
  #
  # This method will return all KDE5 plasma components, via an Array.
  # Whenever you need to obtain all KDE5-plasma components, you should
  # use only this method.
  #
  # In September 2018 there were 45 registered KDE5 plasma components.
  # ========================================================================= #
  def return_kde5_plasma
    result = return_chained_programs['kde5_plasma']
    if result.is_a? String
      result = result.split(',')
    end
    result.map!(&:strip)
    return result
  end; alias return_kde_plasma_components return_kde5_plasma # === return_kde_plasma_components

  # ========================================================================= #
  # === compile_all_kde5_plasma_components
  #
  # Invocation example:
  #   ry --compile-all-kde5-plasma-components
  # ========================================================================= #
  def compile_all_kde5_plasma_components
    _ = return_kde5_plasma
    opne 'We will next compile all '+
         lightgreen(_.size.to_s)+
         ' KDE5 plasma-components:'
    set_compile_these_programs(_)
  end

  # ========================================================================= #
  # === try_to_compile_this_plasma_component
  #
  # The input to this method should typically be a number, such as
  # 1 (or its String-variant "1").
  #
  # Invocation example from the commandline:
  #   ry --plasma3
  # ========================================================================= #
  def try_to_compile_this_plasma_component(
      i = 1
    )
    result = nil
    big_array = return_kde5_plasma       # === KDE5 Plasma
    big_array.flatten!
    if i.include? '..'
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ===================================================================== #
      # Prevent too high numbers next:
      # ===================================================================== #
      i = i.to_i # <- We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result
    end
    set_compile_these_programs(result) # And assign it for compilation purposes.
  end

  # ========================================================================= #
  # === try_to_compile_this_kde_component
  #
  # The input to this method should typically be a number, such as
  # 1 (or its String-variant "1").
  #
  # Invocation examples from the commandline:
  #   ry --kde3
  #   ry --kde1..kde250
  # ========================================================================= #
  def try_to_compile_this_kde_component(
      i = 1
    )
    result = nil
    big_array  = return_kde5_foundation   # === KDE5 Foundation
    big_array << return_kde5_porting_aids # === KDE5 Porting Aids
    big_array << return_kde5_applications # === KDE5 Applications
    big_array << return_kde5_plasma       # === KDE5 Plasma
    big_array.flatten!
    if i.include? '..'
      # ===================================================================== #
      # First we chop away 'kde' if this string exists.
      # ===================================================================== #
      i.sub!(/kde/,'') if i.include? 'kde'
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ===================================================================== #
      # We need numbers past this point. We also have to prevent too
      # high numbers, which the following code will do for us.
      # ===================================================================== #
      i = i.to_i # <- We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result
    end
    set_compile_these_programs(result) # And assign it for compilation purposes.
  end

  # ========================================================================= #
  # === return_kde5_porting_aids
  #
  # Invoke this method like so:
  #
  #   ry --batch-compile=kde5_porting_aids
  #
  # ========================================================================= #
  def return_kde5_porting_aids
    result = return_chained_programs['kde5_porting_aids']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end; alias portingaids         return_kde5_porting_aids # === portingaids
       alias return_porting_aids return_kde5_porting_aids # === return_porting_aids

  # ========================================================================= #
  # === return_kde5_foundation
  #
  # This method will return the Array of programs that constitute the
  # KDE5 foundation.
  # ========================================================================= #
  def return_kde5_foundation
    result = return_chained_programs['kde5_foundation']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === is_a_kde_program?
  #
  # We assume that every program that has the "kde" tag associated,
  # is a KDE program. Thus this method will return true.
  # ========================================================================= #
  def is_a_kde_program?
    tags.include? 'kde'
  end

end; end