#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/lxqt.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === compile_all_lxqt_components
  #
  # This entry point allows us to compile all LxQT components, e. g.
  # for the lxqt-desktop.
  #
  # To compile all LXQT components, try this on the commandline:
  #   ry --compile-lxqt
  # ========================================================================= #
  def compile_all_lxqt_components
    return_lxqt_components.each {|this_component|
      do_compile_this_program(this_component)
    }
  end

  # ========================================================================= #
  # === return_lxqt_components
  #
  # This method will conveniently return the Array holding all programs
  # that belong to the LX-QT pool.
  # ========================================================================= #
  def return_lxqt_components
    result = return_chained_programs['lxqt']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end; alias return_lxqt_desktop_components return_lxqt_components # === return_lxqt_desktop_components

  # ========================================================================= #
  # === try_to_compile_this_lxqt_component
  #
  # The input to this method should typically be a number, such as
  # 1 (or its String-variant "1").
  #
  # Invocation example from the commandline:
  #   ry --lxqt3
  # ========================================================================= #
  def try_to_compile_this_lxqt_component(
      i = 1
    )
    result = nil
    big_array = return_lxqt_components
    big_array.flatten!
    if i.include? '..'
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ===================================================================== #
      # Prevent too high numbers next:
      # ===================================================================== #
      i = i.to_i # <- We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result
    end
    set_compile_these_programs(result) # And assign it for compilation purposes.
  end; alias try_to_compile_this_lxqt_desktop_component try_to_compile_this_lxqt_component # === try_to_compile_this_lxqt_desktop_component 

end; end