#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/mate_desktop.rb'
# =========================================================================== #
# If you wish to compile all mate-components into their own standalone
# prefix via an environment value, you can use the following list:
#
#   env_compile_via_prefix matedesktop MATE_DESKTOP
#   env_compile_via_prefix libmatemixer MATE_DESKTOP
#   env_compile_via_prefix libmateweather MATE_DESKTOP
#   env_compile_via_prefix matemenus MATE_DESKTOP
#   env_compile_via_prefix matepanel MATE_DESKTOP
#   env_compile_via_prefix matenetbook MATE_DESKTOP
#   env_compile_via_prefix matemedia MATE_DESKTOP
#   env_compile_via_prefix marco MATE_DESKTOP
#   env_compile_via_prefix matesettingsdaemon MATE_DESKTOP
#   env_compile_via_prefix libmatekbd MATE_DESKTOP
#   env_compile_via_prefix matecontrolcenter MATE_DESKTOP
#   env_compile_via_prefix matesensorsapplet MATE_DESKTOP
#   env_compile_via_prefix caja MATE_DESKTOP
#   env_compile_via_prefix cajaextensions MATE_DESKTOP
#   env_compile_via_prefix engrampa MATE_DESKTOP
#   env_compile_via_prefix eom MATE_DESKTOP
#   env_compile_via_prefix mateutils MATE_DESKTOP
#   env_compile_via_prefix pluma MATE_DESKTOP
#   env_compile_via_prefix mateterminal MATE_DESKTOP
#
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === try_to_compile_this_mate_desktop_component
  #
  # The input to this method should typically be a number, such as
  # 1 (or its String-variant "1").
  #
  # Invocation example from the commandline:
  #
  #   ry --mate3
  #
  # On my home system I can also use aliases such as these:
  #
  #   mate1
  #   mate2
  #   mate3
  #
  # And so forth.
  # ========================================================================= #
  def try_to_compile_this_mate_desktop_component(
      i = 1
    )
    result = nil
    big_array = return_mate_desktop_components
    big_array.flatten!
    if i.include? '..'
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ===================================================================== #
      # Prevent too high numbers next:
      # ===================================================================== #
      i = i.to_i # <- We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result
    end
    set_compile_these_programs(result) # And assign it for compilation purposes.
  end; alias try_to_compile_this_mate_component try_to_compile_this_mate_desktop_component # === try_to_compile_this_mate_component

  # ========================================================================= #
  # === compile_mate_desktop_components_into_one_standalone_directory
  #
  # To invoke this method from the commandline, do:
  #
  #   ry --mate-desktop-into-standalone-dir
  #
  # ========================================================================= #
  def compile_mate_desktop_components_into_one_standalone_directory
    _ = return_mate_desktop_components
    # ======================================================================= #
    # We currently (Oct 2018) will use a hardcoded path. This may change
    # at a later point in the future.
    # ======================================================================= #
    use_this_prefix = "#{programs_dir?}Mate/1.20/"
    set_use_this_prefix(use_this_prefix)
    batch_compile_the_programs(_)
  end

  # ========================================================================= #
  # === return_mate_desktop_components
  #
  # This method will conveniently return the Array holding all programs
  # that belong to the KDE5 Application pool.
  # ========================================================================= #
  def return_mate_desktop_components
    result = return_chained_programs['all_of_the_mate_desktop']
    result = result.split(',') if result.is_a?(String) and result.include?(',')
    result.map!(&:strip)
    return result
  end; alias return_mate_applications return_mate_desktop_components # === return_mate_applications

end; end