#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/opn.rb'
# =========================================================================== #
require 'rbt/compile/show_and_report.rb'
require 'rbt/toplevel_methods/opn.rb'
require 'rbt/toplevel_methods/do_show_the_classname.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === Do show the classname by default
  # ========================================================================= #
  RBT.do_show_the_classname

  # ========================================================================= #
  # === use_opn?
  # ========================================================================= #
  def use_opn?
    @internal_hash and @internal_hash[:use_opn] # Must be guarded a little.
  end

  # ========================================================================= #
  # === RBT::Compile.do_not_show_names?
  # ========================================================================= #
  def self.do_not_show_names?
    ::RBT.do_not_show_names?
  end

  # ========================================================================= #
  # === RBT::Compile.show_the_classname?
  # ========================================================================= #
  def self.show_the_classname?
    ::RBT.show_the_classname?
  end

  # ========================================================================= #
  # === show_the_classname?
  # ========================================================================= #
  def show_the_classname?
    ::RBT.show_the_classname?
  end

  # ========================================================================= #
  # === disable_opn
  #
  # Use this method to stop reporting any program names.
  #
  # To trigger this, try one of these usage examples:
  #   ry php noopn
  #   ry htop nonames
  # ========================================================================= #
  def disable_opn
    e 'We will not use Opn (class name output)'
    RBT.do_not_show_the_classname
    @internal_hash[:use_opn] = false
  end

  # ========================================================================= #
  # === RBT::Compile.opnn
  #
  # Use this method when you wish to output the RBT::Compile namespace
  # to the user.
  # ========================================================================= #
  def self.opnn(
      i = {
        namespace:   NAMESPACE,
        use_colours: RBT.use_colours?
      }
    )
    if i.is_a? Hash
      unless i.has_key? :use_colours
        i.update(use_colours: RBT.use_colours?)
      end
    end
    if RBT.show_the_classname?
      ::RBT.opnn(i) # Delegate to the module_method here.
    # else # else we will not use Opn at all.
    end
  end; self.instance_eval { alias output_namespace? opnn } # === RBT::Compile.output_namespace?
  
  # ========================================================================= #
  # === opnn                                                       (opnn tag)
  # ========================================================================= #
  def opnn(
      optional_i = { namespace: @namespace }
    )
    if use_opn?
      ::RBT::Compile.opnn(optional_i)
    end
  end; alias opn               opnn # === opn
       alias output_namespace? opnn # === output_namespace?

end; end