#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file deals solely with the $PATH environment variable, which
# is available in ruby through ENV['PATH'].
# =========================================================================== #
# require 'rbt/compile/path.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === query_path?
  #
  # Query the $PATH variable, within ENV['PATH'].
  # ========================================================================= #
  def query_path?
    ENV['PATH'].dup # .dup-ing it is better in the long run.
  end

  # ========================================================================= #
  # === use_traditional_env_path
  #
  # This method will "put" /usr/bin/ at the beginning of the path.
  #
  # It can be invoked from the commandline via:
  #   ry fluxbox --traditional-path
  # ========================================================================= #
  def use_traditional_env_path
    prepend_env_path_with('/usr/bin/')
  end

  # ========================================================================= #
  # === prepend_env_path_with
  # ========================================================================= #
  def prepend_env_path_with(i = '/usr/bin/')
    unless i.end_with? ':'
      i = i.dup if i.frozen?
      i << ':'
    end
    set_path(i+query_path?)
  end

  # ========================================================================= #
  # === set_path
  #
  # Set the $PATH variable here through this setter-method.
  # ========================================================================= #
  def set_path(i)
    i = i.dup # Better to always .dup it when we set it.
    ENV['PATH'] = i # Must re-enable PATH environment as it was.
  end

end; end