#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/permanent.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === permanently_set_programs_dir
  # ========================================================================= #
  def permanently_set_programs_dir(what)
    into = FILE_PROGRAMS_DIRECTORY
    opnn; e 'Storing '+sfancy(what)+' into the file `'+sfile(into)+'`.'
    write_what_into(what, into)
    if is_on_roebe?
      into = RUBY_SRC+'rbt/lib/rbt/yaml/configuration/programs_directory.yml'
      opnn; e 'Storing '+sfancy(what)+' into the file `'+sfile(into)+'`.'
      write_what_into(what, into)
    end
  end; alias permanently_set_programs_directory permanently_set_programs_dir # === permanently_set_programs_directory

  # ========================================================================= #
  # === set_permanently_extract_to
  #
  # This method will permanently set a new directory to which we
  # will extract to.
  #
  # To invoke this method, do something such as:
  #   ry --permanently_extract_to=/Depot
  #   ry --permanently_extract_to=/opt
  #   ry --permanently_extract_to=$TEMP_DIR
  # ========================================================================= #
  def set_permanently_extract_to(
      i = nil
    )
    if i
      target = RBT.permanently_set_temp_directory(i)
      opne "The RBT scripts will henceforthwith be extracted "\
           "towards `#{sdir(target)}`."
    else
      opne 'Please provide an argument.'
    end
  end

  # ========================================================================= #
  # === permanently_add_this_global_flag
  #
  # Use this method in order to permanently add a new global flag, into
  # the file called "global_flags.yml".
  #
  # To test this method, try:
  #   rbt --add-global-flag=--disable-static
  # ========================================================================= #
  def permanently_add_this_global_flag(i)
    i = i.to_s.dup
    into = RBT.file_global_flags
    opne "Now appending the flag `#{sfancy(i)}` into the "\
         "file `#{sfile(into)}`."
    old_file_content = ''.dup
    if File.exist? into
      old_file_content << File.read(into).strip
    end
    old_file_content << " #{i}"
    what = old_file_content
    write_what_into(what, into)
    if is_on_roebe?
      into = ROEBE_FILE_GLOBAL_FLAGS
      opne 'Now appending the flag `'+sfancy(i)+'` into the '\
           'file `'+sfile(into)+'`.'
      write_what_into(what, into)
    end
  end; alias add_this_global_flag permanently_add_this_global_flag # === add_this_global_flag

  # ========================================================================= #
  # === permanently_remove_this_global_flag
  #
  # Use this method in order to permanently remove a new global flag, into
  # the file called "global_flags.yml".
  #
  # To test this method, try:
  #
  #   rbt --remove-global-flag=--disable-static
  #
  # ========================================================================= #
  def permanently_remove_this_global_flag(i)
    i = i.to_s.dup
    into = RBT.file_global_flags
    old_file_content = ''.dup
    if File.exist? into
      old_file_content << File.read(into).strip
    end
    if old_file_content.include? i
      opne 'Now removing the flag `'+sfancy(i)+'` from the '\
           'file `'+sfile(into)+'`.'
      old_file_content.gsub!(/#{i}/, '')
    else
      opne "The flag `#{sfancy(i)}` can not be removed from "\
           "the file `#{sfile(into)}`."
      opne 'because this file does not have that flag.'
    end
    what = old_file_content
    write_what_into(what, into)
    if is_on_roebe?
      into = ROEBE_FILE_GLOBAL_FLAGS
      opne 'Now removing the flag `'+sfancy(i)+'` from the '\
           'file `'+sfile(into)+'`.'
      write_what_into(what, into)
    end
  end; alias remove_this_global_flag permanently_remove_this_global_flag # === remove_this_global_flag

end; end