#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/porg.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === use_porg?
  # ========================================================================= #
  def use_porg?
    @internal_hash[:use_porg]
  end; alias will_we_use_porg? use_porg? # === will_we_use_porg?

  # ========================================================================= #
  # === permanently_disable_porg
  #
  # Permanently disable porg via this method.
  # ========================================================================= #
  def permanently_disable_porg
    if use_porg?
      opne 'Disabling porg.'
      if configuration?.respond_to? :use_porg=
        configuration?.use_porg = false
        if configuration?.respond_to? :save_config
          configuration?.save_config
        else
          what = false
          into = FILE_USE_PORG
          opne "Storing into `#{sfile(into)}`."
          write_what_into(what, into)
        end
      else
        opnn; e 'The configuration-object is not functional - it does not'
        opnn; e 'respond to the method called .use_porg='
      end
    else
      opne 'Porg is already disabled.'
    end
  end

  # ========================================================================= #
  # === disable_porg
  #
  # Disable porg via this method.
  # ========================================================================= #
  def disable_porg
    @internal_hash[:use_porg] = config?.use_porg = false
  end

  # ========================================================================= #
  # === enable_porg
  #
  # Enable porg via this method.
  # ========================================================================= #
  def enable_porg
    @internal_hash[:use_porg] = config?.use_porg = true
  end

  # ========================================================================= #
  # === report_whether_we_will_use_porg
  #
  # To invoke this method, issue:
  #
  #   ry --use-porg?
  #
  # ========================================================================= #
  def report_whether_we_will_use_porg
    opne "Will RBT make use of #{teal('porg')}: "\
         "#{royalblue(verbose_truth(use_porg?.to_s))}"
  end

  # ========================================================================= #
  # === notify_the_user_that_porg_is_not_available
  # ========================================================================= #
  def notify_the_user_that_porg_is_not_available
    opnn; e 'You are trying to use porg, but porg is not installed.'
    opnn; e 'Please either install porg - or disable the configuration '
    opnn; e "option use_porg in the #{slateblue('configuration setting')}."
  end

end; end