#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/preinstall.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === run_preinstall_actions
  #
  # These actions will be run before anything else.
  #
  # An example for such a preinstall action may be the following:
  #
  #     if is_on_32_bit_system?
  #       sed -r "/ifeq.*CompilerTargetArch/s#i386#i686#g" -i projects/compiler-rt/make/platform/clang_linux.mk
  #
  # ========================================================================= #
  def run_preinstall_actions
    unless cookbook_dataset_preinstall?.empty?
      # ===================================================================== #
      # Next we have to handle preinstall-actions. This should typically
      # be an Array.
      # ===================================================================== #
      _ = cookbook_dataset_preinstall?
      # ===================================================================== #
      # Next output that we are doing pre-installation things, but only do
      # so IF we really do so.
      # ===================================================================== #
      first = _.first
      unless first.start_with?('if ') and (first == 'if is_on_32_bit_system?')
        cliner {
          opnn; e "Doing preinstallation actions next, from the directory "
          opnn; e "`#{sdir(return_pwd)}`:"
        }
      end
      if first.start_with? 'if ' # Ok, we assume this to be an if-conditional clause.
        case first # case tag
        when 'if is_on_32_bit_system?'
          if is_on_32_bit_system? # Ok, we are on a 32 bit system.
            _.shift
            unless _.empty?
              _ = _.first
              opnn; e 'A pre-installation instruction was found, '\
                      'which will be run next (from the directory '+sdir_return_pwd+').'
              opnn; esystem _
            end
          end
        end
      else
        opnn
        if _.is_a? Array
          _.each {|entry|
            e sfancy(entry)
            system(entry)
          }
        else
          e sfancy(_)
          system(_)
        end
      end
    end
  end

end; end