#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file collects register and de-register activities for the
# class RBT::Compile.
# =========================================================================== #
# require 'rbt/compile/register.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === register_last_compiled_program
  #
  # This method will try to store the last compiled program into the file
  # determined via the method call RBT.file_compiled_program().
  #
  # We will only store if the entry is DIFFERENT to the last one though.
  #
  # This file is:
  #   $RBT/yaml/last_compiled_program.yml
  # ========================================================================= #
  def register_last_compiled_program(
      i = compile_which_program?
    )
    unless has_a_problem_occurred?
      unless do_not_compile? # Continue only if we do want to compile.
        file = RBT.file_compiled_programs
        base_dir = File.dirname(
          RBT.file_compiled_programs
        )
        mkdir(base_dir) unless Dir.exist? base_dir
        if File.exist? file # Must check whether the file exists first.
          array_dataset = YAML.load_file(file)
        else
          array_dataset = []
        end
        if array_dataset.respond_to? :uniq!
          array_dataset.uniq!
        # else
        #   opne swarn('Some minor problem occurred:')
        end
        unless array_dataset.last == i # Storing only if it is different.
          opne 'Trying to register the last compiled program '\
               '(`'+springgreen(i)+'`) into the file:'
          opne "  `#{sfile(file)}`"
          # ================================================================= #
          # Remove any old entries, if they exist.
          # ================================================================= #
          if array_dataset.include? i 
            array_dataset.reject! {|entry| entry == i }
          end
          what = array_dataset.map {|entry|
            " - #{entry}#{N}"
          }.join
          # ================================================================= #
          # Simply append to the dataset (aka file) at hand.
          # ================================================================= #
          write_what_into(what, file)
        end if array_dataset.respond_to? :last
      end
    end
  end

  # ========================================================================= #
  # === register_all_programs
  # ========================================================================= #
  def register_all_programs
    get_all_programs.each {|entry| register_program(entry) }
  end

  # ========================================================================= #
  # === register_required_dependency
  # ========================================================================= #
  def register_required_dependency(
      name_of_the_program, program_version
    )
    @internal_hash[:array_required_dependencies] << [
      name_of_the_program, program_version
    ]
  end

  # ========================================================================= #
  # === deregistering_from_the_status_file
  #
  # De-register the program that we are currently compiling from the
  # status file again.
  # ========================================================================= #
  def deregistering_from_the_status_file
    into = RBT.file_status
    dataset = {}
    if File.exist? into
      dataset.merge!(YAML.load_file(into))
    end
    if dataset.has_key? 'currently_compiling_these_programs'
      _ = dataset['currently_compiling_these_programs']
      if _.include? compile_which_program?
        dataset['currently_compiling_these_programs'] = dataset['currently_compiling_these_programs'].reject {|line|
          line == compile_which_program?
        }
        what = YAML.dump(dataset)
        write_what_into(what, into)
      end
    end
  end

  # ========================================================================= #
  # === is_registered?
  #
  # Determine whether a given program is registered or whether it is not.
  # ========================================================================= #
  def is_registered?(i)
    return RBT.is_available? i
  end

  # ========================================================================= #
  # === register_that_this_program_is_currently_being_compiled
  #
  # This method will register that a given program is currently being
  # compiled.
  # ========================================================================= #
  def register_that_this_program_is_currently_being_compiled
    into = RBT.file_status # <- This may be e. g. "/Depot/Temp/rbt/status.yml".
    dataset = {}
    if File.exist? into
      dataset.merge!(YAML.load_file(into))
    end
    unless dataset.has_key? 'currently_compiling_these_programs'
      dataset['currently_compiling_these_programs'] = []
    end
    dataset['currently_compiling_these_programs'] << compile_this_program?
    what = YAML.dump(dataset)
    # ======================================================================= #
    # We must sure that the base directory exists.
    # ======================================================================= #
    unless File.directory?(File.dirname(into))
      mkdir(File.dirname(into))
    end
    write_what_into(what, into)
  end

end; end