#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/set.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === set_configure_base_directory
  #
  # This setter-method sets the @internal_hash[:configure_base_directory],
  # which is, for example, a directory such as:
  #
  #   /Depot/Temp/htop-1.0.0/
  #
  # Some projects, such as "xvid" / "xvidcore", may require a specific
  # subdirectory, though.
  #
  # Another example may be the program called "nspr", which requires
  # the following configure base directory:
  #
  #   /Depot/Temp/nspr-4.10.8/nspr/
  #
  # class RBT::Compile will store this in a separate variable so that
  # it can be queried lateron.
  #
  # Do note that the cookbook-dataset will also retain the original
  # configure_base_directory.
  # ========================================================================= #
  def set_configure_base_directory(
      i = rds(
        "#{extract_to?.to_s}"\
        "#{program_name_and_version?.to_s}/"\
        "#{cookbook_dataset_configure_base_directory?.to_s}"
      )
    )
    i = i.to_s.dup
    i = rds(i) # rds() is used here since as of Feb 2012.
    i << '/' unless i.end_with? '/' # It must always have a trailing '/'.
    i = i.to_s.dup
    @internal_hash[:configure_base_directory] = i # <- Finally, assign it.
  end; alias set_configure_base_dir set_configure_base_directory # === set_configure_base_dir

end; end