#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/set_compile_these_programs.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # == set_compile_these_programs
  #
  # This method will designate which programs are to be compiled. Typically
  # this will be an Array of programs, such as ['htop','ruby','php'] and
  # so forth.
  # ========================================================================= #
  def set_compile_these_programs(i)
    if i.is_a? Hash
      if i.has_key? :compile_these_programs
        i = i.delete(:compile_these_programs)
      end
    end
    i = [i].flatten.compact.map {|entry| # Will always be an Array.
      entry = entry.dup if entry.frozen?
      # ===================================================================== #
      # The next line will remove '-' and '_' characters from the given
      # input. This, however had, may NOT be done when there is a
      # locally existing file with that name.
      # ===================================================================== #
      unless File.exist? entry
        entry.delete!('-_') # Don't use '-' characters.
        if entry.end_with? '.yml'
          entry.sub!(/\.yml$/,'')
        elsif entry.end_with? '.pc'
          entry.sub!(/\.pc$/,'')
        end
      end if entry.is_a? String
      # ===================================================================== #
      # Use some exceptions for "kde1" and "mate1", which will be used
      # in the next Array:
      # ===================================================================== #
      this_array = %w(
        kde mate game xfce xorg gnome plasma lxqt portingaids
      )
      if entry.respond_to?(:start_with?) and
         entry.start_with?(*this_array) and 
        (entry =~ /\d{1,4}$/) and
        !is_included?(entry) # <- We have to prioritize on any registered program here.
        menu('--'+entry.to_s)
        nil # Eliminate these lateron.
      else
        entry # <- Keep this as return value here.
      end
    }.compact
    # ========================================================================= #
    # This will always be an Array:
    # ========================================================================= #
    @internal_hash[:compile_these_programs] = i if i and !i.empty?
    # ========================================================================= #
    # We will also initialize the cookbook_dataset for the first program,
    # if possible.
    # ========================================================================= #
    first_program = i.first.to_s.downcase.delete('/')
    if first_program and RBT.is_this_program_included?(first_program)
      # ===================================================================== #
      # Set the initial seeds for the programs at hand.
      # ===================================================================== #
      set_compile_this_program(first_program)
      initialize_cookbook_dataset_for(first_program)
    end
  end

end; end