#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/shell.rb'
# =========================================================================== #
require 'rbt/utility_scripts/generate_shell_completion.rb'
require 'rbt/utility_scripts/generate_shellscript.rb'
require 'rbt/utility_scripts/create_bash_scripts.rb'

module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === generate_shell_completion
  #
  # Generates shell completion code. Right now it will only work for Bash.
  #
  # Invocation example:
  #
  #   ry --generate_shell_completion
  #
  # ========================================================================= #
  def generate_shell_completion(
      for_this_shell = :all
    )
    RBT::GenerateShellCompletion.new(
      for_this_shell
    ) # bl GenerateShellCompletion
  end

  # ========================================================================= #
  # === create_shell_scripts                            (bash tag bashrc tag)
  #
  # This method will convert the yaml dataset to bash scripts, so
  # that the bash shell can make use this information as well.
  #
  # To invoke this method, do something such as this:
  #
  #   ry --bash
  #
  # To get the array content do this (after sourcing the file):
  #   echo ${programs[*]}
  # ========================================================================= #
  # cat /System/Settings/bash_programs_rc
  # ========================================================================= #
  # cd /System/Settings/
  # ========================================================================= #
  def create_shell_scripts
    RBT::CreateBashScripts.new # bl $RBT/create_bash_scripts.rb
  end; alias to_bash create_shell_scripts # === to_bash

  # ========================================================================= #
  # === create_shellscript                           (shell tag, script tag)
  #
  # This is a wrapper over the class GenerateShellscript.
  #
  # It will return a string, which contains all the information to
  # create a specific shell script.
  #
  # The first argument to this method is the name of the program in question.
  #
  # To invoke this method, do:
  #
  #   ry htop create_shellscript
  #   ry php cshell
  #
  # ========================================================================= #
  def create_shellscript(
      program          = compile_which_program?,
      save_into_a_file = true,
      display_result   = false
    )
    _ = GenerateShellscript.new(
      program,
      save_into_a_file,
      display_result
    )
    return _.string?
  end

end; end