#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/static.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === verbose_query_whether_this_program_can_be_compiled_statically
  # ========================================================================= #
  def verbose_query_whether_this_program_can_be_compiled_statically(
      this_program = compile_which_program?
    )
    _ = cookbook_dataset?
    opnn; e 'Can the program '+sfancy(this_program)+' be compiled statically: '+
            lightgreen(verbose_truth(_.can_be_compiled_statically?))
  end

  # ========================================================================= #
  # === verbose_enable_static
  # ========================================================================= #
  def verbose_enable_static
    opne 'Trying to compile statically (no shared libraries).'
    do_enable_static_compilation
    set_cflags(:overrule_static_build)
  end

  # ========================================================================= #
  # === build_static?
  #
  # We have to first check the build_static entry of the cookbook dataset,
  # before we query the value of the @internal_hash.
  # ========================================================================= #
  def build_static?
    _ = cookbook_dataset_build_static?
    unless @internal_hash[:build_static].nil?
      _ = @internal_hash[:build_static]
    end
    _
  end; alias do_we_compile_statically? build_static? # === do_we_compile_statically?

  # ========================================================================= #
  # === do_enable_static_compilation
  #
  # Enable static compilations.
  # ========================================================================= #
  def do_enable_static_compilation
    @internal_hash[:build_static] = true
  end; alias enable_static? do_enable_static_compilation # === enable_static?

  # ========================================================================= #
  # === internal_hash_build_static?
  # ========================================================================= #
  def internal_hash_build_static?
    @internal_hash[:build_static]
  end

  # ========================================================================= #
  # === cookbook_dataset_build_static?
  # ========================================================================= #
  def cookbook_dataset_build_static?
    cookbook_dataset?.build_static?
  end

  # ========================================================================= #
  # === disable_static_compilation
  #
  # Use this method to disable static compilation.
  # ========================================================================= #
  def disable_static_compilation(
      be_verbose = :be_verbose
    )
    case be_verbose
    when :be_verbose
      be_verbose = true
    when :do_not_be_verbose
      be_verbose = false
    end
    if be_verbose
      opne 'Disabling static compilation.'
    end
    @internal_hash[:build_static] = false
  end

end; end