#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/system.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === parse_this_line_obtained_via_io_popen
  #
  # This method is used to parse all output generated via IO.popen().
  # ========================================================================= #
  def parse_this_line_obtained_via_io_popen(
      line,
      use_colours = use_colours?
    )
    # ======================================================================= #
    # Delegate towards class ColourizeParser next.
    # ======================================================================= #
    cparser = @internal_hash[:colourize_parser]
    unless use_colours
      cparser.disable_colours
    end
    line.chomp!
    # ======================================================================= #
    # Feed towards module RBT::Errors::MapLineToASpecificError next:
    # ======================================================================= #
    RBT::Errors::MapLineToASpecificError.parse_this_line(line)
    # ======================================================================= #
    # Next, put the line towards ColourizeParser. The reason why this is
    # done after the error checking above is to simply keep the output
    # as it was originally, before changing it via colours and the like.
    # ======================================================================= #
    cparser.grab_this_line(line)
    line = cparser.line?.dup
    if line.include?('A required package was not found') and
       !RBT::Errors::MapLineToASpecificError.required_packages?.empty?
      name_of_the_package = RBT::Errors::MapLineToASpecificError.required_packages?.first.to_s
      line << ' '+crimson('( -> '+name_of_the_package+' is missing)')
      line << N+N+'You can try to install it via:'+N
      line << N+slateblue('  rbt '+name_of_the_package)+N+N
    end
    # ======================================================================= #
    # The reason we output the line here is so that we can still do
    # modifications to it, if we have to. This is the case e. g. for
    # when we wish to display which package is missing.
    # ======================================================================= #
    e line # Output here.
    if line.include?('configure: No such file or directory')
      opnn; e 'The '+crimson('above error message')+' most likely means '\
              'that no configure script could be found.'
    end
    @internal_hash[:array_sys_command_string] << line # We could also store the lines here.
  end

  # ========================================================================= #
  # === run_this_sys_command
  #
  # This currently uses IO.popen. The alternatives, system() or backticks,
  # are not as useful, in my opinion.
  #
  # The current idea is to work on a line-by-line basis, and to also display
  # what is done, so that the user has an idea what is happening.
  #
  # There is also the possibility to use colours and colourize the input;
  # this allows us to, for example, use specific colours for directories
  # or for files or to indicate failures.
  # ========================================================================= #
  def run_this_sys_command(i)
    use_colours = use_colours?
    # ======================================================================= #
    # I still do not quite understand the second argument. Either way,
    # I have determined that this works better.
    # ffmpeg = IO.popen(cmdline + " 2>&1", "w+")
    # ======================================================================= #
    if i.start_with? 'make'
      unless is_make_available?
        opne "The program \"#{sfancy('make')}\" is "\
             "#{swarn('not available')}, at the least not as  "\
             "part of the $PATH variable."
        opne "RBT::Compile thus can not compile anything. You "\
             "may want to check the #{orange('$PATH')} variable."
        do_not_continue_and_do_not_compile(:be_quiet)
        return
      end
    end
    i = i.dup if i.frozen?
    i << ' 2>&1' unless i.end_with? ' 2>&1'
    io_object = IO.popen(i, {:err => [:child, :out]}).each { |line|
      # ======================================================================= #
      # The next line will also display what has happened.
      # ======================================================================= #
      parse_this_line_obtained_via_io_popen(line, use_colours)
    }
    io_object.close
    # ======================================================================= #
    # Next, we attempt to determine what some of the errors may be and
    # perhaps even how to resolve them.
    # ======================================================================= #
    # Handle "no configure file" next.
    # ======================================================================= #
    if array_sys_command_string? and
       array_sys_command_string?.any? {|line|
        line.include? 'sh: ./configure: No such file or directory'
      }
      problem_is :no_configure_file
    # ======================================================================= #
    # Handle missing headers next.
    # ======================================================================= #
    elsif array_sys_command_string?.any? {|line|
        line.include? 'headers are missing'
      }
      _ = array_sys_command_string?.select {|line|
        line.include? 'headers are missing'
      }
      problem_is _
    end
  end; alias custom_system           run_this_sys_command # === custom_system
       alias run_this_system_command run_this_sys_command # === run_this_system_command

  # ========================================================================= #
  # === colourize_parser?
  # ========================================================================= #
  def colourize_parser?
    @internal_hash[:colourize_parser]
  end

  # ========================================================================= #
  # === last_system_command_was_a_success?
  # ========================================================================= #
  def last_system_command_was_a_success?
    $CHILD_STATUS.success?
  end; alias was_the_configure_run_a_success? last_system_command_was_a_success? # === was_the_configure_run_a_success?
       alias show_success_status?             last_system_command_was_a_success? # === show_success_status?

end; end