#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/tab.rb'
# =========================================================================== #
begin
  require 'roebe/classes/kde/kde_konsole/kde_konsole.rb'
rescue LoadError
  if ENV['IS_ROEBE'].to_s == '1'
    puts 'roebe/classes/kde/kde_konsole/kde_konsole.rb is unavailable (should be part of the roebe gem).'
  end
end # ^^^ it is optional, hence why the check occurs.

module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === rename_tab_title?
  # ========================================================================= #
  def rename_tab_title?
    @internal_hash[:rename_tab_title]
  end

  # ========================================================================= #
  # === do_not_rename_the_kde_konsole_tab
  # ========================================================================= #
  def do_not_rename_the_kde_konsole_tab
    opnn; e 'We will not rename the KDE Konsole tab.'
    @internal_hash[:rename_tab_title] = false
  end

  # ========================================================================= #
  # === consider_changing_the_tab_title                           (shell tag)
  #
  # Use this method to change the title of a tab in a Terminal like Bash.
  # ========================================================================= # 
  def consider_changing_the_tab_title(
      i                             = return_program_name_then_hyphen_then_the_program_version,
      shall_we_rename_the_tab_title = @internal_hash[:rename_tab_title]
    )
    case shall_we_rename_the_tab_title
    when :do_rename
      shall_we_rename_the_tab_title = true
    end
    # ======================================================================= #
    # First check whether we ought to rename the tab title or not.
    # ======================================================================= #
    if shall_we_rename_the_tab_title
      case i
      when :default
        # This is the current hardcoded default. May change at a later time.
        i = return_program_name_then_hyphen_then_the_program_version
      when :clear
        i = '.'
      end
      # ===================================================================== #
      # Query whether Roebe is available or not.
      # ===================================================================== #
      if Object.const_defined?(:Roebe) and
         rename_tab_title? # Only use it if the user enabled this.
        if Roebe.respond_to? :rename_konsole_title_to
          Roebe.rename_konsole_title_to(i, :be_quiet)
        else
          e "The namespace #{sfancy('Roebe')} does not respond to the method"
          e 'rename_konsole_title_to()'
          e 'Either the functionality is missing - or something has not'
          e 'been properly required. Either way, we can not rename the'
          e 'konsole tab for now.'
        end
      end
    end unless i.empty?
  end; alias rename_shell                          consider_changing_the_tab_title # === rename_shell
       alias set_title                             consider_changing_the_tab_title # === set_title
       alias change_tab_title                      consider_changing_the_tab_title # === change_tab_title
       alias consider_renaming_the_kde_konsole_tab consider_changing_the_tab_title # === consider_renaming_the_kde_konsole_tab

end; end