#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/tags.rb'
# =========================================================================== #
require 'rbt/utility_scripts/search_for_tags/search_for_tags.rb'
require 'rbt/utility_scripts/cleanup_system_tags.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === compile_these_tags
  #
  # Invocation example:
  #   rbt --tag_compile=science
  # ========================================================================= #
  def compile_these_tags(i)
    opnn; e "We will next find all programs that have the "\
            "tag `#{sfancy(i)}`."
    _ = ::RBT.return_array_of_this_tag(i) # This depends on ScanForTags.
    set_compile_these_programs(_)
  end

  # ========================================================================= #
  # === search_for_tag                                           (search tag)
  #
  # Use this to search for a tag.
  #
  # Usage example:
  #   ry htop --tags pdf
  #   ry htop --tags term
  # ========================================================================= #
  def search_for_tag(
      this_tag,
      shall_we_exit = true
    )
    results = RBT::Cookbooks::SearchForTags.new(this_tag)
    exit_program if shall_we_exit
    return results
  end

  # ========================================================================= #
  # === show_valid_system_tags
  #
  # Show the allowed, and thus valid, /System/Tags entries.
  #
  # To invoke this method, do:
  #
  #   ry --valid-sys-tags?
  #
  # ========================================================================= #
  def show_valid_system_tags
    _ = RBT.file_allowed_tags
    if File.exist? _
      # ===================================================================== #
      # Since the file exists, we can load up the dataset of it.
      # ===================================================================== #
      dataset_containing_the_allowed_tags = YAML.load_file(_).sort
      opnn; e 'The following '+sfancy(dataset_containing_the_allowed_tags.size.to_s)+' '+
              sdir('/System/Tags/')+' entries are valid:'
      e
      dataset_containing_the_allowed_tags.each_with_index {|tag, index| index += 1
        e mediumvioletred((index.to_s+') ').rjust(4))+
          sfancy(tag)
      }; e
    else
      opnn; e "No file called #{sfile(_)} exists - we thus can not show"
      opnn; e 'any allowed tags.'
    end
  end

  # ========================================================================= #
  # === check_all_tags_under_programs_directory
  #
  # Use this method to check for all entries in the PROGRAMS_DIRECTORY,
  # whether they also have an entry in /System/Tags.
  #
  # To invoke this method, try:
  #
  #   ry --check-all-tags
  #
  # ========================================================================= #
  def check_all_tags_under_programs_directory
    entries = Dir["#{programs_directory?}*"].map {|entry|
      entry << '/Current/'
    }.sort
    entries.each {|entry|
      # ===================================================================== #
      # A particular entry may look like this:
      #   /Programs/Zip/Current/
      # ===================================================================== #
      # Must check whether the directory exists.
      # ===================================================================== #
      if Dir.exist? entry
        splitted = entry.split('/') # <- e. g. "/Programs/Zip/Current/"
        program_name = splitted[2]
        # program_name = ProgramInformation.return_program_name(program_name).downcase
        program_name.downcase!
        dataset = instantiate_a_new_cookbook_for(program_name)
        symlink_into_tags(entry, dataset)
      end
    }
    opnn; e 'All done now.'
  end

end; end