#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/xfce.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === compile_all_of_xfce
  # ========================================================================= #
  def compile_all_of_xfce
    do_compile_these_programs(
      return_xfce_components
    )
  end

  # ========================================================================= #
  # === compile_xfce_components_into_one_standalone_directory
  #
  # To invoke this method from the commandline, do:
  #
  #   ry --xfce-into-standalone-dir
  #
  # ========================================================================= #
  def compile_xfce_components_into_one_standalone_directory
    _ = return_xfce_components
    # ======================================================================= #
    # We currently (Oct 2018) will use a hardcoded path. This may change
    # at a later point in the future.
    # ======================================================================= #
    use_this_prefix = programs_dir?+'Xfce/4.12/'
    set_use_this_prefix(
      use_this_prefix
    )
    batch_compile_the_programs(_)
  end

  # ========================================================================= #
  # === return_xfce_components
  #
  # This method will conveniently return the Array holding all programs
  # that belong to the KDE5 Application pool.
  # ========================================================================= #
  def return_xfce_components
    result = RBT.return_chained_programs['xfce']
    result = result.split(',') if result.is_a?(String) and result.include?(',')
    result.map!(&:strip)
    return result
  end; alias return_xfce return_xfce_components # === return_xfce

  # ========================================================================= #
  # === try_to_compile_this_xfce_component
  #
  # The input to this method should typically be a number, such as
  # 1 (or its String-variant "1").
  #
  # Invocation example from the commandline:
  #   ry --xfce1
  # ========================================================================= #
  def try_to_compile_this_xfce_component(
      i = 1
    )
    result = nil
    big_array = return_xfce_components
    big_array.flatten!
    if i.include? '..'
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ===================================================================== #
      # Prevent too high numbers next:
      # ===================================================================== #
      i = i.to_i # <- We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result
    end
    set_compile_these_programs(result) # And assign it for compilation purposes.
  end; alias try_to_compile_this_xfce try_to_compile_this_xfce_component # === try_to_compile_this_xfce 

end; end