#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/xorg.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === compile_mouse_and_keyboard_xorg_server_components
  #
  # To compile the entries in this method, issue:
  #
  #   ry --compile-mouse-and-keyboard
  #
  # ========================================================================= #
  def compile_mouse_and_keyboard_xorg_server_components
    array_components = %w(
      xf86-input-evdev
      xf86-input-joystick
      xf86-input-libinput
      xf86-input-mtrack
      xf86-input-synaptics
      xf86-input-vmmouse
      xf86-input-wacom
    )
    opnn; e "Next compiling `#{royalblue(array_components.size.to_s)}` programs:"
    array_components.each {|this_component|
      do_compile_this_program(this_component)
    }
  end

  # ========================================================================= #
  # === compile_post_xorg_server_components
  #
  # Invocation example for this method:
  #
  #   ry --post-xorg-server
  #
  # ========================================================================= #
  def compile_post_xorg_server_components
    _ = report_post_xorg_server_components
    opnn; e "Next compiling `#{royalblue(_.size.to_s)}` programs:"
    _.each {|this_program|
      do_compile_this_program(this_program)
    }
  end

  # ========================================================================= #
  # === report_post_xorg_server_components
  # ========================================================================= #
  def report_post_xorg_server_components
    result = return_chained_programs['post_xorg_server']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === return_xorg_utils
  # ========================================================================= #
  def return_xorg_utils
    result = return_chained_programs['xorg_utils']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === return_xorg_components
  #
  # This method will conveniently return the Array holding all programs
  # that belong to the KDE5 Application pool.
  # ========================================================================= #
  def return_xorg_components
    array  = return_xorg_protos
    array << return_xorg_utils
    array << return_xorg_libraries
    array << return_xorg_data
    array << return_xorg_apps
    array << return_xorg_fonts
    array << return_xorg_server
    array.flatten!
    return array
  end; alias return_xorg return_xorg_components # === return_xorg

  # ========================================================================= #
  # === return_xorg_protos
  # ========================================================================= #
  def return_xorg_protos
    result = return_chained_programs['xorg_protos']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === return_xorg_libraries
  # ========================================================================= #
  def return_xorg_libraries
    result = return_chained_programs['xorg_libraries']
    result = result.split(',') if result.include? ','
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === return_xorg_data
  # ========================================================================= #
  def return_xorg_data
    result = return_chained_programs['xorg_data']
    result = result.split(',') if result.include? ','
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === return_xorg_apps
  # ========================================================================= #
  def return_xorg_apps
    result = return_chained_programs['xorg_apps']
    result = result.split(',') if result.include? ','
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === return_xorg_fonts
  # ========================================================================= #
  def return_xorg_fonts
    result = return_chained_programs['xorg_fonts']
    result = result.split(',') if result.include? ','
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === return_xorg_server
  # ========================================================================= #
  def return_xorg_server
    result = return_chained_programs['xorg_server']
    result = result.split(',') if result.include? ','
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === try_to_compile_this_xorg_component
  #
  # The input to this method should typically be a number, such as
  # 1 (or its String-variant "1").
  #
  # Invocation example from the commandline:
  #   ry --xorg1
  # ========================================================================= #
  def try_to_compile_this_xorg_component(
      i = 1
    )
    result = nil
    big_array = return_xorg_components
    big_array.flatten!
    if i.include? '..'
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ===================================================================== #
      # Prevent too high numbers next:
      # ===================================================================== #
      i = i.to_i # <- We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result
    end
    set_compile_these_programs(result) # And assign it for compilation purposes.
  end; alias try_to_compile_this_xorg try_to_compile_this_xorg_component # === try_to_compile_this_xorg 

end; end