#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/configuration/configuration.rb'
# =========================================================================== #
require 'rbt/configuration/initialize_configuration.rb'

module RBT

  # ========================================================================= #
  # Add an alias, called email?. Invokable via: Config.email?
  # This does not work on class SimpleConfigurationLoader - only
  # works on class Configuration::Configuration, hence the check.
  # ========================================================================= #
  if RBT.configuration?.respond_to? :add_alias
    RBT.configuration?.add_alias(:email?, :email)
  end

  # ========================================================================= #
  # === RBT.audio_directory?
  #
  # Where to keep audio/sound files. (Not a very important constant; it is
  # mostly for my own home system really.)
  # ========================================================================= #
  def self.audio_directory?
    @configuration.audio_dir
  end

  # ========================================================================= #
  # === USE_FLAGS
  #
  # The Use flags - whether we will use them or whether we will not use
  # them.
  # ========================================================================= #
  _ = "#{RBT.configuration_directory?}enable_use_flags.yml"
  if File.exist? _
    USE_FLAGS = YAML.load_file(_)
  else
    USE_FLAGS = nil
  end

  # ========================================================================= #
  # === USE_TAGS
  #
  # Whether to use (and thus symlink) tags. Can be changed via the
  # yaml configuration file.
  # ========================================================================= #
  USE_TAGS = @configuration.use_tags

  # ========================================================================= #
  # === RBT::USE_ABSOLUTE_SYMLINKING
  # ========================================================================= #
  USE_ABSOLUTE_SYMLINKING = @configuration.use_absolute_symlinking

  # ========================================================================= #
  # === REPORT_SPECIFIC_PROGRAM_NAME
  # ========================================================================= #
  REPORT_SPECIFIC_PROGRAM_NAME = @configuration.report_specific_program_name

  # ========================================================================= #
  # === CAPITALIZE_PROGRAM_NAME
  #
  # Whether we should capitalize the program names or not. The difference
  # will then be e. g. "htop" versus "Htop".
  # ========================================================================= #
  CAPITALIZE_PROGRAM_NAMES = @configuration.capitalize_program_names
    CAPITALIZE_PROGRAM_NAME = CAPITALIZE_PROGRAM_NAMES

  # ========================================================================= #
  # === STOP_ON_ERROR
  #
  # Whether we stop on error or not.
  # ========================================================================= #
  STOP_ON_ERROR = @configuration.stop_on_error

  # ========================================================================= #
  # === RBT.automatically_create_package?
  #
  # Whether or not to create a package automatically.
  # ========================================================================= #
  def self.automatically_create_package?
    @configuration.automatically_create_package
  end

  # ========================================================================= #
  # === YAML_DATABASE_DIRECTORY
  #
  # Directory definition to use our yaml databases.
  # ========================================================================= #
  YAML_DATABASE_DIRECTORY = RBT.rbt_log_directory?

  # ========================================================================= #
  # === RBT.file_database_installed_files
  #
  # On my system, this will be at:
  #
  #   /Depot/Information/database_installed_files.yml
  #
  # ========================================================================= #
  def self.file_database_installed_files
    "#{YAML_DATABASE_DIRECTORY}database_installed_files.yml"
  end

  # ========================================================================= #
  # === RBT.use_yaml_database_for_tags?
  # ========================================================================= #
  def self.use_yaml_database_for_tags?
    RBT.configuration?.use_yaml_database_for_tags
  end

  # ========================================================================= #
  # === RBT.file_configure_database?
  # ========================================================================= #
  def self.file_configure_database?
    "#{RBT.rbt_log_directory?}configure_command_database.yml"
  end; self.instance_eval { alias file_configure_database file_configure_database? } # === RBT.file_configure_database

  # ========================================================================= #
  # === RBT.file_compiled_programs?
  #
  # Specify the target location for the compiled_programs.yml file.
  # ========================================================================= #
  def self.file_compiled_programs?
    "#{RBT.rbt_log_directory?}compiled_programs.yml"
  end; self.instance_eval { alias file_compiled_programs file_compiled_programs? } # === RBT.file_compiled_programs

  # ========================================================================= #
  # === RBT.file_status
  # ========================================================================= #
  def self.file_status
    "#{RBT.rbt_log_directory?}status.yml"
  end

  # ========================================================================= #
  # === RBT.system_tags?
  #
  # This method will yield towards the Tags/ subdirectory. It defaults to
  # /System/Tags/ on my home system.
  # ========================================================================= #
  def self.system_tags?
    "#{RBT.system_dir?}Tags/" # Keep a trailing / please.
  end

  # ========================================================================= #
  # === RBT.file_compiled_programs
  #
  # This method will return a location such as
  # "/Depot/Temp/rbt/compiled_programs.yml".
  #
  # Note that from this you can also obtain the "last compiled program".
  # ========================================================================= #
  def self.file_compiled_programs
    "#{RBT.rbt_log_directory?}compiled_programs.yml"
  end; self.instance_eval { alias file_last_compiled_programs file_compiled_programs } # === RBT.file_last_compiled_programs

  # ========================================================================= #
  # === RBT::TRY_TO_DOWNLOAD_IF_NOT_FOUND
  #
  # Whether to try and download a program if we can not find it locally.
  # ========================================================================= #
  TRY_TO_DOWNLOAD_IF_NOT_FOUND =
    RBT.configuration?.try_to_download_if_not_found

  # ========================================================================= #
  # === RBT.autoswitch_python?
  #
  # Whether to autoswitch python or whether to not autoswitch python.
  #
  # If set to true then RBT will attempt to switch to another
  # python version, if it is available. It will try to do so only
  # once, though.
  # ========================================================================= #
  def self.autoswitch_python?
    RBT.configuration?.autoswitch_python
  end; self.instance_eval { alias autoswitch_python autoswitch_python? } # === RBT.autoswitch_python

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/colours_e_autoinclude.rb'
  e 'Try to download if not found? '+
    sfancy(RBT::TRY_TO_DOWNLOAD_IF_NOT_FOUND.to_s)
  e 'Will we autoswitch python? '+sfancy(RBT.autoswitch_python?.to_s)
end # rbt_configuration