#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/configuration/configuration_instance_variable.rb'
# =========================================================================== #
require 'rbt/configuration/simple_configuration_loader.rb'

module RBT

  # ========================================================================= #
  # === @configuration                        (configuration tag, config tag)
  #
  # The @configuration object is defined here. The configuration part
  # will be handled by class RBT::SimpleConfigurationLoader.
  #
  # This object can be used to query configuration data for use in
  # the RBT project. A constant also exists, defined in this file,
  # but ideally one should use the @configuration variable. A reader
  # method exists for that - also defined in this directory here.
  #
  # We need to assign a base directory for the configuration to use,
  # however, which explains the first argument passed to class
  # RBT::SimpleConfigurationLoader.
  # ========================================================================= #
  @configuration = nil

  # ========================================================================= #
  # === RBT.configuration?
  #
  # Query method over the configuration-object.
  # ========================================================================= #
  def self.configuration?
    @configuration
  end; self.instance_eval { alias config                configuration? } # === RBT.config
       self.instance_eval { alias config?               configuration? } # === RBT.config?
       self.instance_eval { alias configuration         configuration? } # === RBT.configuration
       self.instance_eval { alias configuration_object? configuration? } # === RBT.configuration_object?

  # ========================================================================= #
  # === RBT.set_configuration
  #
  # This method will automatically invoke class
  # RBT::Configuration::SimpleConfigurationLoader.
  #
  # It has to be manually called to some meaningful value in other code
  # parts.
  # ========================================================================= #
  def self.set_configuration(
      i = RBT::Configuration::SimpleConfigurationLoader.new
    )
    if i.is_a? String
      i = RBT::Configuration::SimpleConfigurationLoader.new(i)
    end
    @configuration = i
  end; self.instance_eval { alias set_configuration_object set_configuration } # === RBT.set_configuration_object

end

if __FILE__ == $PROGRAM_NAME
  pp RBT.configuration?
end