#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# The code in this file here deals mostly with the file chained programs,
# aka "chained_programs.yml".
# =========================================================================== #
# require 'rbt/constants/chained_programs.rb'
# =========================================================================== #
require 'rbt/constants/file_and_directory_constants.rb'
require 'rbt/toplevel_methods/e.rb'

module RBT

  # ======================================================================= #
  # === RBT::CHAINED_PROGRAMS
  # ======================================================================= #
  _ = RBT.file_cookbook_chained_programs
  if File.exist? _
    CHAINED_PROGRAMS = YAML.load_file(_)
    # ===================================================================== #
    # Add a few aliases to CHAINED_PROGRAMS
    # ===================================================================== #
    CHAINED_PROGRAMS['xorg1'] = CHAINED_PROGRAMS['xorg_protos']
    CHAINED_PROGRAMS['xorg2'] = CHAINED_PROGRAMS['xorg_utils']
    CHAINED_PROGRAMS['xorg3'] = CHAINED_PROGRAMS['xorg_libraries']
    CHAINED_PROGRAMS['xorg4'] = CHAINED_PROGRAMS['xorg_data']
    CHAINED_PROGRAMS['xorg5'] = CHAINED_PROGRAMS['xorg_apps']
    CHAINED_PROGRAMS['xorg6'] = CHAINED_PROGRAMS['xorg_fonts']
    CHAINED_PROGRAMS['xorg7'] = CHAINED_PROGRAMS['xorg_server']
  else
    CHAINED_PROGRAMS = {} # Use an empty hash in this case.
    e "The file at `#{COOKBOOK_CHAINED_PROGRAMS_FILE}` does not exist."
  end

  # ========================================================================= #
  # === RBT.chained_programs?
  #
  # Simply return the dataset from the yaml file here.
  #
  # The file that is used here is the file chained_programs.yml,
  # which you can access via:
  #
  #   bl $RSRC/rbt/lib/rbt/yaml/chained_programs.yml
  #
  # ========================================================================= #
  def self.chained_programs?(
      i = RBT.file_cookbook_chained_programs
    )
    YAML.load_file(i)
  end; self.instance_eval { alias chained_programs        chained_programs? } # === RBT.chained_programs
       self.instance_eval { alias return_chained_programs chained_programs? } # === RBT.return_chained_programs

  # ========================================================================= #
  # === return_chained_programs
  # ========================================================================= #
  def return_chained_programs
    RBT.chained_programs?
  end

  # ========================================================================= #
  # === RBT.returned_chained_programs_of_the_mate_desktop
  #
  # This will return the mate-desktop entry, in a sanitized manner,
  # in Array form.
  # ========================================================================= #
  def self.returned_chained_programs_of_the_mate_desktop
    result = return_chained_programs['all_of_the_mate_desktop']
    if result.is_a? String
      result = result.delete("\n").delete(' ')
      # ===================================================================== #
      # Turn it into an Array next.
      # ===================================================================== #
      result = result.split(',').flatten
    end
    return result
  end

end

if __FILE__ == $PROGRAM_NAME
  pp RBT::CHAINED_PROGRAMS
  pp RBT.chained_programs?
  pp RBT.returned_chained_programs_of_the_mate_desktop
end # returned_chained_programs_of_the_mate_desktop