#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/constants/file_and_directory_constants.rb'
# =========================================================================== #
require 'rbt/project/project_base_and_yaml_directory.rb'
require 'rbt/requires/require_yaml.rb'
require 'rbt/constants/standalone_constants.rb'
require 'rbt/constants/standalone_file_constants.rb'
require 'rbt/toplevel_methods/home_dir.rb'
require 'rbt/toplevel_methods/log_directory.rb'
require 'rbt/toplevel_methods/temp_directory.rb'

module RBT

  _ = RBT.project_yaml_directory?

  # ========================================================================= #
  # === RBT::LAST_DOWNLOADED_FILE
  # ========================================================================= #
  LAST_DOWNLOADED_FILE =
    "#{RBT.home_dir?}LAST_DOWNLOADED_FILE"

  # ========================================================================= #
  # === RBT::FILE_USE_FLAGS
  # ========================================================================= #
  FILE_USE_FLAGS =
    "#{_}use_flags.yml"

  # ========================================================================== #
  # === FILE_REGISTERED_HEADERS
  # ========================================================================== #
  FILE_REGISTERED_HEADERS =
    "#{_}registered_headers.yml"

  # ========================================================================= #
  # === FILE_ABBREVIATIONS
  #
  # Note that this file will store only abbreviations to the programs,
  # not the name of the program at hand.
  # ========================================================================= #
  FILE_ABBREVIATIONS =
    "#{_}abbreviations_to_the_programs.yml"

  # ========================================================================= #
  # === RBT::FILE_COOKBOOK_ALIASES
  # ========================================================================= #
  FILE_COOKBOOK_ALIASES =
    "#{_}cookbook_aliases.yml"

  # ========================================================================= #
  # === RBT::FILE_WINDOWS_COOKBOOK
  # ========================================================================= #
  FILE_WINDOWS_COOKBOOK =
    "#{_}windows_cookbook.yml"

  # ========================================================================= #
  # === RBT::FILE_COMPILE_HOOKS
  # ========================================================================= #
  FILE_COMPILE_HOOKS = "#{_}compile_hooks.yml"

  # ========================================================================= #
  # === RBT::FILE_TIME_COMPILATION
  # ========================================================================= #
  FILE_TIME_COMPILATION =
    "#{_}configuration/time_compilation.yml"

  # ========================================================================= #
  # === FILE_PROGRAMS_DIRECTORY
  # ========================================================================= #
  FILE_PROGRAMS_DIRECTORY =
    "#{_}configuration/programs_directory.yml"

  # ========================================================================= #
  # === FILE_TRY_TO_USE_THE_FILE_WINDOWS_COOKBOOK
  # ========================================================================= #
  FILE_TRY_TO_USE_THE_FILE_WINDOWS_COOKBOOK = 
    "#{_}configuration/try_to_use_the_file_windows_cookbook.yml"

  # ========================================================================== #
  # === RBT::FILE_USE_MD5SUM
  # ========================================================================== #
  FILE_USE_MD5SUM = "#{_}configuration/use_md5sum.yml"

  # ========================================================================== #
  # === RBT::FILE_USE_PORG
  # ========================================================================== #
  FILE_USE_PORG = "#{_}configuration/use_porg.yml"

  # ========================================================================== #
  # === RBT::FILE_SHORT_TUTORIAL
  # ========================================================================== #
  FILE_SHORT_TUTORIAL =
    "#{_}tutorial/short_tutorial.md" 

  # ========================================================================== #
  # === RBT.file_short_tutorial?
  # ========================================================================== # 
  def self.file_short_tutorial?
    FILE_SHORT_TUTORIAL
  end

  # ========================================================================= #
  # === RBT::FILE_SPECIFICATION_OF_REGISTERED_COOKBOOK_ENTRIES
  # ========================================================================= #
  FILE_SPECIFICATION_OF_REGISTERED_COOKBOOK_ENTRIES =
    "#{_}specification_of_registered_cookbook_entries.yml"

  # ========================================================================= #
  # === RBT::SPECIFICATION_OF_REGISTERED_COOKBOOK_ENTRIES
  #
  # This constant here depends on the other constant called
  # FILE_SPECIFICATION_OF_REGISTERED_COOKBOOK_ENTRIES. That
  # constant is defined above.
  # ========================================================================= #
  if File.exist? FILE_SPECIFICATION_OF_REGISTERED_COOKBOOK_ENTRIES
    SPECIFICATION_OF_REGISTERED_COOKBOOK_ENTRIES = YAML.load_file(
      FILE_SPECIFICATION_OF_REGISTERED_COOKBOOK_ENTRIES
    )
  end

  # ========================================================================= #
  # === RBT.file_cookbook_aliases
  #
  # Simply return where the cookbook_aliases.yml file resides.
  #
  # This is, for instance, here:
  #
  #   /Programs/Ruby/2.5.1/lib/ruby/site_ruby/2.5.0/rbt/yaml/cookbook_aliases.yml
  #
  # ========================================================================= #
  def self.file_cookbook_aliases
    "#{RBT.project_yaml_directory?}cookbook_aliases.yml"
  end

  # ========================================================================== #
  # === RBT::FILE_PROGRAMS_VERSION
  #
  # This file, when existing, holds a shorter name: version dataset for
  # all programs.
  # ========================================================================== #
  FILE_PROGRAMS_VERSION =
    "#{_}programs_version.yml"

  # ========================================================================= #
  # === FILE_BUILD_SYSTEM_PRIORITIES
  # ========================================================================= #
  FILE_BUILD_SYSTEM_PRIORITIES =
    "#{_}build_systems_priorities.yml"

  # ========================================================================= #
  # === RBT::FILE_DEBUG
  #
  # The debug file.
  # ========================================================================= #
  FILE_DEBUG = "#{RBT.configuration_directory?}debug.yml"

  # ========================================================================= #
  # === RBT.file_use_yaml_database_for_tags
  # ========================================================================= #
  def self.file_use_yaml_database_for_tags
    "#{RBT.configuration_directory?}use_yaml_database_for_tags.yml"
  end

  # ========================================================================= #
  # === FILE_STORING_THE_TIME_IT_TOOK_TO_COMPILE_PROGRAMS
  #
  # Ideally do not use this constant, but the method defined below
  # instead.
  # ========================================================================= #
  FILE_STORING_THE_TIME_IT_TOOK_TO_COMPILE_PROGRAMS = 
    "#{RBT.log_directory?}database_storing_compile_times.yml"

  # ========================================================================= #
  # === RBT.file_storing_the_time_it_took_to_compile_programs
  #
  # Use this method variant here rather than the constant. Reason is that
  # the method assembles its method dynamically.
  # ========================================================================= #
  def self.file_storing_the_time_it_took_to_compile_programs
    "#{RBT.log_directory?}database_storing_compile_times.yml"
  end

  # ========================================================================= #
  # === RBT::FILE_CCACHE
  # ========================================================================= #
  FILE_CCACHE = "#{_}configuration/use_ccache.yml"

  # ========================================================================== #
  # === FILE_BUILD_SYSTEMS_PRIORITIES
  # ========================================================================== #
  FILE_BUILD_SYSTEMS_PRIORITIES =
    "#{_}build_systems_priorities.yml"

  # ========================================================================= #
  # === RBT.directory_expanded_cookbooks?
  #
  # On my home system, this method will return the following String:
  #
  #   "/Depot/Temp/rbt/expanded_cookbooks/"
  #
  # ========================================================================= #
  def self.directory_expanded_cookbooks?
    "#{RBT.rbt_log_directory?}expanded_cookbooks/"
  end; self.instance_eval { alias directory_expanded_cookbooks          directory_expanded_cookbooks? } # === RBT.directory_expanded_cookbooks
       self.instance_eval { alias base_dir_to_store_expanded_cookbooks? directory_expanded_cookbooks? } # === RBT.base_dir_to_store_expanded_cookbooks?
       self.instance_eval { alias expanded_cookbooks?                   directory_expanded_cookbooks? } # === RBT.expanded_cookbooks?
       self.instance_eval { alias path_to_expanded_directory?           directory_expanded_cookbooks? } # === RBT.path_to_expanded_directory?

  # ========================================================================== #
  # === RBT::FILE_ALLOWED_TAGS
  # ========================================================================== #
  FILE_ALLOWED_TAGS =
    "#{_}allowed_tags.yml"

  # ========================================================================== #
  # === RBT.file_allowed_tags
  # ========================================================================== #
  def self.file_allowed_tags
    "#{RBT.project_yaml_directory?}allowed_tags.yml"
  end

  # ========================================================================= #
  # === RBT::FILE_IS_ON_GOBOLINUX
  #
  # This file may be generated by RBT if you issue any of the following
  # commands:
  #
  #   rbt --is-on-gobolinux
  #   rbt --isongobolinux
  #   rbt isongobolinux
  #
  # ========================================================================= #
  FILE_IS_ON_GOBOLINUX = "#{RBT.log_directory?}is_on_gobolinux"

  # ========================================================================= #
  # === RBT.file_is_on_gobolinux
  # ========================================================================= #
  def self.file_is_on_gobolinux
    "#{RBT.log_directory?}is_on_gobolinux"
  end

  # ========================================================================== #
  # === RBT::FILE_COOKBOOK_CHAINED_PROGRAMS
  #
  # Easier constant to keep track of the chained cookbook-programs. In
  # this context, a chain means a successive run of programs that are
  # to be compiled.
  #
  # This will then yield to a path such as the following:
  #
  #   /Programs/Ruby/2.5.1/lib/ruby/site_ruby/2.5.0/rbt/yaml/cookbook_chained_programs.yml
  # ========================================================================== #
  FILE_COOKBOOK_CHAINED_PROGRAMS =
    "#{_}chained_programs.yml"

  # ========================================================================== #
  # === RBT.file_cookbook_chained_programs
  #
  # This is the preferred way to determine where the file
  # "cookbook_chained_programs.yml" can be found.
  # ========================================================================== #
  def self.file_cookbook_chained_programs
    "#{RBT.project_yaml_directory?}chained_programs.yml"
  end; self.instance_eval { alias file_chained_programs file_cookbook_chained_programs } # === RBT.file_chained_programs

  # ========================================================================= #
  # === RBT::FILE_GLOBAL_FLAGS
  # ========================================================================= #
  FILE_GLOBAL_FLAGS =
    "#{_}global_flags.yml"

  # ========================================================================= #
  # == RBT::ROEBE_FILE_GLOBAL_FLAGS
  #
  # This constant here is only for my home system, hence the prefix
  # ROEBE in it.
  # ========================================================================= #
  ROEBE_FILE_GLOBAL_FLAGS =
    "#{RUBY_SRC_DIR_RBT_YAML}"\
    "#{File.basename(FILE_GLOBAL_FLAGS)}"

  # ========================================================================= #
  # === RBT.file_global_flags
  # ========================================================================= #
  def self.file_global_flags
    "#{RBT.project_yaml_directory?}global_flags.yml"
  end

  # ========================================================================= #
  # === FILE_LAST_COMPILED_PROGRAM
  #
  # The YAML file containing the last compiled program. You should use
  # the method RBT.file_last_compiled_program instead, though, as this
  # one is more dynamic.
  # ========================================================================= #
  FILE_LAST_COMPILED_PROGRAM =
    "#{RBT.log_directory?}last_compiled_programs.yml"

  # ========================================================================= #
  # === RBT.file_last_compiled_program
  # ========================================================================= #
  def self.file_last_compiled_program
    "#{RBT.log_directory?}last_compiled_programs.yml"
  end

  # ========================================================================= #
  # === RBT.return_file_specification_of_registered_cookbook_entries
  # ========================================================================= #
  def self.return_file_specification_of_registered_cookbook_entries(
      i = FILE_SPECIFICATION_OF_REGISTERED_COOKBOOK_ENTRIES
    )
    i
  end; self.instance_eval { alias file_specification_of_registered_cookbook_entries return_file_specification_of_registered_cookbook_entries } # === RBT.file_specification_of_registered_cookbook_entries

  # ========================================================================= #
  # === FILE_BASE_TEMPLATE_FOR_COOKBOOK_YAML_FILES
  #
  # This constant specifies the path to the base-template file that
  # can be used for creating new .yml files, for the RBT::Cookbooks
  # namespace.
  # ========================================================================= #
  FILE_BASE_TEMPLATE_FOR_COOKBOOK_YAML_FILES =
    PROJECT_BASE_DIR+'cookbooks/template/base_template_for_cookbook_yaml_files.md'

  # ========================================================================= #
  # === RBT::DIRECTORY_RBT_PROFILES
  # ========================================================================= #
  DIRECTORY_RBT_PROFILES =
    "#{PROJECT_BASE_DIR}misc/profiles/"

  # ========================================================================= #
  # === RBT.file_last_symlinked_program?
  # ========================================================================= #
  def self.file_last_symlinked_program?
    "#{RBT.log_directory?}last_symlinked_program.yml"
  end

end