#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/constants/predefined_installation_instructions.rb'
# =========================================================================== #
require 'rbt/project/project_base_and_yaml_directory.rb'
require 'rbt/requires/require_yaml.rb'

module RBT

  # ========================================================================= #
  # === RBT::HASH_INSTALLATION_INSTRUCTIONS
  #
  # This Hash gives us an easier handle to sanitize our data.
  #
  # It has some constant-shortcuts that are convenient for use lateron.
  #
  # For instance, RINSTALL means "ruby install", PINSTALL means
  # "python install".
  # ========================================================================= #
  HASH_INSTALLATION_INSTRUCTIONS = {}

  # ========================================================================= #
  # === RBT.file_predefined_installation_instructions
  #
  # Method to return where the file location is. There used to be
  # a constant available called FILE_PREDEFINED_INSTALLATION_INSTRUCTIONS
  # but this was removed in January 2019.
  # ========================================================================= #
  def self.file_predefined_installation_instructions(
      use_this_directory = RBT.project_yaml_directory?
    ) 
    "#{use_this_directory}predefined_installation_instructions.yml"
  end

  # ========================================================================= #
  # === RBT::PREDEFINED_INSTALLATION_INSTRUCTIONS
  #
  # Load up the dataset for the predefined installation instructions next.
  # ========================================================================= #
  _ = file_predefined_installation_instructions
  if File.exist? _
    @predefined_installation_instructions = YAML.load_file(_)
    @predefined_installation_instructions.each_pair { |key, value|
      HASH_INSTALLATION_INSTRUCTIONS[key] = value
    }
  else
    @predefined_installation_instructions = {} # Define an empty Hash if the file does not exist.
  end

  # ========================================================================= #
  # === RBT.predefined_installation_instructions?
  # ========================================================================= #
  def self.predefined_installation_instructions?
    @predefined_installation_instructions
  end; self.instance_eval { alias pii? predefined_installation_instructions? } # === RBT.pii?

end

if __FILE__ == $PROGRAM_NAME
  pp RBT::HASH_INSTALLATION_INSTRUCTIONS
end # rbt_predefined_installation_instructions