#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/constants/registered_cookbook_entries.rb'
# =========================================================================== #
require 'rbt/project/project_base_and_yaml_directory.rb'
require 'rbt/requires/require_yaml.rb'

module RBT

  # ========================================================================= #
  # === RBT::FILE_REGISTERED_COOKBOOK_ENTRIES
  #
  # In the past there was a constant that would return the path to the
  # file called 'registered_cookbook_entries.yml'. However had, since as
  # of 25.01.2019 this is discouraged and has been disabled - if you
  # need to obtain the path to this particular .yml file then you
  # are encouraged to use the method called 
  # RBT.file_registered_cookbook_entries instead.
  # ========================================================================= #
  # FILE_REGISTERED_COOKBOOK_ENTRIES =
  #   "#{RBT.yaml_directory?}registered_cookbook_entries.yml"
  # ========================================================================= #

  # ========================================================================= #
  # === RBT.file_registered_cookbook_entries
  #
  # Tell use where the file registered_cookbook_entries.yml is stored.
  # ========================================================================= #
  def self.file_registered_cookbook_entries(
      use_this_directory = RBT.yaml_directory?
    )
    "#{use_this_directory}registered_cookbook_entries.yml"
  end

  # ========================================================================= #
  # === RBT::REGISTERED_COOKBOOK_ENTRIES
  #
  # The following constant contains all registered entries in the cookbook.
  #
  # These can be found specified in the file registered_cookbook_entries.yml.
  #
  # It is an Array.
  #
  # The file itself resides at:
  #
  #   $RSRC/rbt/lib/rbt/yaml/registered_cookbook_entries.yml
  #
  # ========================================================================= #
  _ = RBT.file_registered_cookbook_entries
  if File.exist? _
    REGISTERED_COOKBOOK_ENTRIES =
      YAML.load_file(_)
  else
    REGISTERED_COOKBOOK_ENTRIES = []
  end

  # ========================================================================= #
  # === RBT.registered_cookbook_entries?
  #
  # Simply return all registered cookbook entries through this
  # getter method.
  # ========================================================================= #
  def self.registered_cookbook_entries?
    REGISTERED_COOKBOOK_ENTRIES
  end; self.instance_eval { alias registered_cookbook_entries registered_cookbook_entries? } # === RBT.registered_cookbook_entries?

  # ========================================================================= #
  # === registered_cookbook_entries?
  #
  # Just a pointer to the module-method with the same name.
  # ========================================================================= #
  def registered_cookbook_entries?
    RBT.registered_cookbook_entries?
  end

end

if __FILE__ == $PROGRAM_NAME
  pp RBT.registered_cookbook_entries?
end # rbt_registered_cookbook_entries