#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file will hold the relevant remote URL entries for the classes
# found in this directory.
# =========================================================================== #
# require 'rbt/constants/remote_urls.rb'
# =========================================================================== #
module RBT

  # ========================================================================= #
  # === RBT::REMOTE_URL_FOR_KDE_PLASMA
  #
  # Always point to the latest KDE-Plasma (upstream) URL here,
  # such as https://www.kde.org/info/plasma-5.14.3.php.
  #
  # To invoke updating all KDE-plasma components from the commandline,
  # you may issue the following command:
  #
  #   update_kde_plasma
  #
  # ========================================================================= #
  REMOTE_URL_FOR_KDE_PLASMA =
    'https://download.kde.org/stable/plasma/5.16.3/'

  # ========================================================================= #
  # === RBT::REMOTE_URL_FOR_KDE_APPLICATIONS
  #
  # To batch-update these applications, do:
  #
  #   update_kde_applications
  #
  # ========================================================================= #
  REMOTE_URL_FOR_KDE_APPLICATIONS =
    'https://download.kde.org/stable/applications/19.04.3/src/'

  # ========================================================================= #
  # === RBT::REMOTE_DOWNLOAD_URL_FOR_KDE_PLASMA
  # ========================================================================= #
  REMOTE_DOWNLOAD_URL_FOR_KDE_PLASMA =
    'https://download.kde.org/stable/plasma/'

  # ========================================================================= #
  # === RBT::REMOTE_URL_FOR_KDE_FRAMEWORKS
  #
  # This URL should point to the latest KDE-frameworks release.
  #
  # Presently, (08 September 2018) the latest KDE-frameworks release
  # is 5.50.0 (the URL, oddly enough, does not include the trailing
  # "0").
  #
  # To batch-update from the commandline, try this:
  #
  #   update_kde_frameworks
  #
  # ========================================================================= #
  REMOTE_URL_FOR_KDE_FRAMEWORKS =
    'https://download.kde.org/stable/frameworks/5.59/'

  # ========================================================================= #
  # === RBT::REMOTE_URL_FOR_KDE_PORTING_AIDS
  #
  # This constant must come after the constant REMOTE_URL_FOR_KDE_FRAMEWORKS
  # has been defined.
  #
  # To update these, try:
  #
  #   rbt --update_porting_aids
  #
  # ========================================================================= #
  REMOTE_URL_FOR_KDE_PORTING_AIDS =
    "#{REMOTE_URL_FOR_KDE_FRAMEWORKS}portingAids/"

  # ========================================================================= #
  # === RBT::REMOTE_PUB_MATE_DESKTOP_RELEASES
  # ========================================================================= #
  REMOTE_PUB_MATE_DESKTOP_RELEASES =
    'http://pub.mate-desktop.org/releases/'

  # ========================================================================= #
  # === RBT::REMOTE_PUB_MATE_DESKTOP_RELEASES_LATEST_VERSION       (mate tag)
  #
  # This should be used as the base for the download-functionality in
  # the file check_for_mate_desktop_updates.rb
  #
  # This would correspond to an URL like:
  #
  #   http://pub.mate-desktop.org/releases/1.22/
  #
  # ========================================================================= #
  REMOTE_PUB_MATE_DESKTOP_RELEASES_LATEST_VERSION =
    "#{REMOTE_PUB_MATE_DESKTOP_RELEASES}1.22/"

  # ========================================================================= #
  # === REMOTE_DOWNLOAD_BASE_URL_FOR_LXQT
  # ========================================================================= #
  REMOTE_DOWNLOAD_BASE_URL_FOR_LXQT = 
    'https://downloads.lxqt.org/'

  # ========================================================================= #
  # === CURRENT_VERSION_OF_LXQT
  #
  # Use this constant to keep track as to which lxqt version is the
  # one that is currently tracked by RBT. It has to be manually
  # updated (for now).
  # ========================================================================= #
  CURRENT_VERSION_OF_LXQT = '0.14.1'

  # ========================================================================= #
  # === REMOTE_DOWNLOAD_URL_FOR_LXQT
  #
  # The following constant should have an URL such as:
  #   https://downloads.lxqt.org/0.14.1.html
  # ========================================================================= #
  REMOTE_DOWNLOAD_URL_FOR_LXQT =
    "#{REMOTE_DOWNLOAD_BASE_URL_FOR_LXQT}#{CURRENT_VERSION_OF_LXQT}.html"

  # ========================================================================= #
  # === RBT::REMOTE_SLACKWARE_HOMEPAGE
  # ========================================================================= #
  REMOTE_SLACKWARE_HOMEPAGE =
    'http://www.slackware.com/changelog/current.php?cpu=x86_64'

  # ========================================================================= #
  # === RBT::BASE_URL_TO_GNOME_SOURCES
  # ========================================================================= #
  BASE_URL_TO_GNOME_SOURCES =
    'http://ftp.gnome.org/pub/GNOME/sources/'

  # ========================================================================= #
  # === RBT::BLFS_CHANGELOG_WEBPAGE
  # ========================================================================= #
  BLFS_CHANGELOG_WEBPAGE =
    'http://www.linuxfromscratch.org/blfs/view/cvs/introduction/changelog.html'

  # ========================================================================= #
  # === RBT::REMOTE_URL_TO_THE_MATE_DESKTOP_COMPONENTS
  #
  # This constant specifies where we can find the (remote) source code
  # to the mate-desktop, via HTTP-file listing.
  # ========================================================================= #
  REMOTE_URL_TO_THE_MATE_DESKTOP_COMPONENTS =
    "#{REMOTE_PUB_MATE_DESKTOP_RELEASES_LATEST_VERSION}?C=M&O=D"

end