#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Constants in this file here, may not depend on information derived from
# the outside. It is ok to define shortcuts within this file, though.
# =========================================================================== #
# require 'rbt/constants/standalone_constants.rb'
# =========================================================================== #
module RBT

  # ========================================================================= #
  # === N
  # ========================================================================= #
  N = "\n"

  # ========================================================================= #
  # === RBT::TABULATOR
  # ========================================================================= #
  TABULATOR = "\t"

  # ========================================================================= #
  # === ALPHABET
  # ========================================================================= #
  ALPHABET = ('a'..'z').to_a

  # ========================================================================= #
  # === RBT::GOBOLINUX_LAYOUT
  #
  # This constant is probably only needed for gobolinux systems.
  # ========================================================================= #
  GOBOLINUX_LAYOUT =
    "#{ENV['GOBOLINUX_LAYOUT']}" # Is always a String.

  # ========================================================================= #
  # === RBT::LINUX
  # ========================================================================= #
  LINUX = "#{ENV['LINUX']}"

  # ========================================================================= #
  # === RBT::RBT_HOMEPAGE_URL
  #
  # The RBT Homepage is defined here.
  # ========================================================================= #
  RBT_HOMEPAGE_URL = 'https://rubygems.org/gems/rbt'

  # ========================================================================== #
  # === RBT::MAKE_COMMAND_TO_USE
  #
  # Which make command to use by default.
  # ========================================================================== #
  MAKE_COMMAND_TO_USE = 'make'

  # ========================================================================= #
  # === RBT::FILE_CPUINFO
  # ========================================================================= #
  FILE_CPUINFO = '/proc/cpuinfo'

  # ========================================================================= #
  # === RBT::DEPOT_DIR
  # ========================================================================= #
  DEPOT_DIR = '/Depot/'

  # ========================================================================= #
  # === RBT::USR_INCLUDE_DIR
  #
  # This can be standalone because it will be ALWAYS absolute,
  # pointing towards "/usr/include/".
  # ========================================================================= #
  USR_INCLUDE_DIR = '/usr/include/'

  # ========================================================================= #
  # === SYSBIN_DIRECTORY
  #
  # The following constant should not be used; instead, one should rather
  # use the method RBT.sysbin_directory?, since that method also is
  # set through the yaml-configuration.
  # ========================================================================= #
  SYSBIN_DIRECTORY = '/System/Index/bin/'

  # ========================================================================= #
  # === RBT::DEFAULT_PROGRAM_ENTRY
  # ========================================================================= #
  DEFAULT_PROGRAM_ENTRY = 'htop'
          DEFAULT_ENTRY = DEFAULT_PROGRAM_ENTRY # an "alias" to the above ^^^

  # ========================================================================= #
  # === RBT::SHELL_ERROR_CODE
  #
  # Shell Error code - to append.
  # ========================================================================= # 
  SHELL_ERROR_CODE = ' 2>&1'
        ERROR_LINE = SHELL_ERROR_CODE
  
  # ========================================================================= #
  # === RBT::REGISTER_COMPILED_PROGRAM
  #
  # Whether we should register compiled programs or not.
  # ========================================================================= #
  REGISTER_COMPILED_PROGRAM = true

  # ========================================================================== #
  # === RBT::COMPILE_EVERYTHING_INTO_STANDALONE_DIRECTORY
  #
  # Whether to compile everything into a standalone directory or not.
  # ========================================================================== #
  COMPILE_EVERYTHING_INTO_STANDALONE_DIRECTORY = false

  # ========================================================================= #
  # === RBT::RUBY_SRC_DIR
  #
  # This deliberately hard-coded constant is for my home system - it helps
  # speed up the development process if I can use such constants as
  # shortcuts.
  # ========================================================================= #
  RUBY_SRC_DIR = '/Users/x/DATA/PROGRAMMING_LANGUAGES/RUBY/src/'
    RUBY_SRC_DIR_AT_HOME = RUBY_SRC_DIR # === RUBY_SRC_DIR_AT_HOME
    RUBY_SRC_DIRECTORY   = RUBY_SRC_DIR # === RUBY_SRC_DIRECTORY
    RUBY_SRC             = RUBY_SRC_DIR # === RUBY_SRC

  # ========================================================================= #
  # === RBT::RUBY_SRC_DIR_RBT
  #
  # This constant is only useful on my home system.
  #
  # It corresponds to:
  #
  #   /Users/x/DATA/PROGRAMMING_LANGUAGES/RUBY/src/rbt/lib/rbt/
  #
  # ========================================================================= #
  RUBY_SRC_DIR_RBT = "#{RUBY_SRC_DIR}rbt/lib/rbt/"
    ROEBE_PATH_TO_COOKBOOKS = RUBY_SRC_DIR_RBT # === ROEBE_PATH_TO_COOKBOOKS

  # ========================================================================= #
  # === RBT::RUBY_SRC_DIR_RBT_YAML
  #
  # This constant may correspond to the String
  # "/Users/x/DATA/PROGRAMMING_LANGUAGES/RUBY/src/rbt/lib/rbt/yaml/".
  # ========================================================================= #
  RUBY_SRC_DIR_RBT_YAML = "#{RUBY_SRC_DIR_RBT}yaml/"
    # ======================================================================= #
    # === RUBY_SRC_DIR_RBT_YAML_AT_HOME
    # ======================================================================= #
    RUBY_SRC_DIR_RBT_YAML_AT_HOME = RUBY_SRC_DIR_RBT_YAML

  # ========================================================================= #
  # === DIRECTORY_INDIVIDUAL_COOKBOOKS_AT_ROEBE_HOME
  #
  # This constant is used mostly on my home system.
  # ========================================================================= #
  DIRECTORY_INDIVIDUAL_COOKBOOKS_AT_ROEBE_HOME =
    "#{RUBY_SRC_DIR_RBT_YAML}individual_cookbooks/"

  # ========================================================================= #
  # === URL_TO_REMOTE_CHANGELOG
  #
  # Where to keep the remote changelog, if we make use of one that is.
  # ========================================================================= #
  URL_TO_REMOTE_CHANGELOG =
    'http://shevy.bplaced.net//rbt_changelog.html'

  # ========================================================================= #
  # === RBT::COOKBOOKS_BASE_DIRECTORY
  #
  # The following constant is actually only useful for my home system,
  # and only if I don't use any other setting. So this is mostly just
  # here for legacy reasons.
  # ========================================================================= #
  COOKBOOKS_BASE_DIRECTORY =
    '/Users/x/DATA/PROGRAMMING_LANGUAGES/RUBY/src/rbt/lib/rbt/yaml/individual_cookbooks/'

  # ========================================================================== #
  # === RBT::CHECK_FOR_BINARY_NAMES
  # ========================================================================== #
  CHECK_FOR_BINARY_NAMES  = true

  # ========================================================================== #
  # === RBT::CHECK_FOR_LIBRARY_NAMES
  # ========================================================================== #
  CHECK_FOR_LIBRARY_NAMES = true

  # ========================================================================== #
  # === RBT::SHOW_EXTRA_INFORMATION
  # ========================================================================== #
  SHOW_EXTRA_INFORMATION = true

  # ========================================================================= #
  # === RBT::DEFAULT_COLOUR_TO_USE
  #
  # Specify what is our main colour to use for the Cookbooks project.
  # ========================================================================= #
  DEFAULT_COLOUR_TO_USE = "\e[0;32m" # This is GREEN, which is defined in module Colours.

  # ========================================================================= #
  # === RBT::HOST_CPU
  #
  # The host CPU that the scripts are running on.
  # ========================================================================= #
  HOST_CPU   = RbConfig::CONFIG['host_cpu']

  # ========================================================================= #
  # === TARGET_CPU
  #
  # The target CPU that we are running on.
  # ========================================================================= #
  TARGET_CPU = RbConfig::CONFIG['target_cpu']

  # ========================================================================= #
  # === ARRAY_HELP_OPTIONS                                          (help tag)
  #
  # All the help options are stored here. These are the ones that
  # work on e. g. "rbt --help".
  # ========================================================================= #
  ARRAY_HELP_OPTIONS = %w(
    HELP -h --help --help? help hel he h
    display_standard_help_options
  )+['Display help']

  # ========================================================================= #
  # === HASH_ENGLISH_NUMBERS
  # ========================================================================= #
  HASH_ENGLISH_NUMBERS = {
    'one'   => 1,
    'two'   => 2,
    'three' => 3,
    'four'  => 4,
    'five'  => 5,
    'six'   => 6,
    'seven' => 7,
    'eight' => 8,
    'nine'  => 9
  }

  # ========================================================================= #
  # === ARRAY_HELP_VERSION_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_VERSION_OPTIONS = %w(
    --version?
    --version
    version?
    VERSION
    version
  )

end

if __FILE__ == $PROGRAM_NAME
  puts RBT::RUBY_SRC_DIR  
end