#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Controller.new
#
# The purpose and idea behind this Controller-class is to collect
# all specific actions the user may want to do with the RBT scripts.
#
# Then, this collection is mapped to the specific class in question,
# which will perform the job.
#
# This way the user only has to learn the usage of one tool, in theory -
# the Controller class here.
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class Controller < Base # === RBT::Controller.new

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      cmdline_arguments = nil,
      run_already       = true
    )
    reset
    set_commandline(cmdline_arguments)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === set_commandline
  #
  # Use this method to set the @commandline. We can then respond to what
  # the user instructs us to do.
  # ========================================================================= #
  def set_commandline(i)
    @commandline = i
  end

  # ========================================================================= #
  # === process_commandline
  # ========================================================================= #
  def process_commandline
    _ = @commandline.pop
    case _ # case tag
    # ======================================================================= #
    # === beautify
    # ======================================================================= #
    when 'beautify'
      BeautifyConfigureHelpOutput.new
    # ======================================================================= #
    # === to_bash
    # ======================================================================= #
    when '1','to_bash','tob','tobash'
      Compile.new(nil,nil,false).to_bash # the name doesnt matter here anyway.
    when 'help','?','fhelp','hel'
      feedback_help
    end
  end

  # ========================================================================= #
  # === feedback_help
  #
  # To invoke this, do:
  #   controller fhelp
  # ========================================================================= #
  def feedback_help
    cliner {
      e "Feedbacking help from #{return_cgrey('Controller.rb')} now:#{N}"
      e
      e_table '  tobash       - generate bash shell scripts'
      e_table '  register     - register something'
      e_table '  generate     - generate a makefile'
      e_table '  graph        - run graphviz'
      e_table '  to_trad      - convert a package to traditional'
      e
    }
  end

  # ========================================================================= #
  # === return_cgrey
  # ========================================================================= #
  def return_cgrey(i)
    return Colours::CGREY+i+rev
  end

  # ========================================================================= #
  # === e_table
  # ========================================================================= #
  def e_table(i)
    splitted = i.split '-'
    beautified = ( '%-15s' % splitted[0] ) +  ' - '+splitted[1]
    e '  '+beautified
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    process_commandline
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Controller.new(ARGV)
end
# ======================================================================== #
# Specific usage examples for the Controller class:
#
#   controller compile php5 wdep
#   cont compile php5 wdep
#   rbt compile php5 wdep
#   rbt tutorial
#   controller help
#   rbt 1
# ======================================================================== #