#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/cookbooks/class/constants.rb'
# =========================================================================== #
module RBT

module Cookbooks

class Cookbook # === RBT::Cookbooks::Cookbook

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === DEFAULT_PROGRAM_VERSION
  # ========================================================================= #
  DEFAULT_PROGRAM_VERSION = DEFAULT_VERSION = '1.0.0'

  # ========================================================================= #
  # === REPLACE_PLUS_WITH_LONG_NAME
  #
  # If this constant is true then we will replace '+' with 'plus'
  # if they are part of a program name.
  # ========================================================================= #
  REPLACE_PLUS_WITH_LONG_NAME = false

  # ========================================================================= #
  # === RBT::Cookbooks::Cookbook::ARRAY_FALSE_VALUES              (false tag)
  #
  # This constant collects all values that must be explicitely set to
  # false on startup.
  # ========================================================================= #
  ARRAY_FALSE_VALUES = %w(
    apply_patch
    autosymlink_lib64
    build_static
    can_be_compiled_statically
    copy_source
    do_not_symlink
    do_not_download
    echo_yes
    enable_shared
    enable_static
    keep_extracted
    localstatedir
    modify_the_makefile
    override
    run_make_check
    use_autoconf
    use_autogen
    use_build_directory
    use_glib_schema
    use_this_make_install_command
    sed
    symlink_glib_schema
    symlink_headers
    symlink_pkgconfig_files
  )

  # ========================================================================= #
  # === Cookbooks::ARRAY_NIL_ENTRIES                                (nil tag)
  #
  # All entries that are nil by default.
  # ========================================================================= #
  ARRAY_NIL_ENTRIES = %w(
    short_description
    set_env_variables
    homepage
    meson_configure_options
  )

  # ========================================================================= #
  # === Cookbooks::ARRAY_ARRAY_ENTRIES                            (array tag)
  #
  # These entries should be kept as an Array at all time. They will all
  # allow us to run .each on them.
  #
  # In "pure" ruby code, these would all be set to e. g. "xyz = []", so
  # really just an Array of Arrays.
  # ========================================================================= #
  ARRAY_ARRAY_ENTRIES = %w(
    binaries
    headers
    installation_steps
    libraries
    pkgconfig_files
    postinstall
    pre_configure_steps
    pre_make_commands
    pre_make_install_sed
    required_deps_on
    tags
  )

  # ========================================================================= #
  # === Cookbooks::ARRAY_EMPTY_VALUES
  #
  # All values that will initialize default to an empty string,
  # have to be registered here.
  # ========================================================================= #
  ARRAY_EMPTY_VALUES = %w(
    program_path
    program_full_name
    licence
    url1
    program_version
    configure_command
    use_this_make_command
    configure_base_dir
    optional_start_message
    preinstall
    parameters_to_make
    use_this_build_directory
    configure_options
    svn_url
    git_url
    md5sum
  )

  # ========================================================================= #
  # === Cookbooks::ARRAY_MAIN_PROGRAM_ENTRIES
  #
  # We need the following 8 entries defined here, so that we do not
  # forget them lateron.
  #
  # These are the only entries we care about and describe the most
  # important components of a program.
  # ========================================================================= #
  ARRAY_MAIN_PROGRAM_ENTRIES = %w(
    archive_type
    base_dir
    program_full_name
    program_name
    program_path
    program_version
    real_short_name
    short_name
  )

  # ========================================================================= #
  # === RBT::Cookbooks::Cookbook.main_program_entries?
  # ========================================================================= #
  def self.main_program_entries?
    ARRAY_MAIN_PROGRAM_ENTRIES
  end

  # ========================================================================= #
  # === ARRAY_TRUE_VALUES                                          (true tag)
  #
  # Only a few variables are true for now, by default - prefix,
  # run_ldconfig and has_to_be_compiled at the very least.
  # ========================================================================= #
  ARRAY_TRUE_VALUES = %w(
    active_project
    has_to_be_compiled
    may_we_modify_the_configure_options
    prefix
    run_configure
    run_ldconfig
    run_make
  )

  # ========================================================================= #
  # === is_a_boolean_configuration_value?
  #
  # Query method to determine whether the input is a boolean configuration
  # value. For example, the String "use_glib_schema", would be a boolean
  # configuration value and thus this method would return true.
  # ========================================================================= #
  def is_a_boolean_configuration_value?(i)
    ( ARRAY_FALSE_VALUES + ARRAY_TRUE_VALUES ).flatten.include? i
  end

end; end; end