#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'cookbooks/class/feedback/feedback_format_specification_of.rb'
# =========================================================================== #
require 'rbt/base/prototype.rb'
require 'rbt/constants/file_and_directory_constants.rb'
require 'rbt/cookbooks/class/class_methods/opnn.rb'
require 'rbt/cookbooks/class/class_methods/e.rb'

module RBT

module Cookbooks

class Cookbook < Base

  # ========================================================================= #
  # === Cookbooks::Cookbook.feedback_format_specification_of
  #
  # This is the method that will tell us whether an entry in the cookbooks
  # settings is an Array or a String or a Hash or a Boolean value.
  #
  # Usage example:
  #
  #   Cookbooks::Cookbook.feedback_format_specification_of :headers
  #
  # From the commandline:
  #
  #   scookie --format-specification-of=extract_to
  #   cookbooks --format-specification-of=extract_to
  #
  # ========================================================================= #
  def self.feedback_format_specification_of(
      i = 'extract_to'
    )
    i = i.to_s unless i.is_a? String # We need a String here.
    # ======================================================================= #
    # First, load the YAML file that has our dataset.
    # ======================================================================= #
    dataset = YAML.load_file(
      return_file_specification_of_registered_cookbook_entries # We need this method.
    )
    if dataset.has_key? i
      format_type = dataset[i]
      opnn; e "The format type for the key `#{Colours.simp(i)}` is:"
      opnn; e "  #{Colours.sfancy(format_type)}"
    else
      opnn; e 'No format called `'+Colours.simp(i)+'` is known.'
    end
  end

  # ========================================================================= #
  # === feedback_format_specification_of
  # ========================================================================= #
  def feedback_format_specification_of(i)
    ::Cookbooks::Cookbook.feedback_format_specification_of(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::Cookbook.feedback_format_specification_of
  RBT::Cookbooks::Cookbook.feedback_format_specification_of :headers
end