#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/cookbooks/class/misc.rb'
# =========================================================================== #
module RBT

module Cookbooks

class Cookbook < Base # === RBT::Cookbooks::Cookbook.new

  # ========================================================================= #
  # === sort_by_archive_priority
  # ========================================================================= #
  def sort_by_archive_priority(i)
    array_priority = [
      '.xz',
      '.bz2',
      '.gz',
      '.lz',
      '.gem'
    ]
    begin
      selection = i.select {|entry|
        entry.include?('.') and is_an_archive?(entry)
      }
      i = selection.sort_by {|entry|
        extname = File.extname(entry)
        if array_priority.include? extname
          entry = array_priority.index(extname)
        end
        entry
      }
    rescue ArgumentError => error
      opnn; e 'An error occurred in the method '+
              slateblue(__method__.to_s)+'. (Line: '+royalblue(__LINE__)+')'
      pp i
      pp error
      exit
    end
    i
  end

  # ========================================================================= #
  # === is_github_url?
  #
  # This method will return true if the main url (url1) is a github url.
  # ========================================================================= #
  def is_github_url?(
      i = url1?
    )
    i.start_with?('https://github.com/') or
    i.start_with?('http://github.com/')
  end

  # ========================================================================= #
  # === has_a_dependency_on_meson?
  #
  # This method will return a boolean value, indicating whether the given
  # program at hand has a dependency on the build system called meson or
  # whether it has not.
  # ========================================================================= #
  def has_a_dependency_on_meson?
    required_deps_on?.include? 'meson'
  end

  # ========================================================================= #
  # === update_kde_applications
  # ========================================================================= #
  def update_kde_applications
    unless RBT::Cookbooks.const_defined? :UpdateKdeApplications
      require 'rbt/check_for_updates/update_kde_applications.rb'
    end
    UpdateKdeApplications.new
  end

  # ========================================================================= #
  # === update_mate_desktop_components
  #
  # Update mate-desktop via this method.
  #
  # Invocation example:
  #   cookbooks --update-mate
  # ========================================================================= #
  def update_mate_desktop_components
    unless RBT::Cookbooks.const_defined? :CheckForMateDesktopUpdates
      require 'rbt/check_for_updates/check_for_mate_desktop_updates.rb'
    end
    CheckForMateDesktopUpdates.new
  end

end; end; end