#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/cookbooks/create_cookbook_yaml_file/misc.rb'
# =========================================================================== #
module RBT

module Cookbooks

class CreateCookbookYamlFile < ::RBT::Base # === RBT::Cookbooks::CreateCookbookYamlFile

  # ========================================================================= #
  # === do_not_open_in_editor
  # ========================================================================= #
  def do_not_open_in_editor
    @open_in_editor_after_creation = false
  end

  # ========================================================================= #
  # === do_open_in_editor
  # ========================================================================= #
  def do_open_in_editor
    @open_in_editor_after_creation = false
  end

  # ========================================================================= #
  # === description?
  # ========================================================================= #
  def description?
    @hash_information_about_the_program[:description]
  end

  # ========================================================================= #
  # === file?
  # ========================================================================= #
  def file?
    @file
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
  e
    e 'How to create a new cookbook:' # newcookbook --help
    e
    e 'Simply provide the name of the .yml file, such as:'
    e
    e '   newcookbook php.yml'
    e
    e 'Alternatively, provide a remote URL which we will use to '\
      'generate such a file.'
    e
    e '  --interactive # this will ask you several questions before'
    e '                # the new .yml file will be created'
    e
  end

  # ========================================================================= #
  # === consider_opening_in_editor
  #
  # We may use the newly created file in our main editor.
  # ========================================================================= #
  def consider_opening_in_editor(
      this_file = file?
    )
    if open_in_editor?
      _ = @editor+' '+this_file
      opnn; e 'We will next open this file in our editor, as instructed due'
      opnn; e 'to the ivar '+sfancy('@open_in_editor_after_creation')
      opnn; e 'having been set to true.'
      system(_)
    end
  end; alias edit_this_file consider_opening_in_editor # === edit_this_file

  # ========================================================================= #
  # === add_hint_for_dependencies
  # ========================================================================= #
  def add_hint_for_dependencies
    if is_on_roebe?
      ''
    else # Add this line to help newbies.
      "# The dependencies go in here.#{N}"
    end
  end

  # ========================================================================= #
  # === enter_interactive_part
  #
  # This method allows the user to specifically designate some
  # parts of the recipe at hand, interactively.
  #
  # Invocation example:
  #
  #   ccook --interactive
  #
  # ========================================================================= #
  def enter_interactive_part
    @interactive_mode = true
    e 'A fewquestions may be asked next, about the values that should be'
    e 'added to the yaml file at hand.'
    e
    e slateblue('Remote URL?')
    set_url1($stdin.gets.chomp)
    e
    e slateblue('Description of the program?')
    set_description($stdin.gets.chomp)
    e
    e 'Done!'
  end

  # ========================================================================= #
  # === set_filename
  #
  # This method will store the filename, i.e. where to store the file.
  # ========================================================================= #
  def set_filename(i)
    i = i.to_s.dup
    i = return_program_information(i)
    if i
      i << YAML_EXTENSION unless i.end_with? '.yml'
      # ===================================================================== #
      # Since as of March 2014 we will also downcase the name.
      # ===================================================================== #
      i.downcase!
    end
    set_file(i)
  end; alias set_real_name set_filename # === set_real_name

  # ========================================================================= #
  # === set_file
  #
  # This method is not allowed to do any modifications - it will simply
  # assign towards the @file variable.
  # ========================================================================= #
  def set_file(i)
    @file = i
  end

  # ========================================================================= #
  # === open_in_editor?
  # ========================================================================= #
  def open_in_editor?
    @open_in_editor_after_creation
  end

end; end; end