#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::DisplayCookbookDataset
#
# This class can show the dataset associated to a specific program at
# hand.
# =========================================================================== #
# require 'rbt/cookbooks/display_cookbook_dataset/class.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'
# =========================================================================== #
# Next pull in .rb files that belong to class DisplayCookbookDataset.
# =========================================================================== #
require 'rbt/cookbooks/display_cookbook_dataset/constants.rb'
require 'rbt/cookbooks/display_cookbook_dataset/reset.rb'

module RBT

module Cookbooks

class DisplayCookbookDataset < Base # === RBT::Cookbooks::DisplayCookbookDataset

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      this_program = ARGV,
      run_already  = true
    )
    reset # Defined in reset.rb
    set_program(this_program)
    if block_given?
      yielded = yield
      case yielded
      when :do_not_show_md5sum
        set_show_md5sum false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === set_show_md5sum
  # ========================================================================= #
  def set_show_md5sum(i)
    @internal_hash[:show_md5sum] = i
  end

  # ========================================================================= #
  # === show_md5sum?
  # ========================================================================= #
  def show_md5sum?
    @internal_hash[:show_md5sum]
  end

  # ========================================================================= #
  # === use_only_symbols_for_the_main_hash?
  # ========================================================================= #
  def use_only_symbols_for_the_main_hash?
    false # Hardcoded for now. We keep only Strings.
  end

  # ========================================================================= #
  # === main_program_entries?
  #
  # Query method over ARRAY_MAIN_PROGRAM_ENTRIES.
  # ========================================================================= #
  def main_program_entries?
    self.class.main_program_entries?
  end

  # ========================================================================= #
  # === set_program
  # ========================================================================= #  
  def set_program(i)
    i = i.first if i.is_a? Array
    @program = i
  end

  # ========================================================================= #
  # === program?
  # ========================================================================= #
  def program?
    @program
  end

  # ========================================================================= #
  # === determine_dataset
  # ========================================================================= #
  def determine_dataset(i = program?)
    @dataset = RBT::Cookbooks::Cookbook.new(i) { :bypass_menu_check }
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @dataset
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    determine_dataset
    opnn; e "Displaying information about the program #{sfancy(@program)}."
    dataset = dataset?
    ljust_value = 30
    # ======================================================================= #
    # Display important values.
    # ======================================================================= #
    display_this_option :prefix
    e 'Program path: '.ljust(ljust_value)+sfancy(dataset.program_path?.to_s)
    e 'Dependencies on: '.ljust(ljust_value)+sfancy(dataset.deps_on?.join(',').rstrip.to_s)
    display_this_option :base_dir
    display_this_option :short_name
    display_this_option :real_short_name
    e 'Archive size: '.ljust(ljust_value)+sfancy(dataset.archive_size?.to_s)
    if show_md5sum?
      e 'md5sum: '.ljust(ljust_value)+sfancy(dataset.md5sum?.to_s)
    end
  end

  # ========================================================================= #
  # === display_this_option
  # ========================================================================= #
  def display_this_option(i = :short_name)
    result = ''.dup
    result << case i
    when :prefix
      'Prefix: '
    when :base_dir
      'Base dir: '
    when :short_name
      'Short name: '
    when :real_short_name
      'Real short name: '
    end
    result = result.ljust(30)
    name_of_the_method = "#{i}?"
    result << sfancy(@dataset.send(name_of_the_method.to_sym).to_s)
    e result
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::DisplayCookbookDataset.new(ARGV)
end # scookie2 vim