#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::RequiredDependenciesOn
#
# This class originated, as conceptual idea, from an older class called
# DependencyCollector. We wish to be able to determine the dependencies
# for any given program.
#
# Usage example:
#   RBT::RequiredDependenciesOn.new
# =========================================================================== #
# require 'rbt/cookbooks/required_dependencies_on/required_dependencies_on.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/requires/require_the_cookbook_class.rb'

module RBT

class RequiredDependenciesOn < Base # === RBT::RequiredDependenciesOn.new

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    set_be_verbose
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    i = [i].flatten.compact
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end

  # ========================================================================= #
  # === first?
  # ========================================================================= #
  def first?
    @commandline_arguments.first
  end; alias program_name? first? # === program_name?

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    program_name = program_name?
    if RBT.has_this_program_registered?(program_name)
      @cookbook = RBT::Cookbooks::Cookbook.new(program_name) { :bypass_menu_check }
      report(@cookbook.required_dependencies_on?)
    else
      opne "The program #{sfancy(program_name)} is not "\
           "registered as part of the RBT project."
    end
  end

  # ========================================================================= #
  # === return_dependencies_of_this_program
  #
  # The argument to this method should be the name of the program that
  # we are trying to find.
  # ========================================================================= #
  def return_dependencies_of_this_program(
      i = program_name?
    )
    # ======================================================================= #
    # The input may include strings such as '>=', such as in 'mpc >= 0.8.1'.
    # In this case we will split the input.
    # ======================================================================= #
    if i.include? '='
      i = i.split('=').first.strip
    end
    i.delete!('><')
    i.strip!
    RBT::Cookbooks::Cookbook.new(i) { :bypass_menu_check }.required_dependencies_on?
  end

  # ========================================================================= #
  # === dependencies?
  # ========================================================================= #
  def dependencies?
    @cookbook.required_dependencies_on?
  end

  # ========================================================================= #
  # === report                                                  (report tag)
  # ========================================================================= #
  def report(
      the_dependencies = dependencies?,
      be_verbose       = be_verbose?
    )
    program_name = program_name?
    if be_verbose
      opne "The following dependencies are known (and "\
           "have been registered) for #{royalblue(program_name)}:"
      e
      cliner
      e royalblue(program_name)
    end
    the_dependencies.each {|this_dependency|
      e '  '+slateblue(this_dependency)
      dependencies_for_that_program = return_dependencies_of_this_program(this_dependency)
      if dependencies_for_that_program
        e '    '+slateblue(this_dependency)+' in turn depends on:'
        dependencies_for_that_program.each {|entry|
          e '      '+royalblue(entry.to_s)
        }
      end
    }
    if be_verbose
      e
      cliner
      e
    end
  end; alias report_dependencies report # === report_dependencies

  # ========================================================================= #
  # === RBT::RequiredDependenciesOn[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::RequiredDependenciesOn.new(ARGV)
end # required_dependencies_on gcc