#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This gtk-widget will simply show the compile-time statistics. That's
# about it. It is deliberately kept a small widget.
#
# Todo:
# - Could add an update-button.
# =========================================================================== #
require 'rbt/shell/shell.rb'
require 'gtk_module/base/base_module.rb'
require 'gtk_module/gtk_runner.rb'

module RBT

module GUI

module Gtk

class CompileShellStatistics < ::Gtk::ScrolledWindow

  include Colours::E
  include ::Gtk::BaseModule

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize
    super()
    set_policy(
      ::Gtk::POLICY_AUTOMATIC, ::Gtk::POLICY_AUTOMATIC
    )
    reset
    run
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    # ======================================================================= #
    # The next instance-variable is a Hash.
    # ======================================================================= #
    @compile_time_statistics =
      ::RBT::Compile::Shell.new(:dont_run_yet).return_compile_time
    set_gtk_font_size(:default)
  end

  # ========================================================================= #
  # === hash?
  # ========================================================================= #
  def hash?
    @compile_time_statistics
  end

  # ========================================================================= #
  # === return_time_sorted_dataset
  # ========================================================================= #
  def return_time_sorted_dataset
    _ = hash?
    _ = _.sort_by {|name, time|
      time
    }.reverse
    _
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    treeview = gtk_tree_view
    # treeview.selection.mode = ::Gtk::SELECTION_MULTIPLE
    treeview.headers_clickable = true
    # ======================================================================= #
    # Create a new GtkCellRendererText, add it to the tree
    # view column and append the column to the tree view.
    # ======================================================================= #
    renderer = gtk_cell_renderer_text
    renderer.foreground = '#0c087d' # This is a darkblue colour.
    column   = ::Gtk::TreeViewColumn.new(
      'Name of the program', renderer,  :text => 0
    )
    column.alignment = 0.5
    treeview.append_column(column)
    renderer = gtk_cell_renderer_text
    renderer.foreground = '#037317' # This is a darkgreen colour.
    column   = ::Gtk::TreeViewColumn.new(
      'Time (in seconds)', renderer, :text => 1
    )
    column.alignment = 0.1
    treeview.append_column(column)

    list_store = ::Gtk::ListStore.new(String, String)
    return_time_sorted_dataset.each {|name_of_the_program, compile_time|
      iter = list_store.append
      iter.set_value(0, name_of_the_program)
      _ = ('%.2f' % compile_time.round(2)).rjust(10,' ')
      iter.set_value(1, _)
    }
    treeview.model = list_store
    treeview.signal_connect(:button_press_event) { |widget, event|
      if event.event_type == Gdk::Event::BUTTON2_PRESS and event.button == 1 # event.event_type.name
        selected = treeview.selection.selected.first
        pp selected  
      end
    }
    add(treeview)
  end

  # ========================================================================= #
  # === width?
  # ========================================================================= #
  def width?
    1000
  end

  # ========================================================================= #
  # === height?
  # ========================================================================= #
  def height?
    520
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  widget = RBT::GUI::Gtk::CompileShellStatistics.new
  r = ::Gtk::Runner.new(
    widget.width?,
    widget.height?,
    '/Users/x/DATA/IMG/PC/TABBLE/TABBLE.png',
    'Compile Shell Statistics', 4
  )
  r << widget
  r.is_resizable
  r.top_left
  r.show_all
  r.run
end # rb compile_shell_statistics.rb