#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === Gtk::InstalledPrograms
#
# This widget will display to the user which programs are available
# on the given computer system at hand.
# =========================================================================== #
# require 'rbt/gui/gtk/installed_programs.rb'
# =========================================================================== #
begin
  require 'colours'
rescue LoadError; end
require 'rbt/toplevel_methods/swift_return_version_of_this_program.rb'
require 'gtk_module/base/base_module.rb'

# =========================================================================== #
# Will display installed programs, ie these found in the database.
# =========================================================================== #
module Gtk

class InstalledPrograms < Gtk::VBox

  include Colours
  include ::Gtk::Colours
  include ::Gtk::BaseModule

  alias e puts

  begin
    # ======================================================================= #
    # Pull in the RBT project - note that this is quite a large project.
    # ======================================================================= #
    require 'rbt'
  rescue LoadError; end

  # ========================================================================= #
  # === N
  # ========================================================================= #
  N = "\n"

  # ========================================================================= #
  # === USE_DATABASE
  # ========================================================================= #
  USE_DATABASE = false

  # ========================================================================= #
  # === FEEDBACK_EXTENDED_INFORMATION
  # ========================================================================= #
  FEEDBACK_EXTENDED_INFORMATION = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize
    super('InstalledPrograms')
    reset
    sanitize_data
    create_vbox
    create_scrolled_window
    create_liststore
    create_treeview
    create_buttons
    connect_skeleton
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @use_database = USE_DATABASE
    @array_lsmod_result = []
    @renderer = gtk_cell_renderer_text
    @renderer.editable = true
  end

  # ========================================================================= #
  # === update_listing
  # ========================================================================= #
  def update_listing
    sanitize_data
    update_liststore
  end

  # ========================================================================= #
  # === update_liststore
  # ========================================================================= #
  def update_liststore
    dataset = sanitize_data
    array = []
    # The dataset can be a String or an Array. If it is a String, we must
    # convert it into an Array first.
    _ = dataset
    if _.is_a? String
      _ = dataset.split(N)
      _.each { |entry|
        # =================================================================== #
        # We split on ' ->' Strings.
        # =================================================================== #
        array << [
          entry.split(' ->')[0].downcase.to_s,
          entry.split(' ->')[1].to_s
        ]
      }
      _ = array
    end
    _.sort.each { |entry|
      iter = @list_store.append
      iter.set_value(0, entry[0] )
      iter.set_value(1, entry[1] )
    }
  end

  # ========================================================================= #
  # === create_scrolled_window
  # ========================================================================= #
  def create_scrolled_window
    @scrolled_window = gtk_scrolled_window # Gtk::ScrolledWindow.new
    @scrolled_window.set_shadow_type(Gtk::SHADOW_IN)
    @scrolled_window.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC)
  end

  # ========================================================================= #
  # === create_treeview
  # ========================================================================= #
  def create_treeview
    @tree_view = Gtk::TreeView.new(@list_store)
    @tree_view.insert_column(-1, 'ProgramName',    @renderer, :text => 0)
    @tree_view.insert_column(-1, 'ProgramVersion', @renderer, :text => 1)
    @tree_view.enable_model_drag_source(Gdk::Window::BUTTON1_MASK, 
      [ 
        ['GTK_TREE_MODEL_ROW', 0, 0] 
      ],
      Gdk::DragContext::ACTION_COPY|Gdk::DragContext::ACTION_MOVE
    )
    @tree_view.enable_model_drag_dest([ ['GTK_TREE_MODEL_ROW', 0, 0] ], 
    Gdk::DragContext::ACTION_COPY|Gdk::DragContext::ACTION_MOVE)
    @tree_view.signal_connect(:button_press_event) { |widget, event|
      if    event.event_type == Gdk::Event::BUTTON2_PRESS and event.button == 1 # event.event_type.name
      elsif event.event_type == Gdk::Event::BUTTON_PRESS # left click here.
        # =================================================================== #
        # If the user clicks on the left name, we will feedback
        # extended information, if a top-constant was set to true.
        # =================================================================== #
        if feedback_extended_information?
          program_name = @tree_view.selection.selected[0]
          return_dataset(program_name).report
        end
      end
    }
  end

  # ========================================================================= #
  # === feedback_extended_information?
  # ========================================================================= #
  def feedback_extended_information?
    FEEDBACK_EXTENDED_INFORMATION
  end

  # ========================================================================= #
  # === create_buttons
  # ========================================================================= #
  def create_buttons
    @button_update = gtk_button('_Update')
    @button_update.signal_connect(:clicked) {
      e 'Updating Listing'
      update_listing
    }
    @button_update.modify_bg( Gtk::StateType::ACTIVE, CORAL )
    @button_update.modify_bg( Gtk::StateType::NORMAL, MEDIUMAQUAMARINE)
    @button_update.modify_bg( Gtk::StateType::PRELIGHT, DARKGREEN ) 
  end

  # ========================================================================= #
  # === create_liststore
  # ========================================================================= #
  def create_liststore
    @list_store = Gtk::ListStore.new(String, String, String)
    update_liststore
  end

  # ========================================================================= #
  # === create_vbox
  # ========================================================================= #
  def create_vbox
    @v_box = Gtk::VBox.new
  end

  # ========================================================================= #
  # === connect_skeleton
  # ========================================================================= #
  def connect_skeleton
    @v_box.add(@tree_view).add(@button_update)
    @scrolled_window.add_with_viewport(@v_box)
    self.add(@scrolled_window)
  end

  # ========================================================================= #
  # === width?
  # ========================================================================= #
  def width?
    820
  end

  # ========================================================================= #
  # === height?
  # ========================================================================= #
  def height?
    500
  end

  # ========================================================================= #
  # === sanitize_data
  # ========================================================================= #
  def sanitize_data
    _ = ''
    if @use_database
      x = StdDatabases.new
      x.set_handle
      x.sqlite3_select_all
      _ = x.sqlite3_return_all_programs
    else # else we use RBT.
      _ = RBT.available_programs?
      _.map! {|entry|
        version = RBT.swift_return_version_of(entry) # sanitize_cookbook_dataset.version?
        # =================================================================== #
        # Store the name and the version.
        # =================================================================== #
        [entry, version]
      }
      # We must add the versions in this case though.
    end
    return _ 
  end

  # ========================================================================= #
  # === return_dataset
  # ========================================================================= #
  def return_dataset(i)
    RBT::Cookbooks::Cookbook.new(i) { :bypass_menu_check }
  end

end; end

if __FILE__ == $PROGRAM_NAME
  x = Gtk::InstalledPrograms.new
  r = Gtk.run(x.width?, x.height?, nil, 'Installed Programs')
  r.add(x)
  r.run
end # gtk_installed_programs