#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::GUI::Gtk::TextViewWidget
#
# The purpose of this widget is twofold:
#
#   (a) offer a way to compile a program
#   (b) show the result per line in a textview widget
#
# =========================================================================== #
require 'gtk_module'
require 'cliner/module'
require 'rbt'
require 'gtk_module/base/base_module.rb'

module RBT

module GUI

module Gtk

class TextViewWidget < ::Gtk::VBox

  include ::Gtk::BaseModule

  Thread.abort_on_exception = true

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize
    super()
    reset
    run
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
  end

  # ========================================================================= #
  # === add_textview_widget
  # ========================================================================= #
  def add_textview_widget
    @textview_widget = ::Gtk.return_textview_widget
    @textview_widget.indent = 4
    pack_start(@textview_widget, true, true, 2)
  end

  # ========================================================================= #
  # === return_content_of_input_widget
  # ========================================================================= #
  def return_content_of_input_widget
    @input_widget_on_the_left.content
  end

  # ========================================================================= #
  # === do_compile_this_program          (compile tag, action tag, click tag)
  #
  # The action that is used to compile a given program at hand.
  # ========================================================================= #
  def do_compile_this_program(
      this_program = return_content_of_input_widget
    )
    text = 'Now compiling the program `'+sfancy(this_program)+'`.'
    append_this_text_to_the_textview_widget(
      text
    )
    # ======================================================================= #
    # Also report it on the commandline.
    # ======================================================================= #
    opnn; e text
    append_this_text_to_the_textview_widget(
      this_program
    )
    Thread.new {
      @compile_object = RBT::Compile.new(
        this_program.strip
      )
      set_success_or_failure(
        @compile_object.has_been_successfully_compiled?
      )
    }.join
  end

  # ========================================================================= #
  # === set_success_or_failure
  # ========================================================================= #
  def set_success_or_failure(i)
    @success_or_failure = i.to_s.strip
    if i.to_s == 'true'
      @label_for_success_or_failure.set_text(
        'The program '+return_content_of_input_widget+' has been compiled successfully!'
      )
    else
      @label_for_success_or_failure.set_text(
        'The program '+return_content_of_input_widget+' has NOT been compiled successfully.'
      )
    end
    @label_for_success_or_failure.make_bold
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    Opn.opn(namespace: NAMESPACE)
  end

  # ========================================================================= #
  # === append_this_text_to_the_textview_widget                  (append tag)
  #
  # This is mostly a convenience method - whenever it is called,
  # we will simply append to the textview widget.
  # ========================================================================= #
  def append_this_text_to_the_textview_widget(
      this_text
    )
    unless this_text.end_with? N
      this_text = this_text.dup if this_text.frozen?
      this_text << N
    end
    @textview_widget.append_this_text(this_text)
  end

  # ========================================================================= #
  # === add_input_widget_to_the_left_and_start_compile_button_on_the_right
  # ========================================================================= #
  def add_input_widget_to_the_left_and_start_compile_button_on_the_right
    @input_widget_on_the_left = gtk_entry # entry tag
    @input_widget_on_the_left.on_click_highlight_all
    @input_widget_on_the_left.on_enter_event_select_this
    @compile_button_on_the_right = gtk_button('_Compile')
    @compile_button_on_the_right.signal_connect(:clicked) {
      do_compile_this_program
    }
    @compile_button_on_the_right.tooltip(
      'Press this button to start compiling the program '\
      'listed to the left'
    )
    @widget_with_input_and_compile_button = ::Gtk.hbox(
      @input_widget_on_the_left,
      @compile_button_on_the_right
    )
    pack_start(@widget_with_input_and_compile_button, false, true, 2)
  end

  # ========================================================================= #
  # === add_gui_skeleton
  # ========================================================================= #
  def add_gui_skeleton
    add_textview_widget
    add_input_widget_to_the_left_and_start_compile_button_on_the_right
    add_label_for_success_or_failure
  end

  # ========================================================================= #
  # === add_label_for_success_or_failure
  # ========================================================================= #
  def add_label_for_success_or_failure
    @label_for_success_or_failure = ::Gtk.label()
    pack_start(@label_for_success_or_failure, false, true,2)
  end

  # ========================================================================= #
  # === height?
  # ========================================================================= #
  def height?
    800
  end

  # ========================================================================= #
  # === width?
  # ========================================================================= #
  def width?
    900
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    add_gui_skeleton
    show_all
  end

end

end; end; end

if __FILE__ == $PROGRAM_NAME
  widget = RBT::GUI::Gtk::TextViewWidget.new
  r = ::Gtk::Runner.new(
    widget.height?,
    widget.width?,
    '/Users/x/DATA/IMG/PC/TABBLE/TABBLE.png',
    'Compile Widget', 12
  )
  r << widget
  r.is_resizable
  r.top_left
  r.show_all
  r.run
end