#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# The job of this file is to return all programs that are available
# in the homebrew project. 
# =========================================================================== #
# require 'rbt/homebrew/registered_programs_of_homebrew.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/e.rb'
require 'rbt/colours/colours.rb'

module RBT

  # ========================================================================= #
  # === RBT::REMOTE_URL_TO_THE_HOMEBREW_FORMULAS
  # ========================================================================= #
  REMOTE_URL_TO_THE_HOMEBREW_FORMULAS =
    'https://formulae.brew.sh/formula/'

  # ========================================================================= #
  # === RBT.return_all_registered_programs_of_homebrew
  # ========================================================================= #
  def self.return_all_registered_programs_of_homebrew
    require 'rbt/check_for_updates/base.rb'
    _ = RBT::Cookbooks::CheckForRemoteWebpages.new
    _.obtain_dataset_from_the_remote_webpage(REMOTE_URL_TO_THE_HOMEBREW_FORMULAS)
    result = _.dataset?
    use_this_regex = # See: http://rubular.com/r/ASTY2a3ukx
      /<tr><td><a href="\/formula\/(.+)">.+\s*.+\s*.+\s*<td>(\d{0,2}\.\d{0,2}\.\d{0,2})<\/td>/
    scanned = result.scan(use_this_regex)
    scanned
  end

  # ========================================================================= #
  # === RBT.report_all_registered_programs_of_homebrew
  # ========================================================================= #
  def self.report_all_registered_programs_of_homebrew
    these_entries = RBT.return_all_registered_programs_of_homebrew
    e 'The following entries were found:'
    index = 0
    these_entries.each {|name_of_the_program, program_version| index += 1
      padded_index = (index.to_s.rjust(4)).ljust(6)
      colourized_and_padded_index = RBT.dodgerblue(padded_index)
      e '  '+colourized_and_padded_index+
        name_of_the_program.delete('-').ljust(40)+
        program_version
    }
  end

  # ========================================================================= #
  # === RBT.return_programs_that_are_exlusively_registered_in_the_homebrew_programs
  #
  # Usage example:
  #   x = RBT.return_programs_that_are_exlusively_registered_in_the_homebrew_programs
  # ========================================================================= #
  def self.return_programs_that_are_exlusively_registered_in_the_homebrew_programs
    require 'rbt/toplevel_methods/available_programs.rb'
    available_programs = RBT.available_programs?
    remote_registered_programs = RBT.return_all_registered_programs_of_homebrew.map {|a,b|
      a.delete('-_')
    }
    remote_registered_programs.reject! {|entry| available_programs.include? entry }
    remote_registered_programs
  end

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/colours_e_autoinclude.rb'
  # RBT.return_all_registered_programs_of_homebrew
  # RBT.report_all_registered_programs_of_homebrew
  x = RBT.return_programs_that_are_exlusively_registered_in_the_homebrew_programs
  pp x
  n_programs = x.size
  e 'There are a total of '+sfancy(n_programs)+' programs available in homebrew.'
end # rbt_registered_programs_of_homebrew