#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/infobox/run.rb'
# =========================================================================== #
module RBT

class Infobox < RBT::Base # === RBT::Infobox.new

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = compile_this_program?
    if _.nil?
      opnn; e 'Please supply the name of the program to show.'
      return
    end
    # ======================================================================= #
    # Right now, only .program_path? can lead to a longer padding size.
    # ======================================================================= #
    determine_padding_size_to_use(_.program_path?)
    ascii_hliner_with_this_text { :box_drawings_heavy_down_and_left }
    blank_line
    display extracting_compiling_or_installing+' this program',
            _.short_program_name?
    # ======================================================================= #
    # === Prefix
    #
    # Show the prefix, but not if it is a .gem file.
    # ======================================================================= #
    display 'Prefix', _.prefix? unless _.program_path?.end_with? '.gem'
    display 'Program version', _.program_version?
    display 'Program path', _.program_path?
    # ======================================================================= #
    # === Display where we will extract the archive into
    # ======================================================================= #
    display 'Will be extracted towards', extract_to?
    # ======================================================================= #
    # === Keep the archive extracted
    #
    # Note that this setting may be removed at a later time - not sure
    # yet (June 2018).
    #
    # We will not display this information if it is a .gem file though.
    # ======================================================================= #
    unless _.program_path?.end_with? '.gem'
      display 'Keep the archive extracted', colourized_verbose_truth(keep_extracted?)
    end
    # ======================================================================= #
    # === Show whether we will use ccache or not
    # ======================================================================= #
    e complete_left_padding_with_this_text('Will ccache be used')+
      colourized_verbose_truth(@use_ccache.to_s)+' '+return_vertical_bar_while_honouring_unicode
    # ======================================================================= #
    # === Show the CFLAGS in use
    # ======================================================================= #
    display 'CFLAGS in use', cflags_in_use?
    # ======================================================================= #
    # === Show if we try to use a static build
    # ======================================================================= #
    if _.build_static?
      e complete_left_padding_with_this_text('Compile statically')+
      colourized_verbose_truth(_.build_static?.to_s)+' '+
      return_vertical_bar_while_honouring_unicode
    end
    # ======================================================================= #
    # === Display whether we will use a build directory
    #
    # This used to be a conditional check, via "if _.use_build_directory?",
    # but since as of 18.09.2018 we will show this no matter what.
    # ======================================================================= #
    display 'Use a build directory', colourized_verbose_truth(_.use_build_dir?)
    # ======================================================================= #
    # === Display the configure options
    # ======================================================================= #
    use_these_configure_options = _.configuration_options?
    if use_these_configure_options
      unless _.configuration_options?.empty?
        display 'configure options', use_these_configure_options
      end
    end
    # ======================================================================= #
    # === Display postinstall step, but only if it exists
    # ======================================================================= #
    if postinstall? and !postinstall?.empty?
      display 'Postinstall action', postinstall?.join(' ')
    end
    # ======================================================================= #
    # === Display x86 or x64 type
    # ======================================================================= #
    display 'Host architecture in use', RBT.determine_host_architecture
    if @show_required_dependencies
      display 'Required dependencies on', _.dependencies?
    end
    # ======================================================================= #
    # === Show whether we will use autogen, but only if we really do
    # ======================================================================= #
    if _.use_autogen?
      display 'Use autogen', colourized_verbose_truth(true)
    end
    # ======================================================================= #
    # === Show the build system in use next
    #
    # We have to be mindful of situations such as when "configure" would
    # be displayed, but the .yml file has a "run_configure: no" entry.
    # ======================================================================= #
    use_this_build_system = _.use_which_build_system?
    if (input?.run_configure? == false) and (use_this_build_system == 'configure')
      # ===================================================================== #
      # In this case the program does not use configure at all, so it
      # does not make a whole lot of sense to insinuate that 'configure'
      # is used, naturally.
      # ===================================================================== #
      use_this_build_system = '(unknown / unspecified)'
    end
    display 'Build system to be used',
            padded_colourize_of_this_input(use_this_build_system, :seagreen)

    # ======================================================================= #
    # === Parameters for make
    # ======================================================================= #
    unless parameters_to_make?.reject {|entry| entry.nil? or entry.empty? }.empty?
      joined = parameters_to_make?
      if joined.is_a? Array
        joined = joined.join(',').strip
      end
      joined.chop! if joined.end_with? ','
      display 'Parameters for make',
              padded_colourize_of_this_input(joined, :seagreen)
    end
    # ======================================================================= #
    # === Show the compiler that will be used for the current run
    # ======================================================================= #
    display 'Compiler to be used:',
      padded_colourize_of_this_input(compiler_to_use?, :mediumpurple)
    blank_line
    ascii_hliner(:default, :box_drawings_heavy_up_and_left_and_right) { :heavy_up_and_horizontal }
    e
  end

end; end