#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Libtool
#
# Usage example:
#   RBT::Libtool.new
# =========================================================================== #
# require 'rbt/libtool/libtool.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
# =========================================================================== #
# Next, require .rb files that belong to class RBT::Libtool.
# =========================================================================== #
require 'rbt/libtool/constants.rb'
require 'rbt/libtool/help.rb'
require 'rbt/libtool/initialize.rb'
require 'rbt/libtool/menu.rb'
require 'rbt/libtool/remove_this_entry.rb'
require 'rbt/libtool/reset.rb'
require 'rbt/libtool/run.rb'
require 'rbt/libtool/search_in_these_directories.rb'

module RBT

class Libtool < Base # === RBT::Libtool.new

  # ========================================================================= #
  # === log_these_files
  # ========================================================================= #
  def log_these_files
    @array_log_these_files
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.join(' ').strip if i.is_a? Array
    i = i.to_s.dup
    if i.end_with? ':'
      i.chop! # Trailing ':' seem pointless.
    end
    @input = i
    menu(@input)
  end

  # ========================================================================= #
  # === ensure_that_we_have_found_la_files
  # ========================================================================= #
  def ensure_that_we_have_found_la_files
    if @array_all_libtool_files.empty?
      obtain_all_the_la_files
    end
  end

  # ========================================================================= #
  # === report_how_many_la_files_were_found
  # ========================================================================= #
  def report_how_many_la_files_were_found
    opnn; e "There are a total of #{sfancy(n_la_files?.to_s)}"\
            " .la files on this system."
    opnn; e 'The directories '+sdir(target_directories?.join(', '))+
            ' were searched.'
  end
  
  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === show_every_la_file
  #
  # Invocation example:
  #   roebelibtool --show-every-la-file
  # ========================================================================= #
  def show_every_la_file
    ensure_that_we_have_found_la_files
    opnn; e 'Every .la file on this computer will be shown next'
    e
    @array_all_libtool_files.each_with_index {|this_libtool_file, index| index += 1
      index = (index.to_s+')').ljust(6)
      e '  '+sfancy(index)+sfile(this_libtool_file)
    }; e
  end

  # ========================================================================= #
  # === Libtool.autofix_every_la_file
  #
  # Use this method to fix every .la file on your system.
  #
  # Invocation example:
  #   rubylibtool --autofix
  # ========================================================================= #
  def self.autofix_every_la_file
    ensure_that_we_have_found_la_files
    if Libtool.faulty_la_files?.empty?
      # ===================================================================== #
      # In this case, fill up the Array first.
      # ===================================================================== #
      Libtool.fill_up_array_that_holds_faulty_la_files
    end
    Libtool.faulty_la_files?.each {|file|
      opnn; e "Now working on the #{orange('.la')} file `#{sfile(file)}`."
      dataset = File.readlines(file)
      new_file_content = ''
      dataset.each {|line|
        if line.include?('dependency_libs') and 
           line.include? PROGRAMS_DIRECTORY.chop # .chop because we have a trailing '/'.
          splitted = line.split(' ')
          splitted.map! {|entry|
            if entry.include? PROGRAMS_DIRECTORY.chop
              entry = rds(entry)
              # ============================================================= #
              #   /usr//Programs/Libx11/1.6.3/lib/libX11.la
              # or
              #   /usr//usr//usr//Programs/Atspi2core/2.18.2/lib/libatspi.la
              # really strange
              # ============================================================= #
              regex = REGEX_TO_DISCOVER_PROGRAMS_ENTRIES_IN_DEPENDENCY_LIBS
              entry =~ regex
              match = $1.to_s.dup
              entry.sub!(/#{match}/, '/usr/')
            end
            entry
          }
          line = splitted.join(' ')+N
        end
        new_file_content << line
      }
      what = new_file_content
      into = file
      opnn; e 'Now storing into `'+sfile(into)+'`.'
      write_what_into(what, into)
    }
  end; self.instance_eval { alias fix_libtool_files autofix_every_la_file } # === RBT.fix_libtool_files

  # ========================================================================= #
  # === n_la_files?
  # ========================================================================= #
  def n_la_files?
    @array_all_libtool_files.size
  end

  # ========================================================================= #
  # === show_statistics
  # ========================================================================= #
  def show_statistics
    ensure_that_we_have_found_la_files
    report_how_many_la_files_were_found
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end

# =========================================================================== #
# === RBT.libtool
#
# Easier module-level method towards Libtool.
# =========================================================================== #
def self.libtool(i = ARGV)
  RBT::Libtool.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Libtool.new(ARGV)
end # rbt_libtool --remove-this-entry=/usr/lib/libgobject-2.0.la