#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/libtool/menu.rb'
# =========================================================================== #
module RBT

class Libtool < Base

  # ========================================================================= #
  # === menu                                                       (menu tag)
  #
  # The input to this should ideally be the whole commandline, as String.
  #
  # Reason being that we can apply a regex on the whole line.
  # ========================================================================= #
  def menu(i)
    case i # case tag
    # ======================================================================= #
    # === Remove a faulty .la entry that was stored in a local .md file
    #
    # This entry point can be used to simply remove the last stored .la
    # file.
    #
    # Invocation example:
    #   rubylibtool --remove-from-stored-file
    # ======================================================================= #
    when /^-?-?remove(_|-)?from(_|-)?stored(_|-)?file$/i,
         /^-?-?remove(_|-)?from(_|-)?file$/i
      remove_the_last_faulty_libtool_entry_stored_in_a_file
    # ======================================================================= #
    # === Remove a single, faulty libtool entry from every .la file
    #
    # This method will modify offending .la files. The input argument
    # will be removed from these files.
    #
    # Trigger the following via:
    #   rubylibtool --remove-this-entry=/usr/lib/libpng16.la
    #   rubylibtool --remove-this-entry=/Programs/Sqliteautoconf/3110000/lib/libsqlite3.la
    # ======================================================================= #
    when /-?-?remove-?this-?entry=(.+)/,
         /-?-?remove=(.+)/,
         /-?-?purge=(.+)/,
         /-?-?purge (.+)/,
         /-?-?purge-?this-?entry=(.+)/,
         /-?-?purge-?this=(.+)/,
         /-?-?purge-?entry=(.+)/,
         /-?-?fix-?this-?entry=(.+)/
      remove_this_entry($1.to_s.dup)
      exit
    # ======================================================================= #
    # === Show the help section
    # ======================================================================= #
    when /-?-?help/,'HELP'
      show_help
      exit
    # ======================================================================= #
    # === rubylibtool --search-in-this-directory=/opt/lib
    # ======================================================================= #
    when /(_|-)?(_|-)?search(_|-)?in(_|-)?this(_|-)?directory=(.+)/
      append_this_directory_to_the_list_of_directories_to_search_for(
        $6.to_s.dup
      )
    # ======================================================================= #
    # === rubylibtool --search-in-which-directories?
    # ======================================================================= #
    when /(_|-)?(_|-)?search(_|-)?in(_|-)?which(_|-)?directories\??/
      notify_the_user_which_directories_will_be_searched
    # ======================================================================= #
    # === Designate another directory to use
    #
    # Specific usage example:
    #   rubylibtool --from-this-directory=PWD --purge=/usr/lib/libstdc++.la
    # ======================================================================= #
    when /--from-this-directory=(.+)/
      use_this_directory($1.to_s.dup)
    # ======================================================================= #
    # === rubylibtool --autocorrect
    #
    # The idea about autocorrecting the .la files is to convert
    # /Programs/ entries into /usr/ entries.
    # ======================================================================= #
    when /-?-?autofix/, /-?-?autocorrect/
      autofix_every_la_file
    # ======================================================================= #
    # === Show every .la file on this machine
    #
    # Invocation example:
    #   roebelibtool --show-every-la-file
    # ======================================================================= #
    when /-?-?show(_|-)?every(_|-)?la(_|-)?file/,
         /-?-?all$/,'ALL'
      show_every_la_file
    # ========================================================================= #
    # === roebe_libtool --stats
    # ========================================================================= #
    when /-?-?stats/,/-?-?statistics/,/-?-?report/
      show_statistics
      exit
    else # else nothing was found.
      if i.end_with?('.la')
        remove_this_entry(i)
      end
      # opnn; e 'Unsure which action to perform. Pass in "--help" for the '\
      #         'available options.'
    end
  end

end; end