#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/libtool/remove_this_entry.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/libtool/class_methods/use_this_directory.rb'
require 'rbt/libtool/constants.rb'
require 'rbt/libtool/reset.rb'
require 'rbt/libtool/search_in_these_directories.rb'

module RBT

class Libtool < Base # === RBT::Libtool

  # ========================================================================= #
  # === notify_the_user_which_directories_will_be_searched
  # ========================================================================= #
  def notify_the_user_which_directories_will_be_searched
    opnn; e 'The following directories will be searched:'
    e "  #{sfancy(@search_in_these_directories.join(', '))}"
  end

  # ========================================================================= #
  # === append_this_directory_to_the_list_of_directories_to_search_for
  # ========================================================================= #
  def append_this_directory_to_the_list_of_directories_to_search_for(i)
    i = i.dup if i.frozen?
    i << '/' unless i.end_with? '/'
    @search_in_these_directories << i unless @search_in_these_directories.include? i
  end

  # ========================================================================= #
  # === search_in_these_directories?
  # ========================================================================= #
  def search_in_these_directories?
    @search_in_these_directories
  end; alias target_directories?          search_in_these_directories? # === target_directories?
       alias search_in_which_directories? search_in_these_directories? # === search_in_which_directories?

  # ========================================================================= #
  # === obtain_all_the_la_files
  # ========================================================================= #
  def obtain_all_the_la_files
    @search_in_these_directories.each {|this_dir|
      these_la_files = Dir["#{this_dir}**/**.la"]
      @array_all_libtool_files << these_la_files
      @array_all_libtool_files.flatten!
    }
    return @array_all_libtool_files
  end; alias obtain_all_libtool_files obtain_all_the_la_files # === obtain_all_libtool_files
       alias every_la_file            obtain_all_the_la_files # === every_la_file

  # ========================================================================= #
  # === clear_array_log_these_files
  # ========================================================================= #
  def clear_array_log_these_files
    @array_log_these_files = []
  end

  # ========================================================================= #
  # === remove_this_entry
  #
  # Use this method to remove a specific entry within all .la files.
  # ========================================================================= #
  def remove_this_entry(i)
    clear_array_log_these_files # Reset the Array first.
    if i.end_with?(':') and !File.exist?(i)
      i.chop!
    end
    opnn; e 'The following entry will be removed from every '\
            'encountered .la file on this computer:'
    e
    e "  #{seagreen(i)}"
    e
    if ::RBT::Libtool.use_this_directory?
      la_files = Dir["#{RBT::Libtool.use_this_directory?}**/**.la"]
    else
      la_files = obtain_all_libtool_files
    end
    notify_the_user_which_directories_will_be_searched
    # ======================================================================= #
    # Next, reject those who do not contain the given search term "i".
    # We will also log these files.
    # ======================================================================= #
    la_files.select! {|file|
      # ===================================================================== #
      # Must first check whether it is a symlink that exists.
      # ===================================================================== #
      if File.symlink?(file) and !File.exist?(File.readlink(file))
        symlink_target = File.readlink(file)
        opnn; e 'No file exists at '+sfile(file)+' (a symlink pointing '\
                'at '+sfile(symlink_target)+')'
      end
      if File.exist? file
        dataset = File.read(file)
        dataset.include?(i)
      else
        opnn; e "No file could be found at `#{sfile(file)}`."
      end
    }
    if la_files.empty?
      opnn; e "We did not find any .la file that includes "\
              "the entry `#{simp(i)}`."
    else
      la_files.each {|file|
        opnn; e "Working on `#{sfile(file)}` next."
        dataset = File.readlines(file)
        _ = ''.dup
        dataset.each {|line|
          if line.include?('dependency_libs') and line.include?(i)
            line.gsub!(/#{Regexp.quote(i)}/,'')
            @array_log_these_files << i
          end
          _ << line
        }
        # =================================================================== #
        # Write the new content into that .la file.
        # =================================================================== #
        write_what_into(_, file)
      }
      unless @array_log_these_files.empty?
        what = N+@array_log_these_files.join(N)+('-' * 40)+N
        # ===================================================================== #
        # Store the modified files in a file:
        # ===================================================================== #
        into = "#{rbt_log_directory?}libtool_modified_these_files.md"
        opnn; e 'Keeping a backup of which libtool files were '\
                'modified, at `'+sfile(into)+'`.'
        append_what_into(what, into)
      end
    end
  end

  # ========================================================================= #
  # === remove_the_last_faulty_libtool_entry_stored_in_a_file
  #
  # Invocation example:
  #   rubylibtool --remove-from-file
  # ========================================================================= #
  def remove_the_last_faulty_libtool_entry_stored_in_a_file
    _ = rbt_log_directory?+'libtool/last_faulty_libtool_file.md'
    # ======================================================================= #
    # (1) Read the entry from the .md file, if it exists
    # ======================================================================= #
    if File.exist? _
      this_is_a_faulty_libtool_entry = File.read(_).strip
      # ===================================================================== #
      # (2) pass this into remove_this_entry()
      # ===================================================================== #
      remove_this_entry(this_is_a_faulty_libtool_entry)
    else
      opnn; e 'No file exists at `'+sfile(_)+'`.'
    end
  end

end

# =========================================================================== #
# === RBT.fix_libtool_la_file_from_saved_file
# =========================================================================== #
def self.fix_libtool_la_file_from_saved_file
  RBT::Libtool.new.remove_the_last_faulty_libtool_entry_stored_in_a_file
end

end