#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CreateDebianControlFile
#
# This class can generate a control-file for use in debian.
#
# The control-file contains various values which dpkg, dselect, apt-get,
# apt-cache, aptitude, and other package management tools will use to
# manage the package, on a debian system. It is defined by the Debian
# Policy Manual, 5 "Control files and their fields".
# =========================================================================== #
# require 'rbt/linux/debian/create_control_file.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Linux

module Debian

class CreateControlFile < Base # === RBT::Linux::Debian::CreateControlFile

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  #
  # The first argument is the name of the program.
  # ========================================================================= #
  def initialize(
      name_of_the_program = Dir.pwd, # By default we will obtain information from the current working directory.
      run_already         = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @_ = ''.dup # <- This is the string that will be stored.
    @filename = 'control'
  end

  # ========================================================================= #
  # === create_the_control_file
  # ========================================================================= #
  def create_the_control_file
    opnn; e 'Now storing in the file '+sfile(@filename)+'.'
    write_what_into(@_, @filename)
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    create_the_control_file
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Linux::Debian::CreateControlFile.new(ARGV)
end # create_debian_control_file