#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ColourMakeInstall
#
# This class can be used if you just want a colourized way to invoke
# "make". We will make use of RBT::ColourizeParser for this.
#
# Do note that while this class is called ColourMakeInstall, and
# the default is to run the action "make install", you can actually
# tell the class to use another command, other than "make".
#
# Usage example:
#   RBT::ColourMakeInstall.new
# =========================================================================== #
# require 'rbt/make/colour_make_install.rb'
# RBT::ColourMakeInstall.new
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/make/make.rb'

module RBT

class ColourMakeInstall < Base # === ColourMakeInstall.new

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    if block_given?
      # ===================================================================== #
      # Merge the given block into the commandline-arguments next:
      # ===================================================================== #
      @hash_commandline_arguments.update(yield)
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # The following Hash will store commandline arguments given to this
    # class. I use this rather than yield directly because I feel an
    # instance variable is less transient than depending on block_given?
    # calls primarily.
    # ======================================================================= #
    @hash_commandline_arguments = {
      run_make_install: true
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    # ======================================================================= #
    # We will not handle anything here but instead simply delegate
    # the block towards class RBT::Make.
    # ======================================================================= #
    RBT::Make.new(@hash_commandline_arguments).run
  end

  # ========================================================================= #
  # === RBT::ColourMakeInstall[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end; self.instance_eval { alias run [] } # === ColourMakeInstall.run

end

# =========================================================================== #
# === RBT.colour_make_install
# =========================================================================== #
def self.colour_make_install(&block)
  RBT::ColourMakeInstall.new(&block)
end; self.instance_eval { alias colourized_make_install colour_make_install } # === RBT.colourized_make_install

end

if __FILE__ == $PROGRAM_NAME
  RBT::ColourMakeInstall.new(ARGV)
end # colour_make_install