#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# ============================================================================ #
# require 'rbt/requires/require_yaml.rb'
# ============================================================================ #
require 'rbt/toplevel_methods/configuration_directory.rb'
require 'yaml'

# =========================================================================== #
# We will also enable a custom variant to load yaml files.
# =========================================================================== #
module RBT

  # ========================================================================= #
  # === RBT.set_yaml_engine
  #
  # This method will assign (and thus keep track of) the yaml engine
  # that the RBT project will use. Since some time this is standard
  # psych; in the past it used to be syck.
  # ========================================================================= #
  def self.set_yaml_engine(i)
    case i # case tag
    when :psych, :default # <- psych is the default since some time.
      # ===================================================================== #
      # pass through because psych is the default yaml engine that will
      # work without modification.
      # ===================================================================== #
    when :syck
      begin
        require 'syck'
      rescue LoadError; end
      begin
        YAML::ENGINE.yamler = 'syck'
      rescue NameError; end
    end
    @yaml_engine = i
  end; self.instance_eval { alias yaml_engine= set_yaml_engine } # === RBT.yaml_engine=

  # ========================================================================== #
  # === RBT.yaml_engine?
  # ========================================================================== #
  def self.yaml_engine?
    @yaml_engine
  end

  # ========================================================================== #
  # Set to a default value upon startup of the RBT project.
  # ========================================================================== #
  _ = RBT.file_use_psych_or_syck
  if File.exist? _
    set_yaml_engine(YAML.load_file(_))
  else
    puts "No file exists at `#{_}`."
  end

  # ========================================================================== #
  # === RBT.load_yaml
  #
  # You may use this method when loading a yaml file in the RBT project.
  # ========================================================================== #
  def self.load_yaml(i)
    YAML.load_file(i)
  end

  # ========================================================================== #
  # === load_yaml
  # ========================================================================== #
  def load_yaml(i)
    ::RBT.load_yaml(i)
  end

end

if __FILE__ == $PROGRAM_NAME
  pp RBT.yaml_engine?
end