#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === FindLikelyDirectory
#
# This class will find a most likely directory if the input match
# is not 100% accurate.
# =========================================================================== #
class FindLikelyDirectory

  # ========================================================================= #
  # === FindLikelyDirectory::SOURCE_DIRECTORY
  # ========================================================================= #
  SOURCE_DIRECTORY = ENV['MY_SRC'].to_s

  # ========================================================================= #
  # === FindLikelyDirectory::TEMP_DIRECTORY
  # ========================================================================= #
  TEMP_DIRECTORY   = ENV['MY_TEMP'].to_s

  # ========================================================================= #
  # === FindLikelyDirectory::DEFAULT_CHOICE
  # ========================================================================= #
  DEFAULT_CHOICE   = "#{TEMP_DIRECTORY}/binutils-2.*"

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i = DEFAULT_CHOICE
    )
    set_find_similarities_for_this_string(i)
    find_similarities
  end

  # ========================================================================= #
  # === string?
  # ========================================================================= #
  def string?
    @find_similarities_for_this_string
  end

  # ========================================================================= #
  # === set_find_similarities_for_this_string
  # ========================================================================= #
  def set_find_similarities_for_this_string(i = DEFAULT_CHOICE)
    i = DEFAULT_CHOICE if i.nil?
    @find_similarities_for_this_string = i
  end

  # ========================================================================= #
  # === find_similarities
  #
  # This method does the searching part.
  # ========================================================================= #
  def find_similarities
    splitted = @find_similarities_for_this_string.split('-')
    _ = splitted.first # get all chars before the first - to use as glob. 
    obtain_matches(_)
  end

  # ========================================================================= #
  # === obtain_matches
  # ========================================================================= #
  def obtain_matches(path) # This sets @matches
    @matches = Dir["#{path}*"]
    @matches.select! {|entry| File.directory?(entry) } # We only care about directories.
  end

  # ========================================================================= #
  # === match
  # ========================================================================= #
  def match
    return @matches.first # right now will always return the first member.
  end

  # ========================================================================= #
  # === found_match?
  # ========================================================================= #
  def found_match? # return bool
    if @matches.empty?
      return false
    else
      return true
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  pp _ = FindLikelyDirectory.new(ARGV.first)
  puts "\n\n"+'We tried to find similarities for: '+
       "\n\n"+'  '+_.string?+"\n\n"
end # fld