#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ShowStatistics
#
# This class can be used to output anything that may be related to
# statistics, within the RBT project/namespace.
#
# Usage example:
#   RBT::ShowStatistics.new
# =========================================================================== #
# require 'rbt/statistics/show_statistics.rb'
# RBT::ShowStatistics.new
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/statistics/compile_strategies.rb'
require 'rbt/compile/time.rb'

module RBT

class ShowStatistics < Base # === RBT::ShowStatistics.new

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    i = [i].flatten.compact
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    cliner
    RBT::CompileStrategies.new
    cliner
    RBT.show_compile_time_statistics
    cliner
  end

  # ========================================================================= #
  # === RBT::ShowStatistics[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::ShowStatistics.new(ARGV)
end # show_statistics