#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::SymlinkThisProgram
#
# This class can symlink a given program into the corresponding
# GoboLinux-like hierarchy (aka /Programs/Foo).
#
# This allows us to use the appdir-compiled program in a FHS system.
#
# The class must be flexible enough to accomodate for different
# invocation styles.
#
# For instance, the following variants must all work:
#
#   cd /Programs/Htop/Current; rnsymc
#   rnsymc php 7.3.7
#   rnsymc php
#
# Usage example:
#
#   RBT::SymlinkThisProgram.new(ARGV)
#
# =========================================================================== #
# require 'rbt/symlink_this_program/initialize.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/symlink_this_program/constants.rb'
require 'rbt/symlink_this_program/menu.rb'
require 'rbt/symlink_this_program/misc.rb'
require 'rbt/symlink_this_program/reset.rb'
require 'rbt/symlink_this_program/run.rb'

module RBT

class SymlinkThisProgram < Base # === RBT::SymlinkThisProgram

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    case run_already
    when :do_not_run_yet,
         :dont_run_yet
      run_already = false
    end
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :do_not_run_yet
      # ===================================================================== #
      when :do_not_run_yet
        run_already = false
      # ===================================================================== #
      # === :do_not_register
      # ===================================================================== #
      when :do_not_register
        do_not_register_into_the_local_yaml_database
      end
    end
    run if run_already
  end

end; end