#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/symlink_this_program/symlink.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class SymlinkThisProgram < Base # === RBT::SymlinkThisProgram

  # ========================================================================= #
  # === consider_symlinking_pkgconfig_files
  #
  # This method can be used to symlink .pc files into the /usr/lib/pkgconfig
  # hierarchy.
  # ========================================================================= #
  def consider_symlinking_pkgconfig_files
    appdir_target = full_path_to_the_target_program?
    # ======================================================================= #
    # The appdir_target variable may be incorrect, such as when we symlink
    # another version.
    # ======================================================================= #
    if appdir_target.respond_to? :split
      splitted = appdir_target.split('/')
      unless splitted.last == full_path_to_the_target_program?.split('/').last
        appdir_target = full_path_to_the_target_program?
      end
    else
      opnn; e 'A minor problem - appdir_target does not respond to the method .split.'
      opnn; e 'We will still continue, though.'
    end
    if @dataset.symlink_pkgconfig_files? and
       appdir_target.include?(programs_dir?)
      require 'rbt/utility_scripts/symlink_pkgconfig_files.rb'
      opnn; e "Next, we will symlink all .pc files found "\
              "under `#{sdir(appdir_target)}`."
      SymlinkPkgconfigFiles.new(appdir_target)
    end
  end

  # ========================================================================= #
  # === symlink_these_entries
  #
  # The second argument can be a Hash or a String - at the least this is
  # what we expect. The second argument, if it is a String, will then
  # be assumed to be the target onto which we will symlink entries onto.
  # ========================================================================= #
  def symlink_these_entries(
      entries, hash = {}
    )
    if hash.is_a? Hash
      target = hash.delete(:to) if hash.has_key? :to
    end
    entries.each {|entry|
      symlink(
        entry,
        target+File.basename(entry)
      )
    }
  end

  # ========================================================================= #
  # === do_symlink_the_headers
  # ========================================================================= #
  def do_symlink_the_headers
    @shall_we_symlink_the_headers = true
  end

  # ========================================================================= #
  # === will_we_symlink?
  # ========================================================================= #
  def will_we_symlink?
    @will_we_symlink
  end; alias we_will_symlink? will_we_symlink? # === we_will_symlink?

  # ========================================================================= #
  # === do_not_symlink=
  # ========================================================================= #
  def do_not_symlink=(i)
    if i == true
      do_not_symlink
    else
      @will_we_symlink = true
    end
  end

  # ========================================================================= #
  # === do_not_symlink
  # ========================================================================= #
  def do_not_symlink
    @will_we_symlink = false
  end

  # ========================================================================= #
  # === symlink_this_program?
  # ========================================================================= #
  def symlink_this_program?
    @symlink_this_program
  end; alias program_name?        symlink_this_program? # === program_name?
       alias input?               symlink_this_program? # === input?
       alias name_of_the_program? symlink_this_program? # === name_of_the_program?
       alias program?             symlink_this_program? # === program?

end; end