#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/all_binaries.rb'
# =========================================================================== #
require 'rbt/project/project_base_and_yaml_directory.rb'
require 'rbt/requires/require_yaml.rb'

module RBT

  # ========================================================================= #
  # === FILE_REGISTERED_BINARIES
  #
  # This may be a file path such as:
  #
  #   /Programs/Ruby/2.4.1/lib/ruby/site_ruby/2.4.0/cookbooks/yaml/registered_binaries.yml
  #
  # cat $COOKBOOKS/yaml/registered_binaries.yml
  # ========================================================================= #
  FILE_REGISTERED_BINARIES =
    "#{RBT.yaml_directory?}registered_binaries.yml"

  # ========================================================================= #
  # === YAML_FILE_REGISTERED_BINARIES
  #
  # This is equivalent to:
  #   data = YAML.load_file(RBT.yaml_directory?+'registered_binaries.yml')
  # ========================================================================= #
  if File.exist? FILE_REGISTERED_BINARIES
    YAML_FILE_REGISTERED_BINARIES = YAML.load_file(FILE_REGISTERED_BINARIES)
  else
    YAML_FILE_REGISTERED_BINARIES = {} # Otherwise, initialize as empty hash.
  end; ALL_BINARIES = YAML_FILE_REGISTERED_BINARIES # === ALL_BINARIES

  # ========================================================================= #
  # === RBT.file_registered_binaries
  # ========================================================================= #
  def self.file_registered_binaries
    "#{RBT.yaml_directory?}registered_binaries.yml"
  end; self.instance_eval { alias location_registered_binaries file_registered_binaries } # === RBT.location_registered_binaries

  # ========================================================================= #
  # === RBT.all_binaries?
  #
  # Get back all registered binaries.
  #
  # This is guaranteed to return a Hash.
  # ========================================================================= #
  def self.all_binaries?
    ALL_BINARIES
  end; self.instance_eval { alias all_binaries all_binaries? } # === RBT.all_binaries

  # ========================================================================= #
  # === all_binaries?
  # ========================================================================= #
  def all_binaries?
    RBT.all_binaries?
  end

  # ========================================================================= #
  # === RBT.does_include_this_binary?
  #
  # This will return true if we have a binary with that name.
  #
  # Usage example:
  #   RBT.does_include_this_binary? 'xxd'
  #   RBT.has_this_binary? 'xzcmp'
  # ========================================================================= #
  def self.does_include_this_binary?(i)
    all_binaries?.has_key?(i)
  end; self.instance_eval { alias has_this_binary? does_include_this_binary? } # === RBT.has_this_binary?

end

if __FILE__ == $PROGRAM_NAME
  require 'opn'
  require 'colours/colours_e_autoinclude.rb'
  pp RBT.all_binaries?
  e
  e 'This was read from the file:'
  e
  e "  #{RBT.file_registered_binaries}"
  e
  is_it_included = RBT.does_include_this_binary?('xxd')
  opn; e 'Is `'+sfancy('xxd')+'` a part of the rbt project? '+
         simp(is_it_included.to_s)
  if is_it_included
    opn; e 'It belongs to the program called `'+
           sfancy(
             RBT::YAML_FILE_REGISTERED_BINARIES['xxd']
           )+'`.'
  end
  e
end # all_binaries?