#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/all_libraries.rb'
# =========================================================================== #
require 'rbt/project/project_base_and_yaml_directory.rb'
require 'rbt/requires/require_yaml.rb'

module RBT

  # ========================================================================= #
  # === FILE_REGISTERED_LIBRARIES
  #
  # This may be a file path such as:
  #
  #   /Programs/Ruby/2.4.1/lib/ruby/site_ruby/2.4.0/rbt/yaml/registered_libraries.yml
  #
  # cat $RBT/yaml/registered_libraries.yml
  # ========================================================================= #
  FILE_REGISTERED_LIBRARIES =
    "#{RBT.yaml_directory?}registered_libraries.yml"

  # ========================================================================= #
  # === YAML_FILE_REGISTERED_LIBRARIES
  #
  # This is equivalent to:
  #   data = YAML.load_file(RBT.yaml_directory?+'registered_libraries.yml')
  # ========================================================================= #
  if File.exist? FILE_REGISTERED_LIBRARIES
    YAML_FILE_REGISTERED_LIBRARIES = YAML.load_file(FILE_REGISTERED_LIBRARIES)
  else
    YAML_FILE_REGISTERED_LIBRARIES = {} # Otherwise, initialize as empty hash.
  end; ALL_LIBRARIES = YAML_FILE_REGISTERED_LIBRARIES # === ALL_LIBRARIES

  # ========================================================================= #
  # === RBT.file_registered_libraries
  # ========================================================================= #
  def self.file_registered_libraries
    FILE_REGISTERED_LIBRARIES
  end; self.instance_eval { alias location_registered_libraries file_registered_libraries } # === RBT.location_registered_libraries

  # ========================================================================= #
  # === RBT.all_libraries?
  #
  # Get back all registered libraries.
  #
  # This is guaranteed to return a Hash.
  # ========================================================================= #
  def self.all_libraries?
    ALL_LIBRARIES
  end; self.instance_eval { alias all_libraries all_libraries? } # === RBT.all_libraries

  # ========================================================================= #
  # === all_libraries?
  # ========================================================================= #
  def all_libraries?
    RBT.all_libraries?
  end

  # ========================================================================= #
  # === RBT.does_include_this_library?
  #
  # This will return true if we have a binary with that name.
  #
  # Usage example:
  #   RBT.does_include_this_binary? 'xxd'
  #   RBT.has_this_binary? 'xzcmp'
  # ========================================================================= #
  def self.does_include_this_library?(i)
    all_libraries?.has_key?(i)
  end; self.instance_eval { alias has_this_library? does_include_this_library? } # === RBT.has_this_library?

end

if __FILE__ == $PROGRAM_NAME
  pp RBT.all_libraries?
  puts 'This was read from the file: '
  puts
  puts '  '+RBT.file_registered_libraries
  puts
end # all_libraries?