#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/archive_type.rb'
# =========================================================================== #
require 'rbt/constants/archive_types.rb'

module RBT

  # ========================================================================= #
  # === RBT.is_an_archive?
  #
  # This method will return true if the given input is considered to be
  # an archive; and false otherwise.
  # ========================================================================= #
  def self.is_an_archive?(i)
    ARRAY_REGISTERED_ARCHIVES.any? {|entry|
      i.end_with? entry
    }
  end

  # ========================================================================= #
  # === RBT.return_archive_type
  #
  # This method will feedback the archive type in use, for the given
  # input at gabd.
  #
  # It is used in class RBT::UpdateEntry.
  #
  # The default will be, as you can see from the default argument,
  # .tar.xz. Note that this assumption is not always correct, e. g.
  # if you do not repackage.
  #
  # Note that the simple archives, such as .gem or .zip, could be
  # detected automatically, but entries such as .tar.gz are harder
  # detect, hence why we will use a hardcoded case menu here.
  # ========================================================================= #
  def self.return_archive_type(
      i = '.tar.xz'
    )
    if i.is_a? Array
      i = i.first
    end
    if    i.end_with? '.tar.xz'
      '.tar.xz'
    elsif i.end_with? '.xz'
      '.tar.xz'
    elsif i.end_with? '.tar.gz'
      '.tar.gz'
    elsif i.end_with? '.tar.bz2'
      '.tar.bz2'
    elsif i.end_with? '.tar.lz'
      '.tar.lz'
    elsif i.end_with? '.gem'
      '.gem'
    elsif i.end_with? '.zip'
      '.zip'
    elsif i.end_with? '.jar'
      '.jar'
    elsif i.end_with? '.phar'
      '.phar'
    else # else try to make an "educated guess" - via File.extname().
      # ===================================================================== #
      # But this can also return faulty results. For example, if the input
      # is "27.01.2019" then it does not make any sense to return ".2019" as
      # an archive type, since that is not an archive type. This is the
      # reason why specifically numbers are excluded in the following
      # check.
      # ===================================================================== #
      extname = File.extname(i)
      if extname.delete('.') =~ /^\d+$/ # If only numbers.
        extname = ''.dup
      end
      extname
    end
  end; self.instance_eval { alias archive_type?    return_archive_type } # === RBT.archive_type?
       self.instance_eval { alias archive_type_of? return_archive_type } # === RBT.archive_type_of?
       self.instance_eval { alias archive_type_of  return_archive_type } # === RBT.archive_type_of

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/colours_e_autoinclude.rb'
  if ARGV.empty?
    e 'Please provide at the least one argument.'
  else
    e 'The archive type for this given input is:'
    e
    e '  '+sfancy(RBT.return_archive_type(ARGV))
    e
  end
end # atype libdrm-2.38.tar.bz2