#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/array_all_rubygems.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/cookbook_directory.rb'

module RBT

  # ========================================================================= #
  # === RBT.array_all_rubygems
  #
  # This method will return all rubygems, as an Array.
  #
  # This can then be used in other programs, such as in RBT to
  # batch-install all rubygems.
  # ========================================================================= #
  def self.array_all_rubygems
    # ======================================================================= #
    # This is a bit hack-ish, but it is also fast. We will scan for entries
    # that hold the content 'url1:'; and also the substring '.gem' within
    # that entry. Other entries will be ignored.
    # ======================================================================= #
    array = []
    # ======================================================================= #
    # Obtain all .yml files first:
    # ======================================================================= #
    all_yaml_files = Dir["#{RBT.individual_cookbooks_directory?}*.yml"]
    all_yaml_files.each {|path_to_the_yaml_file|
      name_of_the_program = File.basename(path_to_the_yaml_file.sub(/\.yml$/,''))
      dataset_of_the_program = File.read(path_to_the_yaml_file)
      if dataset_of_the_program =~ /^ url1: .*\.gem$/
        # =================================================================== #
        # In this case we will assume that it is a ruby .gem file, so we
        # add it into the Array that will be returned.
        # =================================================================== #
        array << name_of_the_program
      end
    }
    array.sort # Keep it sorted, for easier reading.
  end

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/colours_e_autoinclude.rb'
  pp RBT.array_all_rubygems
  e
  e sfancy(RBT.array_all_rubygems.size.to_s)+' gems are available/registered.'
  e
end