#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/batch_compile.rb'
# =========================================================================== #
require 'rbt/compile/compile.rb'

module RBT

  # ========================================================================= #
  # === RBT.batch_compile
  #
  # This method can be used to batch compile programs.
  #
  # The second argument to this method controls as to whether we will
  # make use of chained programs or not.
  # ========================================================================= #
  def self.batch_compile(
      i = ARGV, try_to_use_chained_programs = true
    )
    i = [i].flatten.compact
    compile_object = RBT::Compile.new { :do_not_run_yet }
    i.each {|compile_this_program|
      if compile_this_program.is_a?(String) and compile_this_program.include?(',')
        compile_this_program = compile_this_program.split(',').map(&:strip)
      end
      # ===================================================================== #
      # Obtain the Array next that will hold our chained programs.
      # ===================================================================== #
      if RBT.chained_programs?.include? compile_this_program and
        try_to_use_chained_programs 
        compile_this_program = RBT.chained_programs?[compile_this_program]
      end
      compile_object.set_compile_these_programs(i)
      compile_object.run
      compile_object.reset
    }
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.batch_compile(ARGV)
end