#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/check_for_binary_duplicates.rb'
# =========================================================================== #
require 'rbt/colours/colours.rb'
require 'rbt/toplevel_methods/e.rb'
require 'rbt/constants/standalone_constants.rb'

module RBT

  # ========================================================================= #
  # === RBT.array_binary_duplicates
  #
  # This method will return an Array of all "/usr/bin/" binary duplicates,
  # or from wherever else you wish to get the listing of duplicates. The
  # default target is "/usr/bin/" but you could also pass in the argument
  # "/usr/bin/", as second argument to this method.
  #
  # The first argument to this method is the directory where your main
  # binaries should reside at. On GoboLinux-like systems, this will
  # default to '/System/Index/bin'.
  #
  # Invocation example for this method from within ruby code:
  #
  #   array = RBT.array_binary_duplicates
  #
  # ========================================================================= #
  def self.array_binary_duplicates(
      from_this_directory = :default,
      to                  = :default
    )
    # ======================================================================= #
    # The following two case-menus will handle the :default arguments
    # passed. The default is, on my home system, for '/System/Index/bin/'
    # as the first argument, and for '/usr/bin/' as the second argument
    # to this method here.
    # ======================================================================= #
    case from_this_directory
    when :default
      from_this_directory = SYSBIN_DIRECTORY
    end
    case to
    when :default
      to = '/usr/bin/'
    end
    array = []
    Dir["#{from_this_directory}*"].each {|this_file| # Return the Array here.
      name_of_the_program = File.basename(this_file)
      target_file = "#{to}#{name_of_the_program}"
      if File.exist? target_file
        array << target_file
      end
    }
    return array
  end

  # ========================================================================= #
  # === RBT.check_for_binary_duplicates
  #
  # This method will check for binary duplicates in the directory
  # /usr/bin/.
  # ========================================================================= #
  def self.check_for_binary_duplicates
    array_binary_duplicates.each {|target_file|
      surrounded_target_file = "`#{target_file}`"
      colourized_target_file = sfile(surrounded_target_file)
      padded_and_colourized_target_file = colourized_target_file.ljust(70)
      e "Yes, #{padded_and_colourized_target_file} is a duplicate."
    }
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.check_for_binary_duplicates
end # check_for_binary_duplicates