#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/cliner.rb'
# =========================================================================== #
module RBT

  # ========================================================================= #
  # === RBT.unicode_cliner
  #
  # This is like RBT.cliner(), but it will make use of unicode horizontal
  # building block.
  # ========================================================================= #
  def self.unicode_cliner(
      use_this_token  = '=',
      use_this_colour = nil,
      &block
    )
    yielded = :unicode
    if block_given?
      yielded = yield
    end
    RBT.cliner(use_this_token, use_this_colour) { yielded }
  end

  # ========================================================================= #
  # === RBT.cliner
  #
  # This method accepts block-parameters such as :unicode, which
  # will be "converted" into a corresponding horizontal line via
  # a certain unicode block-symbol.
  # ========================================================================= #
  def self.cliner(
      use_this_token  = '=',
      use_this_colour = nil,
      &block
    )
    case use_this_token
    when :default
      use_this_token = '='
    end
    case use_this_colour
    when :default
      use_this_colour = nil
    end
    if block_given?
      yielded = yield
      case yielded
      when :unicode_middle_horizontal_bar
        begin
          # ================================================================= #
          # Fetch the unicode block here.
          # ================================================================= #
          require 'roebe/requires/require_unicode.rb'
          use_this_token = ::Roebe.horizontal_double_bar
        rescue LoadError
        end
      when :unicode
        begin
          # ================================================================= #
          # Fetch the unicode block here.
          # ================================================================= #
          require 'roebe/requires/require_unicode.rb'
          use_this_token = ::Roebe.return_horizontal_unicode_block
        rescue LoadError
        end
      end
    end
    use_this_colour = nil
    if use_this_token.is_a? Hash
      if use_this_token.has_key? :use_colour
        use_this_colour = use_this_token[:use_colour]
        use_this_token = '=' # Restore the default in this case.
      end
    end
    case use_this_token
    when nil, :default
      use_this_token = '=' # <- Restore to the default in this case.
    end
    if use_this_colour
      if use_this_colour.to_s.start_with?('konsole_dodgerblue') and
        !use_this_colour.to_s.include?('colour_')
        # ================================================================= #
        # In this case, rewrite the colour-code a little, such as from
        # 'konsole_dodgerblue' towards 'konsole_colour_blue'.
        # ================================================================= #
        use_this_colour = 'konsole_colour_'+use_this_colour.to_s.split('_').last
      end
      puts ::Colours::AllColourMethods.send(use_this_colour, (use_this_token * 80))
    else
      if use_this_token.is_a? Symbol
        use_this_token = ::Colours::AllColourMethods.send(use_this_token, '=')
      end
      puts (use_this_token * 80)
    end 
  end
    
end

if __FILE__ == $PROGRAM_NAME
  RBT.cliner
  puts
  puts 'And now with text:'
  puts
  RBT.cliner { puts 'Hi there' }
end # rbt_cliner