#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file keeps track where the cookbook directory is. It will use
# a top-level instance variable for this.
# =========================================================================== #
# require 'rbt/toplevel_methods/cookbook_directory.rb'
# =========================================================================== #
require 'rbt/project/project_base_and_yaml_directory.rb'
require 'colours'
require 'rbt/namespace/namespace.rb'
  
module RBT

  # ========================================================================= #
  # === @cookbook_directory
  #
  # This variable will tell us where to find the individual cookbooks.
  # ========================================================================= #
  @cookbook_directory = 
    "#{RBT.project_yaml_directory?}individual_cookbooks/"

  # ========================================================================= #
  # In the past, the file had constants such as
  # INDIVIDUAL_COOKBOOKS_DIRECTORY (defined in 16.09.2014), but when we
  # transitioned into the above top-level instance variable, I felt that
  # the constant approach that was used before, is no longer as useful.
  # So I deprecated the use of constants here, then removed constants
  # such as INDIVIDUAL_COOKBOOKS_DIRECTORY, INDIVIDUAL_COOKBOOKS,
  # DIRECTORY_INDIVIDUAL_COOKBOOKS, COOKBOOKS_DIRECTORY, COOKBOOK_DIRECTORY,
  # COOKBOOKS_DIR and COOKBOOKS_BASE_DIRECTORY. In particular the latter
  # may have been defined elsewhere on top of that, which was quite
  # confusing - hence the removal was the right decision to be made.
  # ========================================================================= #

  # ========================================================================= #
  # === RBT.set_cookbook_directory
  #
  # This setter-method can be used to determine whether the individual
  # cookbooks are to be found normally.
  # ========================================================================= #
  def self.set_cookbook_directory(i)
    if i # <- Assignment is only done if the user supplies a valid argument.
      unless i.end_with? '/'
        i = i.dup if i.frozen?
        i << '/'
      end
      @cookbook_directory = i
    end
  end

  # ========================================================================= #
  # === RBT.cookbook_directory?
  #
  # Provide a pointer, through this query method, towards the directory
  # where the individual cookbooks are kept.
  # ========================================================================= #
  def self.cookbook_directory?
    @cookbook_directory
  end; self.instance_eval { alias cookbook_directory              cookbook_directory? } # === RBT.cookbook_directory
       self.instance_eval { alias cookbooks_directory?            cookbook_directory? } # === RBT.cookbooks_directory?
       self.instance_eval { alias individual?                     cookbook_directory? } # === RBT.individual?
       self.instance_eval { alias individual_cookbooks?           cookbook_directory? } # === RBT.individual_cookbooks?
       self.instance_eval { alias individual_cookbooks_dir?       cookbook_directory? } # === RBT.individual_cookbooks_dir?
       self.instance_eval { alias individual_cookbooks_directory? cookbook_directory? } # === RBT.individual_cookbooks_directory?

  # ========================================================================= #
  # === RBT.report_where_the_cookbook_directory_can_be_found
  #
  # This method will report where the cookbook directory can be found.
  # ========================================================================= #
  def self.report_where_the_cookbook_directory_can_be_found
    opn(namespace: RBT::NAMESPACE[:namespace])
    e 'The cookbook directory can be found at this location:'
    e Colours.sfancy("  #{RBT.individual_cookbooks_directory?}")
  end

  # ========================================================================= #
  # === individual_cookbooks_directory?
  # ========================================================================= #
  def individual_cookbooks_directory?
    ::RBT.cookbook_directory?
  end; alias cookbooks_directory? individual_cookbooks_directory? # === cookbooks_directory?

end

if __FILE__ == $PROGRAM_NAME
  puts RBT.cookbook_directory?
  puts RBT.cookbooks_directory?
end # rbt_cookbook_directory