#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/create_directory.rb'
# =========================================================================== #
require 'fileutils'
require 'rbt/colours/sdir.rb'
require 'rbt/toplevel_methods/opn.rb'

module RBT

  # ========================================================================= #
  # === RBT.create_directory
  #
  # Internal Wrapper to create a directory.
  #
  # We will also check if a Dir exists before actually creating it. This
  # is the method of choice for creating a directory. Use it for all the
  # RBT scripts, whenever you need to create a directory.
  # ========================================================================= #
  def self.create_directory(
      where         = '/Users/Packages/',
      use_this_mode = 0755,
      be_verbose    = :be_quiet,
      do_use_opnn   = true
    )
    case use_this_mode
    when :be_verbose
      be_verbose = :be_verbose
      use_this_mode = :default
    end
    if where.is_a? Array
      where.each {|entry|
        create_directory(
          entry, use_this_mode, be_verbose, do_use_opnn
        )
      }
    else
      if File.directory? where
        unless where.end_with? '/' # Append a /.
          where = where.dup if where.frozen?
          where << '/'
        end
      end
      case use_this_mode
      when :default # Default value is 0755.
        use_this_mode = 0755
      when Hash
        if use_this_mode.has_key? :verbosity
          use_this_mode = use_this_mode.delete(:verbosity)
        end
        use_this_mode = 0755
      when '', true
        use_this_mode = 0755
      when :be_verbose
        use_this_mode = 0755
        be_verbose = true
      end
      # ===================================================================== #
      # Handle special symbols given to be_verbose.
      # ===================================================================== #
      case be_verbose
      when Hash
        if be_verbose.has_key? :verbosity
          be_verbose = be_verbose.delete(:verbosity)
        end
      when :be_quiet, :be_silent
        be_verbose = false
      end
      case do_use_opnn
      when :do_not_use_opn,
           :do_not_use_opnn
        do_use_opnn = false
      end
      if File.exist? where
        if be_verbose
          opnn if do_use_opnn
          e "Directory `#{sdir(where)}` "\
            "already exists, thus"
          opnn if do_use_opnn
          e 'skipping creating a new one.'
        end
      else
        if where
          if be_verbose
            opnn if do_use_opnn
            e "Creating the directory `#{sdir(where)}` next."
          end
          unless File.directory? where
            # basedir = File.dirname(where)
            FileUtils.mkdir_p(where,
              {
                mode:    use_this_mode,
                verbose: false
              } )
            if be_verbose
              opnn if do_use_opnn
              e "Can not write into the directory `#{sdir(where)}` "\
                "due to insufficient permissions."
            end
          end
        else
          opnn if do_use_opnn; e 'No directory was given.'
        end
      end
    end
  end; self.instance_eval { alias mkdir create_directory } # === RBT.mkdir

end

if __FILE__ == $PROGRAM_NAME
  RBT.create_directory(ARGV, :be_verbose)
end # rbt_create_directory foo