#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/determine_appdir_prefix_from_this_input.rb'
# RBT.determine_appdir_prefix_from_this_input
# =========================================================================== #
require 'rbt/toplevel_methods/programs_directory.rb'

begin
  require 'program_information'
rescue LoadError; end

module RBT

  # ========================================================================= #
  # === RBT.determine_appdir_prefix_from_this_input
  #
  # This method will return a String such as "/Programs/Htop/1.1.1/".
  # ========================================================================= #
  def self.determine_appdir_prefix_from_this_input(
      name_of_the_program    = 'htop',
      version_of_the_program = nil
    )
    if name_of_the_program.is_a? Array
      name_of_the_program = name_of_the_program.first
    end
    name_of_the_program = 'htop' if name_of_the_program.nil?
    new_prefix = ''.dup
    # ======================================================================= #
    # Append the "/Programs/" string next:
    # ======================================================================= #
    new_prefix << programs_directory? 
    if name_of_the_program.nil?
      e 'Please supply a valid argument to this method.'
      e
      e 'A valid input-argument may be:'
      e
      e '  foobar-1.0'
      e
      raise 'A program-name (String) is required.' 
    end
    # ======================================================================= #
    # === Safeguard against improper versions
    # ======================================================================= #
    if version_of_the_program.nil? and name_of_the_program.include?('-')
      version_of_the_program = ProgramInformation.return_version(name_of_the_program)
      name_of_the_program = ProgramInformation.return_name(name_of_the_program)
    end
    new_prefix << name_of_the_program.capitalize
    new_prefix << '/'
    if version_of_the_program
      new_prefix << "#{version_of_the_program}/"
    end
    return new_prefix
  end; self.instance_eval { alias static_appdir_prefix_of? determine_appdir_prefix_from_this_input } # === RBT.static_appdir_prefix_of?

end

if __FILE__ == $PROGRAM_NAME
  # puts RBT.determine_appdir_prefix_from_this_input(ARGV)
  # puts RBT.determine_appdir_prefix_from_this_input('htop')
  # puts RBT.determine_appdir_prefix_from_this_input('htop', '2.3.4')
  puts RBT.determine_appdir_prefix_from_this_input('htop-1.1.1')
  puts RBT.determine_appdir_prefix_from_this_input('cmake-3.4.2')
end