#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/determine_host_architecture.rb'
# =========================================================================== #
module RBT

  # ========================================================================= #
  # === RBT.determine_host_architecture
  #
  # If you ever want to determine the host architecture, make use
  # of this method here. This method will return a String.
  #
  # Invocation example:
  #   RBT.determine_host_architecture # => "x86_64"
  # ========================================================================= #
  def self.determine_host_architecture
    # ======================================================================= #
    # We use uname to query the host architecture. This will thus
    # not work on Windows. An alternative would be to use:
    #   RbConfig::CONFIG['host']
    # This is now what we will use.
    # ======================================================================= #
    # this_architecture = `uname -m`.to_s.chomp
    this_architecture = RbConfig::CONFIG['host']
    if this_architecture.include? '-'
      this_architecture = this_architecture.split('-').first
    end
    return this_architecture
  end

end

if __FILE__ == $PROGRAM_NAME
  puts RBT.determine_host_architecture
end # determine_host_architecture