#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/does_this_file_exist.rb'
# =========================================================================== #
require 'rbt/constants/file_and_directory_constants.rb'
require 'rbt/toplevel_methods/configure_has_a_higher_priority_than_meson.rb'

module RBT

  # ========================================================================= #
  # === RBT.meson_build_file_exists?
  #
  # When we check for the file called "meson.build", we must also
  # keep in mind that we may use a build directory. This is the
  # reason why the following code uses a variable.
  # ========================================================================= #
  def self.meson_build_file_exists?(
      check_for_this_file        = 'meson.build',
      prepend_this_for_the_check = nil
    )
    case check_for_this_file
    when :default
      check_for_this_file = 'meson.build'
    end
    does_this_file_exist?(
      check_for_this_file, prepend_this_for_the_check
    )
  end

  # ========================================================================= #
  # === RBT.does_this_file_exist?
  #
  # This method will check whether a given file exists or whether it does
  # not.
  #
  # The second argument can be used to determine whether we shall use
  # a file at a particular position on the filesystem, e. g. like
  # '../include/' that can be passed. This would check for the file
  # at the directory below the current one, within the include/
  # subdirectory.
  # ========================================================================= #
  def self.does_this_file_exist?(
      i                          = FILE_SCONS,
      prepend_this_for_the_check = nil
    )
    if i.is_a? Array
      i = i.first
    end
    i = i.to_s unless i.is_a? String # We need a String past this point.
    if prepend_this_for_the_check
      unless prepend_this_for_the_check.is_a? String
        if prepend_this_for_the_check == true
          # ================================================================= #
          # true values are converted into '../' strings.
          # ================================================================= #
          prepend_this_for_the_check = '../'.dup
        end
      end
      i = i.dup.prepend(
        prepend_this_for_the_check.dup
      )
    end
    File.exist? i
  end

  # ========================================================================= #
  # === RBT.depends_on_cmake?
  #
  # This method will only query the original cookbook-dataset, whether
  # the project at hand depends on cmake or whether it does not.
  # ========================================================================= #
  def self.depends_on_cmake?(i = nil)
    i and i.include? 'cmake'
  end; self.instance_eval { alias program_uses_cmake? depends_on_cmake? } # === RBT.program_uses_cmake?

  # ========================================================================= #
  # === RBT.depends_on_python?
  #
  # Whether python is registered as a dependency.
  # ========================================================================= #
  def self.depends_on_python?(i = nil)
    i and i.include? 'python'
  end

  # ========================================================================= #
  # === RBT.depends_on_ninja?
  # ========================================================================= #
  def self.depends_on_ninja?(i = nil)
    i and i.include?('ninja')
  end

  # ========================================================================= #
  # === RBT.depends_on_waf?
  # ========================================================================= #
  def self.depends_on_waf?(i = nil)
    i and i.include? 'waf'
  end

  # ========================================================================= #
  # === RBT.does_this_file_exist_and_is_it_a_file?
  #
  # This method will add an additional File.file? check.
  # ========================================================================= #
  def self.does_this_file_exist_and_is_it_a_file?(i)
    does_this_file_exist?(i) && File.file?(i)
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.does_this_file_exist?(ARGV)
end