#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/ensure_that_the_required_directories_exist.rb'
# =========================================================================== #
require 'rbt/colours/sdir.rb'
require 'rbt/toplevel_methods/log_directory.rb'
require 'rbt/toplevel_methods/opn.rb' # We need opn in this file here.

module RBT

  # ========================================================================== #
  # === RBT.ensure_that_the_required_directories_exist
  #
  # This method ensures that the base-directory exists.
  #
  # This can also be invoked from the commandline, by issuing:
  #
  #   rbt --ensure-that-the-required-directories-exist
  #
  # ========================================================================== #
  def self.ensure_that_the_required_directories_exist(
      use_this_as_directory = RBT.store_into_this_directory?,
      be_verbose            = false
    )
    if block_given?
      yielded = yield
      case yielded
      when :be_verbose
        be_verbose = true
      when :be_quiet
        be_verbose = false
      end
    end
    if Dir.exist?(use_this_as_directory)
      if be_verbose
        opn; e "The directory at `#{use_this_as_directory}` "\
               "already exists."
      end
    else
      target = File.dirname(use_this_as_directory)
      if File.writable?(target)
        require 'fileutils'
        opn; e "Creating directory `#{use_this_as_directory}`."
        FileUtils.mkdir_p(use_this_as_directory)
      else
        e "Can not write into the directory "\
          "`#{use_this_as_directory}` as it is not writable. "\
          "Consider using"
        e 'another setting for temp_directory.yml perhaps.'
        e
        e 'This file can be found here:'
        e
        e "  #{file_temp_directory}"
        e
      end
    end
  end

end