#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/fast_return_file_size_of_this_program.rb'
# =========================================================================== #
require 'rbt/requires/require_yaml.rb'
require 'rbt/toplevel_methods/e.rb'
require 'rbt/configuration/configuration.rb'
require 'rbt/constants/file_and_directory_constants.rb'

module RBT

  # ========================================================================= #
  # === RBT.fast_return_file_size_of_this_program
  #
  # This method will quickly return the file size of the given input
  # program.
  #
  # Invocation example from ruby:
  #   RBT.fast_return_file_size_of_this_program(:htop) # => 213004
  # ========================================================================= #
  def self.fast_return_file_size_of_this_program(i)
    if i.is_a? Array
      i = i.join(' ').strip
    end
    i = i.to_s unless i.is_a? String
    if File.directory? RBT.base_dir_to_store_expanded_cookbooks?
      target_file = "#{RBT.base_dir_to_store_expanded_cookbooks?}#{i}.yml"
      if File.exist? target_file
        dataset = YAML.load_file(target_file)
        archive_size = dataset['archive_size']
        return archive_size
      else
        e "No file exists at `#{target_file}`"
      end
    else
      e 'No directory exists at '+
        RBT.base_dir_to_store_expanded_cookbooks?
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  puts RBT.fast_return_file_size_of_this_program(ARGV)
end # fast_return_file_size_of_this_program htop