#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/find_url_for.rb'
# =========================================================================== #
require 'rbt/cookbooks/class/class.rb'
require 'rbt/toplevel_methods/return_location_to_this_programs_yaml_file.rb'

module RBT

  # ========================================================================= #
  # === RBT.find_url_for
  #
  # Use this method to find a specific URL quickly. We will return 
  # the first URL always, also called url1. The method will also
  # try to return url2 - and hence return an Array.
  #
  # Usage example:
  #   RBT.find_url_for()
  #   RBT.find_url_for(:htop) # => ["http://hisham.hm/htop/releases/2.2.0/htop-2.2.0.tar.gz", "http://hisham.hm/htop/index.php?page=downloads"]
  #   RBT.url_for? :patch # => ["https://ftp.gnu.org/pub/gnu/patch/patch-2.7.6.tar.xz", "http://ftp.gnu.org/gnu/patch/?C=M;O=D"]
  # ========================================================================= #
  def self.find_url_for(
      i = ARGV
    )
    i = i.first if i.is_a? Array
    i = i.to_s
    # ======================================================================= #
    # Next, we will try to use the expanded dataset, if it exists. This
    # will be MUCH faster than instantiating a new ruby object, as is
    # done in the else clause.
    # ======================================================================= #
    expanded_file = RBT.return_location_to_this_programs_yaml_file(i)
    if File.exist? expanded_file
      if i.end_with? '.yml'
        i = i.dup if i.frozen?
        i.sub!(/\.yml$/,'')
      end
      dataset = YAML.load_file(expanded_file)
      if dataset.has_key?(i)
        dataset = dataset[i]
      end
      url1 = dataset['url1'] # Is a String.
      url2 = dataset['url2'] # Is a String.
    else
      _ = RBT::Cookbooks::Cookbook.new(i) { :skip_menu_interface }
      url1 = _.url1? # Return the URL here.
      url2 = _.url2?
    end
    url1 = url1.to_s.strip
    url2 = url2.to_s.strip
    return [ url1, url2 ]
  end; self.instance_eval { alias url1         find_url_for } # === RBT.url1
       self.instance_eval { alias location_of? find_url_for } # === RBT.location_of?
       self.instance_eval { alias url_for?     find_url_for } # === RBT.url_for?

end

if __FILE__ == $PROGRAM_NAME
  RBT.find_url_for(ARGV).each {|remote_url|
    puts "  #{remote_url}"
  }
end # rbt_find_url_for htop