#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/generate_programs_version_yaml_file.rb'
# =========================================================================== #
require 'rbt/project/project_base_and_yaml_directory.rb'
require 'rbt/cookbooks/class/class.rb'
require 'rbt/colours/colours.rb'
require 'rbt/version/version.rb'
require 'rbt/toplevel_methods/available_programs.rb'
require 'rbt/toplevel_methods/roebe.rb'
require 'rbt/toplevel_methods/save_file.rb'

module RBT

  # ========================================================================= #
  # === RBT.generate_programs_version_yaml_file
  #
  # This method will create one big yaml file that is a Hash. This big
  # Hash will have the name of all programs and the corresponding
  # version of said programs.
  #
  # The key will be the name of the program and the value will
  # be the respective program version.
  #
  # You can also pass in your own dataset. If you do so, remember to
  # use a hash, with specific key->value pairs that reflect the
  # name of the program and the specific version of said program.
  #
  # You can invoke this method from the commandline, by issuing:
  #
  #   cookbook --generate_version_file
  #
  # ========================================================================= #
  def self.generate_programs_version_yaml_file(
      use_this_dataset = nil
    )
    _ = ''.dup # We will append onto this String.
    unless use_this_dataset # This is the default behaviour/clause.
      hash = {} # We store into this hash here.
      all_programs = RBT.available_programs
      all_programs.each {|this_program|
        opn; e "Now working on the program "\
               "`#{Colours.lightgray(this_program)}`."
        cookbook = RBT::Cookbooks::Cookbook.new(this_program) { :bypass_menu }
        # =================================================================== #
        # Add onto the Hash next. Should keep it as a String.
        # =================================================================== #
        hash[this_program] = cookbook.version?.to_s
      }
      use_this_dataset = hash
    end
    use_this_dataset.each_pair {|key, value|
      value = '"'+value.to_s+'"' unless value.include? '"'
      _ << key.to_s+': '+value.to_s+N
    }
    # ======================================================================= #
    # Ok, now we should have all programs and the respective version
    # at hand - we can store it next. In the past, this made use of
    # YAML.dump() but for some strange reason, YAML.dump() gave strange
    # results; which is why I decided to generate the .yml file
    # on my own.
    # ======================================================================= #
    what = _
    into = RBT.yaml_directory?+'programs_version.yml'
    opn; e "Will also store into `#{sfile(into)}`."
    RBT.write_what_into(what, into) # Store into the file programs_version.yml
    # ======================================================================= #
    # Store for my home-system as well.
    # ======================================================================= #
    if is_roebe? # On my system.
      into = RUBY_SRC_DIR_AT_HOME+'rbt/lib/rbt/yaml/programs_version.yml'
      opn; e "Will also store into `#{sfile(into)}`."
      RBT.write_what_into(what, into)
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.generate_programs_version_yaml_file
end # generate_programs_version_yaml_file