#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/gobolinux.rb'
# =========================================================================== #
require 'rbt/constants/file_and_directory_constants.rb'

module RBT

  # ========================================================================= #
  # === RBT.is_this_a_real_gobolinux_system?
  #
  # This method queries whether we are on a real GoboLinux system. In order
  # to determine this, we will ONLY check on local file-paths and
  # possibly directory-paths,
  # ========================================================================= #
  def self.is_this_a_real_gobolinux_system?
    File.exist?('/System/Aliens') and 
    File.exist?('/System/Kernel/Boot/System.map') and
    File.symlink?('/System/Kernel/Boot/System.map')
  end

  # ========================================================================= #
  # === RBT.are_we_on_gobolinux?
  #
  # This method can help us when we wish to determine whether we are on
  # GoboLinux or whether we are not.
  #
  # Note that this will return a "static" result. If you need a value
  # that is dynamic, you ought to query an instance variable instead.
  # ========================================================================= #
  def self.are_we_on_gobolinux?
    return_this = false
    use_this_env_key = 'IS_ON_GOBOLINUX'
    if ENV.has_key? use_this_env_key
      if ENV[use_this_env_key].to_s == '1'
        return_this = true
      end
    elsif File.exist? FILE_IS_ON_GOBOLINUX
      return_this = true
    elsif GOBOLINUX_LAYOUT.to_s == '1' # The constant GOBOLINUX_LAYOUT is a String.
      return_this = true
    elsif File.exist?(RBT.file_is_on_gobolinux) and
          (File.read(RBT.file_is_on_gobolinux).strip.to_s == '1')
      return_this = true
    end
    return return_this
  end; self.instance_eval { alias is_on_gobolinux? are_we_on_gobolinux? } # === RBT.is_on_gobolinux?

end

if __FILE__ == $PROGRAM_NAME
  puts RBT.are_we_on_gobolinux?
end # are_we_on_gobolinux?