#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/install_which_binaries.rb'
# =========================================================================== #
require 'rbt/project/project_base_and_yaml_directory.rb'
require 'rbt/constants/standalone_constants.rb'
require 'rbt/toplevel_methods/roebe.rb'

module RBT

  # ========================================================================= #
  # === RBT.will_install_which_binaries?
  #
  # This method will simply return an Array of binaries that belong to
  # the rbt project.
  # ========================================================================= #
  def self.will_install_which_binaries?
    results = []
    if is_on_roebe?
      results = Dir["#{RUBY_SRC_DIR_AT_HOME}rbt/bin/*"]
    else # Else we handle the case for regular users.
         # This depends on information stored in the file called
         # rbt_installs_these_executables.yml, which has to be
         # upgraded every now and then.
      results = YAML.load_file(
        "#{RBT.project_yaml_directory?}rbt_installs_these_executables.yml"
      )
    end
    # ======================================================================= #
    # Use only the basename of any results:
    # ======================================================================= #
    results.map! {|entry|
      File.basename(entry)
    }
    results.sort # <- Keep the entries alphabetically sorted.
  end

end

if __FILE__ == $PROGRAM_NAME
  pp RBT.will_install_which_binaries?
end