#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/log_directory.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/convert_global_env.rb'
require 'rbt/toplevel_methods/temp_directory.rb'

module RBT

  # ========================================================================= #
  # === @log_directory
  # ========================================================================= #
  @log_directory = "#{@temp_directory}rbt/"

  # ========================================================================= #
  # === RBT.log_directory?
  #
  # Query method for the @log_directory variable. Prefer to use this method
  # or any of its aliases rather than the constant called LOG_DIR.
  #
  # On my home system this defaults to "/Depot/Temp/rbt/".
  # ========================================================================= #
  def self.log_directory?
    @log_directory
  end; self.instance_eval { alias log_directory              log_directory? } # === RBT.log_directory
       self.instance_eval { alias rbt_temp_directory?        log_directory? } # === RBT.rbt_temp_directory?
       self.instance_eval { alias rbt_log_directory?         log_directory? } # === RBT.rbt_log_directory?
       self.instance_eval { alias rbt_log_dir?               log_directory? } # === RBT.rbt_log_dir?
       self.instance_eval { alias log_dir?                   log_directory? } # === RBT.log_dir?
       self.instance_eval { alias log_dir                    log_directory? } # === RBT.log_dir
       self.instance_eval { alias store_into_this_directory? log_directory? } # === RBT.store_into_this_directory?
       self.instance_eval { alias store_into_this_directory  log_directory? } # === RBT.store_into_this_directory
       self.instance_eval { alias store_where?               log_directory? } # === RBT.store_where?

  # ========================================================================= #
  # === RBT.set_log_directory
  #
  # This defaults to "/Depot/Temp/rbt/" on my system.
  # ========================================================================= #
  def self.set_log_directory(
      i = "#{RBT.temp_directory}rbt/"
    )
    i = i.dup if i.frozen?
    i = convert_global_env(i) if i.include? '$'
    i << '/' unless i.end_with? '/'
    # ======================================================================= #
    # Next, we must ensure that the required directories exist.
    # ======================================================================= #
    i = i.dup if i.frozen? # <- Need to do so again, e. g. in a .cgi environment context.
    # ======================================================================= #
    # In the past we did ensure that the required directories exist,
    # whenever RBT.set_log_directory() is called. But this does not
    # make that much sense, and can lead to other problems, so it
    # was disabled here at 24.01.2019. Do note that the functionality
    # of ensuring that the required directories exist will still be
    # handled by RBT, but in other code parts instead.
    # ======================================================================= #
    # require 'rbt/toplevel_methods/ensure_that_the_required_directories_exist.rb'
    # ^^^ this used to be on top
    # ======================================================================= #
    # ensure_that_the_required_directories_exist(
    #   i
    # )
    @log_directory = i
  end

  RBT.set_log_directory # Set it up once.

  # ========================================================================= #
  # === RBT.store_the_last_configuration_options_in_this_file
  #
  # You can also use this alias-method for invoking the method:
  #
  #   RBT.store_the_last_configuration_options_into_which_file? # => "/Depot/Temp/rbt/last_configure_options_used.md"
  #
  # ========================================================================= #
  def self.store_the_last_configuration_options_in_this_file
    "#{RBT.store_into_this_directory?}"\
    "last_configure_options_used_through_class_show_configuration_options.md"
  end; self.instance_eval { alias store_the_last_configuration_options_into_which_file? store_the_last_configuration_options_in_this_file } # === RBT.store_the_last_configuration_options_into_which_file?
       self.instance_eval { alias file_last_configure_options                           store_the_last_configuration_options_in_this_file } # === RBT.file_last_configure_options

end

if __FILE__ == $PROGRAM_NAME
  puts RBT.store_the_last_configuration_options_into_which_file?  
end