#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Code in this file is for toplevel-functionality related to the meson
# build system (and ninja).
#
# In particular it was necessary to quickly compile a certain program,
# such as atk, in an appdir-like approach, so the code elements here in
# this file were started in May 2019.
# =========================================================================== #
# require 'rbt/toplevel_methods/meson.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/esystem.rb'
require 'rbt/toplevel_methods/change_directory.rb'
require 'rbt/toplevel_methods/programs_directory.rb'
require 'rbt/toplevel_methods/return_pwd.rb'
require 'rbt/colours/sfancy.rb'

module RBT

  # ========================================================================= #
  # === RBT.meson_appdir_prefix
  # ========================================================================= #
  def self.meson_appdir_prefix(
      i = :use_cwd_as_input
    )
    case i
    when :use_cwd_as_input
      i = return_pwd
    end
    i = File.basename(i)
    require 'program_information'
    this_program_name    = ProgramInformation.return_name(i).delete('-_')
    this_program_version = ProgramInformation.return_version(i)
    use_this_build_directory = 'BUILD_DIRECTORY_FOR_MESON'
    use_this_prefix = programs_dir?+
                      this_program_name.capitalize+
                      '/'+
                      this_program_version+
                      '/'
    esystem 'meson --prefix='+use_this_prefix+' '+use_this_build_directory
    cd use_this_build_directory
    esystem 'ninja'
    esystem 'ninja install'
    e "If everything went fine then the program "\
      "#{sfancy(this_program_name)} has been compiled"
    e "successfully into the prefix `#{sfancy(use_this_prefix)}`."
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.meson_appdir_prefix
end # meson_appdir_prefix