#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/misc.rb'
# =========================================================================== #
require 'rbt/constants/standalone_constants.rb'
require 'rbt/toplevel_methods/save_file.rb'
require 'rbt/toplevel_methods/e.rb'
require 'rbt/colours/colours.rb'

module RBT

  # ========================================================================= #
  # === RBT.host_cpu?
  # ========================================================================= #
  def self.host_cpu?
    HOST_CPU
  end

  # ========================================================================= #
  # === RBT.return_remote_url_from_this_input
  #
  # Give the below input, this method should return a String like:
  #
  #   http://ftp.gnu.org/gnu/glibc/glibc-2.28.tar.xz
  #
  # ========================================================================= #
  def self.return_remote_url_from_this_input(
      remote_url = 'http://ftp.gnu.org/gnu/glibc/glibc-2.29.tar.xz',
      version    = '2.28'
    )
    require 'program_information'
    program_version = ProgramInformation.return_version(remote_url)
    result = remote_url.sub(
      program_version, version
    )
    result
  end

  # ========================================================================= #
  # === RBT.symlink_appdir_libraries_into_usr_lib_directory
  #  
  # The following method is very specialized - it will symlink all
  # entries under lib/* into the /usr/lib/ hierarchy.
  #
  # This will be invoked as part of the postinstallation-steps, but
  # only if we make use of an AppDir layout, and only if we are
  # on a roebe-system (aka my home directory normally).
  # ========================================================================= #
  def self.symlink_appdir_libraries_into_usr_lib_directory(
      from_this_appdir_library_target
    )
    e "Now obtaining all entries from the directory `"\
      "#{sdir(from_this_appdir_library_target)}`:"
    entries = Dir["#{from_this_appdir_library_target}*"]
    entries.each {|this_library|
      symlink(
        this_library,
        "/usr/lib/#{File.basename(this_library)}"
      )
    }
  end

  # ========================================================================= #
  # === RBT.fix_chown_root_entries_in_this_file
  #
  # This method can be used to replace all "chown root:root" entries
  # with "chown 0:0" entries. It may be necessary for when we have
  # setup a chroot.
  # ========================================================================= #
  def self.fix_chown_root_entries_in_this_file(
      this_file = 'Makefile'
    )
    what = ''.dup
    if File.exist? this_file
      File.readlines(this_file).each {|this_entry|
        if this_entry.include? 'chown root:root'
          this_entry.gsub!(/root:root/, '0:0')
        end
        what << this_entry
      }
      e "Now modifying the file `#{sfile(this_file)}`."
      write_what_into(what, this_file)
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.fix_chown_root_entries_in_this_file
end # fix_chown_root_entries_in_this_file