#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/programs_directory.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/convert_global_env.rb'

module RBT

  # ========================================================================= #
  # === @programs_directory
  # ========================================================================= #
  @programs_directory = nil

  # ========================================================================= #
  # === RBT.set_programs_directory
  # ========================================================================= #
  def self.set_programs_directory(
      i = '/Programs/' # This is hardcoded default, but can be changed by the user through later calls.
    )
    i = i.dup if i.frozen?
    # ======================================================================= #
    # Replace all $ with proper env-variables if they are included.
    # ======================================================================= #
    i = convert_global_env(i) if i.include? '$'
    # ======================================================================= #
    # If there is still at the least one '$' entry then this variable does
    # not exist. In this case, use a hardcoded entry instead. It is up to
    # the user to resolve this problem.
    # ======================================================================= #
    i = '/Programs/' if i.include? '$' # <- See above for the explanation.
    i = i.dup if i.frozen? # <- Need to do so again, e. g. in a .cgi environment context.
    i << '/' unless i.end_with? '/'
    i.squeeze!('/')
    @programs_directory = i
  end

  # ========================================================================= #
  # Also set it at once, so that we have a proper start value.
  # ========================================================================= #
  RBT.set_programs_directory

  # ========================================================================= #
  # === RBT.programs_directory?
  #
  # Query the value of the toplevel @programs_directory instance variable.
  # ========================================================================= #
  def self.programs_directory?
    @programs_directory
  end; self.instance_eval { alias programs_dir?             programs_directory? } # === RBT.programs_dir?
       self.instance_eval { alias programs_dir              programs_directory? } # === RBT.programs_dir
       self.instance_eval { alias programs_directory?       programs_directory? } # === RBT.programs_directory?
       self.instance_eval { alias programs_directory        programs_directory? } # === RBT.programs_directory
       self.instance_eval { alias program_directory         programs_directory? } # === RBT.program_directory
       self.instance_eval { alias return_programs_directory programs_directory? } # === RBT.return_programs_directory
       self.instance_eval { alias program_directory?        programs_directory? } # === RBT.program_directory?
       self.instance_eval { alias program_dir?              programs_directory? } # === RBT.program_dir?
       self.instance_eval { alias program_dir               programs_directory? } # === RBT.program_dir

end