#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/remove_file.rb'
# =========================================================================== #
require 'rbt/colours/sfile.rb'

module RBT

  # ========================================================================= #
  # === RBT.remove_file
  #
  # Use this method to remove a file, but check if the file exists
  # before actually deleting it. Additionally, also check for proper
  # file permissions permissions as well.
  # ========================================================================= #
  def self.remove_file(
      file,
      report_action = false
    )
    file = file.dup if file.frozen?
    file.squeeze!('/') # We do not want double '/' entries.
    case report_action
    when :be_quiet
      report_action = false
    when :be_verbose
      report_action = true
    end
    can_we_delete_this_entry = false
    can_we_delete_this_entry = true if File.exist? file
    begin
      if File.ftype(file) == 'link'
        can_we_delete_this_entry = true
      end
    rescue Errno::ENOENT
      can_we_delete_this_entry = true # Failsafe.
    end
    # ======================================================================= #
    # We also have to handle symlinks, aka 'link', entries. These may no
    # longer exist, but we still will to delete such stray links, hence
    # why the above check has to be the way it is.
    # ======================================================================= #
    if can_we_delete_this_entry
      # ===================================================================== #
      # We need to check whether we can write to the file at hand.
      # ===================================================================== #
      base_directory = File.dirname(file)
      if File.writable? base_directory
        unless File.directory? file # Safeguard.
          if report_action
            e "Now removing the file `#{sfile(file)}`."
          end
          if File.exist? file
            File.delete(file)
          else
            if report_action
              e "No file exists at `#{sfile(file)}`."
            end
          end
        end
      else
        opnn(__FILE__)
        ewarn "Insufficient permission to remove file #{sfile(file)}"
      end
    else # Not sure if we should report this.
      if report_action
        opnn(__FILE__)
        ewarn "Trying to remove a file, but the file #{sfile(where)}"
        ewarn ' does not exist.'
      end    
    end
  end

end