#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/report_total_size_of_all_archives.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/available_programs.rb'
require 'rbt/configuration/configuration.rb'
require 'rbt/constants/file_and_directory_constants.rb'
require 'rbt/colours/colours.rb'

module RBT

  # ========================================================================= #
  # === RBT.report_total_size_of_all_archives
  #
  # This method can be used to report the total size of all locally
  # available archives.
  #
  # Invocation example from the commandline:
  #   rbt --report_total_size_of_all_archives
  # ========================================================================= #
  def self.report_total_size_of_all_archives
    total_size = 0
    available_programs = available_programs?
    available_programs.each {|this_program|
      possible_target = "#{RBT.directory_expanded_cookbooks?}#{this_program}.yml"
      if File.exist? possible_target
        dataset = YAML.load_file(possible_target)
        program_path = dataset['program_path']
      else
        # =================================================================== #
        # The expanded-cookbooks file does not exist, so use
        # Cookbooks::Cookbook instead.
        # =================================================================== #
        require 'rbt/cookbooks/class/class.rb'
        program_path = RBT::Cookbooks::Cookbook.new(this_program) { :bypass_menu_check }.program_path?
      end
      if File.exist? program_path
        total_size += File.size(program_path)
      end
    }
    n_kilobytes = total_size  / 1024.0
    n_megabytes = n_kilobytes / 1024.0
    n_gigabytes = n_megabytes / 1024.0
    # ======================================================================= #
    # Finally, report it here.
    # ======================================================================= #
    e "The total file size of all #{sfancy(available_programs.size.to_s)}"\
      " programs is #{total_size.to_s} bytes."
    e "  (#{n_kilobytes.round(2)} KB; "+
      n_megabytes.round(2).to_s+' MB; '+
      crimson(n_gigabytes.round(2).to_s+' GB')+')'
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.report_total_size_of_all_archives
end # report_total_size_of_all_archives