#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/return_array_of_programs_without_a_blfs_entry.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/available_programs.rb'

module RBT

  # ========================================================================= #
  # === RBT.return_array_of_programs_without_a_blfs_entry
  #
  # This method will return an Array that keeps track of all local
  # cookbook files without a BLSF entry.
  #
  # This functionality can then be used to output this Array on the
  # commandline
  # ========================================================================= #
  def self.return_array_of_programs_without_a_blfs_entry
    array_of_programs_without_a_blfs_entry = []
    # ======================================================================= #
    # First, obtain all available programs.
    # ======================================================================= #
    all_available_programs = RBT.available_programs?
    all_available_programs.each {|this_program|
      location = individual_cookbooks_directory?+this_program+'.yml'
      dataset = File.read(location)
      unless dataset =~ /^ blfs:/
        array_of_programs_without_a_blfs_entry << this_program
      end
    }
    return array_of_programs_without_a_blfs_entry
  end

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/colours_e_autoinclude.rb'
  RBT.return_array_of_programs_without_a_blfs_entry.each_with_index {|a,b| b += 1
    e '  '+sfancy(b.to_s)+') '+simp(a.to_s)
  }
  e
  e '^^^ These are the entries '+yellow('without')+' a BLFS entry.'
  e
end # return_array_of_programs_without_a_blfs_entry