#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# The code in this file, depends on a file in the Cookbooks project.
#
# Since, in principle, the RBT script can be used without the Cookbooks
# project as well, we need to protect the require line.
# =========================================================================== #
# require 'rbt/toplevel_methods/return_blfs_page_of_this_program.rb'
# =========================================================================== #
require 'rbt/utility_scripts/blfs.rb'

module RBT

  # ========================================================================= #
  # === RBT.return_blfs_page_of_this_program
  #
  # This method will either return a String or nil.
  #
  # If a String is returned then we will assume that this is to the
  # remote BLFS page entry of a (locally registered) program.
  #
  # This method will return the BLfS page of a program, or nil if it
  # was not registered.
  #
  # Returns:
  #   - either a string
  #   - or nil
  #
  # Invocation example:
  #   result = RBT.return_blfs_page_of_this_program('gcc')
  # ========================================================================= #
  def self.return_blfs_page_of_this_program(
      this_program = 'gcc'
    )
    this_program = this_program.first if this_program.is_a? Array
    _ = RBT::Blfs.new(this_program) { :be_quiet_and_do_not_tamper_with_the_xorg_buffer }
    result = _.remote_url?
    result = result.first if result.is_a? Array
    return result
  end; self.instance_eval { alias return_blfs_entry_for return_blfs_page_of_this_program } # === RBT.return_blfs_entry_for

end

if __FILE__ == $PROGRAM_NAME
  p RBT.return_blfs_page_of_this_program('gcc')
  p RBT.return_blfs_page_of_this_program('libx11')
  unless ARGV.empty?
    p RBT.return_blfs_page_of_this_program(ARGV)
  end
end # return_blfs_page_of_this_program htop