#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/return_programs_from_this_url.rb'
# =========================================================================== #
require 'open-uri'
require 'program_information'

module RBT

  # ========================================================================= #
  # === RBT.return_programs_from_this_url
  #
  # The task of this method is to return an Array of programs that can
  # be found on a remote URL. If we take the example below for KDE's plasma
  # related code, then the Array that will be returned will include the
  # names of all the programs that can be found at that particular remote
  # URL.
  #
  # This then allows us to do various further checks, such as whether the
  # RBT project is tracking all these programs or not, and so forth.
  #
  # The first input argument to this method is assumed to be a remote URL,
  # as a String - so only pass an URL to the method here.
  #
  # Specific invocation example in ruby:
  #
  #   pp RBT.return_program_from_this_url('https://download.kde.org/stable/plasma/5.10.5/'); ''
  #
  # ========================================================================= #
  def self.return_programs_from_this_url(i)
    begin
      require 'program_information' unless Object.const_defined? :ProgramInformation
    rescue LoadError; end
    i = i.first if i.is_a? Array
    regex_to_use = /<tr><td valign="top">&nbsp;<\/td><td><a href="(.+?)">/ # See: http://rubular.com/r/fIcvzXfPi1
    dataset = open(i).read
    scanned = dataset.scan(regex_to_use).flatten
    scanned.reject! {|entry| entry.end_with? '.sig' or entry.start_with? '/' }
    scanned.map! {|entry|
      entry.sub!(/\.xz$/,'')
      entry.sub!(/\.tar$/,'')
      program_name = ProgramInformation.return_program_name(entry)
      program_name
    }
    scanned.uniq!
    return scanned.sort
  end; self.instance_eval { alias return_program_from_this_url return_programs_from_this_url } # === RBT.return_program_from_this_url

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/colours_e_autoinclude.rb'
  if ARGV.empty?
    array = RBT.return_programs_from_this_url(
      'https://download.kde.org/stable/plasma/5.10.5/'
    )
    # ======================================================================= #
    # The next line can be used if you want to batch-open the files locally
    # or batch-compile something.
    # ======================================================================= #
    # array.each {|entry| e 'rbt '+entry+' trad; sleep 2' }
    pp array
    # pp array.size
  else
    pp RBT.return_programs_from_this_url(ARGV); ''
  end  
end # rbt_return_programs_from_this_url https://download.kde.org/stable/plasma/5.10.5/