#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/run_makepkg_command.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/esystem.rb'
require 'rbt/toplevel_methods/determine_host_architecture.rb'

module RBT

  # ========================================================================= #
  # === RBT.run_makepkg_command
  #
  # This method will invoke the slackware "makepkg" command, in order
  # to create a slackware package.
  #
  # By default, this method will work from the current working directory,
  # so it is recommended to have already extracted the target archive
  # in question.
  # ========================================================================= #
  def self.run_makepkg_command(
      i                        = :from_the_current_working_directory,
      optional_program_version = nil,
      use_this_architecture    = RBT.determine_host_architecture # <= "x86_64"
    )
    # ======================================================================= #
    # Determine which makepkg command variant to use next.
    # ======================================================================= #
    use_this_makepkg_command = 'makepkg --chown n --linkadd y'
    case i
    when :from_the_current_working_directory,
         :default # This is the default.
      _ = File.basename(Dir.pwd)
      name_of_the_package, the_program_version = _.split('-')
    else # else depend on TWO input arguments: (a) name and (b) version.
      name_of_the_package = i
      the_program_version = optional_program_version
    end
    # ======================================================================= #
    # === Host Architecture
    #
    # The host architecture is currently "x86_64". This may change at
    # a later time, though; at the least it may be more flexible.
    # ======================================================================= #
    full_name_of_the_package = "#{name_of_the_package}-"\
                               "#{the_program_version}-"\
                               "#{use_this_architecture}"\
                               "-1.tgz"
    _ = use_this_makepkg_command+' ../'+full_name_of_the_package
    esystem(_)
    path_to_the_package = File.expand_path('../'+full_name_of_the_package)
    return path_to_the_package
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.run_makepkg_command
end