#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::SaveFile
#
# This file provides file-saving capabilities. This is already possible
# within ruby itself via File.open but I wanted to use a simpler API
# to memorize.
# =========================================================================== #
# require 'rbt/toplevel_methods/save_file.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/e.rb'
require 'rbt/toplevel_methods/create_directory.rb'

module RBT

module SaveFile

  # ========================================================================= #
  # === RBT::SaveFile.write_what_into
  #
  # This method can be used to write content into a file, via
  # ruby's File.open() functionality.
  #
  # The official documentation can be found here:
  #   https://ruby-doc.org/core/File.html#method-c-open
  # ========================================================================= #
  def self.write_what_into(
      what,
      into,
      use_these_permissions = 0755
    )
    case use_these_permissions
    when :default
      use_these_permissions = 0755
    end
    base_dir = File.dirname(into)
    RBT.mkdir(base_dir) unless File.exist? base_dir
    File.open(into, 'w+', use_these_permissions.to_i) { |file|
      file.write(what)
    }
    # RBT.e "No file exists at #{into}."
  end

  # ========================================================================= #
  # === RBT::SaveFile.append_what_into
  # ========================================================================= #
  def self.append_what_into(what, into)
    File.open(into, 'a') { |file| file.write(what) }
  end

end

  # ========================================================================= #
  # === RBT.write_what_into
  # ========================================================================= #
  def self.write_what_into(what, into, use_these_permissions = :default)
    RBT::SaveFile.write_what_into(what, into, use_these_permissions)
  end

  # ========================================================================= #
  # === RBT.append_what_into
  # ========================================================================= #
  def self.append_what_into(what, into)
    RBT::SaveFile.append_what_into(what, into)
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT::SaveFile.write_what_into('Hello world!', '/Depot/Temp/test.md')
  RBT::SaveFile.write_what_into('test', '/here.md')
  RBT::SaveFile.append_what_into('Hello world!', '/Depot/Temp/test.md')
end