#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/split_at.rb'
# =========================================================================== #
module RBT

  # ========================================================================= #
  # === RBT::N_CHARACTERS_PER_LINE
  #
  # Default value to be used for word-wrap splitting.
  # ========================================================================= #
  N_CHARACTERS_PER_LINE = 76

  # ========================================================================= #
  # === RBT.wrap_at                 (Begrenzung tag, limit tag, boundary tag)
  #
  # Wraps at a specified position.
  #
  # Specific examples:
  #   puts "abc abc abdc abc abc abc abc abc abc abc abcdef abc abc abc abc abc abc abc abc abc ".wrap_at(20)
  #   puts RBT.wrap_at(("x"*200), 30)
  #   ("x"*200).wrap_at(30,true,5)
  # ========================================================================= #  
  def self.wrap_at(
      this_string,
      my_limit    = N_CHARACTERS_PER_LINE,
      add_newline = true,
      add_padding = nil
    )
    case my_limit
    when :default
      my_limit = N_CHARACTERS_PER_LINE
    end
    begin
      add_padding = add_padding.to_i unless add_padding.nil?
      if add_newline
        _ = "\\1\\3\n" # newline at end
        unless add_padding.nil?
          _ = _.linsert(' ' * add_padding)
        end
        return this_string.gsub(/(.{1,#{my_limit}})( +|$)\n?|(.{#{my_limit}})/,_)
      else
        _ = "\\1\\3"
        unless add_padding.nil?
          _ = _.linsert(' ' * add_padding)
        end
        return this_string.gsub(
          /(.{1,#{my_limit}})( +|$)\n?|(.{#{my_limit}})/,
          _
        ) # No newline at the end.
      end
    rescue Exception => error
      puts error
    end
  end; self.instance_eval { alias split_at wrap_at } # === RBT.split_at

end

if __FILE__ == $PROGRAM_NAME
  puts RBT.wrap_at(("x"*200), 30)
end