#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/symlink.rb'
# =========================================================================== #
begin
  require 'symlink' # We need class Symlink.
rescue LoadError; end
require 'rbt/colours/use_colours.rb'
require 'rbt/toplevel_methods/do_show_the_classname.rb'
require 'rbt/toplevel_methods/roebe.rb'

module RBT

  # ========================================================================= #
  # === RBT.symlink                                             (symlink tag)
  #
  # Wrapper to symlink something, by delegating towards class Symlink.
  #
  # This could also be adapted to work on Windows.
  #
  # Please use this and only this method consistently when wanting to
  # create a symlink from within the RBT project.
  #
  # Here we have to be careful with the arguments, because File.symlink
  # will fail if the file already exists. Right now we default to
  # the behaviour that we will ALWAYS remove any file that exists
  # at `new_location`.
  # ========================================================================= #
  def self.symlink(
      existing,
      new_location = nil,
      be_verbose   = true,
      use_colours  = RBT.use_colours? # Here we need colours/colours.rb
    )
    case be_verbose
    when :be_quiet, :be_silent
      be_verbose = false
    when :default
      be_verbose = true
    end
    if existing.is_a? Hash
      if existing.has_key? :to
        new_location = existing.delete(:to)
      end
      if existing.has_key? :from
        existing = existing.delete(:from)
      end
    end
    # ======================================================================= #
    # === Delegate towards class Symlink next, after a check
    # ======================================================================= #
    unless Object.const_defined? :Symlink
      if is_on_roebe?
        e '(from rbt/toplevel_methods/symlink.rb): class Symlink '\
          'is not available - consider installing it.'
      end
    end
    _ = Symlink.new(
      existing,
      new_location,
      be_verbose,
      :do_not_run_yet
    )
    unless _.namespace?.start_with? 'RBT->'
      _.prepend_this_to_namespace 'RBT->'
    end
    _.shall_we_use_colours?(use_colours)
    # ======================================================================= #
    # For understanding the next line look at USE_ABSOLUTE_SYMLINKING above.
    # ======================================================================= #
    _.use_absolute_symlinking if USE_ABSOLUTE_SYMLINKING
    _.disable_opn if RBT.do_not_show_names?
    _.run
  end; self.instance_eval { alias do_symlink symlink } # === RBT.do_symlink

end