#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/temp_directory.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/configuration_directory.rb'
require 'rbt/toplevel_methods/return_pwd.rb'
require 'rbt/toplevel_methods/save_file.rb'

module RBT

  # ========================================================================= #
  # === @temp_directory
  #
  # We set @temp_directory to an initial value, which may be changed at
  # a later time.
  # ========================================================================= #
  @temp_directory = '/tmp/'

  # ========================================================================= #
  # === RBT.temp_directory?
  #  
  # Preferentially use this method if you wish to find out the path
  # to the temporary directory in use by the RBT project.
  # ========================================================================= #
  def self.temp_directory?
    @temp_directory
  end; self.instance_eval { alias temp_directory temp_directory? } # === RBT.temp_directory
       self.instance_eval { alias temp_dir?      temp_directory? } # === RBT.temp_dir?
       self.instance_eval { alias rbt_temp_dir?  temp_directory? } # === RBT.rbt_temp_dir?

  # ========================================================================= #
  # === RBT.file_temp_directory
  #
  # This reader method will return the location to the file named
  # "temp_directory.yml", as a String.
  #
  # On my system, the method may return a String such as
  # "/Programs/Ruby/2.5.1/lib/ruby/site_ruby/2.5.0/rbt/yaml/configuration/temp_directory.yml"
  # ========================================================================= #
  def self.file_temp_directory
     "#{RBT.configuration_directory?}temp_directory.yml"
  end; self.instance_eval { alias file_temp_directory_yaml file_temp_directory } # === RBT.file_temp_directory_yaml

  # ========================================================================= #
  # === RBT.set_temp_directory
  # ========================================================================= #
  def self.set_temp_directory(
      i = '/tmp/'
    )
    i = convert_global_env(i) if i.include? '$'
    if i.include? '$'
      i = '/tmp/' # Use a default value in this case.
    end
    unless i.end_with? '/' # We must ensure that the last charcter is always a '/'.
      i = i.dup if i.frozen?
      i << '/' 
    end
    @temp_directory = i
  end; self.instance_eval { alias set_temp_dir set_temp_directory } # === RBT.set_temp_dir

  # ========================================================================= #
  # === RBT.permanently_set_temp_directory
  #
  # Use this method if you wish to permanently modify the temp directory,
  # that is, the directory we will extract to (or to a subdirectory in
  # that directory).
  # ========================================================================= #
  def self.permanently_set_temp_directory(i)
    case i
    when :pwd,'pwd', :default
      i = return_pwd.dup # Return the current working directory here.
    when :home_directory
      i = (ENV['HOME'].to_s+'/').squeeze('/')
    end
    # ======================================================================= #
    # Next, ensure trailing '/' character - unless the input includes
    # a '$' character.
    # ======================================================================= #
    unless i.include? '$'
      i << '/' unless i.end_with? '/'
    end
    opn; e "Setting the new temp directory to point towards `#{sdir(i)}`."
    # ======================================================================= #
    # Modify the yaml file next:
    # ======================================================================= #
    what = i
    into = RBT.file_temp_directory_yaml
    RBT.write_what_into(what, into)
    # ======================================================================= #
    # And also modify it on my home system, if IS_ROEBE is set to true.
    # ======================================================================= #
    if is_on_roebe?
      into = "#{RUBY_SRC_DIR_RBT_YAML_AT_HOME}configuration/"\
             "#{File.basename(into)}"
      RBT.write_what_into(what, into)
    end
    RBT.set_temp_directory(i) # And also sync it.
    unless File.directory? i
      opnn; e 'Since that directory does not exist, it will be created now.'
      mkdir(i) { :be_quiet }
    end
    i # Also return the variable, so we can re-use it in other methods.
  end; self.instance_eval { alias set_permanently_extract_to permanently_set_temp_directory } # === RBT.set_permanently_extract_to
       self.instance_eval { alias permanently_extract_to     permanently_set_temp_directory } # === RBT.permanently_extract_to

end

if __FILE__ == $PROGRAM_NAME
  pp RBT.file_temp_directory
  pp RBT.temp_directory?  
end