#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ApplySedOperations
#
# This class can apply sed-operations, as specified in a particular
# cookbook file.
#
# Usage example:
#   RBT::ApplySedOperations.new
# =========================================================================== #
# require 'rbt/utility_scripts/apply_sed_operations.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ApplySedOperations < Base # === RBT::ApplySedOperations.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  #
  # The first argument to this method will be the sed-operations that
  # this class will perform. It should be an Array, but if a String is
  # given then it will be converted into an Array anyway.
  # ========================================================================= #
  def initialize(
      use_these_sed_operations = ARGV,
      run_already              = true
    )
    reset
    set_use_these_sed_operations(
      use_these_sed_operations
    )
    case run_already
    when :do_not_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_use_these_sed_operations
  # ========================================================================= #
  def set_use_these_sed_operations(i = get_pwd)
    i = [i].flatten.compact
    @use_these_sed_operations = i
    sanitize_sed_operations
  end

  # ========================================================================= #
  # === use_these_sed_operations?
  # ========================================================================= #
  def use_these_sed_operations?
    @use_these_sed_operations
  end; alias use_which_sed_operations? use_these_sed_operations? # === use_which_sed_operations?

  # ========================================================================= #
  # === sanitize_sed_operations
  # ========================================================================= #
  def sanitize_sed_operations
    @use_these_sed_operations.map! {|entry|
      if entry.is_a?(String) and entry.start_with?(':') # <- Assume a Symbol in this case.
        entry = obtain_sed_entries_for_this_program(entry)
      end
      entry
    }
    @use_these_sed_operations.flatten! # <- Must invoke this again due to the ^^^ above code.
  end

  # ========================================================================= #
  # === obtain_sed_entries_for_this_program
  # ========================================================================= #
  def obtain_sed_entries_for_this_program(i)
    if i.start_with? ':'
      i = i.dup if i.frozen?
      i[0,1] = ''
    end
    require 'rbt/cookbooks/class/class.rb'
    dataset = RBT::Cookbooks::Cookbook.new(i) { :bypass_menu_check }
    dataset.sed?
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = use_which_sed_operations?
    unless _.empty?
      opnn; e 'Trying to use the "'+teal('sed')+'" '\
              'binary next from `'+sdir_return_pwd+'`.'
      _.each {|run_this_sed_operation|
        e skyblue("  #{run_this_sed_operation}")
        system(run_this_sed_operation)
      }
    end 
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::ApplySedOperations.new(ARGV)
end # apply_sed_operations :krb5