#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::AutoupdateThisProgram
#
# This class will try to autoupdate the given programs.
#
# Usage example:
#   RBT::AutoupdateThisProgram.new
# =========================================================================== #
# require 'rbt/utility_scripts/autoupdate_this_program.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/compile/compile.rb'

module RBT

class AutoupdateThisProgram < Base # === RBT::AutoupdateThisProgram.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      update_these_programs = ARGV,
      run_already           = true
    )
    reset
    update_these_programs = consider_setting_commandline_arguments(update_these_programs)
    # ======================================================================= #
    # Designate which programs are to be auto-updated next.
    # ======================================================================= #
    set_update_these_programs(
      update_these_programs
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    @commandline_arguments = []
    # ======================================================================= #
    # Whether to update the libraries/binaries of the given program at hand.
    # ======================================================================= #
    @update_the_libraries_of_the_program = true
    @update_the_binaries_of_the_program  = true
  end

  # ========================================================================= #
  # === consider_setting_commandline_arguments
  # ========================================================================= #
  def consider_setting_commandline_arguments(i)
    if i.any? {|line| line.start_with?('--') }
      @commandline_arguments << i.select {|line| line.start_with?('--') }
      @commandline_arguments.flatten!
      i.reject! {|line| line.start_with?('--') }
    end
    i
  end

  # ========================================================================= #
  # === set_update_these_programs
  #
  # This will set which programs ought to be automatically updated.
  # ========================================================================= #
  def set_update_these_programs(i)
    @update_these_programs = [i].flatten.compact.map(&:downcase)
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = @commandline_arguments
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === autoupdate vala --only-libs
      # === autoupdate vala --only-libraries
      # ===================================================================== #
      when /^-?-?only(-|_)?libs$/i,
           /^-?-?only(-|_)?libraries$/i
        @update_the_libraries_of_the_program = false
      # ===================================================================== #
      # === autoupdate vala --only-bin
      # === autoupdate vala --only-binaries
      # ===================================================================== #
      when /^-?-?only(-|_)?bin$/i,
           /^-?-?only(-|_)?bins$/i,
           /^-?-?only(-|_)?binaries$/i
        @update_the_binaries_of_the_program = false
      end
    end 
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    menu
    @update_these_programs.each {|this_program|
      cd_to_the_log_directory
      _ = RBT::Compile.new(this_program) { :do_not_run }
      opnn; e "Now updating the libraries of the "\
              "program called `#{orange(this_program)}`:"
      if @update_the_libraries_of_the_program
        _.update_libraries_of_this_program(this_program)
      end
      if @update_the_binaries_of_the_program
        _.update_binaries_of_this_program(this_program)
      end
    }
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::AutoupdateThisProgram.new(ARGV)
end # autoupdate_this_program 