#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::BackupProgram
#
# Use this class to backup several programs. In actual relevance, you
# can basically copy selected packages into a tarball with this script,
# by passing in an array.
#
# The array lists which programs should be backuped.
#
# This class was created at 13.10.2007.
#
# Simple Usage example for the latter part:
#
#   require 'rbt'; RBT::BackupProgram.new ['php','ruby']
# =========================================================================== #
# require 'rbt/utility_scripts/backup_programs.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class BackupProgram < Base # 

  # ========================================================================= #
  # === initialize
  #
  # The first argument is an array, listing which programs to backup.
  # ========================================================================= #
  def initialize(
      array_these_programs = []
    )
    reset
    unless array_these_programs.empty?
      set_array_these_programs(array_these_programs)
      backup_these_programs
    end
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @time_now = Time.now.strftime '%d.%m.%Y'
  end

  # ========================================================================= #
  # === set_array_these_programs
  #
  # We must keep in mind that i should be an Array.
  # ========================================================================= #
  def set_array_these_programs(i)
    i = [i].flatten.compact # <- Ensure that it is an Array.
    @array_these_programs = i
  end

  # ========================================================================= #
  # === backup_these_programs
  #
  # This is the actual powerhorse of the class.
  # ========================================================================= #
  def backup_these_programs
    # ======================================================================= #
    # Copy to new directory.
    # ======================================================================= #
    @target_dir = RBT.temp_directory?+'BackupProgram_'+@time_now+'/'
    remove(@target_dir) if File.exist? @target_dir # If it exists.
    create_directory @target_dir
    cd @target_dir
    e @target_dir
    @array_these_programs.each { |program|
      target = programs_directory?+program.capitalize
      @target_dir << program+'/'
      cmd = 'cp -rv '+target
      cmd << ' .'
      if File.exist? target
        esystem cmd
      else
        warn target+' does not exist.'
      end 
    }
    create_log
    notify_user_of_task_being_finished
  end; alias run backup_these_programs # === run

  # ========================================================================= #
  # === notify_user_of_task_being_finished
  # ========================================================================= #
  def notify_user_of_task_being_finished
    e 'If you wish to change directory to that target, do:'
    e
    e "  cd #{sdir(@target_dir)}"
    e
  end

  # ========================================================================= #
  # === create_log
  #
  # Create a log file here.
  # ========================================================================= #
  def create_log
    into = @target_dir+'/ProgramsFile'
    e sfile(into)
    write_what_into(
      'We stored these programs: '+
      @array_these_programs.join(' ').downcase,
      into
    )
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::BackupProgram.new(ARGV)
end # Usage examples:
    #   pbackup
    #   pbackup php