#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::BeautifyConfigureHelpOutput
#
# Usage examples:
#   RBT::BeautifyConfigureHelpOutput.new
# =========================================================================== #
# require 'rbt/utility_scripts/beautify_configure_help_output.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class BeautifyConfigureHelpOutput < Base # RBT::BeautifyConfigureHelpOutput.new

  T = "\t"

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === R
  # ========================================================================= #
  R = "\r"

  # ========================================================================= #
  # === RUN_THIS_COMMAND
  #
  # This is the command we run to get some help.
  # ========================================================================= #
  RUN_THIS_COMMAND = './configure --help'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @result = nil
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === find_all_entries_with
  #
  # All entries without a '--' will be rejected from our dataset.
  # ========================================================================= #
  def find_all_entries_with(
      i = '--'
    )
    @result.select! {|entry| entry.include? i }
  end

  # ========================================================================= #
  # === green
  # ========================================================================= #
  def green
    Colours::GREEN
  end

  # ========================================================================= #
  # === rev
  # ========================================================================= #
  def rev
    Colours.rev
  end

  # ========================================================================= #
  # === reformat_result_string
  #
  # We will reformat the @result string here.
  # ========================================================================= #
  def reformat_result_string
    _ = @result
    _ = _.split(N) unless _.is_a? Array
    return _
  end

  # ========================================================================= #
  # === result?
  # ========================================================================= #
  def result?
    @result
  end

  # ========================================================================= #
  # === determine_result
  #
  # @result will keep the result from running a sys-command.
  # ========================================================================= #
  def determine_result
    begin
      @result = `#{RUN_THIS_COMMAND}`.split("\n").map {|entry|
        entry.chomp.strip
      }
      reformat_result_string
    rescue Errno::ENOENT; end
  end

  # ========================================================================= #
  # === colourize_this                                        (colourize tag)
  #
  # This method will colourize some words.
  # ========================================================================= #
  def colourize_this(i)
    if i.include? 'DATAROOTDIR'
      i.gsub!(/(DATAROOTDIR)/, slateblue('\1')+green)
    end
    if i.include? 'EPREFIX'
      i.gsub!(/(EPREFIX)/, orange('\1')+green)
    elsif i.include? 'PREFIX'
      i.gsub!(/(PREFIX)/, orchid('\1')+green)
    end
    if i.include? 'SUFFIX'
      i.gsub!(/(SUFFIX)/, powderblue('\1')+green)
    end
    return i
  end

  # ========================================================================= #
  # === report_result
  # ========================================================================= #
  def report_result
    n_options = @result.size
    opnn; e 'These ('+simp(n_options.to_s)+') options are available '+
            'for '+simp(File.basename(Dir.pwd))+':'
    n_ljust = 38
    l_padding = '    '.dup # Left padding.
    if @result.is_a? Array
      cliner {
        @result.sort.each {|entry|
          if entry.include? '  '
            splitted = entry.split('  ').reject {|inner_entry|
              inner_entry.empty?
            }
            splitted.map! {|inner_entry| inner_entry.tr(R,'').tr(T,'') }
            # We have a little leading '  ' padding.
            splitted[0]  = l_padding+(splitted[0].ljust(n_ljust))+green
            splitted[-1] = colourize_this(splitted[-1])
            splitted[-1] = splitted[-1].strip.ljust(40)+rev
            entry = splitted.join('  ')
          else # Else we have to pad too.
            entry = l_padding+entry.ljust(n_ljust)
          end
          efancy '  '+entry
        }
      }
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    determine_result
    if result?
      find_all_entries_with('--')
      report_result
    end
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::BeautifyConfigureHelpOutput.new(ARGV)
end # beautify