#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::BeautifySystem
#
# This class will attempt to make a Linux system more "beautiful".
#
# What this means is to, currently, turn it a bit into a GoboLinux
# hybrid system.
#
# Usage example:
#   RBT::BeautifySystem.new
# =========================================================================== #
# require 'rbt/utility_scripts/beautify_system.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/compile/compile.rb'
require 'rbt/constants/chained_programs.rb'

module RBT

class BeautifySystem < Base # === RBT::BeautifySystem.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === RBT::BeautifySystem::USE_THIS_AS_DEFAULT_INPUT
  # ========================================================================= #
  USE_THIS_AS_DEFAULT_INPUT = '

    m4
    make
    mpfr
    gmp
    mpc
    libffi
    atk
    cairo
    freetype
    fontconfig
    harfbuzz
    pango
    glib
    gtk
    binutils
    coreutils
    pixman
    cmake
    lcms
    lcms2
    libassuan
    libx11
    libgweather
    gcc

  '

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @use_this_as_default_input = USE_THIS_AS_DEFAULT_INPUT
    # ===================================================================== #
    # Add some aliases there.
    # ===================================================================== #
    chained_programs = RBT.chained_programs
    # ===================================================================== #
    # You can add more aliases to this entry.
    # ===================================================================== #
    array_add_these_aliases_from_the_chained_programs = %w(
      autotools
      audio_suite
      audio_suite2
      cfamily_addons
      alsa
      editors
      perl
      video_base
      video_suite
      metagtkmm
      databases
      fluxbox
      graphics_suite
      irc
      my_python
      sdl
      viewers
      xml
      ruby_addons
      xorg_protos
      xorg_utils
      xorg_libraries
      xorg_data
      xorg_apps
      xorg_fonts
      xorg_server
      terminals
      all_of_the_mate_desktop
      kde5_foundation
      kde5_plasma
      kde5_porting_aids
      kde5_applications
      xfce
    )
    array_add_these_aliases_from_the_chained_programs.each {|entry|
      add_this = chained_programs[entry].strip.tr(',',N)
      @use_this_as_default_input << add_this
    }
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    do_compile @use_this_as_default_input
  end

  # ========================================================================= #
  # === do_compile
  # ========================================================================= #
  def do_compile(i)
    if i.is_a? String
      i.strip!
      if i.include? N
        i = i.split(N)
      end
    end
    if i.is_a? String
      i = [i]
    end
    i.map!(&:strip)
    i.reject!(&:empty?)
    i.each {|entry|
      begin
        RBT::Compile.new(entry.strip)
      rescue SystemExit
        opnn; e 'User requested to exit, thus we will exit now.'
        exit
      rescue Exception => error
        opnn; e 'An error was encountered:'
        pp error
        pp error.class
      end
    }
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::BeautifySystem.new(ARGV)
end # beautify_system