#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::BinaryNameBelongsToWhichProgram
#
# This small class will report to which a particular program a specific
# binary belongs to. The name of the binary has to be used as main
# argument to this class.
#
# Usage example:
#   RBT::BinaryNameBelongsToWhichProgram.new
# =========================================================================== #
# require 'rbt/utility_scripts/binary_name_belongs_to_which_program.rb'
# RBT::BinaryNameBelongsToWhichProgram.new
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/utility_scripts/registered_binaries.rb'

module RBT

class BinaryNameBelongsToWhichProgram < Base # === RBT::BinaryNameBelongsToWhichProgram.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    i = [i].flatten.compact
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    first = @commandline_arguments.first
    all_registered_binaries = RBT.hash_registered_binaries?
    if all_registered_binaries.has_key? first
      belongs_to = all_registered_binaries[first]
      opnn; e "The binary called `#{royalblue(first)}` belongs to the "\
              "program `#{orange(belongs_to)}`."
    else
      opnn; e "The program called `#{royalblue(first)}` does not appear "\
              "to be registered "
      opnn; e 'with any particular program.'
    end
  end

  # ========================================================================= #
  # === RBT::BinaryNameBelongsToWhichProgram[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::BinaryNameBelongsToWhichProgram.new(ARGV)
end # binary_name_belongs_to_which_program