#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CompileIntoHomeDir
#
# This will compile the given input at hand into the home dir of
# the user.
#
# Usage example:
#   RBT::CompileIntoHomeDir.new
# =========================================================================== #
# require 'rbt/utility_scripts/compile_into_home_dir.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/compile/compile.rb'

module RBT

class CompileIntoHomeDir < Base # === RBT::CompileIntoHomeDir.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_compile_these_programs(
      i
    )
    if block_given?
      yielded = yield
      set_commandline_arguments(yielded)
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    set_commandline_arguments
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = nil)
    @commandline_arguments = [i].flatten.compact
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end

  # ========================================================================= #
  # === set_compile_these_programs
  # ========================================================================= #
  def set_compile_these_programs(i)
    @compile_these_programs = [i].flatten.compact
  end

  # ========================================================================= #
  # === compile_which_programs?
  # ========================================================================= #
  def compile_which_programs?
    @compile_these_programs
  end

  # ========================================================================= #
  # === without_symlinking?
  # ========================================================================= #
  def without_symlinking?
    commandline_arguments?.any? {|entry|
      entry == :without_symlinking
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    # ======================================================================= #
    # Instantiate our compile-object next:
    # ======================================================================= #
    _ = RBT::Compile.new(:do_not_run_yet)
    compile_these_programs = compile_which_programs?
    compile_these_programs.each {|this_program|
      opnn; e "Now attempting to compile the program `#{orange(this_program)}`."
      _.reset
      _.set_compile_this_program(this_program)
      _.prepare_dataset_for(this_program)
      _.set_user_prefix('$HOME')
      if without_symlinking?
        _.do_not_symlink { :be_verbose }
      end
      _.do_compile_this_program(this_program)
    }
  end

  # ========================================================================= #
  # === RBT::CompileIntoHomeDir[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::CompileIntoHomeDir.new(ARGV)
end # compile_into_home_dir sed