#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# ========================================================================== #
# === RBT::ConvertToNonTraditional
#
# This class will gather all files from /usr/ etc... and copy them
# to some other location. In order for this to work, we must make
# use of the information available in the individual cookbook file.
# ========================================================================== #
# require 'rbt/convert_to_non_traditional.rb'
# ========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ConvertToNonTraditional < Base # === RBT::ConvertToNonTraditional

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      convert_this_program,
      run_already = false
    )
    reset
    set_this_program(
      convert_this_program
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @database = nil
    @convert_already = true
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === load_yaml_database
  #
  # Load the yaml database. We will only continue if the Database file 
  # exists.
  # ========================================================================= #
  def load_yaml_database
    _ = YAML_DATABASE_DIRECTORY+'Database_InstalledFiles.yml'
    if File.exist? _
      opnn; e 'Loading Database from '+sfile(_)+' next.'
      @database = load_yaml(_)
      fetch_keys_and_convert
    else
      opnn; warn 'But file '+sfile(_)+swarn(' does not exist.')
      opnn; warn 'This script currently requires that it must exist.'
    end
  end; alias start load_yaml_database # === start
       alias run   load_yaml_database # === run

  # ========================================================================= #
  # === fetch_keys_and_convert
  # ========================================================================= #
  def fetch_keys_and_convert(
      convert_already = @convert_already
    )
    @keys = @database.keys
    result = @keys.grep(/#{@program}/)
    @key_name = result.first.to_s
    # splitted = @key_name.split('-')
    if result
      opnn; e 'Is dabei '+@key_name
      # _ = '/'+splitted[0].capitalize
      # __ = splitted[1].capitalize
      sanitize_all_files # needed
      reject_directories # needed
      do_the_conversion if convert_already
    end
  end

  # ========================================================================= #
  # === do_the_conversion
  # ========================================================================= #
  def do_the_conversion
    create_program(@key_name) # not needed to act on InstalledFiles
    copy_from_usr_to_programs # not needed to act on InstalledFiles
    symlink_program(@program_dir) # not needed to act on InstalledFiles
    remove_usr_directories # not needed to act on InstalledFiles
  end

  # ========================================================================= #
  # === sanitize_all_files
  # ========================================================================= #
  def sanitize_all_files
    @all_files = @database[@key_name].map {|f|
      remove_double_slashes(
        f.gsub(/\/Programs/, '')
      )
    }
  end

  # ========================================================================= #
  # === copy_from_usr_to_programs
  # ========================================================================= #
  def copy_from_usr_to_programs
    @all_files.each { |key|
      @program_dir = programs_directory?+'/'+@key_name.capitalize.tr('-','/')
      _ = @program_dir+File.dirname(key)
      copy_file('/usr'+key, _)
    }
  end

  # ========================================================================= #
  # === remove_usr_directories
  # ========================================================================= #
  def remove_usr_directories # remove the /usr directories.
    @all_files.each { |file|
      remove_file('/usr'+file)
    }
  end

  # ========================================================================= #
  # === symlink_program
  # ========================================================================= #
  def symlink_program(i)
    @advanced_symlink = AdvancedSymlink.new(i)
    @advanced_symlink.wrap_action
  end

  # ========================================================================= #
  # === reject_directories
  # ========================================================================= #
  def reject_directories
    @all_files.reject! {|entry| File.directory? '/usr/'+entry }
    @all_files.reject! {|f| f.include? 'CompileTimeInformation'}
  end

  # ========================================================================= #
  # === set_this_program
  # ========================================================================= #
  def set_this_program(i)
    @program = i
  end

  # ========================================================================= #
  # === convert_already =
  # ========================================================================= #
  def convert_already=(i)
    @convert_already = i
  end

  # ========================================================================= #
  # === create_program
  # ========================================================================= #
  def create_program(
      key_name = @key_name
    )
    _ = RBT::CreateProgram.new(key_name, true, true)
    _.create_directories
  end

end; end

if __FILE__ == $PROGRAM_NAME
  _ = RBT::ConvertToNonTraditional.new(ARGV[0])
  _.start
end # ctnt htop