#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ConvertToTraditional
#
# This class will take a program from the $PROGRAMS directory and convert
# it into a FHS-ready installation, with the prefix being /usr.
# =========================================================================== #
# require 'rbt/utility_scripts/convert_to_traditional.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ConvertToTraditional < Base

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === USE_THIS_TARGET_PREFIX
  # ========================================================================= #
  USE_THIS_TARGET_PREFIX = '/usr/'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      full_location_of_package_or_name = nil,
      run_already                      = true
     )
    reset
    set_full_location_of_package_or_name(full_location_of_package_or_name)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    @target_prefix_to_use = USE_THIS_TARGET_PREFIX
  end

  # ========================================================================= #
  # === set_full_location_of_package_or_name
  # ========================================================================= #
  def set_full_location_of_package_or_name(i)
    i = i.first if i.is_a? Array
    @full_location_of_package_or_name = i
  end

  # ========================================================================= #
  # === set_start_location
  # ========================================================================= #
  def set_start_location(
      i = programs_directory?
    )
    if RBT.configuration?.capitalize_program_names # Whether to capitalize the directory.
      i << @full_location_of_package_or_name.capitalize
    else
      i << @full_location_of_package_or_name
    end
    @start_location = i
  end

  # ========================================================================= #
  # === prefix?
  # ========================================================================= #
  def prefix?
    @target_prefix_to_use
  end

  # ========================================================================= #
  # === convert_to_fhs
  #
  # This method will "convert" the package to a FHS install.
  # ========================================================================= # 
  def convert_to_fhs(i)
    i = i.dup if i.frozen?
    i << '/Current/'
    if File.exist?(i)
      opnn; e "Now #{springgreen(converting)} `#{sfile(i)}`."
      opnn; e "to a FHS package (into --prefix #{prefix?})."
      copy_recursively(i, prefix?, true) # true for "be verbose".
      opnn; e 'Done!'
    else
      opnn; e 'Sorry, we can not continue. The directory at:'
      opnn; e sdir(i)
      opnn; e 'does not seem to exist.'
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    if @full_location_of_package_or_name # Was given.
      set_start_location(programs_directory?)
      if File.exist? @start_location
        convert_to_fhs(@start_location)
      end
    end
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::ConvertToTraditional.new(ARGV.first)
end
# Test it here:
#   ctt ATK
#   ctt Atk
#   ctt xfontsel