#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CreateBashScripts
#
# This script will generate several bash scripts. 
# =========================================================================== #
# require 'rbt/create_bash_scripts.rb'
# =========================================================================== #
require 'rbt/utility_scripts/generate_shellscript.rb'
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

class CreateBashScripts < Base

  # ========================================================================= #
  # === WHERE_TO
  # ========================================================================= #
  WHERE_TO = DEPOT_DIR+'Information/'

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      for_this_program = nil,
      run_already      = true
    )
    reset
    set_for_this_program(for_this_program)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @cookbook_dataset = RBT::Cookbooks::Cookbook.new { :bypass_menu }
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_for_this_program
  # ========================================================================= #
  def set_for_this_program(i)
    @this_program = i
  end

  # ========================================================================= #
  # === copy_archive_shell_script
  # ========================================================================= #
  def copy_archive_shell_script
    copy '$LINUX/SHELL/SCRIPTS/extract_archive.sh', WHERE_TO
  end

  # ========================================================================= #
  # === notify_user
  #
  # Use this method to notify the user.
  # ========================================================================= #
  def notify_user(file)
    cliner
    opnn; e sfancy('Finished creation! The file was: '+N+'  ')+sfile(file)
    e
    opnn; e 'If you want to cd to the directory, use this command:'
    opnn; e sfancy('  cd ')+sdir('/System/Settings/Scripts/')
    opnn; e 'And if you want to source this file, use this command:'
    e sfancy('  source ')+sfile(file)
    exit_program
  end

  # ========================================================================= #
  # === exit_program
  # ========================================================================= #
  def exit_program
    exit
  end

  # ========================================================================= #
  # === create_shellscript
  #
  # This delegates the task to RBT::GenerateShellscript.
  #
  #   bl $RBT/utility_scripts/generate_shellscript.rb
  #
  # ========================================================================= #
  def create_shellscript(for_this_program = @this_program)
    @cookbook_dataset = RBT::Cookbooks::Cookbook.new(for_this_program) { :bypass_menu }
    _ = GenerateShellscript.new(
      @cookbook_dataset, true, false, false
    ) {{
      :also_run_at_once => false
    }}
    return _.dataset+N
  end

  # ========================================================================= #
  # === run
  #
  # Everything is bundled here in this method.
  # ========================================================================= #
  def run
    reset
    copy_archive_shell_script
    e
    dir = system_directory?+'Settings/Scripts/'
    file = dir+'bash_programs_rc'
    cliner {
      e sfancy('Now creating a bash file. The name and file path will be:')
      e '  '+sfile(file)
    }
    create_directory(dir) unless File.exist?(dir) # create dir if it doesnt exist yet.
    remove_file(file) if File.exist? file
    string = ''.dup
    # ======================================================================= #
    # First, lets collect which programs actually are available. This will 
    # be an array.
    # ======================================================================= #
    string << '# Store all available programs in an array, so that we '+N
    string << '# can use this information in our shell scripts lateron.'+N
    string << 'programs=('
    all_programs.each { |program|
      string << '  "'+program.to_s+'"'+N
    }; string << ')'+N+N
    # ======================================================================= #
    # Second, we write their info :>
    # It will lead to a lot of "c_" functions.
    # 
    # Example entries start like so:
    #   function c_python() {
    # ======================================================================= #
    all_programs.each { |program|
      opnn; e "Creating a shellscript for `#{sfancy(program)}` next."
      string << create_shellscript(program) # This is defined in this file here.
    }
    write_what_into(string, file)
    # ======================================================================= #
    # The following 3 lines are specific for my machine.
    # ======================================================================= #
    _ = ENV['LINUX'].to_s+'/SHELL/bash_programs_rc'
    remove_file(_) if File.exist? _
    write_what_into(string, _)
    # ======================================================================= #
    # At this point, all data has been written and we can notify the user
    # about it.
    # ======================================================================= #
    notify_user(file)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::CreateBashScripts.new(ARGV.first)
end # cbs