#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CreateDatabase
#
# The purpose of this script is to take all the various .yml
# files and generate one SQL database from it.
#
# In principle this SQL database should be more efficient than various
# .yml files since it should be faster to query from it.
#
# Usage example:
#   RBT::Cookbooks::CreateDatabase.new
# =========================================================================== #
# require 'rbt/utility_scripts/create_database.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

module Cookbooks

class CreateDatabase < ::RBT::Base # === RBT::Cookbooks::CreateDatabase.new

  begin
    require 'sql_paradise'
  rescue LoadError
    if RBT.is_on_roebe?
      puts 'sql_paradise project is not available. Please install it.'
    end
  end

  # ========================================================================= #
  # Guard against the end-user missing the sql_paradise gem.
  # ========================================================================= #
  if Object.const_defined? :SqlParadise
    include ::SqlParadise
    include ::SqlParadise::Commands
  end

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_input = nil,
      run_already    = true
    )
    reset
    set_input(
      optional_input
    )
    case run_already
    when :do_not_report
      @shall_we_report = false
      run_already = true
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @_ = ''
    @shall_we_report = true
    @query = Cookbooks::Cookbook.new(:do_not_run_yet)
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = nil)
    @input = i
  end

  # ========================================================================= #
  # === _
  # ========================================================================= #
  def _(i)
    @_ << i+N
  end

  # ========================================================================= #
  # === shall_we_report?
  # ========================================================================= #
  def shall_we_report?
    @shall_we_report
  end

  # ========================================================================= #
  # === report_result
  # ========================================================================= #
  def report_result
    e @_ if shall_we_report?
  end

  # ========================================================================= #
  # === data?
  # ========================================================================= #
  def data?
    @_
  end

  # ========================================================================= #
  # === save_result
  # ========================================================================= #
  def save_result
    opnn; e "We will now save the result into #{sfile(save_where?)}"
    save_file(data?, save_where?)
  end

  # ========================================================================= #
  # === populate_database
  # ========================================================================= #
  def populate_database
    available_programs?.each {|the_program|
      @query = RBT::Cookbooks::Cookbook.new(the_program) { :bypass_menu }
      @query.feedback
      what = InsertInto[@name_of_the_table, @query.program_path?]
      append_what_into(what, file?)
    }
  end

  # ========================================================================= #
  # === display_generated_file
  #
  # This is an optional method.
  # ========================================================================= #
  def display_generated_file
    if File.exist? save_where?
      cliner {
        e File.readlines(save_where?)
      }
    end
  end

  # ========================================================================= #
  # === save_where?
  # ========================================================================= #
  def save_where?
    "#{temp_dir?}database_for_programs.sql"
  end; alias file? save_where? # === file

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    if Object.const_defined? :SqlParadise # The functionality depends on SqlParadise.
      _ SqlParadise::CreateDatabase['cookbooks']
      # ===================================================================== #
      # Populate the table next.
      # ===================================================================== #
      dataset = RBT.registered_cookbook_entries?
      dataset.map! {|entry| entry+' varchar(120)' }
      @name_of_the_table = 'cookbooks'
      _ CreateTable[@name_of_the_table]
      entries_in_the_table = '('+dataset.join(', ')+')'
      _ InsertInto[@name_of_the_table, entries_in_the_table]
      report_result
      save_result
      populate_database # We add our programs next.
    end
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  _ = RBT::Cookbooks::CreateDatabase.new(ARGV, :do_not_report)
  _.display_generated_file
end # cdatabase