#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::FeedbackBinariesOf
#
# This class will yield to us all binaries of a given program.
# =========================================================================== #
# require 'cookbooks/utility_scripts/feedback_binaries_of.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

class FeedbackBinariesOf < Base

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = 'ruby',
      run_already = true
    )
    super()
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i)
    i = i.join(' ').strip if i.is_a? Array
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end; alias search_term? input? # === search_term?

  # ========================================================================= #
  # === obtain_dataset
  # ========================================================================= #
  def obtain_dataset
    @dataset = RBT::Cookbooks::Cookbook.new(input?) { :bypass_menu_check }
  end

  # ========================================================================= #
  # === binaries?
  # ========================================================================= #
  def binaries?
    @binaries
  end

  # ========================================================================= #
  # === report_binaries
  # ========================================================================= #
  def report_binaries
    if binaries?.empty?
      opnn; e 'We do not know whether `'+sfancy(search_term?)+
              '` will install any binaries.'
    else
      @binaries.each {|entry|
        e '  - '+entry
      }
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    obtain_dataset
    @binaries = @dataset.binaries
    report_binaries
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::FeedbackBinariesOf.new(ARGV)
end # binaries_of? ruby