#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::FeedbackLicenses
#
# This will feedback all licenses. Optionally you can search for only a 
# specific version.
# =========================================================================== #
# require 'cookbooks/utility_scripts/feedback_licenses.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

module Cookbooks

class FeedbackLicenses < Base # === RBT::Cookbooks::FeedbackLicenses

  # ======================================================================= #
  # === PADDING_TO_USE
  # ======================================================================= #
  PADDING_TO_USE = '25'

  # ======================================================================= #
  # === PADDING_LEFT
  # ======================================================================= #
  PADDING_LEFT = '| ' # The leading padding to use.

  # ======================================================================= #
  # === DEFAULT_FILTER_TO_USE
  # ======================================================================= #
  DEFAULT_FILTER_TO_USE = '*' # By default, we show ALL known licenses.

  # ======================================================================= #
  # === NAMESPACE
  # ======================================================================= #
  NAMESPACE = inspect

  # ======================================================================= #
  # === initialize
  #
  # Usage example:
  #   FeedbackLicenses.new(filter_for_licenses = DEFAULT_FILTER_TO_USE, run_already = true)
  #
  # If the filter is * then we will filter for ALL licenses.
  # ======================================================================= #
  def initialize(
      filter_for_licenses = DEFAULT_FILTER_TO_USE,
      run_already         = true
    )
    super()
    reset
    set_filter_for_licenses(filter_for_licenses)
    run if run_already
  end

  # ======================================================================= #
  # === reset
  # ======================================================================= #
  def reset
    super()
    @cookbook = RBT::Cookbooks::Cookbook.new { :bypass_menu }
    @filter   = ''.dup # Set the filter to use.
    @namespace = NAMESPACE
  end

  # ======================================================================= #
  # === set_filter_for_licenses
  # ======================================================================= #
  def set_filter_for_licenses(i = DEFAULT_FILTER_TO_USE)
    i = i.first if i.is_a? Array
    i = DEFAULT_FILTER_TO_USE if i.nil?
    i = DEFAULT_FILTER_TO_USE if i.empty?
    i = i.to_s.dup
    i = i.downcase
    @filter = i
  end; alias filter_for set_filter_for_licenses # === filter_for

  # ======================================================================= #
  # === set_which_program
  # ======================================================================= #
  def set_which_program(i) # Set which program we are seeking.
    @which_program = i
  end

  # ======================================================================= #
  # === output_header
  # ======================================================================= #
  def output_header
    # ===================================================================== #
    # This is the header content.
    # ===================================================================== #
    e PADDING_LEFT+('%-25s' % 'Name  ')+
      '  | License                                         |'
  end

  # ======================================================================= #
  # === filter?
  # ======================================================================= #
  def filter?
    @filter
  end

  # ======================================================================= #
  # === feedback_licenses
  # ======================================================================= #
  def feedback_licenses
    cliner {
      _ = 'known licenses' # Placeholder for now.
      _ = filter? unless filter? == DEFAULT_FILTER_TO_USE
      opnn; e 'Feedback licenses matching search-criteria for'
      opnn; e '`'+sfancy(_.to_s)+'` in alphabetical order next:'
    }
    output_header
    cliner {
      programs = available_programs?
      programs.each { |program|
        @cookbook = RBT::Cookbooks::Cookbook.new(program) { :bypass_menu }
        license = return_license
        if license.is_a? Array
          license = license.join(', ')
        end
        next if license == 'unknown'
        full_license_text = PADDING_LEFT+'%-25s' % program+'  | '+'%-10s' % license
        if @filter == DEFAULT_FILTER_TO_USE
          e full_license_text
        else # else we assume user wants filtering done
          unless license.to_s.downcase.scan(/#{filter?}/).empty?
            e full_license_text 
          end
        end
      }
    }
  end

  # ======================================================================= #
  # === return_license
  # ======================================================================= #
  def return_license
    @cookbook.license?.to_s # .downcase
  end

  # ======================================================================= #
  # === run                                                       (run tag)
  # ======================================================================= #
  def run
    feedback_licenses
  end

  # ======================================================================= #
  # === RBT::Cookbooks::FeedbackLicenses[]
  # ======================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

# =========================================================================== #
# === RBT.feedback_licenses
# =========================================================================== #
def self.feedback_licenses
  RBT::Cookbooks::FeedbackLicenses.new
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::FeedbackLicenses.new(ARGV)
end # flic gpl # Feedback all GPL-licenses.