#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::GenerateMachomebrewFormula
#
# Usage examples:
#   RBT::GenerateMachomebrewFormula.new
# ============================================================================ #
# require 'rbt/utility_scripts/generate_machomebrew_formula.rb'
# ============================================================================ #
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT # === RBT::GenerateMachomebrewFormula

class GenerateMachomebrewFormula < Base 

  # ========================================================================= #
  # === DEFAULT_PROGRAM
  # ========================================================================= #
  DEFAULT_PROGRAM = 'htop'

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      for_this_program = ARGV.first,
      run_already      = true
    )
    reset
    set_program(
      for_this_program
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @program = nil
    @_ = ''.dup # This is the string we use to generate a machomebrew formula.
    setup_dataset
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_program
  # ========================================================================= #
  def set_program(
      i = DEFAULT_PROGRAM
    )
    i = DEFAULT_PROGRAM if i.nil?
    @program = i.to_s.dup
  end

  # ========================================================================= #
  # === program?
  # ========================================================================= #
  def program?
    @program
  end; alias program_name? program? # === program_name?

  # ========================================================================= #
  # === setup_dataset
  #
  # @program must exist before calling this method.
  # ========================================================================= #
  def setup_dataset(i = program?)
    @data = RBT::Cookbooks::Cookbook.new(i) { :bypass_menu_check }
  end

  # ========================================================================= #
  # === find_remote_homepage
  #
  # This method finds the last URL.
  # ========================================================================= #
  def find_remote_homepage
    @data.remote_homepage?.to_s
  end

  # ========================================================================= #
  # === generate_main_string
  # ========================================================================= #
  def generate_main_string
    short_name        = @data.short_name?.to_s.capitalize
    if short_name.empty?
      short_name = program_name?.to_s.dup.capitalize
    end
    link_to_remote_package = @data.url1?
    dependencies      = @data.required_deps_on?
    configure_options = @data.configure_options?
    program_homepage  = find_remote_homepage
    @_ << "

require 'formula'

class #{short_name.to_s} < Formula

  homepage '"+program_homepage+"'
  url '"+link_to_remote_package+"'
  sha1 '692669243433c55384a54b397a1cc926e582e9f2'

"
  # Next, add the dependencies:
  dependencies.each {|dep|
    @_ << "  depends_on '"+dep+"'"+N
  }

@_ << "

  skip_clean :all

  def options
    [['--disable-etcdir', 'Disable the reading of Zsh rc files in /etc']]
  end

  def install
    args = %w(
"

  # ========================================================================= #
  # Next, add the dependencies:
  # ========================================================================= #
  configure_options.each {|option|
    @_ << option.to_s+N
  } unless configure_options.empty?

@_ << "
    )

    args << '--disable-etcdir' if ARGV.include? '--disable-etcdir'
    system './configure', *args
    system 'make install'
  end

  def caveats; <<-EOS.undent

Bla.

EOS
  end
end
"
  end

  # ========================================================================= #
  # === save_string
  # ========================================================================= #
  def save_string
    _ = "#{rbt_log_directory?}homebrew_formulas/#{program?}.rb"
    unless File.directory? File.dirname(_)
      mkdir(File.dirname(_))
    end
    opnn; e "Saving into `#{sfile(_)}`."
    save_what_to(@_, _)
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    setup_dataset # Set it up anew again.
    generate_main_string
    save_string
  end

end; end

if __FILE__ == $PROGRAM_NAME
  _ = RBT::GenerateMachomebrewFormula.new(ARGV.first, false)
  _.run
end # gmf htop