#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::GenerateRbtConfiguration
#
# This class will generate a new Configuration file for use in the
# RBT project.
#
# Usage example:
#   RBT::GenerateRbtConfiguration.new
# =========================================================================== #
# require 'rbt/utility_scripts/generate_rbt_configuration.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class GenerateRbtConfiguration < Base # === RBT::GenerateRbtConfiguration.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === LAST_UPDATE
  # ========================================================================= #
  LAST_UPDATE = 'Sep 2018'

  # ========================================================================= #
  # === STORE_WHERE_BASE_DIR
  # ========================================================================= #
  STORE_WHERE_BASE_DIR = RBT.temp_dir?

  # ========================================================================= #
  # === SAVE_NEW_FILE_HERE
  # ========================================================================= #
  SAVE_NEW_FILE_HERE = "#{STORE_WHERE_BASE_DIR}/RBT_configuration.yml"

  # ============================================================================ #
  # === initialize
  # ============================================================================ #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ============================================================================ #
  # === reset                                                        (reset tag)
  # ============================================================================ #
  def reset
    super()
    set_store_here SAVE_NEW_FILE_HERE
    @_ = ''.dup # The result string.
    @namespace = NAMESPACE
  end

  # ============================================================================ #
  # === set_store_here
  # ============================================================================ #
  def set_store_here(i)
    @store_here = rds(i.to_s)
  end

  # ============================================================================ #
  # === add
  #
  # This will append to @_ and will also append two newlines.
  # ============================================================================ #
  def add(i)
    @_ << i+N
  end; alias _ add # === _

  # ============================================================================ #
  # === get_user_input
  # ============================================================================ #
  def get_user_input
    @user_input = $stdin.gets.chomp
  end

  # ============================================================================ #
  # === show_welcome_message
  # ============================================================================ #
  def show_welcome_message
    cliner {
      opnn; e 'Welcome to the RBT Project, class GenerateRbtConfiguration.'
    }
  end

  # ============================================================================ #
  # === run_through_all_configure_options
  #
  # Here we will grab all configuration entries.
  # ============================================================================ #
  def run_through_all_configure_options
    _ = RBT.config_directory?
    # ========================================================================== #
    # Load it up next.
    # ========================================================================== #
    available_keys = Dir[_+'*.yml'].map {|entry|
      File.basename(entry).gsub!(/\.yml$/,'')
    }
    available_keys.each {|entry|
      _ entry+': t'
    }
  end

  # ============================================================================ #
  # === e_get_user_input_then_add
  #
  # Here we will combine get_user_input() with add().
  # ============================================================================ #
  def e_get_user_input_then_add(variable_name, desc = '')
    desc = "Description: #{variable_name}" if desc.to_s.empty?
    print desc+' '
    get_user_input
    add ' '+variable_name+': '+@user_input
  end

  # ============================================================================ #
  # === show_last_update
  # ============================================================================ #
  def show_last_update
    opnn; e 'This was last updated at: '+simp(LAST_UPDATE)
  end

  # ============================================================================ #
  # === save_result
  # ============================================================================ #
  def save_result # Save the result.
    opnn; e 'Storing into `'+sfile(@store_here)+'`.'
    save_file(@_, @store_here)
  end

  # ============================================================================ #
  # === run                                                            (run tag)
  # ============================================================================ #
  def run
    show_welcome_message
    opnn; e 'We will now generate a new configuration file.'
    opnn; e 'We will store this configuration file here: '+sfile(@store_here)
    opnn; e 'We will guide you through the process of creating a'
    opnn; e 'new configuration file from scratch.'
    show_last_update
    run_through_all_configure_options
    save_result # store it into a new file.
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::GenerateRbtConfiguration.new
end # gen_rbt_config