#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::PackageRbtScripts
#
# This class will simply package the RBT Scripts.
#
# Commandline invocation example:
#   package_rbt_scripts
# =========================================================================== #
# require 'rbt/utility_scripts/package_rbt_scripts.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
begin
  require 'totarxz'
rescue LoadError; end

module RBT

class PackageRbtScripts < Base # === RBT::PackageRbtScripts

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === ARCHIVES_BASE_DIRECTORY
  # ========================================================================= #
  ARCHIVES_BASE_DIRECTORY = '/Depot/Archives/'

  # ========================================================================= #
  # === NAME_OF_THE_PROJECT
  # ========================================================================= #
  NAME_OF_THE_PROJECT = 'rbt'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_archive_dir
  # ========================================================================= #
  def set_archive_dir(
      i = ARCHIVES_BASE_DIRECTORY
    )
    i = ARCHIVES_BASE_DIRECTORY if i.nil?
    @archive_dir = i
  end

  # ========================================================================= #
  # === report_where_the_archive_dir_will_be
  # ========================================================================= #
  def report_where_the_archive_dir_will_be
    archive_dir = ARCHIVES_BASE_DIRECTORY+'rbt-'+return_date
    set_archive_dir(archive_dir)
    opnn; e 'The archive directory will be stored '\
            'at `'+sdir(@archive_dir)+'`.'
  end

  # ========================================================================= #
  # === ensure_that_archive_directory_exists
  # ========================================================================= #
  def ensure_that_archive_directory_exists
    mkdir(ARCHIVES_BASE_DIRECTORY) unless Dir.exist? ARCHIVES_BASE_DIRECTORY
  end

  # ========================================================================= #
  # === archive_dir?
  # ========================================================================= #
  def archive_dir?
    @archive_dir
  end

  # ========================================================================= #
  # === run_everything
  # ========================================================================= #
  def run_everything
    ensure_that_archive_directory_exists # The archive dir must exist.
    report_where_the_archive_dir_will_be
    _ = RUBY_SRC_DIR # /Users/x/DATA/PC/PROGRAMMING_LANGUAGES/RUBY/src/
    rbt_base_dir = _+'rbt/' # Keep a trailing / here.
    # ======================================================================= #
    # Now it is time to (1) delete an old archive directory, and then to
    # (2) create the archive directory.
    # ======================================================================= #
    remove_directory(archive_dir?) if Dir.exist? archive_dir?
    opnn; create_directory(archive_dir?, :be_verbose)
    # ======================================================================= #
    # The next step is to enter the RBT base directory.
    # ======================================================================= #
    opnn; change_directory(rbt_base_dir, :be_verbose)
    opnn; e 'We are now in the directory `'+sdir(Dir.pwd)+'`.'
    opnn; e '  Will copy '+sdir(rbt_base_dir)+' into the directory '
    opnn; e sdir('  '+archive_dir?)
    # ======================================================================= #
    # Copy recursively next.
    # ======================================================================= #
    copy_recursively(rbt_base_dir, archive_dir?)
    # ======================================================================= #
    # Ok, we have created the target directory, and a pointer to the
    # old directory. Next, we copy setup.rb but only into the newly
    # created file.
    # ======================================================================= #
    Roebe.copy_setup_rb_file
    # ======================================================================= #
    # Next, change to the dir below archive_dir?.
    # ======================================================================= #
    cd File.dirname(archive_dir?)
    # ======================================================================= #
    # Remove setup.rb first. No longer necessary I think.
    # ======================================================================= #
    # remove(archive_dir?+'setup.rb')
    # ======================================================================= #
    # === Package it up via ToTarXt
    #
    # We will delegate towards class ToTarXz for the packaging.
    # ======================================================================= #
    # new_dir_name is now the new name of the directory.
    _ = File.basename(archive_dir?)
    result = ToTarXz.new(_){ :be_verbose }.location?.dup
    set_archive_dir log_dir?.dup # Reassign here.
    opnn; e 'Now moving '+sfancy(result)+' to '+sdir(archive_dir?)
    move_file(result, archive_dir?)
    opnn; e 'Finished packaging up the '+orange('RBT Scripts')+
            ' from '
    e sdir("   #{rbt_base_dir}")+' into '
    target = archive_dir?+File.basename(result).dup+'.tar.xz'
    e sdir('   '+target)
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #  
  def run
    run_everything
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::PackageRbtScripts.new
end # package_rbt