#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ParseConfigureHelp
#
# This class is part of the RBT project.
#
# It can be used to parse the --help output from programs.
# =========================================================================== #
# require 'rbt/utility_scripts/parse_help.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ParseConfigureHelp < Base

  # ========================================================================= #
  # === CONFIGURE_STRING
  # ========================================================================= #
  CONFIGURE_STRING = './configure --help'

  # ========================================================================= #
  # === BE_VERBOSE
  # ========================================================================= #
  BE_VERBOSE = false

  # ========================================================================= #
  # Some hardcoded-colour definitions.
  # ========================================================================= #
  RED   = "\e[31m"
  CYAN  = "\e[36m"
    DEFAULT_COLOUR = CYAN
  GREEN = "\e[32m"

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = false
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @string = ''.dup # empty on default
    @be_verbose = BE_VERBOSE
  end

  # ========================================================================= #
  # === string?
  # ========================================================================= #
  def string?
    @string
  end; alias string string? # === string

  # ========================================================================= #
  # === cyan
  # ========================================================================= #
  def cyan
    CYAN
  end

  # ========================================================================= #
  # === red
  # ========================================================================= #
  def red
    RED
  end
  
  # ========================================================================= #
  # === sanitize_string
  # ========================================================================= #
  def sanitize_string(
      this_string = CONFIGURE_STRING
    ) # should this be removed??
    @string = `#{this_string}`.to_s.strip if File.exist? 'configure'
  end

  # ========================================================================= #
  # === include?
  #
  # Will return true if our flag could be found.
  # ========================================================================= #
  def include?(substring, be_verbose = @be_verbose)
    substring = substring.dup if substring.frozen?
    substring.strip! # sanitize our input string
    substring.prepend '--' unless substring.start_with? '-'
    return false if @string.empty? # can never be true if @string is empty
    if @string.include? substring
      if be_verbose
        e '%-5s' % red+'  YES, the configure option `'+substring+'` is included.'+revert
      end
      return true 
    else
      if be_verbose
        e '%-5s' % cyan+'  No, the configure option `'+substring+'` is not included.'+revert
      end
      return false
    end
  end

  # ========================================================================= #
  # === revert
  # ========================================================================= #
  def revert # Return to default colour.
    DEFAULT_COLOUR
  end

  # ========================================================================= #
  # === return_terse_version
  #
  # Return a terse version of the string, only if it contains two '-'.
  # ========================================================================= #
  def return_terse_version
    @string.split(N).reject {|x| ! x.include? '--' }
  end
  
  # ========================================================================= #
  # === verbose_include?
  #
  # Wrapper towards the method above.
  # ========================================================================= #
  def verbose_include?(s)
    @be_verbose = true
    include?(s)
  end; alias vi? verbose_include? # === vi?

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    sanitize_string
  end

end; end

if __FILE__ == $PROGRAM_NAME
  require 'cliner'
  _ = RBT::ParseConfigureHelp.new(true)
  cliner
  shall_we_debug = false
  # The next code is only used for debugging.
  if shall_we_debug
    _.vi? '--with-arts'
    _.vi? '--with-libiconv-prefix'
    _.vi? '--enable-php'
    _.vi? '--enable-ogg'
    _.vi? '--disable-arts'
    _.vi? '--without-png'
    _.vi? '--enable-sdl'
    _.vi? '--enable-filters'
    _.vi? 'enable-warnings' 
    _.vi? '--disable-plugin'
    _.vi? '--disable-dbus'
    _.vi? '--disable-mmx'
    _.vi? '-openvg'
    _.vi? '-xinerama'
    _.vi? '-xinput'
    cliner
    _.vi? '--help'
    cliner
  end
  puts _.return_terse_version
  cliner
end # phelp