#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::PurgeBinariesOfThisProgram
#
# Use this class if you want to purge binaries in /usr/bin/ and in
# /usr/sbin/.
#
# Be careful - and be sure to really want to use it, before using it.
# Ideally also have backups available.
#
# Usage example:
#   RBT::PurgeBinariesOfThisProgram.new
# =========================================================================== #
# require 'rbt/utility_scripts/purge_binaries_of_this_program.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

class PurgeBinariesOfThisProgram < Base # === RBT::PurgeBinariesOfThisProgram.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    # ======================================================================= #
    # Use a default target for @target_base_directory:
    # ======================================================================= #
    @target_base_directory = '/usr/bin/'
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = [i].flatten.compact
    if i.any? {|entry| entry.start_with? '--' }
      menu(
        i.select {|inner_entry| inner_entry.start_with? '--' }
      )
      i = i.reject {|inner_entry| inner_entry.start_with? '--'}
    end
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === purgebinof binutils --pwd
      # ===================================================================== #
      when /^-?-?pwd$/
        use_current_working_directory_as_base_directory
      end
    end
  end

  # ========================================================================= #
  # === use_current_working_directory_as_base_directory
  # ========================================================================= #
  def use_current_working_directory_as_base_directory
    @target_base_directory = return_pwd
  end

  # ========================================================================= #
  # === try_usr_sbin_variant
  # ========================================================================= #
  def try_usr_sbin_variant
    if @target_base_directory.start_with? '/usr/bin/'
      '/usr/sbin/'
    else
      @target_base_directory.gsub(/\/bin\//,'/sbin/')
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    @cookbook = RBT::Cookbooks::Cookbook.new(input?) { :bypass_menu_check }
    all_binaries = @cookbook.binaries? # <- Obtain all registered binaries.
    opnn; e 'This program has the following binaries registered:'
    e
    all_binaries.each {|entry| e "  - #{entry}" }
    e
    unless all_binaries.empty?
      all_binaries.each {|file|
        target = "#{@target_base_directory}#{File.basename(file)}"
        if File.exist? target
          opnn; remove_file(target, :be_verbose)
        elsif File.exist?(sbin_target = "#{try_usr_sbin_variant}#{File.basename(file)}")
          opnn; remove_file(sbin_target, :be_verbose)
        end
      }
    end
  end

  # ========================================================================= #
  # === RBT::PurgeBinariesOfThisProgram[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::PurgeBinariesOfThisProgram.new(ARGV)
end # purge_binaries