#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::PurgeHeadersOfThisProgram
#
# Use this class if you want to purge header files (.h files) from the
# /usr/include/ directory.
#
# Be careful - and be sure to really want to use this class, before using it.
# Ideally also have some backups available.
#
# Usage example:
#   RBT::PurgeHeadersOfThisProgram.new
# =========================================================================== #
# require 'rbt/utility_scripts/purge_headers_of_this_program.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

class PurgeHeadersOfThisProgram < Base # === RBT::PurgeHeadersOfThisProgram.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    @cookbook = RBT::Cookbooks::Cookbook.new(input?) { :bypass_menu_check }
    all_headers = @cookbook.headers? # <- Obtain all registered headers.
    opnn; e 'This program has the following headers registered:'
    e
    all_headers.each {|entry| e "  - #{entry}" }
    e
    unless all_headers.empty?
      all_headers.each {|file|
        target = "/usr/include/#{File.basename(file)}"
        if File.exist? target
          opnn; remove_file(target, :be_verbose)
        end
      }
    end
  end

  # ========================================================================= #
  # === RBT::PurgeHeadersOfThisProgram[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::PurgeHeadersOfThisProgram.new(ARGV)
end # purge_headers