#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::RemoveAllSymlinks
#
# We will remove all symlinks from either:
#
#  (1) The /Programs hierarchy
#  (2) The current directory
#
# Warning: use this class only when you are sure that you wish
# to get rid of every symlink in the /Programs directory.
#
# Usage example:
#   RBT::RemoveAllSymlinks.new
# =========================================================================== #
# require 'rbt/utility_scripts/remove_all_symlinks.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/utility_scripts/remove_symlinks.rb'

module RBT

class RemoveAllSymlinks < Base # === RBT::RemoveAllSymlinks

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_commandline_arguments = nil,
      run_already                    = true
    )
    reset
    set_input(
      optional_commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @from_here = programs_directory? # We default to /Programs, or whatever other value is stored there.
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = nil)
    i = i.first if i.is_a? Array
    @input = i
  end

  # ========================================================================= #
  # === check_commandline
  # ========================================================================= #
  def check_commandline
    case @input
    when 'PWD','Dir.pwd' # This means to work on the current directory.
      @from_here = (Dir.pwd+'/').squeeze '/'
    end
  end

  # ========================================================================= #
  # === get_all_entries
  # ========================================================================= #
  def get_all_entries(from_where = @from_here)
    _ = from_where
    @entries = Dir["#{_}*"].reject {|entry|
      ! File.directory?(entry)
    }.map {|entry| entry << '/Current' }
  end

  # ========================================================================= #
  # === remove_current_symlinks
  # ========================================================================= #
  def remove_current_symlinks
    @entries.each {|entry|
      if File.symlink? entry
        opnn; e 'Now deleting '+sfile(entry)+'.'
        remove(entry)
      end
    }
  end

  # ========================================================================= #
  # === remove_symlinks_in_main_paths
  # ========================================================================= #
  def remove_symlinks_in_main_paths
    begin
      self.new
    rescue Exception => error
      p error
    end
  end

  # ========================================================================= #
  # === notify_the_user_what_we_will_do_next
  # ========================================================================= #
  def notify_the_user_what_we_will_do_next
    opnn; e "Now removing all Symlinks from #{sdir(from_where?)}."
  end

  # ========================================================================= #
  # === from_here?
  # ========================================================================= #
  def from_here?
    @from_here
  end; alias from_where? from_here? # === from_where?

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    check_commandline
    notify_the_user_what_we_will_do_next
    get_all_entries
    remove_current_symlinks
    remove_symlinks_in_main_paths
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::RemoveAllSymlinks.new(ARGV)
end # remove_all_symlinks