#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::RemoveEmptyDirectories
#
# This class will remove empty directories. By default it will operate
# on the current working directory.
# =========================================================================== #
# require 'rbt/utility_scripts/remove_empty_directory.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class RemoveEmptyDirectories < Base # === RBT::RemoveEmptyDirectories

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      this_directory = Dir.pwd,
      run_already    = true
    )
    reset
    set_base_directory(
      this_directory
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_base_directory
  # ========================================================================= #
  def set_base_directory(i = Dir.pwd)
    i = i.dup if i.frozen?
    i << '/' unless i.end_with? '/'
    @base_directory = i
  end

  # ========================================================================= #
  # === obtain_all_directories
  # ========================================================================= #
  def obtain_all_directories
    # ======================================================================= #
    # @directories will hold the directories that will be removed.
    # They will, however had, only be removed when they are empty.
    # ======================================================================= #
    @directories = Dir["#{@base_directory}*"].select {|entry|
      File.directory? entry
    }.map {|entry| rds(entry) }
    # reject_illegal_directories
  end

  # ========================================================================= #
  # === reject_illegal_directories
  #
  # Here you can disallow certain directories. This is, however had, no
  # longer in use.
  # ========================================================================= #
  def reject_illegal_directories
    @directories.reject! {|entry|
      entry.include? '/Depot' # So that we won't remove in /Depot/Temp.
    }
  end

  # ========================================================================= #
  # === remove_empty_directories
  #
  # This will remove empty directories.
  #
  # However had, we will not remove directories that start with a '/'
  # character and have only one '/' in total.
  # ========================================================================= #
  def remove_empty_directories
    @directories.each {|entry|
      if Dir.empty?(entry)
        shall_we_remove_this_directory = true
        msg_to_the_user = 'The directory at `'+sdir(entry)+'` is empty.'
        if entry.start_with?('/') and
           entry.count('/') == 1
          shall_we_remove_this_directory = false
        else
          msg_to_the_user << ' We will remove it now.'
        end
        opnn; e msg_to_the_user
        if shall_we_remove_this_directory
          remove_directory(entry) # Only remove it if it is not something like '/run'.
        end
      end
    }
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    obtain_all_directories
    remove_empty_directories
  end

end; end

if __FILE__ == $PROGRAM_NAME 
  RBT::RemoveEmptyDirectories.new
end # rempty