#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ReportMateDesktopVersion
#
# This class will report the components of the mate desktop available
# on the given host computer.
#
# Usage example:
#   RBT::ReportMateDesktopVersion.new
# =========================================================================== #
# require 'rbt/utility_scripts/report_mate_desktop_version.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ReportMateDesktopVersion < Base # === RBT::ReportMateDesktopVersion.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === N_LJUST
  # ========================================================================= #
  N_LJUST = 22

  # ========================================================================= #
  # === ERROR_LINE
  # ========================================================================= #
  ERROR_LINE = ' 2>&1'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    if block_given?
      yielded = yield
      case yielded
      when :do_not_report,
           :do_not_report_anything
        @do_report = false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    @n_ljust   = N_LJUST
    @string_to_display = ''.dup
    @do_report = true
  end

  # ========================================================================= #
  # === add                                                         (add tag)
  # ========================================================================= #
  def add(
      i           = '',
      add_newline = true
    )
    case add_newline
    when :no_newline
      add_newline = false
    end
    if add_newline
      i = i.dup if i.frozen?
      i << "\n"
    end
    @string_to_display << i
  end

  # ========================================================================= #
  # === string_to_display?
  # ========================================================================= #
  def string_to_display?
    @string_to_display
  end; alias string? string_to_display? # === string?
       alias result? string_to_display? # === result?

  # ========================================================================= #
  # === add_newline
  # ========================================================================= #
  def add_newline
    add
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    i = [i].flatten.compact
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end

  # ========================================================================= #
  # === report_mate_common_version
  # ========================================================================= #
  def report_mate_common_version
    add left_padded+'Mate-Common: '.ljust(@n_ljust)+slateblue(mate_common?)
  end

  # ========================================================================= #
  # === left_padded
  # ========================================================================= #
  def left_padded
    grey+'  '
  end

  # ========================================================================= #
  # === report_caja_version
  # ========================================================================= #
  def report_caja_version
    add left_padded+'Caja: '.ljust(@n_ljust)+slateblue(cajav?)
  end

  # ========================================================================= #
  # === report_caja_dropbox_version
  # ========================================================================= #
  def report_caja_dropbox_version
    add left_padded+'Caja-dropbox: '.ljust(@n_ljust)+slateblue(cajadropboxv?)
  end

  # ========================================================================= #
  # === report_engrampa_version
  # ========================================================================= #
  def report_engrampa_version
    add left_padded+'Engrampa: '.ljust(@n_ljust)+slateblue(engrampav?)
  end

  # ========================================================================= #
  # === report_eom_version                                          (eom tag)
  # ========================================================================= #
  def report_eom_version
    add left_padded+'Eom: '.ljust(@n_ljust)+slateblue(eomv?)
  end

  # ========================================================================= #
  # === report_pluma_version
  # ========================================================================= #
  def report_pluma_version
    add left_padded+'Pluma: '.ljust(@n_ljust)+slateblue(plumav?)
  end

  # ========================================================================= #
  # === report_atril_version
  # ========================================================================= #
  def report_atril_version
    add left_padded+'Atril: '.ljust(@n_ljust)+slateblue(atrilv?)
  end

  # ========================================================================= #
  # === report_marco_version
  # ========================================================================= #
  def report_marco_version
    add left_padded+'Marco: '.ljust(@n_ljust)+slateblue(marcov?)
  end

  # ========================================================================= #
  # === marcov?
  # ========================================================================= #
  def marcov? 
    return_sanitized_sys_command(:marco)
  end

  # ========================================================================= #
  # === cajav?
  # ========================================================================= #
  def cajav? # Return the version of caja
    return_sanitized_sys_command(:caja)
  end

  # ========================================================================= #
  # === plumav?
  # ========================================================================= #
  def plumav?
    return_sanitized_sys_command(:pluma)
  end

  # ========================================================================= #
  # === cajadropboxv?
  # ========================================================================= #
  def cajadropboxv?
    return_sanitized_sys_command(:caja_dropbox)
  end

  # ========================================================================= #
  # === atrilv?
  # ========================================================================= #
  def atrilv?
    return_sanitized_sys_command(:atril)
  end

  # ========================================================================= #
  # === eomv?
  # ========================================================================= #
  def eomv?
    return_sanitized_sys_command(:eom)
  end

  # ========================================================================= #
  # === engrampav?
  # ========================================================================= #
  def engrampav?
    return_sanitized_sys_command(:engrampa)
  end

  # ========================================================================= #
  # === mate_utils?
  # ========================================================================= #
  def mate_utils?
    return_sanitized_sys_command(:mate_utils)
  end

  # ========================================================================= #
  # === mate_common?
  # ========================================================================= #
  def mate_common?
    return_sanitized_sys_command(:mate_common)
  end

  # ========================================================================= #
  # === mate_desktopv?
  # ========================================================================= #
  def mate_desktopv?
    return_sanitized_sys_command(:mate_desktop)
  end

  # ========================================================================= #
  # === report_mate_keyboard_version
  # ========================================================================= #
  def report_mate_keyboard_version
    add left_padded+'Mate-Keyboard: '.ljust(@n_ljust)+slateblue(mate_keyboard?)
  end

  # ========================================================================= #
  # === report_mate_mixer_version
  # ========================================================================= #
  def report_mate_mixer_version
    add left_padded+'Mate-Mixer: '.ljust(@n_ljust)+slateblue(mate_mixer?)
  end

  # ========================================================================= #
  # === report_mate_media_version
  # ========================================================================= #
  def report_mate_media_version
    add left_padded+'Mate-Media: '.ljust(@n_ljust)+slateblue(mate_media?)
  end

  # ========================================================================= #
  # === report_mate_weather_version
  # ========================================================================= #
  def report_mate_weather_version
    add left_padded+'Mate-Weather: '.ljust(@n_ljust)+slateblue(mate_weather?)
  end

  # ========================================================================= #
  # === report_mate_applets_version
  # ========================================================================= #
  def report_mate_applets_version
    add left_padded+'Mate-Applets: '.ljust(@n_ljust)+slateblue(mate_applets?)
  end

  # ========================================================================= #
  # === report_mate_control_center_version
  # ========================================================================= #
  def report_mate_control_center_version
    add left_padded+'Mate-Control-Center: '.ljust(@n_ljust)+slateblue(mate_control_center?)
  end

  # ========================================================================= #
  # === mate_keyboard?
  #
  # Return the version of libmatekbd.
  # ========================================================================= #
  def mate_keyboard?
    return_sanitized_sys_command(:mate_keyboard)
  end

  # ========================================================================= #
  # === mate_applets?
  # ========================================================================= #
  def mate_applets?
    return_sanitized_sys_command(:mate_applets)
  end

  # ========================================================================= #
  # === mate_control_center?
  # ========================================================================= #
  def mate_control_center?
    return_sanitized_sys_command(:mate_control_center)
  end

  # ========================================================================= #
  # === mate_mixer?
  # ========================================================================= #
  def mate_mixer?
    return_sanitized_sys_command(:mate_mixer)
  end

  # ========================================================================= #
  # === mate_media?
  # ========================================================================= #
  def mate_media?
    return_sanitized_sys_command(:mate_media)
  end

  # ========================================================================= #
  # === mate_weather?
  # ========================================================================= #
  def mate_weather?
    return_sanitized_sys_command(:mate_weather)
  end

  # ========================================================================= #
  # === return_pkgconfig_result_for
  # ========================================================================= #
  def return_pkgconfig_result_for(
      i = :libmatekbd
    )
    result = `pkg-config --modversion #{i} 2>&1`.strip
    if result.include?('No package ') and result.include?(' found')
      result = ''
    end
    return result
  end

  # ========================================================================= #
  # === return_sanitized_sys_command
  #
  # This method will return the proper version of an installed
  # mate-desktop component.
  # ========================================================================= #
  def return_sanitized_sys_command(
      i = :caja
    )
    case i # case tag
    when :mate_common
      i = :mate_doc_common
    when :mate_utils
      i = :mate_screenshot
    when :mate_desktop
      i = :mate_about
    end
    # ======================================================================= #
    # Next, some packages such as libmatekbd, only come with a .pc file
    # rather than a commandline way to query its version, which we will
    # determine first.
    # ======================================================================= #
    case i
    when :mate_media
      result = return_mate_media_version
    when :mate_keyboard
      result = return_pkgconfig_result_for(:libmatekbd)
    when :mate_mixer
      result = return_pkgconfig_result_for(:libmatemixer)
    when :mate_weather
      result = return_pkgconfig_result_for(:mateweather)
    when :mate_applets
      result = return_pkgconfig_result_for(:libmatepanelapplet)
    when :mate_control_center
      result = return_pkgconfig_result_for('mate-window-settings-2.0')
    else
      i = i.to_s.tr('_','-')
      result = `#{i} --version #{ERROR_LINE}`.split(N).first # system tag
    end
    result.to_s.
           sub(/MATE /,'').
           sub(/caja /,'').
           sub(/marco /,'').
           sub(/mate-screenshot /,'').
           sub(/mate-doc-common \(mate-common\) /,''). 
           sub(/Desktop Environment /,'').
           sub(/Document Viewer /,'').
           sub(/Eye of Image Viewer /,'').
           sub(/pluma - Version /,'').
           strip
  end

  # ========================================================================= #
  # === return_mate_media_version
  # ========================================================================= #
  def return_mate_media_version
    result = `mate-volume-control --version #{ERROR_LINE}`.to_s
    if result.include? ' '
      result = result.split(' ').last
    end
    result
  end

  # ========================================================================= #
  # === report_mate_desktop_version
  # ========================================================================= #
  def report_mate_desktop_version
    add left_padded+'Mate-Desktop: '.ljust(@n_ljust)+slateblue(mate_desktopv?)
  end

  # ========================================================================= #
  # === report_mate_utils_version
  # ========================================================================= #
  def report_mate_utils_version
    add left_padded+'Mate-Utils: '.ljust(@n_ljust)+slateblue(mate_utils?)
  end

  # ========================================================================= #
  # === report
  # ========================================================================= #
  def report
    e @string_to_display if @do_report
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    add grey, :no_newline
    add 'Reporting the installed versions of the '+
        royalblue('mate-desktop components')+
        grey(':')
    add_newline
    report_mate_desktop_version        # === mate-desktop
    report_mate_utils_version          # === mate-utils
    report_mate_common_version         # === mate-common
    report_mate_keyboard_version       # === libmatekbd
    report_mate_media_version          # === mate-media
    report_mate_mixer_version          # === libmatemixer
    report_mate_weather_version        # === libmateweather
    report_mate_applets_version        # === mate-applets
    report_mate_control_center_version # === mate-control-center
    report_marco_version               # === marco
    report_atril_version               # === atril
    report_eom_version                 # === eom
    report_pluma_version               # === pluma
    # ======================================================================= #
    # Engrampa presently does not work via --version switch; and no other
    # work around either. We have to wait until support for --version is
    # added.
    # report_engrampa_version          # === engrampa
    # ======================================================================= #
    report_caja_version                # === caja
    # report_caja_dropbox_version      # === caja-dropbox
    add_newline
    report
  end

  # ========================================================================= #
  # === RBT::ReportMateDesktopVersion[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end

# =========================================================================== #
# === RBT.report_mate_desktop_version.
# =========================================================================== #
def self.report_mate_desktop_version
  RBT::ReportMateDesktopVersion.new
end

# =========================================================================== #
# === RBT.return_mate_desktop_version_string
#
# This method will return the String that contains the information which
# mate-desktop components are available. That way we can then embed
# this String in other programs.
# =========================================================================== #
def self.return_mate_desktop_version_string
  RBT::ReportMateDesktopVersion.new { :do_not_report_anything }.string?
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::ReportMateDesktopVersion.new(ARGV)
end # report_mate_desktop_version