#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::ReportTheRegisteredPrograms
#
# This class will report the registered programs.
#
# It will output a line similar to the following one:
#
#   2845 programs are registered in our cookbooks as of 19.05.2016.
#   2988 programs are registered in the cookbooks as of 20.08.2017.
#
# Usage example:
#   RBT::Cookbooks::ReportTheRegisteredPrograms.new
# =========================================================================== #
# require 'rbt/utility_scripts/report_the_registered_programs.rb'
# =========================================================================== #
require 'rbt/base/prototype.rb'
require 'rbt/toplevel_methods/n_programs_available.rb'

module RBT

module Cookbooks

class ReportTheRegisteredPrograms < ::RBT::Base # === RBT::Cookbooks::ReportTheRegisteredPrograms.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === SHOW_CLASS_NAME
  # ========================================================================= #
  SHOW_CLASS_NAME = false

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    if commandline_arguments
      unless commandline_arguments.empty?
        case commandline_arguments.first # Use --short if you want to have a shorter variant instead.
        when 'do_not_show_the_class_name','--short'
          @show_class_name = false
        end
      end
    end
    if block_given?
      yielded = yield
      case yielded
      when :short
        @show_class_name = false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    @show_class_name = SHOW_CLASS_NAME
  end

  # ========================================================================= #
  # === show_class_name?
  # ========================================================================= #
  def show_class_name?
    @show_class_name
  end

  # ========================================================================= #
  # === do_report_how_many_programs_are_registered
  # ========================================================================= #
  def do_report_how_many_programs_are_registered
    opnn if show_class_name?
    e "#{sfancy(RBT.n_programs_available?)} programs are "\
      "registered in the #{orange('RBT project')} as of "\
      "#{cadetblue(return_date)}."
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    do_report_how_many_programs_are_registered
  end

end; end

  # ========================================================================= #
  # === RBT.report_the_registered_programs
  # ========================================================================= #
  def self.report_the_registered_programs(i = ARGV)
    RBT::Cookbooks::ReportTheRegisteredPrograms.new(i)
  end

  # ========================================================================= #
  # === RBT.report_n_programs_available
  # ========================================================================= #
  def self.report_n_programs_available
    RBT::Cookbooks::ReportTheRegisteredPrograms.new { :short }
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::ReportTheRegisteredPrograms.new(ARGV)
end # nprograms
    # nprograms do_not_show_the_class_name