#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::ShowDependenciesOf
#
# This class will simply show all dependencies of a given program.
#
# This is different to the class ShowDependenciesOn, which shows
# all programs that depend on the given input-program.
#
# Usage examples:
#   sdeps_of ruby
#   sdeps_of libx11
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

module Cookbooks

class ShowDependenciesOf < RBT::Base # RBT::Cookbooks::ShowDependenciesOn

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      of_this_program = ARGV,
      run_already     = true
    )
    reset
    set_of_this_program(of_this_program)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_of_this_program
  # ========================================================================= #
  def set_of_this_program(of_this_program)
    if of_this_program.is_a? Array
      of_this_program = of_this_program.first
    end
    @of_this_program = of_this_program
  end

  # ========================================================================= #
  # === program?
  # ========================================================================= #
  def program?
    @of_this_program
  end

  # ========================================================================= #
  # === report_these_dependencies
  # ========================================================================= #
  def report_these_dependencies(deps)
    if deps.empty?
      opnn; e 'No dependencies have been found '\
              'for `'+sfancy(program?.to_s)+'`.'
    else
      opnn; e 'The dependencies of `'+sfancy(program?.to_s)+'` are:'; e
      deps.each_with_index {|entry, index| index += 1
        index = index.to_s.rjust(deps.size.to_s.size)
        index = slateblue(index.to_s)
        e "  (#{index.to_s}) #{entry}"
      }; e
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    _ = RBT::Cookbooks::Cookbook.new(program?) { :bypass_menu_check }
    _.run
    deps = _.required_deps_on? # This will be an array.
    report_these_dependencies(deps)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::ShowDependenciesOf.new(ARGV)
end # sdof ruby
    # sdof gtk+