#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::SimpleVersionComparer
#
# This class will compare two versions and determine whether a statement
# such as '1.0.0 >= 2.0.0' is true or false.
#
# Usage example:
#   RBT::SimpleVersionComparer.new
# =========================================================================== #
# require "simple_version_comparer.rb"
# =========================================================================== #
require 'rbt/base/prototype.rb'

module RBT

class SimpleVersionComparer < Base # SimpleVersionComparer.new

  attr_writer :be_extra_verbose

  # ========================================================================= #
  # === TO_COMPARE
  # ========================================================================= #
  TO_COMPARE = '2.6.31 >= 2.2.2'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      to_compare  = TO_COMPARE,
      run_already = true
    )
    reset
    scan_then_compare(to_compare) if run_already
  end

  # ========================================================================= #
  # === return_value
  # ========================================================================= #
  def return_value
    @return_value
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @be_extra_verbose = false
    @return_value = false # defaults to false value.
  end

  # ========================================================================= #
  # === scan_then_compare
  #
  # By convention, valid input to this method must include a ' '.
  #
  # This is needed so that we can split on it.
  #   <=
  #   <
  #   >=
  #   >
  #   ==
  #   <>=
  #
  # The last one means "smaller or larger than", as in not equal to. So
  # it is the same as !=.
  # ========================================================================= #
  def scan_then_compare(i) # run tag
    @string_to_compare = i
    a, operator, b = @string_to_compare.split ' '
    if i.include? '<>=' # Hack for now. Always true.
      @return_value = ! (a == b)
    elsif i.include? '!='
      @return_value = ! (a == b) # Same reason as above.
    else
      @return_value = Gem::Version.new(a).send(operator, Gem::Version.new(b))
    end
    e '`'+('%15s' % @string_to_compare)+
      '` was: `'+('%3s' % operator.to_s)+ # Beautify the input here.
      '`. This is a '+@return_value.to_s+' statement.' if @be_extra_verbose
    return @return_value
  end; alias run scan_then_compare # === run

end; end

if __FILE__ == $PROGRAM_NAME # Now testing this class.
  require 'colours/autoinclude'
  _ = RBT::SimpleVersionComparer.new
  _.be_extra_verbose = true # Some tests here.
  _.scan_then_compare '1.0.1 > 2.0.0'
  _.scan_then_compare '1.2.3 >= 1.2.3'
  _.scan_then_compare '1.2.3 >= 1.2.1'
  _.scan_then_compare '1.2.3 >= 1.2.2'
  _.scan_then_compare '1.2.3 > 1.2.3'
  _.scan_then_compare '1.2.3 <>= 1.2.3'
  _.scan_then_compare '1.2.3 <>= 1.2.4'
  _.scan_then_compare '1.2.3 != 1.2.4'
  _.be_extra_verbose = false
  # value = _.scan_then_compare('2 < 3')
  # puts value
  # value = _.scan_then_compare('2 > 3')
  # puts value
end # scomp