#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::SymlinkAllPrograms
#
# This class will symlink all entries under /Programs/, one after the
# other.
# =========================================================================== #
# To require this file, do something like:
#   require 'rbt/utility_scripts/symlink_all_programs.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/symlink_this_program/initialize.rb'

module RBT

class SymlinkAllPrograms < Base # === RBT::SymlinkAllPrograms

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    register_sigint
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    set_program_dir
  end

  # ========================================================================= #
  # === symlink_all
  # ========================================================================= #
  def symlink_all
    @directories.each { |dir|
      begin
        SymlinkThisProgram.new(dir, true)
      rescue Exception => error
        opnn; pp error.to_s+' <-- found an error in class SymlinkAllPrograms.'
        opnn; ewarn 'We will continue.'
      end
    }
  end

  # ========================================================================= #
  # === report_that_we_are_finished
  # ========================================================================= #
  def report_that_we_are_finished
    opnn; e "Finished in #{orange('class RBT::SymlinkAllPrograms')}."
  end

  # ========================================================================= #
  # === set_program_dir
  # ========================================================================= #
  def set_program_dir(
      i = programs_dir?
    )
    @program_dir = i # Internal handler.
  end

  # ========================================================================= #
  # === program_dir?
  # ========================================================================= #
  def program_dir?
    @program_dir
  end

  # ========================================================================= #
  # === get_directories
  #
  # Fetch all directories in the $PROGRAMS directory.
  # ========================================================================= #
  def get_directories(of_this_directory = @program_dir)
    @directories = Dir["#{of_this_directory}*"].sort
    @directories.map! {|dir| dir = dir+'/Current/'} # @directories now contains all directories.
  end

  # ========================================================================= #
  # === run_everything
  # ========================================================================= #
  def run_everything
    reset                       # 1
    get_directories             # 2
    symlink_all                 # 3
    report_that_we_are_finished # 4
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run # run tag
    run_everything
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::SymlinkAllPrograms.new
end # sprograms