#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::SystemCompilePossibilities
#
# The job of this class is to gather the compile-possibilities that the
# given system has. This can then be made available to other classes.
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/toplevel_methods/swift_return_version_of_this_program.rb'
  
module RBT

class SystemCompilePossibilities < Base

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === TRY_TO_COMPILE_THE_PROGRAM_IF_IT_IS_OUTDATED
  #
  # If the following constant is set to true, then this class will also
  # try to compile the program at hand if it is outdated.
  # ========================================================================= #
  TRY_TO_COMPILE_THE_PROGRAM_IF_IT_IS_OUTDATED = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(commandline_arguments)
    run if run_already
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i)
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === check_gcc_version
  # ========================================================================= #
  def check_gcc_version(i = :gcc)
    local_version = EnvironmentInformation.return_gcc_version
    e "GCC Version: #{sfancy(local_version)}"
    # ======================================================================= #
    # Next check if we could update the GCC version:
    # ======================================================================= #
    current_version_is = RBT.swift_return_version_of_this_program(i)
    if local_version < current_version_is
      consider_compiling_this_program(i)
    end
  end

  # ========================================================================= #
  # === check_sed_version
  # ========================================================================= #
  def check_sed_version(i = :sed)
    local_version = EnvironmentInformation.return_sed_version
    e 'Sed Version: '+sfancy(local_version)
    current_version_is = RBT.swift_return_version_of_this_program(i)
    if local_version < current_version_is
      consider_compiling_this_program(i)
    end
  end

  # ========================================================================= #
  # === consider_compiling_this_program
  #
  # This method will honour a constant, before it will attempt to compile
  # anything.
  # ========================================================================= #
  def consider_compiling_this_program(i)
    if TRY_TO_COMPILE_THE_PROGRAM_IF_IT_IS_OUTDATED
      opnn; e 'Next trying to compile `'+sfancy(i.to_s)+'`.'
      RBT::Compile.new(i)
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    require 'environment_information'
    check_sed_version # (1)
    check_gcc_version # (2)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::SystemCompilePossibilities.new(ARGV)
end # system_compile_possibilities