#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Users
#
# This simple class contains information about Users on the host system.
# =========================================================================== #
require 'etc'
require 'rbt/base/base.rb'

module RBT # RBT::Users

class Users < Base

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      shall_we_report = false
    )
    reset
    unless is_on_windows? # skip this if we are on windows
      Etc.passwd { |user|         # user id, group id
        @passwd_info[user.name] = [ user.uid, user.gid ]
        @superuser_name = user.name if user.uid == 0
      }
    end
    report if shall_we_report
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @passwd_info = {}
  end

  # ========================================================================= #
  # === passwd_info?
  # ========================================================================= #
  def passwd_info?
    @passwd_info
  end; alias passwd_info passwd_info? # ===passwd_info

  # ========================================================================= #
  # === superuser_name?
  #
  # This method will simply return the name of the superuser.
  # ========================================================================= #
  def superuser_name?
    @superuser_name
  end
  
  # ========================================================================= #  
  # === fetch_user
  #
  # This method will specifically fetch a user.
  # ========================================================================= #
  def fetch_user(name)
    @passwd_info[name] if @passwd_info.keys.include? name
  end
  
  # ========================================================================= #
  # === report
  #
  # We will report with some useful information to the user.
  # ========================================================================= #
  def report
    cliner {
      opn; e 'Available Users ('+
      simp(@passwd_info.keys.size.to_s)+' in total):'
    }
    sort_by_uid = @passwd_info.sort_by {|key, value|
      uid = value[0]
      uid
    } 
    e; sort_by_uid.each { |entry|
      user_name = entry.first
      uid = entry[1][0].to_s
      gid = entry[1][1].to_s
      e '  User: '+Colours.bold_yellow(user_name.ljust(10))+
         sfancy('   UID: '+uid.to_s.rjust(4)+' GID: '+gid.to_s.rjust(4))
    }; e
  end; private :report
  
end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Users.new(true)
end # rusers