#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::BatchValidateTheCookbookRecipes
#
# This class will batch-validate the various cookbooks files, by invoking
# the other .rb files in this directory. It is thus some kind of "master
# control" file in regards to validation-related queries to the cookbooks
# recipes.
#
# Usage example:
#   RBT::Cookbooks::BatchValidateTheCookbookRecipes.new
# =========================================================================== #
# require 'rbt/cookbooks/validation/batch_validate_the_cookbook_recipes.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class BatchValidateTheCookbookRecipes < Base # === RBT::Cookbooks::BatchValidateTheCookbookRecipes.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(
      i
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @array_run_these_files = []
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === run_this_file
  # ========================================================================= #
  def run_this_file(i)
    if i.is_a? Array
      # ===================================================================== #
      # First, prevent self-validation .rb file.
      # ===================================================================== #
      i.reject! {|entry| entry.include? 'batch_validate_the_cookbook_recipes.rb' }
      i.each {|entry| run_this_file(entry) }
    else
      # ===================================================================== #
      # We must determine the name of the class. The usual input is
      # something like:
      #   /Programs/Ruby/2.4.2/lib/ruby/site_ruby/2.4.0/cookbooks/validation/scan_for_missing_last_update_entries.rb
      # ===================================================================== #
      load(i)
      _ = File.basename(i).sub(/\.rb$/,'').dup
      _ = to_camelcase(_)
      namespace = 'RBT::Cookbooks::'+_
      opnn; e "Now invoking #{slateblue('class '+namespace)}"
      constant = RBT::Cookbooks.const_get(_)
      constant.send(:new) # Instantiate it here.
    end
  end; alias run_these_files run_this_file # === run_these_files

  # ========================================================================= #
  # === determine_which_files_to_run
  # ========================================================================= #
  def determine_which_files_to_run
    @array_run_these_files << Dir["#{directory_validation?}*.rb"]
    @array_run_these_files.flatten!
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    determine_which_files_to_run
    run_these_files(
      @array_run_these_files
    )
  end

  # ========================================================================= #
  # === RBT::Cookbooks::BatchValidateTheCookbookRecipes[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

# =========================================================================== #
# === RBT.validate_entries
#
# Batch-valide the cookbook recipes here.
# =========================================================================== #
def self.validate_entries
  RBT::Cookbooks::BatchValidateTheCookbookRecipes.new
end
  
end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::BatchValidateTheCookbookRecipes.new(ARGV)
end # batch_validate_the_cookbook_recipes