#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Sinatra
#
# require 'rbt/www/app.rb'
# class Foobar < Rbt::Base
# =========================================================================== #
require 'sinatra'
require 'colours'
require 'html_tags'
require 'beautiful_url'
require 'open-uri'
require 'json'
require 'rbt/base/base.rb'
require 'rbt/toplevel_methods/report_extended_information_about_this_program.rb'
require 'rbt/toplevel_methods/available_programs.rb'

module RBT

class Sinatra < ::Sinatra::Base

  include Colours::E
  include HtmlTags::BaseModule

  # ========================================================================= #
  # === USE_THIS_PORT
  # ========================================================================= #
  USE_THIS_PORT = '5580'

  set :port, USE_THIS_PORT

  # ========================================================================= #
  # Try to enable sessions:
  # ========================================================================= #
  enable :sessions

  # ========================================================================= #
  # === before                                                     (before tag)
  # ========================================================================= #
  before {
    content_type(:html, 'charset' => 'ISO-8859-1')
  }

  # ========================================================================= #
  # === IS_ROEBE
  # ========================================================================= #
  IS_ROEBE = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize
    super()
    reset
    if IS_ROEBE
      require 'open_in_browser'
      # ===================================================================== #
      # Tell us which port will be used:
      # ===================================================================== #
      target = "http://localhost:#{USE_THIS_PORT}/"
      e sfancy(target)
      Thread.new {
        sleep 1.0
        OpenInBrowser[target]
      }
    end
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
  end

  # ========================================================================= #
  # === / Entry point
  #
  # This is the root of our web-application.
  # ========================================================================= #
  get('/') {
    return_main_index
  }

  # ========================================================================= #
  # === return_main_index
  # ========================================================================= #
  def return_main_index
    'This is the www-interface for the RBT project, using <b>sinatra</b>.'+
    return_available_actions
  end

  # ========================================================================= #
  # === return_available_actions
  # ========================================================================= #
  def return_available_actions
    p('The following entry points are available:')+
    p(css_style: 'padding:10px; margin-left: 1em') {
      a('/view', content: '/view', css_style: :bold)+br  # /view
    }
  end

  # ========================================================================= #
  # === return_to_root_link
  # ========================================================================= #
  def return_to_root_link
    p {
      a('/', content: 'Home')
    }
  end

  # ========================================================================= #
  # === /view
  #
  # http://localhost:5580/view
  # ========================================================================= #
  get('/view') {
    p('View the following package:')
    route_to_this_action = '/view/'
    div(css_style: 'padding: 0.2em') {
      p('<b>Display this program:</b>', css_style: 'padding:0.75em')+
      '<form id="view" style="margin-left:2em; margin-top:2px" action="'+route_to_this_action+'">
      <input type="text" name="user_input" style="border:3px solid slateblue; padding: 4px"><br>
      <input type="submit" name="user_input_submit" value="Display" style="2px dotted royalblue; '\
      'color: white; background-color: olive; font-size: 1.50em; margin:2px; padding: 4px">
      </form>'+
      return_to_root_link
    }
  }
  # ========================================================================= #
  # === /view
  #
  # http://localhost:5580/view/php
  # ========================================================================= #
  get('/view/*'){
    use_these_params = params[:splat]
    if params.has_key?('user_input')
      use_these_params = params.fetch('user_input')
    end
    result = ''.dup
    if RBT.does_include? use_these_params
      _ = RBT::Cookbooks.new(use_these_params)
      program_path = _.program_path?
      description  = _.description?
      program_name = _.program_name?
      result << p(
        '<b>Program path</b>: '+program_path+"<br>\n"+
        '<b>Description</b>: '+description+"<br>\n"+
        '<b>Program name</b>: '+program_name+"<br>\n"
      )
    else
      result << p('The program '+use_these_params+' is not included.')
    end
    result
  }

end

# =========================================================================== #
# === rBT.run_app
# =========================================================================== #
def self.run_app
  RBT::Sinatra.run!
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.run_app
end